/*
 * Copyright 2008  Rivo Laks <rivolaks@hot.ee>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ASTEROIDGEN_H
#define ASTEROIDGEN_H

#include <QString>
#include <QList>
#include <QHash>
#include <QPair>

#include <Eigen/Core>

using namespace Eigen;


class Triangle
{
public:
    Triangle()  { a = b = c = -1; }
    Triangle(int s, int t, int u)  { a = s; b = t; c = u; }

    int a, b, c;
    Vector3f normal;
};

class Vertex
{
public:
    Vertex()  {}
    Vertex(const Vector3f& p, const Vector2f& t)  { pos = p; tex = t; }
    Vertex(const Vector3f& p, const Vector3f& n, const Vector2f& t)  { pos = p; normal = n; tex = t; }
    Vertex(const Vertex& a, const Vertex& b)
    {
        pos = (a.pos + b.pos) / 2;
        pos *= ((a.pos.norm() + b.pos.norm()) / 2) / pos.norm();  // Make pos's length equal to avg of lengths of a.pos and b.pos
        tex = (a.tex + b.tex) / 2;
    }

    //int id;
    Vector3f pos;
    Vector3f normal;
    Vector2f tex;
};


/**
 * @author Rivo Laks <rivolaks@hot.ee>
 */
class AsteroidGen
{
public:
    AsteroidGen();
    ~AsteroidGen();

    bool generate();

    bool saveLod(int lod, const QString& filename);
    bool saveModel(const QString& filenameTemplate, const QString& indexfile);
    bool saveModel(const QString& filenamePrefix);

    QString modelParameters() const;
    void setModelParameters(const QString& params);


    QList<Vertex> vertices;
    QList<Triangle>* faces;
    int currentLodCount;

    int seed;
    int lodCount;
    float randomInitial;
    float randomIterative;
    float randomDamping;
    float randomPost;

protected:
    int getVertexIndex(int parenta, int parentb);

private:
    QHash<QPair<int, int>, int> edgeMapper;
};

#endif
