/*
 * Copyright 2008  Rivo Laks <rivolaks@hot.ee>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "modelviewer.h"

#include "asteroidgen.h"

#include <kgllib/camera.h>

#include <QMouseEvent>
#include <QtDebug>


ModelViewer::ModelViewer(QWidget* parent) : GLWidget(parent)
{
}

void ModelViewer::mousePressEvent(QMouseEvent* e)
{
    lastMousePos = e->pos();
}

void ModelViewer::mouseMoveEvent(QMouseEvent* e)
{
    int dx = e->x() - lastMousePos.x();
    int dy = e->y() - lastMousePos.y();
    if (e->buttons() & Qt::LeftButton) {
        mTrackball.rotate(dx, dy);
        update();
    }
    lastMousePos = e->pos();
}

void ModelViewer::setGenerator(AsteroidGen* generator)
{
    mGenerator = generator;
    update();
}

void ModelViewer::initializeGL()
{
    GLWidget::initializeGL();

    QString datapath = "data/";
    mTexture = new Texture(datapath + "textures/asteroid.png");

    // Load shaders
    QString shaderdir = datapath + "shaders/";
    // asteroid
    mProgram = new Program(shaderdir + "asteroid.vert", shaderdir + "asteroid.frag");
    if (!mProgram->isValid()) {
        qDebug() << "Invalid program";
    }
    mProgram->bind();
    mProgram->setUniform("diffuseTexture", 0);
    mProgram->unbind();
    glLightfv(GL_LIGHT0, GL_POSITION, Vector4f(-100000.0, 60000.0, 20000.0, 1.0).data());

    camera()->setPosition(Vector3f(0, 0, 3));
}

void ModelViewer::render()
{
    if (!mGenerator || !mGenerator->faces) {
        return;
    }
    checkGLError("render begin");

    mTrackball.transform(*camera()).applyView();

    mProgram->bind();
    mTexture->enable();

    glBegin(GL_TRIANGLES);
    QList<Triangle> faces = mGenerator->faces[mGenerator->currentLodCount-1];
    foreach (Triangle t, faces) {
        renderVertex(mGenerator->vertices[t.a]);
        renderVertex(mGenerator->vertices[t.b]);
        renderVertex(mGenerator->vertices[t.c]);
    }
    glEnd();

    mProgram->unbind();
    mTexture->disable();
}

void ModelViewer::renderVertex(const Vertex& v)
{
    glNormal3fv(v.normal.data());
    glTexCoord2fv(v.tex.data());
    glVertex3fv(v.pos.data());
}

