uniform sampler2D diffuseTexture;
uniform sampler2D glossTexture;
uniform sampler2D emissionTexture;

varying vec4 vertexEye;


vec3 calculateMaterial(vec3 L, vec3 Lcolor, vec3 N)
{
    vec3 texColor = texture2D(diffuseTexture, gl_TexCoord[0].xy).rgb;
    float gloss = texture2D(glossTexture, gl_TexCoord[0].xy).r;

    vec3 specular = vec3(3.0)*gloss;
    float specularity = 30.0;

    float NdotL = max(dot(N, L), 0.0);

    float spec = 0.0;
    if (NdotL > 0.0) {
        vec3 R = 2.0 * NdotL * N - L;
        vec3 V = normalize(-vertexEye.xyz);
        spec = pow(max(dot(R, V), 0.0), specularity);
    }

    return (texColor * NdotL + specular*spec) * Lcolor;
}

vec3 calculateMaterialAmbient(vec3 Lcolor)
{
    vec3 texColor = texture2D(diffuseTexture, gl_TexCoord[0].xy).rgb;
    
    return Lcolor * texColor;
}
