/*
    Copyright 2009  Rivo Laks <rivolaks@hot.ee>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef LOADINGVIEW_H
#define LOADINGVIEW_H

#include "glview.h"

#include <Eigen/Core>

#include <QRect>
#include <QFont>
#include <QHash>

class QDomElement;
class GameWorld;
class GameObject;

namespace KGLLib
{
    class GeometryBuffer;
}

class LoadingView : public GLView
{
    Q_OBJECT
    public:
        LoadingView(KGLLib::GLWidget* parent, GameWorld* world, const QString& levelname);

        virtual void init();
        virtual void render();
        virtual void resize(int w, int h);

        bool load();

    signals:
        void loadingComplete();


    protected:
        bool loadLevel(const QString& levelname);

        bool loadObjects(const QDomElement& elem);
        bool loadAsteroidField(const QDomElement& elem);
        bool loadObject(const QDomElement& elem);
        bool loadVector(const QDomElement& elem, Eigen::Vector3f* pos);
        bool loadTrack(const QDomElement& elem);
        bool loadPlayer(const QDomElement& elem);
        bool loadPlanet(const QDomElement& elem);

        bool tryLoad();

    protected slots:
        void loadingAsteroid(QString name, int i, int total);

    protected:
        enum LoadingState
        {
            Load_NotStarted = 0,
            Load_InitWorld,
            Load_AsteroidModelInfos,
            Load_LevelXML,
            Load_ObjectModelInfos,
            Load_FinalizeModels,
            Load_PrecompileShaders,
            Load_AssignObjectModels,
            Load_AssignAsteroidModels,
            Load_Done
        };
        struct LoadingStateInfo
        {
            LoadingStateInfo(float p, const QString& s)
            {
                progress = p;
                status = s;
            }
            LoadingStateInfo()
            {
                progress = 0.0f;
            }

            float progress;
            QString status;
        };

        void updateProgress(LoadingState currentState, int substate = -1, int substateCount = -1);

        GameWorld* mWorld;
        QString mLevelName;

        QHash<LoadingState, LoadingStateInfo> mStateInfos;
        float mProgress;
        QString mStatus;
        bool mFailed;

        QRect mProgressRect;
        QRect mStatusTextRect;
        QFont mStatusTextFont;
        QRect mLoadingTextRect;
        QFont mLoadingTextFont;

        // For each model name, stores all objects using that model
        QHash<QString, QList<GameObject*> > mModel2Objects;
};

#endif // LOADINGVIEW_H
