/***************************************************************************
 *   Copyright (C) 2009 by Alexander Smirnov                               *
 *   alexander.v.smirnov@gmail.com                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 ***************************************************************************/

#ifndef BACKGAMMONSCENE_H
#define BACKGAMMONSCENE_H

#include "games/backgammon/backgammongame.h"
#include <QGraphicsScene>
#include <kdebug.h>

/**
	@author Alexander Smirnov <alexander.v.smirnov@gmail.com>
*/
class Checker;

class BackgammonScene : public QGraphicsScene
{
    Q_OBJECT
public:
    BackgammonScene(BackgammonGame * game=NULL);

    ~BackgammonScene();

    /**
     *  Performs scene resize - rescales all corresponding svg's,
     *  recalcs playfield rect etc etc
     */
    void resizeScene( int width, int height );

    /**
     * Sets a game to be visualised with the scene
     * Scene takes ownership and will delete the game on destruction
     */
    void setGame(IGameEngine * game);
    
public slots:
    void slotShowDice( qint8 d1, qint8 d2);
    void moveChecker (qint8 cellFrom, qint8 cellTo);
    void arrangeCheckers( const QStringList& newBoardState );
    void collectCheckers();
    
    void showEngineMessage( const QString& message ); 
    void showProposedMoves( const QList<int>& indexes, Checker::Color color );
    void unshowProposedMoves();
    
signals:
    void mPressed(QString info);

protected:
    virtual void drawBackground( QPainter *p, const QRectF& rect );

    void updateObjects();

    void stopGame();

private:

    QPointF getCheckerPosition(Checker *checker) const;

    QPointF getCheckerPosition(int cellIdx, int order) const;

    void setCheckerPosition(Checker *checker);

    void setCheckerZValue(Checker *checker);

    void addCheckersToScene(Checker::Color col );

//     TODO: add board edit mode
//     void addCheckerToBoard( qint8 cellId, Checker::Color color);

    int numberOfCheckersInARow(const qint8 row) const;

    int checkerRow(const int order) const;

    int checkerSeq(const int order) const;

    qreal getCheckerY(int cellIdx, int order) const;

    double checkerSize() const;

    double dieSize() const;

    void moveChecker(Checker * checker, int cellTo);

    /**
     *  Mouse presses event handler
     */
    virtual void mousePressEvent( QGraphicsSceneMouseEvent* );

    // smiling: )
    class Private;
    Private *d;
};

#endif
