/***************************************************************************
 *   Copyright (C) 2009 by Alexander Smirnov                               *
 *   alexander.v.smirnov@gmail.com                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 ***************************************************************************/

#ifndef BOARD_H
#define BOARD_H

#include <QVector>
#include <QList>
#include <QMultiMap>

#include "dice.h"
#include "checker.h"
/**
	@author Alexander Smirnov <alexander.v.smirnov@gmail.com>
*/
class Board{
public:

    static qint8 numberOfDice() {return m_numberOfDice;}
    static Die* die(qint8 n);
    static void showDice();
    static void hideDice();

    static qint8 numberOfCheckersAt(qint8 cellIdx);
    static QList<Checker*> checkersAt(qint8 cellIdx);
    static void addChecker(qint8 cellIdx, Checker * checker);
    static Checker* getLastChecker(qint8 cellIdx);
    static void setCheckerSize(int size);
    static qint8 numberOfCheckers() {return m_numberOfCheckers;}
    static void moveChecker(Checker * checker, qint8 cellTo);
    static Checker::Color checkerColor(qint8 checker); //TODO; move it upper?

    //test idx if it is valid cell index
    static bool indexValid(qint8 idx);
    static Checker::Color cellColor(qint8 idx);
    static qint8 cellNumber(const QPointF& point) ;

    static qint8 home(Checker::Color);
    static qint8 bar(Checker::Color);

    static void resize(int width, int height );
    static const QRectF& rect() {return self()->m_boardRect;}

    static const QRectF& cellRect(const qint8 cellIdx);

    static const qint8 BLACK_POOL = 26;
    static const qint8 WHITE_POOL = 27;
    static const qint8 BLACK_BAR = 24;
    static const qint8 WHITE_BAR = 25;

    static QString state();
    
    static void setCheckerPositions();

private:
    static Board* self();
    Board();
    ~Board(){}

//prohibited:
    Board( const Board & );
    Board& operator= ( const Board & );

private:
    static const qint8 m_numberOfCells = 28;
    static const qint8 m_numberOfCheckers = 15;
    static const qint8 m_numberOfDice = 2;

    //stores checkers for every board cell
    // 0..23 - board cells
    // 24,25 - bar
    // 26,27 - home
    QMultiMap<qint8, Checker*> m_checkers;

    /**
     * Position of the board within the scene
     */
    QRectF m_boardRect;

    QVector<Die*> m_dice;

    /**
     * this array caches info for every board cell
     * Cell bounds should be updated every time the scene gets resized
     */
    mutable QRectF m_cellBounds[m_numberOfCells+1];
};

#endif
