/***************************************************************************
 *   Copyright 2006-2007 Dmitry Suzdalev <dimsuz@gmail.com>                *
 *                                                                         *
 *   This file is part of the KDE project "KReversi"                       *
 *   Was adapted for needs of KDE backgammon game                          *
 *   alexander.v.smirnov@gmail.com                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 ***************************************************************************/
#include "boardrenderer.h"
#include <kstandarddirs.h>
#include <ksvgrenderer.h>
#include <kpixmapcache.h>

#include <kdebug.h>

#include <QPainter>
#include <QApplication>

BoardRenderer* BoardRenderer::self()
{
    static BoardRenderer instance;
    return &instance;
}

BoardRenderer::BoardRenderer():Renderer()
{
    checkerElementId.insert(Checker::Black, "black_checker");
    checkerElementId.insert(Checker::White, "white_checker");
}

BoardRenderer::~BoardRenderer()
{
}

void BoardRenderer::renderBackground( QPainter *p, const QRectF& r ) 
{
    self()->renderElement(p, "background", r);
}

void BoardRenderer::renderBoard( QPainter *p, const QRectF& r ) 
{
    self()->renderElement(p, "board", r);
}

void BoardRenderer::renderBoardLabels( QPainter *p, const QRectF& r ) 
{
    self()->renderElement(p, "board_numbers", r);
}

void BoardRenderer::renderChip( QPainter *p, const QRectF& r, Checker::Color color ) 
{
    self()->renderElement(p, self()->checkerElementId.value(color), r);
}

void BoardRenderer::renderDie( QPainter *p, const QRectF& r, int die ) 
{
    self()->renderElement(p, "die"+QString::number(die), r);
}

void BoardRenderer::renderCube( QPainter *p, const QRectF& r, int cube ) 
{
    self()->renderElement(p, "cube"+QString::number(cube), r);
}

void BoardRenderer::renderPossibleMove( QPainter *p, const QRectF& r  ) 
{
    self()->renderElement( p, "move_hint", r );
}

QRectF BoardRenderer::defaultBoardRect() 
{
    return self()->elementRect("board");
}

QRectF BoardRenderer::defaultCellRect(qint8 cellIdx) 
{
    return self()->elementRect("cell"+ QString::number(cellIdx));
}

QSize BoardRenderer::defaultBoardSize() 
{
    return self()->elementSize("board");
}

QSize BoardRenderer::defaultBoardNumbersSize() 
{
    return self()->elementSize("board_numbers");
}

QSize BoardRenderer::defaultTableSize() 
{
    return self()->elementSize("background");
}

QSize BoardRenderer::defaultCheckerSize() 
{
    return self()->elementSize("checker_bw_1");
}

QSize BoardRenderer::defaultDiceSize() 
{
    return self()->elementSize("die1");
}

QSize BoardRenderer::defaultCubeSize() 
{
    return self()->elementSize("cube2");
}
