/***************************************************************************
 *   Copyright (C) 2009 by Alexander Smirnov                               *
 *   alexander.v.smirnov@gmail.com                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 ***************************************************************************/

#include "dice.h"
#include "boardrenderer.h"
#include "settings.h"

#include <QPainter>
#include <kdebug.h>

Die::Die(qint8 order)
 : QObject(), m_order(order),m_opacity(1),m_timer(0)
{
    setVisible(false);
}


Die::~Die()
{
}

void Die::updatePixmap(int num, double size)
{
    QGraphicsPixmapItem::setPixmap(QPixmap::fromImage(getImage(num,size)));
    m_number = num;
    setVisible(true);
}

void Die::updatePixmap(double size)
{
    QGraphicsPixmapItem::setPixmap(QPixmap::fromImage(getImage(m_number,size)));
}

void Die::animate(QTimeLine::Direction direction)
{
    slotStopAnimation();
    m_timer = new QTimeLine(1000/Settings::animationSpeed());
    m_timer->setFrameRange(0, 30);
    m_timer->setDirection(direction);
    connect( m_timer, SIGNAL( valueChanged(qreal) ), SLOT( slotFade(qreal) ));
    connect( m_timer, SIGNAL( finished() ), SLOT( slotStopAnimation() ) );
    m_timer->start();
}

void Die::slotStopAnimation()
{
    if (!m_timer)
        return;

    m_timer->stop();

    if (m_timer->direction() == QTimeLine::Forward)
    {
        slotFade(1.0); //show
    }
    else
    {
        slotFade(0.0); //hide
    }
    delete m_timer;
    m_timer = 0;
}

void Die::show()
{
    if (!Settings::animationEnabled())
    {
        slotFade( 1 );
    }
    else
    {
        m_opacity = 0;
        animate(QTimeLine::Forward);
    }
}

void Die::hide()
{
    if (!Settings::animationEnabled())
    {
        slotFade( 0 );
    }
    else
    {
        m_opacity = 1;
        animate(QTimeLine::Backward);
    }
}

void Die::slotFade(qreal val)
{
    m_opacity = val;
    update();
}



void Die::paint( QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget )
{
    painter->setOpacity( m_opacity );
    QGraphicsPixmapItem::paint( painter, option, widget );
}

QImage Die::getImage (int num, double size)
{
    QImage img(size, size, QImage::Format_ARGB32_Premultiplied);
    img.fill(0);

    QPainter p(&img);
    const QRectF rectangle(0,0,size,size);
    BoardRenderer::renderDie( &p, rectangle, num );
    p.end();

    return img;
}

void Die::mousePressEvent ( QGraphicsSceneMouseEvent * event )
{
    kDebug() << "die mouse press";
}
