/*
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA  02110-1301, USA.

    ---
    Copyright (C) 2011, Dmitry Chernov diman4ik.chernov@gmail.com
 */

#include "backgammon_fibs.h"
#include "backgammon_fibs_connect.h"


#include "clip.h"

#include "iplayer.h"
#include "humanplayer.h"

#include <KDebug>
#include <klocale.h>
#include <QDateTime>
#include <QGraphicsSceneMouseEvent>


BackgammonFibs::BackgammonFibs( QObject *parent ) : BackgammonGame(parent),
                                                    m_needBoard(false)
{
    /*
     * Pattern setup - rather long and boring
     */
    QString pattern;

    m_patternBanner           = QRegExp( "^WELCOME TO THE" );
    m_patternYourLogin        = QRegExp( "^login:" );
    m_patternServerTime       = QRegExp( " UTC" );
    m_patternWelcome          = QRegExp( pattern.sprintf( "^%d ", CLIP_WELCOME ) );
    m_patternOwnInfo          = QRegExp( pattern.sprintf( "^%d ", CLIP_OWN_INFO ) );
    m_patternMotdBegin        = QRegExp( pattern.sprintf( "^%d", CLIP_MOTD_BEGIN ) );
    m_patternMotdEnd          = QRegExp( pattern.sprintf( "^%d", CLIP_MOTD_END ) );
    m_patternWhoInfoBegin     = QRegExp( pattern.sprintf( "^%d ", CLIP_WHO_INFO ) );
    m_patternWhoInfoEnd       = QRegExp( pattern.sprintf( "^%d", CLIP_WHO_END ) );
    m_patternShouts           = QRegExp( pattern.sprintf( "^%d ", CLIP_SHOUTS ) );
    m_patternSays             = QRegExp( pattern.sprintf( "^%d ", CLIP_SAYS ) );
    m_patternWhispers         = QRegExp( pattern.sprintf( "^%d ", CLIP_WHISPERS ) );
    m_patternMessage          = QRegExp( pattern.sprintf( "^%d ", CLIP_MESSAGE ) );
    m_patternMessDelivered    = QRegExp( pattern.sprintf( "^%d ", CLIP_MESSAGE_DELIVERED ) );
    m_patternMessSaved        = QRegExp( pattern.sprintf( "^%d ", CLIP_MESSAGE_SAVED ) );
    m_patternKibitzes         = QRegExp( pattern.sprintf( "^%d ", CLIP_KIBITZES ) );
    m_patternYouSays          = QRegExp( pattern.sprintf( "^%d ", CLIP_YOU_SAY ) );
    m_patternYouShouts        = QRegExp( pattern.sprintf( "^%d ", CLIP_YOU_SHOUT ) );
    m_patternYouWhispers      = QRegExp( pattern.sprintf( "^%d ", CLIP_YOU_WHISPER ) );
    m_patternYouKibitzes      = QRegExp( pattern.sprintf( "^%d ", CLIP_YOU_KIBITZ ) );
    m_patternUnknown          = QRegExp( pattern.sprintf( "^%s ", "**" ) );
    m_patternLogins           = QRegExp( pattern.sprintf( "^%d ", CLIP_LOGIN ) );
    m_patternLogouts          = QRegExp( pattern.sprintf( "^%d ", CLIP_LOGOUT ) );    
    m_patternInviteConfirm    = QRegExp( "You invited" );
    m_patternGameReload1      = QRegExp( "^You are now playing with " );
    m_patternGameReload2      = QRegExp( " has joined you. Your running match was loaded." );
    m_patternMatchBegin1      = QRegExp( " has joined you for a [0-9]+ point match" );
    m_patternMatchBegin2      = QRegExp( " has joined you for an unlimited match\\.$" );
    m_patternMatchBegin3      = QRegExp( "^\\*\\* You are now playing a [0-9]+ point match with " );
    m_patternMatchBegin4      = QRegExp( "^\\*\\* You are now playing an unlimited match with " );
    m_patternYourTurn         = QRegExp( "^It's your turn" );
    m_patternPleaseMove       = QRegExp( "^Please move [1-6]+ pie" );
    //m_patternBoard            = QRegExp( "^board:" );
    /*m_patternGamesBeg1        = QRegExp("start a [0-9]+ point match");
    m_patternGamesBeg2        = QRegExp("start an unlimited match");*/
    m_patternGameBegin        = QRegExp( "^Starting a new game with " );
    
    //m_patternGamesRes1        = QRegExp( "are resuming their [0-9]+-point match" );
    //m_patternGamesRes2        = QRegExp( "are resuming their unlimited match" );
    m_patternBoardBegin       = QRegExp( "^   +" );
    m_patternBoardEnd         = QRegExp( "^   BAR" );
    //m_patternRoll             = 
    
    m_patternInviteResume     = QRegExp( " wants to resume a saved match with you" );
    m_patternInvitePlay       = QRegExp( " wants to play a [0-9]+ point match with you" );
    m_patternInviteEnd        = QRegExp( "Type 'join" );
    
    m_patternOpponentFirst    = QRegExp( "makes the first move." );
    m_patternOpponentMove     = QRegExp( "moves [0-9]{1,2}-[0-9]{1,2}|moves bar-[0-9]{1,2}|moves [0-9]{1,2}-off" );
    m_patternRoll             = QRegExp( "You roll [0-9] and [0-9]" );
    m_patternOpponentRoll     = QRegExp( " rolls [0-9] and [0-9]." );
    m_patternOpponentDoubles  = QRegExp( " doubles. Type" );
    
    m_patternAcceptDone       = QRegExp( "You accept the double" );
    
    m_patternInfo             = QRegExp( "^\\*\\*" );
    
    m_patternYouWin           = QRegExp( "You win the " );
    m_patternYouLost          = QRegExp( " wins the game" );
    
    m_patternTurn             = QRegExp( "^turn: " );
    m_patternMatchLength      = QRegExp( "^match length: " );
    m_patternPointsFor        = QRegExp( "^points for " );
    
    m_patternCantMove         = QRegExp( "can't move" );
    
    m_patternBlind            = QRegExp("^\\*\\* You blind ");
    m_patternUnblind          = QRegExp("^\\*\\* You unblind ");
    m_patternStartWatch       = QRegExp("^You're now watching ");
    m_patternStopWatch        = QRegExp("^You stop watching ");

    m_patternWave1            = QRegExp(" waves goodbye again.$");
    m_patternWave2            = QRegExp(" waves goodbye.$");
    m_patternWave3            = QRegExp("^You wave goodbye.$");

    m_patternMatchEndLost     = QRegExp(" wins the [0-9]+ point match [0-9]+-[0-9]+");
    m_patternMatchEndWon      = QRegExp(" win the [0-9]+ point match [0-9]+-[0-9]+");
    
    m_connection = new BackgammonFibsConnect( this );

    /* 
     * connect the various slots
     */
    connect( m_connection, SIGNAL(connectionDown()), this, SLOT(serverDisconnected()) );
    connect( m_connection, SIGNAL( newData( QString& ) ), this,  SLOT( handleServerData( QString& ) ) );
    connect( this, SIGNAL(serverString(const QString&)), m_connection,  SLOT(sendData(const QString&)) );
    
    // Player plays white
    m_players[0] = new HumanPlayer(Checker::Black, "Player1");
    // Opponent plays red
    m_players[1] = new HumanPlayer(Checker::White, "Player2");
    
    getYou().setDirection( IPlayer::negative );
    getOpponent().setDirection( IPlayer::positive );
    
    //showMove( 0, 1 );
    m_yourTurn = false;
}
    
BackgammonFibs::~BackgammonFibs()
{
    if ( m_connection ) 
        delete m_connection;
}

/*
 * Parse an incoming line and notify all interested parties - first match 
 * decides.
 */
void BackgammonFibs::handleServerData( QString& data )
{
    //kDebug() << "data " << data;
    
    QString line;
    
    while( ( line = getLine( data ) ).length() > 0 )
    {
        pattern next = getPattern( line );
        
        if( line.contains( m_patternWhoInfoBegin ) ) 
        {
                
            kDebug() << "who info";
            
            line.replace( m_patternWhoInfoBegin, "" );
            getWhoInfo(line);
            
            for(;;)
            {
                QString new_line = getLine( data );
                
                if( new_line.contains( m_patternWhoInfoEnd ) )
                    break;
                if( new_line.length() == 0 )
                    break;

                new_line.replace( m_patternWhoInfoBegin, "" );
                getWhoInfo( new_line );
            }
            continue;
        }
        
        if( line.contains( m_patternWhoInfoEnd ) )
            continue;
   
        if( line.contains( m_patternBanner ) )
        {
            m_state = connected;
            kDebug() << "banner";
            emit( serverInfo(line) );
            QString banner = getAll( data, m_patternYourLogin );
            emit( serverInfo(banner) );
            continue;
        }
        
        if( line.contains( m_patternYourLogin ) )
            continue;
        
        if( line.contains( m_patternWelcome ) )
        {
            kDebug() << "welcome";
            line.replace( m_patternWelcome, "" ); 
            emit( serverInfo( line ) );
            continue;
        }
        
        if( line.contains( m_patternOwnInfo ) )
        {
            m_state = authorized;
            line.replace( m_patternOwnInfo, "" );
            getMyOwnInfo(line);
            continue;
        }
        
        if( line.contains( m_patternMotdBegin ) )
        {
            // Read all of the message and show it
            kDebug() << "message of the day begin";
            
            QString motd = getAll( data, m_patternMotdEnd );
            emit( serverInfo( motd ) );
            continue;
        }
        
        if( line.contains( m_patternShouts ) ) 
        {
            line.replace( m_patternShouts, "" );
            emit( fibsChatInformation( line ) );
            continue;
        }
        
        if( line.contains( m_patternSays ) ) 
        {
            line.replace( m_patternSays, "" );
            emit( fibsChatInformation( QString( "says: " ) + line ) );
            continue;
        }
        
        if( line.contains( m_patternWhispers ) ) 
        {
            line.replace( m_patternWhispers, "" );
            emit( fibsChatInformation( QString( "whispers: " ) + line ) );
            continue;
        }
        
        if( line.contains( m_patternMessage ) )
        {
            line.replace( m_patternMessage, "" );
            emit( fibsMessage( QString( "message: " ) + line ) );
            continue;
        }
        
        if( line.contains( m_patternMessDelivered ) )
        {
            line.replace( m_patternMessDelivered, "" );
            emit( serverInfo( QString( "delivered message to " ) + line ) );
            continue;
        }
        
        if( line.contains( m_patternMessSaved ) )
        {
            line.replace( m_patternMessSaved, "" );
            emit( serverInfo( QString( "saved message to " ) + line ) );
            continue;
        }
        
        if( line.contains( m_patternKibitzes ) ) 
        {
            line.replace( m_patternKibitzes, "" );
            emit( fibsChatInformation( line ) );
            continue;
        }
        
        if( line.contains( m_patternYouSays ) )
        {
            line.replace( m_patternYouSays, "" );
            emit( serverInfo( QString( "delivered : " ) + line ) );
            continue;
        }
        
        if( line.contains( m_patternYouShouts ) ) 
        {
            line.replace( m_patternYouShouts, "" );
            emit( serverInfo( QString( "delivered : " ) + line ) );
            continue;
        }
        
        if( line.contains( m_patternYouWhispers ) ) 
        {
            line.replace( m_patternYouWhispers, "" );
            emit( serverInfo( QString( "delivered : " ) + line ) );
            continue;
        }
        
        if( line.contains( m_patternYouKibitzes ) ) 
        {
            line.replace( m_patternYouKibitzes, "" );
            emit( serverInfo( QString( "delivered + " ) + line ) );
            continue;
        }
        
        if( line.contains( m_patternUnknown ) )
        {
            line.replace( m_patternUnknown, "" );
            emit( serverInfo(line) );
            continue;
        }
        
        if( line.contains( m_patternLogins ) ) 
        {
            line.replace( m_patternLogins, "" );
            emit( serverInfo(line) );
            continue;
        }
        
        if( line.contains( m_patternLogouts ) ) 
        {
            line.replace( m_patternLogouts, "" );
            emit( serverInfo(line) );
            
            QString name = getFirstWord( line );
            
            kDebug() << name << " logouts";
            
            emit( logouts( name ) );
            
            continue;
        }
        
        if( line.contains( m_patternInviteConfirm ) )
        {
            line.replace( "** ", "" );
            emit( serverInfo(line) );
            continue;
        }
        
        if( ( next == patternInGameYouInvited ) ||
            ( next == patternInGameOpponentInvited ) ||
            ( next == patternInGameYouInvitedReload ) ||
            ( next == patternInGameOpponentInvitedReload ) )
        {
            kDebug() << "you are now playing";
            line.replace( "** ", "" );
            emit( fibsGamesInformation(line) );
            
            QString opponent_name;// = line.mid( line.lastIndexOf( ' ' ) + 1 );
            
            //kDebug() << "opponent_name = " << opponent_name;
            
            //getOpponent().setName( opponent_name );*/
            
            // Get opponent name
            if( next == patternInGameYouInvited )
            {
                // Opponent name is at the beginning
                opponent_name = getFirstWord(line);
            }  
            else if( next == patternInGameOpponentInvited )
            {
                // Opponent name is in the end
                opponent_name = getLastWord(line); //line.mid( line.lastIndexOf( ' ' ) + 1 )
            }
            else if( next == patternInGameYouInvitedReload )
            {
                // Opponent name is at the beginning
                opponent_name = getFirstWord(line);
            }
            else if( next == patternInGameOpponentInvitedReload )
            {
                // Opponent name is in the end
                //opponent_name = line.mid( line.lastIndexOf( ' ' ) + 1 )
                int with_index = line.indexOf( "with" );
                int length = line.indexOf( '.', with_index + 5 ) - ( with_index + 5 );
                
                opponent_name = line.mid( with_index + 5, length );
                //getLastWord(line);
            }
            
            getOpponent().setName( opponent_name );
            m_needReverse = false; 
            
            startNewGame();
            
            if( next == patternInGameOpponentInvited )
                m_needReverse = true;
            
            if( ( next == patternInGameYouInvitedReload ) ||
                ( next == patternInGameOpponentInvitedReload ) )
                m_needBoard = true;
            
            if( m_needBoard )
                emit( serverString( "board" ) );
            
            continue;
        }
        
        if( line.contains( m_patternBoardBegin ) )
        {
            kDebug() << "board begin " << line;
            
            if( m_needBoard )
            {
            //emit( fibsGamesInformation( getAll( data, m_patternBoardEnd ) ) );
                QVector<QString> board;
                board.push_back( line );
                getAll( data, m_patternBoardEnd, board );
                readBoard( board );
            }
            /*else
                skipAll( data, m_patternBoardEnd );*/
            
            continue;
        }
        
        if( line.contains( m_patternGameBegin ) )
        {
            m_state = playing;
            emit( fibsGamesInformation(line) );
            getGameBegin(data);
            continue;
        }
        
        if( line.contains( m_patternInviteResume ) )
        {
            getInvite( line );
            skipAll( data, m_patternInviteEnd );
            continue;
        }
        
        if( line.contains( m_patternInvitePlay ) )
        {
            getInvite( line );
            skipAll( data, m_patternInviteEnd );
            continue;
        }
        
        if( line.contains( m_patternOpponentMove ) )
        {
            kDebug() << "opponent move";
            getOpponentMove( line );
            continue;
        }
        
        if( line.contains( m_patternYourTurn ) )
        {
            kDebug() << "your turn";
            if( line.contains( "Please roll or double" ) )
                emit( showRollOrDoubleQuestion() );
            
            emit( fibsGamesInformation( line ) );
            continue;
        }
        
        if( line.contains( m_patternRoll ) )
        {
            Rolls got_roll = getRolls( line );

            setDice( got_roll );
            switchToState(Move1);
            
            //m_yourTurn = true;
            
            continue;
        }
        
        if( line.contains( m_patternPleaseMove ) )
        {
            emit( fibsGamesInformation( line ) );
            continue;
        }
        
        if( line.contains( m_patternOpponentRoll ) )
        {            
            Rolls got_roll = getRolls( line );
            setDice( got_roll );
            emit( fibsGamesInformation( line ) );
            continue;            
        }
        
        if( line.contains( m_patternOpponentDoubles ) )
        {
            // Show dialog
            emit( showAcceptRejectQuestion() );
            emit( fibsGamesInformation( "Your opponent doubles. Please accept or reject" ) );
            continue;
        }
        
        if( line.contains( m_patternAcceptDone ) )
        {
            emit( fibsGamesInformation( line ) );
            continue;
        }
        
        if( line.contains( m_patternServerTime ) )
        {
            emit( serverInfo( QString( "server time : " + line ) ) );
            continue;
        }
        
        if( line.contains( m_patternOpponentFirst ) )
        {
            emit( fibsGamesInformation( "Opponent makes first move" ) );
            continue;            
        }
        
        if( line.contains( m_patternYouWin ) )
        {
            emit( fibsGamesInformation( line ) );
            continue;
        }
        
        if( line.contains( m_patternInfo ) )
        {
            line.replace( "** ", "" );
            if( m_state == playing )
                emit( fibsGamesInformation( line ) );
            continue;
        }
        
        if( line.contains( m_patternTurn ) )
        {
            emit( fibsGamesInformation( line ) );
            m_yourTurn = true;
            continue;
        }
        
        if( line.contains( m_patternMatchLength ) )
        {
            emit( fibsGamesInformation( line ) );
            continue;
        }
    
        if( line.contains( m_patternPointsFor ) )
        {
            emit( fibsGamesInformation( line ) );
            continue;
        }
        
        if( line.contains( m_patternYouLost ) )
        {
            emit( fibsGamesInformation( line ) );
            continue;
        }
        
        if( line.contains( m_patternCantMove ) )
        {
            emit( fibsGamesInformation( line ) );
            continue;
        }
        
        if( line.contains( m_patternMatchEndLost ) )
        {
            emit( serverInfo( line ) );
            continue;
        }
        
        if( line.contains( m_patternMatchEndWon ) )
        {
            emit( serverInfo( line ) );
            continue;
        }
        
        if( line.contains( m_patternBlind ) )
        {
            emit( serverInfo( line ) );
            continue;
        }
            
        if( line.contains( m_patternUnblind ) )
        {
            emit( serverInfo( line ) );
            continue;            
        }
        
        if( line.contains( m_patternStartWatch ) )
        {
            emit( serverInfo( line ) );
            continue;            
        }
        
        if( line.contains( m_patternStopWatch ) )
        {
            emit( serverInfo( line ) );
            continue;            
        }

        if( line.contains( m_patternWave1 ) )
        {
            emit( fibsChatInformation( line ) );
            continue;
        }
        
        if( line.contains( m_patternWave2 ) )
        {
            emit( fibsChatInformation( line ) );
            continue;
        }
        
        if( line.contains( m_patternWave3 ) )
        {
            emit( fibsChatInformation( line ) );
            continue;
        }
        
        if( line.contains( "Done" ) )
            continue;
        
        if( line.compare("\n") ==  0 )
            continue;
        
        kDebug() << "unparsed line " << line;
        kDebug() << "unparsed data " << data;
        
        continue;
    }    
}

void BackgammonFibs::fibsRequestInvitation( const QString &player )
{
    /*if( !invitationDlg ) {
        QString p = player;
        invitationDlg = new KFibsInvite(0, "invite");
        connect(invitationDlg, SIGNAL(inviteCommand(const QString &)), 
                this, SLOT(handleCommand(const QString &)));
        connect(invitationDlg, SIGNAL(dialogDone()), this, SLOT(invitationDone()));
    }
    invitationDlg->setPlayer(player);
    invitationDlg->show();    */
}

/*
 * Send a disconnection request to the server. As soon as the server has 
 * closed the connection, the serverDisconnected() slot is called
 */
void BackgammonFibs::disconnectFIBS()
{
    kDebug() << "fibs disconnected";
    emit( serverString("quit") );
    /*emit( infoText(i18n("Closing connection...")) );
    emit( fibsConnectionClosed() );*/
}

void BackgammonFibs::connectFIBS( const QString& serv, qint16 port )
{
    m_connection->makeConnect( serv, port );
}

void BackgammonFibs::setAllowed(int, bool)
{
}

void BackgammonFibs::newBoard(QString*)
{
}

void BackgammonFibs::setupOk()
{
    
}

void BackgammonFibs::invitationDone()
{
    
}

void BackgammonFibs::prepareMenu()
{
    
}

void BackgammonFibs::serverDisconnected()
{
    
}

void BackgammonFibs::roll()
{
    emit( serverString("roll") );
}

void BackgammonFibs::doubleCube()
{
    emit( serverString("double") );
}

void BackgammonFibs::load()
{
    
}

void BackgammonFibs::done()
{
    
}

void BackgammonFibs::sendLine( QString line )
{
    /*if( line.compare( "login" ) == 0 )
    {
        QString login = "mio_first";
        QString password = "gfhjkm";
    
        this->login( login, password );
    }
    else
    {*/
        emit( serverString(line) );
    //}
}

void BackgammonFibs::login( const QString& login, const QString& password )
{
    kDebug() << "trying to login";
    
    getYou().setName(login);
    
    emit( serverString( QString("login ") + PROG_NAME + " " + "1008" + " "
                         + login + " " + password ) );
}

QString BackgammonFibs::getLine( QString& line_given )
{
    int index = line_given.indexOf( "\r" );
    
    if( index == -1 )
        return "";
    
    QString line( line_given.left( index ) );
    line_given.replace( 0, index + 2, "" );
    
    if( index == 0 )
        return "\n";
    
    return line + "\n";
}

void BackgammonFibs::say( const QString& what )
{
    emit( serverString( QString( "shout " ) + what ) );
}

void BackgammonFibs::kibitz( const QString& what )
{
    emit( serverString( QString( "kibitz " ) + what ) );
}

void BackgammonFibs::getMyOwnInfo( const QString& line )
{
    int togglesStart = line.indexOf( " " ) + 1;
    
    QStringList toggles = line.mid( togglesStart ).split(" ");

    m_toggles[allowpip] = toggles[0].toInt() > 0;
    m_toggles[autoboard] = toggles[1].toInt() > 0;
    m_toggles[autodouble] = toggles[2].toInt() > 0;
    m_toggles[automove] = toggles[3].toInt() > 0;
    m_toggles[away] = toggles[4].toInt() > 0;
    m_toggles[bell] = toggles[5].toInt() > 0;
    m_toggles[crawford] = toggles[6].toInt() > 0;
    m_toggles[edouble] = toggles[7].toInt() > 0;
    m_experience = toggles[8].toInt();
    m_toggles[greedy] = toggles[9].toInt() > 0;
    m_toggles[moreboards] = toggles[10].toInt() > 0;
    m_toggles[moves] = toggles[11].toInt() > 0;
    m_toggles[notify] = toggles[12].toInt() > 0;
    m_rating = toggles[13].toDouble();     
    m_toggles[ratings] = toggles[14].toInt() > 0;
    m_toggles[ready] = toggles[15].toInt() > 0;
    
    if( toggles[16] == "unlimited" )
        m_redoubles = -1;
    else
        m_redoubles = toggles[16].toInt();
    
    m_toggles[report] = toggles[17].toInt() > 0;
    m_toggles[silent] = toggles[18].toInt() > 0;
    m_timezone = toggles[19];
    
    emit( togglesLoaded(m_toggles) );
}

void BackgammonFibs::message( const QString& name, const QString& text )
{
    emit( serverString( QString( "message " ) + name + " " + text ) );
}

void BackgammonFibs::tell( const QString& name, const QString& text )
{
    emit( serverString( QString( "tell " ) + name + " " + text ) );
}

void BackgammonFibs::invite( const QString& name, int length )
{
    if( length != 0 )
        emit( serverString( QString( "invite " ) + name + " " + QString::number(length) ) );
    else
        emit( serverString( QString( "invite " ) + name ) );   
}

void BackgammonFibs::join( QString name )
{
    //getOpponent().setName( name );
    emit( serverString( QString( "join " ) + name ) );
}

QString BackgammonFibs::getAll( QString& line_given, QRegExp end_pattern )
{
    QString ret;
    
    for(;;)
    {
        QString new_line = getLine( line_given );
        
        if( new_line.contains( end_pattern ) )
            break;
        
        if( new_line.length() == 0 )
            break;
       
        ret += new_line;
    }
    
    return ret;
}

void BackgammonFibs::toggle( int toggle, bool state )
{
    kDebug() << "toggle " << toggle << " : " << state;
    
    if( m_toggles[toggle] == state )
        return;
    else
        m_toggles[toggle] = state;
    
    switch( toggle )
    {
        case allowpip:
            emit( serverString( QString( "toggle allowpip" ) ) );
            break;
            
        case autoboard:
            emit( serverString( QString( "toggle autoboard" ) ) );
            break;
            
        case autodouble:
            emit( serverString( QString( "toggle autodouble" ) ) );
            break;
            
        case automove:
            emit( serverString( QString( "toggle automove" ) ) );
            break;
            
        case away:
            emit( serverString( QString( "toggle away" ) ) );
            break;
            
        case bell:
            emit( serverString( QString( "toggle bell" ) ) );
            break;
            
        case crawford:
            emit( serverString( QString( "toggle crawford" ) ) );
            break;
            
        case edouble:
            emit( serverString( QString( "toggle double" ) ) );
            break;
            
        case greedy:
            emit( serverString( QString( "toggle greedy" ) ) );
            break;
            
        case moreboards:
            emit( serverString( QString( "toggle moreboards" ) ) );
            break;
            
        case moves:
            emit( serverString( QString( "toggle moves" ) ) );
            break;
            
        case notify:
            emit( serverString( QString( "toggle notify" ) ) );
            break;
            
        case ratings:
            emit( serverString( QString( "toggle ratings" ) ) );
            break;
            
        case ready:
            emit( serverString( QString( "toggle ready" ) ) );
            break;
            
        case report:
            emit( serverString( QString( "toggle report" ) ) );
            break;
            
        case silent:
            emit( serverString( QString( "toggle silent" ) ) );
            break;
    }
}

void BackgammonFibs::getWhoInfo( const QString& line )
{
    QStringList infos = line.split(" ");
    
    if( infos.size() != 12 )
        return;
    
    WhoInfo info;
    
    info.m_name = infos[0];
    
    info.m_opponent = infos[1];
    
    info.m_watching = infos[2];
    
    info.m_ready = infos[3].toInt() > 0;
    
    info.m_away = infos[4].toInt() > 0;
    
    info.m_rating = infos[5].toDouble();
    
    info.m_experience = infos[6].toInt();
    
    info.m_idle = infos[7].toInt();
    
    info.m_login = QDateTime::fromTime_t( infos[8].toInt() );
    
    info.m_hostname = infos[9];
    
    info.m_client = infos[10];
    
    info.m_email = infos[11];
    
    emit( newWhoInfo(info) );
}

void BackgammonFibs::getInvite( const QString& line )
{
    int space_ind = line.indexOf( " " );
    QString name = line.left( space_ind );
    
    if( line.contains("resume") )
    {
        m_invites.push_back(name);
        emit( resumeInvite(name) );
        /*emit( showYesNoDialog(  "Match invitation", 
                                "Player " + name + " want's to resume saved match with you ",
                                "Join", "Reject" ) );*/
    }
    else
    {
        m_invites.push_back(name);
        int length = line.mid( space_ind + 17, 1 ).toInt();
        emit( playInvite( name, length ) );
        /*int res = KMessageBox::questionYesNo( 0, "Player " + name + " invites you for " + QString::number(length), 
                                              "Match Invitation", 
                                              KGuiItem( "Join" ), KGuiItem( "Reject" ) );*/
        /*emit( showYesNoDialog(  "Player " + name + " want's to play a " + QString::number(length) + " match with you",
                                "Match invitation",
                                "Join", "Reject" ) );*/
    }
}

void BackgammonFibs::getOpponentMove( const QString& line )
{
    int name_end = line.indexOf( " " );
    
    int moves_beg = name_end + 7;
    
    QStringList moves = line.mid( moves_beg ).split( " " );
    
    QVector<Move> moves_parsed;
    
    for( int i = 0; i < moves.length(); ++i )
    {
        Move move = getMoveFromStr( moves[i] );
        
        if( move.src != move.dst )      // TODO: Smthin like move legal?
            moves_parsed.append( move );
        
        kDebug() << "move " << move.src << ":" << move.dst;
    }
    
    emit( opponentMoves( moves_parsed ) );
    
    QVector<Move> moves_done;
    
    for( int i = 0; i < moves_parsed.size(); ++i )
    {
        setCurrentMoveSrc( moves_parsed[i].src );
        setCurrentMoveDst( moves_parsed[i].dst );
        
        makeMove( getOpponent(), moves_done );
        showBoardChanges( getOpponent(), moves_done );
        
        moves_done.clear();
    }
    
    Board::hideDice();
    
    showField();
    
    m_yourTurn = true;
}

Move BackgammonFibs::getMoveFromStr( const QString& line_part )
{
    int dash_ind = line_part.indexOf("-");
    
    QString src_str = line_part.left( dash_ind );
    QString dst_str = line_part.mid( dash_ind + 1 );
    
    int src = 0, dst = 0;
    
    if( src_str.compare( "bar" ) != 0 )
    {
        src = src_str.toInt() - 1; // 0 based index
        
        if( m_needReverse )
            src = reverseIndex( src );
    }
    else
        src = getBarIndex( getOpponent() ); // bar
        
    if( dst_str.compare( "off" ) != 0 )
    {
        dst = dst_str.toInt() - 1; // 0 based index
        
        if( m_needReverse )
            dst = reverseIndex( dst );
    }
    else
        dst = getOutIndex( getOpponent() );

    return { src, dst };
}

void BackgammonFibs::getRoll( const QString& line )
{
    int first_dice_ind = 9;
    
    int first_dice = line.mid( first_dice_ind, 1 ).toInt();
    
    int second_dice = line.mid( first_dice_ind + 6, 1 ).toInt();
    
    //emit( roll( first_dice, second_dice ) );
}

BackgammonFibs::Rolls BackgammonFibs::getRolls( const QString& line )
{
    int first_roll_ind = line.indexOf( QRegExp( "[0-9]" ) );
    int first_roll = line.mid( first_roll_ind, 1 ).toInt();
    int second_roll = line.mid( line.indexOf( QRegExp( "[0-9]" ), first_roll_ind + 1 ), 1 ).toInt();
    
    return qMakePair( first_roll, second_roll );
}

void BackgammonFibs::getGameBegin( QString& line )
{
    int your_roll = 0;
    int opponent_roll = 0;
    
    while( your_roll == opponent_roll )
    {
        QString next = getLine( line );
        Rolls first_roll = getRolls( next );
    
        your_roll = first_roll.first;
        opponent_roll = first_roll.second;
        
        kDebug() << "roll " << your_roll << " : " << opponent_roll;
        
        setDice( first_roll );
    }
    
    if( your_roll > opponent_roll )
    {
        emit( fibsGamesInformation( "It's your turn." ) );
        //setDice( Rolls( your_roll, opponent_roll ) );
        switchToState( Move1 );
        m_yourTurn = true;
    }
    else
        emit( fibsGamesInformation( "Opponent's turn" ) );    
}

void BackgammonFibs::accept()
{
    emit( serverString( QString( "accept" ) ) );
}

void BackgammonFibs::reject()
{
    emit( serverString( QString( "reject" ) ) );
}

void BackgammonFibs::processUIEvent( QGraphicsSceneMouseEvent* event, BackgammonScene* board )
{
    if( !m_yourTurn )
        return;
    
    IPlayer& player = getYou();

    qint8 cell = Board::cellNumber( event->scenePos() );
    
    kDebug() << "cell = " << cell << " state " << getState();
    
    if( cell == Board::bar( player.color() ) )
        cell = getBarIndex(player);
    

    switch( getState() )
    {
        case BackgammonGame::Roll:
        {
            roll();
            break;
        }
        case BackgammonGame::Move1:
        case BackgammonGame::Move2:
        case BackgammonGame::Move3:
        case BackgammonGame::Move4:
            if( !gotCurrentMoveSrc() )
            {
                // Need extra specification to know what die to use
                setCurrentMoveSrc(cell);
                showProposedMoves( cell, player );
                break;
            }
            else if( !gotCurrentMoveDst() )
            {
                setCurrentMoveDst(cell);
            }
            if( moveReady( player ) )
            {                    
                unshowProposedMoves();
                if( createMove( player ) )
                {
                    kDebug() << "move creation succesessful";
                    QVector<Move> changes;
                    
                    Move mv = makeMove( player, changes );
                    
                    if( changes.size() > 0 )
                        showBoardChanges( player, changes );
                    
                    m_myLastMoves.push_back( mv );
                    switchToNextState();
                }
                else
                    emit( engineMessage( i18n( "Illegal move" ) ) );
            }
            else
                kDebug() << "move not ready";
            
            if( !gotMoves( player ) )
            {
                emit( engineMessage( i18n( "Done" ) ) );
                switchToState( Commit );
            }
            break;
        case BackgammonGame::Commit:
        {
            kDebug() << "action = commit";
            
            Board::hideDice();
            sendMove();
            emit( moveFinished("") );
            //updatePip();
            showField();
            
            m_yourTurn = false;
            switchToState(Roll);
        }
        default:
            Q_ASSERT_X(false,"processUIEvent", "Incorrect game state reached");
    }
}

void BackgammonFibs::sendMove()
{
    if( m_myLastMoves.size() == 0 )
        return;
    
    QString line;
    
    for( int i = 0; i <  m_myLastMoves.size(); ++i )
    {
        int src = m_myLastMoves[i].src; 
        int dst = m_myLastMoves[i].dst;
        
        /*if( m_needReverse )
        {
            src = reverseIndex(src);
            dst = reverseIndex(dst);
        }
        
        src++;  // Convert to 1 based index
        dst++;*/
        
        if( src > bar_ind )
            line += "b";
        else
        {
            if( m_needReverse )
                src = reverseIndex(src);
                
            src++;      // Convert to 1 based index
            line += QString::number( src );
        }
        
        line += " ";
        
        if( dst > thrown_ind )
            line += "h";
        else
        {
            if( m_needReverse )
                dst = reverseIndex(dst);
            
            dst++;      // Convert to 1 based index
            line += QString::number( dst );
        }
        
        line += " ";
    }
    
    kDebug() << "move " << line;
    m_myLastMoves.clear();
    emit( serverString(line) );
}

BackgammonFibs::pattern BackgammonFibs::getPattern( const QString& line )
{
    // Here we can use statates to avoid parsing
    
    if( line.contains( m_patternGameReload1 ) )
    {
        kDebug() << "patternInGameOpponentInvitedReload " << line;
        return patternInGameOpponentInvitedReload;
    }
    
    if( line.contains( m_patternGameReload2 ) )
    {
        kDebug() << "patternInGameYouInvitedReload";
        return patternInGameYouInvitedReload;
    }
    
    if( line.contains( m_patternMatchBegin1 ) )
    {
        kDebug() << "patternInGameYouInvited";
        return patternInGameYouInvited;
    }
    
    if( line.contains( m_patternMatchBegin2 ) )
    {
        kDebug() << "patternInGameYouInvited";
        return patternInGameYouInvited;
    }
    
    if( line.contains( m_patternMatchBegin3 ) )
    {
        kDebug() << "patternInGameOpponentInvited";
        return patternInGameOpponentInvited;
    }
    
    if( line.contains( m_patternMatchBegin4 ) )
    {
        kDebug() << "patternInGameOpponentInvited";
        return patternInGameOpponentInvited;
    }
    
    if( line.contains( m_patternBoardBegin ) )
    {
        kDebug() << "patternBoard";
        return patternBoard;
    }
    
    return noPattern;
}

void BackgammonFibs::startNewGame()
{
    if( !m_needBoard )
        emit( setCheckerPositions( BackgammonGame::initialBoardState() ) );
    
    resetBoard();
    
    switchToState(Roll);
}

void BackgammonFibs::readBoard( const QVector<QString>& board )
{
    /*
    
    Board is smth like this:
      1  2  3  4  5  6        7  8  9 10 11 12
    +------------------------------------------+ O: mio_second - score: 0
    |    X     X     O |   |     O           X |
    |                O |   |     O           X |
    |                O |   |     O           X |
    |                O |   |                 X |
    |                O |   |                 X |
    |                  |BAR|                   |v    2-point match
    |                X |   |                 O |
    |                X |   |                 O |
    |                X |   |     X           O |
    | O              X |   |     X           O |
    | O              X |   |     X           O |
    +------------------------------------------+ X: mio_first - score: 1
     24 23 22 21 20 19       18 17 16 15 14 13

    BAR: O-0 X-0   OFF: O-0 X-0   Cube: 1  mio_second rolled 5 4.
    */
    kDebug() << board;
    
    IPlayer* playerX = 0; 
    IPlayer* playerO = 0;
    
    // Get upper player info
    kDebug() << getYou().name() << " : " << getOpponent().name();
    
    if( board[0].contains( "+-1-" ) )
        m_needReverse = true;
    
    for( int k = 0, j = 0; j < 2; k = board.size() - 3, ++j )
    {
        int score = board[k].mid( board[k].lastIndexOf(':') + 2, 1 ).toInt();
        
        int sign_index = board[k].indexOf( QRegExp( "[XO]" ) );
        
        //kDebug() << "board[k] " << board[k];
    
        QChar sign = board[k][sign_index];
        
        kDebug() << "score = " << score << " sign " << sign;
        
        if( board[k].contains( getYou().name() ) )
        {
            if( sign == 'X' )
                playerX = &getYou();
            else if( sign == 'O' )
                playerO = &getYou();
        }
        else if( board[k].contains( getOpponent().name() ) )
        {
            if( sign == 'X' )
                playerX = &getOpponent();

            else if( sign == 'O' )
                playerO = &getOpponent();
        }
    }
    
    kDebug() << "player X name " << playerX->name() << " player 0 name " << playerO->name();
    
    // TODO: m_needReverse must go next >>
    // convert the abovelong lines to a macros
    
    qint8 cells[ FIELD_SIZE ];
    
    memset( cells, 0, sizeof(cells) );
    
    for( int k = 0, j = 0; j < 2; k = board.size() - 3, ++j )
    { 
        QString line = board[k];
        
        //kDebug() << line;
        
        int number_length = 1;
        
        for( int i = 0; i < line.length(); i += number_length )
        {
            number_length = 1;
            
            if( line[i] == ':' )
                break;
            
            if( line[i].isDigit() )
            {
                if( line[i + 1].isDigit() )
                    number_length = 2;
                
                int cell = line.mid( i, number_length ).toInt();
                
                /*if( ( k == 0 ) && ( cell == 1 ) )
                    m_needReverse = true;*/
                
                kDebug() << "cell = " << cell;
                
                cell--; // to 0 based index
                
                if( m_needReverse )
                {
                    cell = reverseIndex( cell ) + 1;
                    kDebug() << "after reverse " << cell;
                    cell--; // to 0 based index
                }
                
                int j = k;
                
                if( k == 0 )
                        j++;
                    else
                        j--;
                    
                int index = i + ( number_length - 1 );

                while( board[j][index].isLetter() )
                {
                    if( board[j][index] == 'X' )
                    {
                        /*if( m_needReverse )
                            cells[cell] += playerO->getDirection();
                        else
                            cells[cell] += playerX->getDirection();*/
                        cells[cell] -= playerX->getDirection();
                    }
                    else if( board[j][index] == 'O' )
                    {
                        /*if( m_needReverse )
                            cells[cell] += playerX->getDirection();
                        else
                            cells[cell] += playerO->getDirection();*/
                        cells[cell] -= playerO->getDirection();
                    }
                    
                    if( k == 0 )
                        j++;
                    else
                        j--;
                }
                
                if( board[j][index].isDigit() )
                {
                    // Number of checkers at cell is written
                    kDebug() << "digit " << j << " " << index;

                    if( cells[cell] > 0 )
                        cells[cell] = board[j][index].digitValue();
                    else
                        cells[cell] = - board[j][index].digitValue();

                    if( board[j][index - 1].isDigit() )
                    {
                        if( cells[cell] > 0 )
                            cells[cell] += board[j][index].digitValue()*10;
                        else
                            cells[cell] += - board[j][index].digitValue()*10;
                    }
                }
            }
        }
    }
    

    
    // get BAR and OFF
    // BAR: O-0 X-0   OFF: O-0 X-0   Cube: 1  mio_second rolled 5 4.
    QString last = board[ board.size() - 1 ];
    
    int barO = last.mid( 10, 1 ).toInt();
    int barX = last.mid( 14, 1 ).toInt();

    int offO = last.mid( 25, 1 ).toInt();
    int offX = last.mid( 29, 1 ).toInt();
    
    kDebug() << "bar" << barO << ":" << barX;
    kDebug() << "off" << offO << ":" << offX;
    
    // Write bar and off info to cells
    
    int bar_ind_X = getBarIndex( *playerX );
    int bar_ind_O = getBarIndex( *playerO );
    
    int off_ind_X = getOutIndex( *playerX );
    int off_ind_O = getOutIndex( *playerO );
    
    /*if( m_needReverse )
    {
        std::swap( bar_ind_O, bar_ind_X );
        std::swap( off_ind_O, off_ind_X );
    }*/
    
    cells[ bar_ind_X ] = - barX*playerX->getDirection();
    cells[ bar_ind_O ] = - barO*playerO->getDirection();
    cells[ off_ind_X ] = - offX*playerX->getDirection();
    cells[ off_ind_O ] = - offO*playerO->getDirection();    
    
    for( int i = 0; i < FIELD_SIZE; ++i )
        kDebug() << "cell " << i << " = " << cells[i];
    
    int roll_start_index = last.indexOf( "rolled" );
    
    Rolls roll( last.mid( roll_start_index + 7, 1 ).toInt(),
                last.mid( roll_start_index + 9, 1 ).toInt() );
                
    kDebug() << "rolls " << roll.first << " : " << roll.second;
        
    if( last.contains( "You" ) )
    {
        if( ( roll.first == 0 ) && ( roll.second == 0 ) )
            switchToState( Roll );
        else
            switchToState( Move1 );
        
        m_yourTurn = true;
    }
    
    setBoard( cells );
    
    showField();
    
    setDice( roll );
    
    m_needBoard = false;
}

int BackgammonFibs::reverseIndex( int index )
{
    return qAbs( 23 - index );
}

void BackgammonFibs::skipAll( QString& line_given, QRegExp end_pattern )
{
    for(;;)
    {
        QString new_line = getLine( line_given );
        
        if( new_line.contains( end_pattern ) )
            break;
        
        if( new_line.length() == 0 )
            break;
    }
}

void BackgammonFibs::getAll( QString& line_given, QRegExp end_pattern, QVector<QString>& vec )
{
    for(;;)
    {
        QString new_line = getLine( line_given );
        
        if( new_line.contains( end_pattern ) )
        {
            vec.push_back( new_line );
            break;
        }
        
        if( new_line.length() == 0 )
            break;

        vec.push_back( new_line );
    }
}

QString BackgammonFibs::getFirstWord( const QString& line )
{
    int index = 0;
    
    while( line[index] == ' ' )
        index++;
    
    return line.mid( 0, line.indexOf( ' ' ) );
}
    
QString BackgammonFibs::getLastWord( const QString& line )
{
    return line.mid( line.lastIndexOf( ' ' ) + 1 );
}
