/*
This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.

---
Copyright (C) 2011, Dmitry Chernov diman4ik.chernov@gmail.com
*/

#ifndef _WhoInfo_h_
#define _WhoInfo_h_

#include <QDateTime>


class WhoInfo 
{
public:

    // name:       The login name for the user this line is referring to.
    QString m_name;
    
    // opponent:       The login name of the person the user is currently playing against, 
    // or a hyphen if they are not playing anyone.
    QString m_opponent;
    
    // watching:       The login name of the person the user is currently watching, or a hyphen 
    // if they are not watching anyone.
    QString m_watching;
    
    // ready:          1 if the user is ready to start playing, 0 if not. Note that the ready 
    // status can be set to 1 even while the user is playing a game and thus, technically unavailable. 
    // Refer to Toggle Ready.
    bool m_ready;
    
    // away:   1 for yes, 0 for no. Refer to Away.
    bool m_away;
    
    // rating:         The user's rating as a number with two decimal places.
    double m_rating;
    
    // experience:     The user's experience.
    int m_experience;
    
    // idle:   The number of seconds the user has been idle.
    int m_idle;
    
    // login:          The time the user logged in as the number of seconds since midnight, January 1, 1970 UTC.
    // int m_login;
    QDateTime m_login;
    
    // hostname:       The host name or IP address the user is logged in from. Note that the host name can 
    // change from an IP address to a host name due to the way FIBS host name resolving works.
    QString m_hostname;
    
    // client:         The client the user is using (see login) or a hyphen if not specified. See notes below.
    QString m_client;
    
    // email:          The user's email address, or a hyphen if not specified. Refer to Address.
    QString m_email;
    
    bool operator == ( const WhoInfo& info )
    {
        return (*this ).m_name.compare( info.m_name ) == 0;
    }
};

#endif
