/***************************************************************************
 *   Copyright (C) 2009 by Alexander Smirnov                               *
 *   alexander.v.smirnov@gmail.com                                         *
 *                                                                         *
 *   Copyright (C) 2011, Dmitry Chernov                                    *
 *   diman4ik.chernov@gmail.com                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef IGAMEENGINE_H
#define IGAMEENGINE_H

#include <QObject>
#include "checker.h"

class BackgammonScene;
class IPlayer;
class QGraphicsSceneMouseEvent;

/**
 * Interface specifying some general actions for backgammon-like games
 * We are going to support backgammon, fevga, nackgammon and probably more in future
 * @author Alexander Smirnov <alexander.v.smirnov@gmail.com>
 */
class IGameEngine : public QObject
{
    Q_OBJECT
public:
    IGameEngine( QObject* parent = 0 );
    virtual ~IGameEngine();

    enum EGameState{ Roll, Move1, Move2, Move3, Move4, Commit, _Max };

    /**
     * Starts new game loop
     */
    virtual void startNewGame() = 0;

    /**
     * Handle event, received from UI
     */
    virtual void processUIEvent(QGraphicsSceneMouseEvent* event, BackgammonScene* board) = 0;

    /**
     * Places checkers on the board
     * Initial checkers placement is different for different kinds of games.
     */
    virtual QStringList initialBoardState() const = 0;

    /**
     * Returns true if checker of specified color can be placed on specified cell
     * Depends on concrete game rules
     */
    virtual bool canPlaceChecker(const Checker::Color color, qint8 cellDest) const = 0;

    /**
     * Returns true if checker can be moved by player from cell1 to cell2
     * Depends on concrete game rules
     */
    virtual bool canMoveChecker(IPlayer* player, qint8 cell1, qint8 cell2) const = 0;
    
    /**
     * Roll dice
     *
     */
    virtual void roll() = 0;

signals:
    /**
     * notifies everybody who is interested what dice have been rolled
     */
    void diceRolled(qint8, qint8);

    /**
     * emited when significant game event has occured
     */
    void gameEvent(QString);

    void pipCountChanged(QString);

    /**
     * informs that player has commited his move
     */
    void moveFinished(QString);
    
    
    void showMove( qint8, qint8 );
    
    void setCheckerPositions( const QStringList& positions );
    
    void proposedMoves( const QList<int>& , Checker::Color );
    void hideProposedMoves();
};

#endif
