/***************************************************************************
 *   Copyright 2006-2007 Dmitry Suzdalev <dimsuz@gmail.com>                *
 *                  2009 Alexander Smirnov <alexander.v.smirnov@gmail.com> *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
 ***************************************************************************/
#include "renderer.h"

#include <QApplication>

#include <kdebug.h>
#include <kpixmapcache.h>
#include <ksvgrenderer.h>

Renderer::Renderer(int cacheSize, QString themeURL)
{
    m_renderer = new KSvgRenderer();
    const bool res = m_renderer->load( themeURL );

    if (res)
    {
        onThemeLoadSuccess();
    }
    else
    {
        onThemeLoadFailure();
    }

    m_cache = new KPixmapCache("svg-cache");
    m_cache->setCacheLimit( cacheSize );
}

/**
 * Override it, if you need another behaviour
 */
void Renderer::onThemeLoadFailure()
{
    kDebug() <<"Failed to load SVG";
    qApp->quit();
}

/**
 * Override it, if you need another behaviour
 */
void Renderer::onThemeLoadSuccess()
{
    kDebug() <<"SVG successfuly loaded";
}

Renderer::~Renderer()
{
    delete m_renderer;
    delete m_cache;
}

void Renderer::renderElement (QPainter *p, const QString& elementid, const QRectF& r ) const
{
    QPixmap pix;
    QString cacheStr = elementid+QString("_%1x%2").arg(r.width()).arg(r.height());
    if(!m_cache->find(cacheStr, pix))
    {
        pix = QPixmap(r.size().toSize());
        pix.fill(Qt::transparent);
        QPainter paint(&pix);
        m_renderer->render(&paint, elementid);
        paint.end();
        m_cache->insert(cacheStr, pix);
    }

    p->drawPixmap(static_cast<int>(r.x()), static_cast<int>(r.y()), pix);
}

QRectF Renderer::elementRect(const QString& elementId) const
{
    return m_renderer->boundsOnElement(elementId);
}

QSize Renderer::elementSize(const QString& elementId) const
{
    return elementRect(elementId).toRect().size();
}
