/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/

#ifndef BOARD_H
#define BOARD_H

#include <QByteArray>
#include <QPoint>

/**
  *@author Piotr Banasiak
  */
  

class Board 
{
public:
 enum BitMask
 {
  MAN= 1,
  KING= 2,
  WHITE= 4,
  BLACK= 8,
  CAPTURED= 16,// that happens during taking the opponent piece
  MOVEABLE=32 // indicates that given piece can move
  };
  
enum Pieces
{
  EmptySq=0,
  whiteM=MAN | WHITE,
  blackM=MAN | BLACK,
  whiteK=KING | WHITE,
  blackK=KING | BLACK
};

	Board(int bSize=0);
	Board(const Board *const);
	~Board();

	bool operator== ( const Board & other ) const;
  void startPos();//setup start position
	void clear();//removes all pieces
  int sqNumber( int xx, int yy )const; 
	int startingPiecesNumber()const;
	void countPieces(int &white_M, int& white_K, int &black_M, int& black_K)const;
	int moveNr;//indicates is white or black move
	int movesToDraw;
  
	QByteRef at(int x, int y);
  QByteRef at( QPoint &xy );
  char get(int x, int y)const
  {
      Q_ASSERT( x>=0 );Q_ASSERT( x<m_size );
      Q_ASSERT( y>=0 );Q_ASSERT( y<m_size );
      int idx=x*m_size+y;
      return b.at( idx );
  };
  char get(QPoint xy)const
  {
      Q_ASSERT( xy.x()>=0 );Q_ASSERT( xy.x()<m_size );
      Q_ASSERT( xy.y()>=0 );Q_ASSERT( xy.y()<m_size );
      int idx = xy.x()*m_size + xy.y();
      // Returns reference to temporary ( b.at(idx) gives a const char,
      // stored as temporary - this function should either not return
      // a reference or be replaced by at(). Same applies to get(int,int).
      return b.at( idx );
  };
  int size()const {return m_size;}
  bool isWhiteTurn()const { return (moveNr+1)%2; }
  //----------------------------
  // true if coordinates are inside the board
  // excluding border lines    
  bool inMiddle(const QPoint& xy)const
  {
    if(xy.x()<1)
      return false;
    if(xy.y()<1)
      return false;
    if(xy.x()>=m_size-1)
      return false;
    if(xy.y()>=m_size-1)
      return false;
    return true;
  };

  //----------------------------
  // if coordinates are on the board
  bool isInside(const QPoint& xy)const
  {
    if(xy.x() < 0)
      return false;
    if(xy.y() < 0)
      return false;
    if(xy.x() >= m_size)
      return false;
    if(xy.y() >= m_size)
      return false;
    return true;
  }  
   
private:
	int m_size;
	QByteArray b;
  
  friend uint qHash ( const Board& B );
};
extern uint qHash ( const Board& B );
#endif
