/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/

#ifndef BOARD64_H
#define BOARD64_H

#include <QtGlobal>
#include "board.h"

/**
  *@author Piotr Banasiak
   Board implemented as template
   // T is long enough unsigned integer
  */



template <class T>
class BoardT 
{
public:
  BoardT()
  {
    captured=0;
    movesToDraw=9999;
  }
  //-------------------------------------------- 
 ~BoardT()
 {
 } 
 //--------------------------------------------  
  BoardT(const BoardT& B)
  {
    blackK=B.blackK;
    blackM=B.blackM;
  
    whiteK=B.whiteK;
    whiteM=B.whiteM;
  
    movesToDraw=B.movesToDraw;
    moveNr=B.moveNr;
    captured=B.captured;
  //memcpy( this,&B,sizeof( B));
  }
  //-------------------------------------------- 
  // from external board this is only temporary
  // untill class Board ( not template ) is removed completely from project
  BoardT(const Board& B)
  { 
    int sqNr=0;
  
    whiteM=0;
    whiteK=0;
    blackM=0;
    blackK=0;
    captured=0;
    movesToDraw= B.movesToDraw;
    moveNr=B.moveNr;
    
    int size=B.size();
  
    for(int x=0;x< size;x++)
      for(int y=0;y<size;y++)
      {
          int piece=B.get( x,y );
          sqNr=B.sqNumber( x, y );
          if( sqNr <0 ) continue;
  
          T M=1;//maskM
          M <<= sqNr;
          switch(piece)
          {
            case Board::whiteM:
            whiteM |= M;
            break;
            
            case Board::whiteK:
            whiteK |= M;
            break;
            
            case Board::blackM:
            blackM |= M;
            
            break;
            case Board::blackK:
            blackK |= M;
            break;
          }
      }
    return ;
  }
  //-----------------------------------

  inline BoardT & operator=(const BoardT& B)
  {
    blackK=B.blackK;
    blackM=B.blackM;
  
    whiteK=B.whiteK;
    whiteM=B.whiteM;
  
    movesToDraw=B.movesToDraw;
    moveNr=B.moveNr;
    captured=B.captured;
  
    return *this;
  }
  //----------------------------------------------
  inline bool operator== ( const BoardT & B ) const
  { 
    if( blackK!=B.blackK )return false;
    if( blackM!=B.blackM )return false;
    if( whiteK!=B.whiteK )return false;
    if( whiteM!=B.whiteM )return false;
    
    if( isWhiteTurn() != B.isWhiteTurn() )return false;
    return true;
  }
  //----------------------------------------------
  bool isWhiteTurn()const { return (moveNr+1)%2; }///@todo reimplement by bitwise and with  0x1
  //-------------------------------------------------
  void countPieces( int &cntWM, int& cntWK, int &cntBM, int& cntBK )const
  {
    cntWM=cntWK=cntBM=cntBK=0;
    T sq= ( whiteM | whiteK | blackM | blackK );
    T mask=1;
    
    while( sq )
    {
        if( whiteM & mask ) cntWM++;
        if( whiteK & mask ) cntWK++;
        if( blackM & mask ) cntBM++;
        if( blackK & mask ) cntBK++;
        
        mask<<=1;
        sq>>=1;
    }
  }
  //-----------------------------------------------
  T whiteM;
  T whiteK;
  T blackM;
  T blackK;
  T captured;
  
  int movesToDraw;
  int moveNr;
};

//-----------------------------------------------------
// ignoring captured and moveNr FIXME debug that function 
// this is global function not class member
template <class T>
uint qHash ( const BoardT<T>& B )
{
  T k=0;///@todo use Template type

  int tsize=sizeof( B.whiteM );
  int usize=sizeof( uint );
  int tt=tsize/usize;
  
  for( int j=0 ; j<tt ; j++ )
  {
     k ^= (B.whiteM >> j*8*usize );
     k ^= (B.whiteK >> j*8*usize );
     k ^= (B.blackM >> j*8*usize );
     k ^= (B.blackK >> j*8*usize );
  }

  if( B.isWhiteTurn() )
  {
    k= ~k;
  }

  return k;
}
#endif

