/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/

#include <kmainwindow.h>
#include <kstatusbar.h>
#include <kstandardaction.h>
#include <ktoggleaction.h>
#include <kicon.h>
#include <kmenubar.h>
#include <ktoolbar.h>
#include <kactioncollection.h>

#include "draughts.h"
#include "settingsdialog.h"
#include "config.h"
#include "boardview.h"
#include "boardmodel.h"
#include "game.h"
#include "piecedelegate.h"
#include "netplayer.h"
#include "humanplayer.h"
#include "engineplayer.h"
#include "networkgamedialog.h"
#include "newgamedialog.h"
#include "movelistmodel.h"
#include "movelistview.h"
#include "engineinfowidget.h"
#include "engineintt.h"
#include "engineamt.h"


class netPlayer;
class humanPlayer;

draughts::draughts()
{
    //ui.setupUi(this);
    ui_actionStop_thinking=0;
// ui_actionNew=0;
// ui_actionBack=0;
// ui_actionForward=0; 
// ui_actionConfiguration=0;
// ui_actionForce_move=0;
// 
// ui_actionOpen_Game=0;
// ui_actionSave_As=0;
// ui_actionSave_Game=0;
// ui_actionSetPlayers=0;
// ui_actionNetGame=0;
// ui_actionInvert_board=0;

    
    pEngine=0;
    engineFactory();
    pGame=new Game ( pEngine );

    connect( pGame, SIGNAL(gameOver( int )), this, SLOT(onGameOver(int)));
    connect( pGame, SIGNAL(documentChanged()), this, SLOT(onDocumentChanged()));
    
    
    //loadBook();
    
    createView();
    
    pGame->newGame();
    
    connectActions(  );
    
    pWhitePlayer = new humanPlayer( this,boardModel );//new humanPlayer(this);
    connect( pWhitePlayer, SIGNAL(moveFinished(LegalMove)), this, SLOT(onMoveFinished(LegalMove)));
    
    pBlackPlayer = new enginePlayer(this,pGame);
    pBlackPlayer->setLevel( Conf().m_BlackLevel ,Conf().m_BlackLevelType );
    connect( pBlackPlayer, SIGNAL(moveFinished(LegalMove)), this, SLOT(onMoveFinished(LegalMove)));
    pNetPlayer=0;
    wakeUp();
}

void draughts::loadBook()
{
    QString fileName="/home/piotr/src/draughts/b1.book";
    QFile file(fileName);
    if (!file.open(QFile::ReadOnly | QFile::Text)) {
        QMessageBox::warning(this, tr("Application"),
                             tr("Cannot read file %1:\n%2.")
                             .arg(fileName)
                             .arg(file.errorString()));
        return;
    }

   pGame->loadBook( file );
}

draughts::~draughts()
{
   if( pGame) delete pGame;
   pGame=0;
   
   if( pEngine) delete pEngine;
   pEngine=0;
}

void draughts::connectActions()
{
    // File
    QAction *quitAct = KStandardAction::quit(this, SLOT(close()), actionCollection());
    actionCollection()->addAction("quit",quitAct);
    QAction *ui_actionNew = KStandardAction::openNew(this, SLOT(newGame()), actionCollection());
    actionCollection()->addAction("newGame",ui_actionNew);
    QAction *ui_actionOpen_Game = KStandardAction::open(this, SLOT(open()), actionCollection());
    actionCollection()->addAction("open",ui_actionOpen_Game);
    QAction *ui_actionSave_Game = KStandardAction::save(this, SLOT(save()), actionCollection());
    actionCollection()->addAction("save", ui_actionSave_Game);
    QAction *ui_actionSave_As = KStandardAction::saveAs(this, SLOT(saveAs()), actionCollection());
    actionCollection()->addAction("saveAs",ui_actionSave_As);
    
    // GAME
    QAction *ui_actionNetGame =  actionCollection()->addAction("networkGame");
    ui_actionNetGame->setText(i18n("network game"));
    ui_actionNetGame->setIcon(KIcon(":/images/connect.png"));
    connect(ui_actionNetGame, SIGNAL(triggered()), this, SLOT(onNetGame()));
    
    QAction *ui_actionBack =KStandardAction::undo(this, SLOT(takeBack()), actionCollection());
    actionCollection()->addAction("takeBack", ui_actionBack);
    QAction *ui_actionForward = KStandardAction::redo(this, SLOT(forward()), actionCollection());
    actionCollection()->addAction("forward",ui_actionForward);
    ///@todo implement logic behind
    //ui_actionBack->setEnabled(false);
    //ui_actionForward->setEnabled(false);
    
    QAction *ui_actionForce_move = actionCollection()->addAction("forceMove");
    ui_actionForce_move->setText(i18n("force move"));
    connect(ui_actionForce_move, SIGNAL(triggered()), this, SLOT(onForceMove()) );

    QAction *ui_actionSetPlayers = actionCollection()->addAction("setPlayers");
    ui_actionSetPlayers->setText(i18n("set players"));
    connect(ui_actionSetPlayers, SIGNAL(triggered()), this, SLOT(onSetPlayers()) );
    
    QAction *ui_actionConfiguration = KStandardAction::preferences(this, SLOT(preferencesDialog()), actionCollection());
    actionCollection()->addAction("Preferences",ui_actionConfiguration);
    
    ui_actionStop_thinking =  actionCollection()->addAction("stopThinking");
    ui_actionStop_thinking->setText(i18n("stop thinking"));
    ui_actionStop_thinking->setIcon(KIcon(":/images/stop.png"));
    ui_actionStop_thinking->setEnabled(false);
    connect(ui_actionStop_thinking, SIGNAL(triggered()), this, SLOT(stopThinking()));
    
    // View
    KToggleAction *ui_actionInvert_board = actionCollection()->add<KToggleAction>("invertBoard");
    ui_actionInvert_board->setText(i18n("invert board"));
    connect(ui_actionInvert_board, SIGNAL(toggled(bool)), boardView, SLOT(invertBoard(bool)));
    
     setupGUI();
     
    toolBar()->addAction(ui_actionBack);
    toolBar()->addAction(ui_actionForward);
    toolBar()->addAction(ui_actionStop_thinking);
}

void draughts::closeEvent(QCloseEvent *event)
{
    if (maybeSave()) {
       // writeSettings();
        event->accept();
    } else {
        event->ignore();
    }
}

void draughts::newFile()
{
    if (maybeSave()) 
    {
        setCurrentFile("");
    }
}

void draughts::open()
{
  
  
//   {
//      QString fileName("/home/piotr/src/draughts/DXP/dxpGameReq.txt");
//      QFile file(fileName);
//      if(!file.open(QFile::ReadWrite))
//      {
//         QMessageBox::warning(this, tr("Application"),
//                              tr("Cannot read file %1:\n%2.")
//                              .arg(fileName)
//                              .arg(file.errorString()));
//     }
//     //dxpGameReq req;
//     QDataStream in(&file);
//     DxpMsg::readF( in );
//     //req.writeTo( out );
//   }
  
  
//if (pBlackPlayer)pNetPlayer->sendGameReq();
    if (maybeSave()) 
    {
        QString fileName = QFileDialog::getOpenFileName(this);
        if (!fileName.isEmpty())
            loadFile(fileName);
    }
}

bool draughts::save()
{
    //statusBar()->showMessage("save not implemented", 2000);
    if (curFile.isEmpty()) 
    {
        return saveAs();
    } 
    else 
    {
        return saveFile(curFile);
    }
}

bool draughts::saveAs()
{
    QString fileName = QFileDialog::getSaveFileName(this);
    if (fileName.isEmpty())
        return false;

    return saveFile(fileName);
}

bool draughts::maybeSave()
{
   bool isDocumentChanged=0;
    if (isDocumentChanged) 
    {
        int ret = QMessageBox::warning(this, tr("Application"),
                     tr("The document has been modified.\n"
                        "Do you want to save your changes?"),
                     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No,
                     QMessageBox::Cancel | QMessageBox::Escape);
        if (ret == QMessageBox::Yes)
            return save();
        else if (ret == QMessageBox::Cancel)
            return false;
    }
    return true;
}

void draughts::loadFile(const QString &fileName)
{
    QFile file(fileName);
    if (!file.open(QFile::ReadOnly | QFile::Text)) {
        QMessageBox::warning(this, tr("Application"),
                             tr("Cannot read file %1:\n%2.")
                             .arg(fileName)
                             .arg(file.errorString()));
        return;
    }
    int oldSize=Conf().m_boardSize;
    QDataStream in(&file);
    qint32 size=8;
    QString rules;
    in >> size;
    in >> rules;
    Conf().m_boardSize=size;
    Conf().m_Rules=rules;
    Conf().write();
    onSizeChanged( oldSize );
    
    pGame->load( file );

    setCurrentFile(fileName);
    //statusBar()->showMessage(tr("File loaded"), 2000);
}

bool draughts::saveFile(const QString &fileName)
{
    QFile file(fileName);
    if (!file.open(QFile::WriteOnly | QFile::Text)) {
        QMessageBox::warning(this, tr("Application"),
                             tr("Cannot write file %1:\n%2.")
                             .arg(fileName)
                             .arg(file.errorString()));
        return false;
    }

    
    QDataStream out(&file);
    out << qint32( Conf().m_boardSize );
    out << Conf().m_Rules;
    pGame->save( file );
    
    setCurrentFile(fileName);
    //statusBar()->showMessage(tr("File saved"), 2000);
    return true;
}

void draughts::setCurrentFile(const QString &fileName)
{
    curFile = fileName;

    setWindowModified(false);

    if (curFile.isEmpty())
        setWindowTitle(tr("Application"));
    else
        setWindowTitle(tr("%1[*] - %2").arg(strippedName(curFile))
                                       .arg(tr("Application")));
}

QString draughts::strippedName(const QString &fullFileName)
{
    return QFileInfo(fullFileName).fileName();
}


/*!
    \fn draughts::preferencesDialog()
 */
void draughts::preferencesDialog()
{
   QDialog *d=new SettingsDialog(this);
   connect(d, SIGNAL(sizeChanged(int)), this, SLOT(onSizeChanged(int)));
   d->show();
}

/*!
    \fn draughts::newGame()
 */
void draughts::newGame()
{
    //statusBar()->showMessage("newGame", 2000);

    int resp=0;
    newGameDialog *d=new newGameDialog(this);
    
    d->resize( d->minimumSize() );
    d->setWindowTitle( tr("New Game"));
    d->setNetPlayerEnabled( pNetPlayer );
    d->setWhitePlayer( pWhitePlayer, false );
    d->setBlackPlayer( pBlackPlayer, false );
    
    
    resp=d->exec();
    if(resp !=QDialog::Accepted) {delete d;return;}
    
    setPlayers( d );
    bool netPlayWhite= (d->getWhitePlayer() == AbstractPlayer::networkType );
    if( pNetPlayer )
    {
       connect( pNetPlayer, SIGNAL(gameAcc(bool)), this, SLOT(onGameAcc(bool)) );
       pNetPlayer->sendGameReq(netPlayWhite);
    }
    
    pGame->newGame();
    if( !pNetPlayer )wakeUp();
}

/*!
    \fn draughts::back()
 */
void draughts::takeBack()
{
    if( pNetPlayer ) return;
    
    //statusBar()->showMessage("half move takeBack", 2000);
    if (pGame->takeBack())
    {
       AbstractPlayer *p=pWhitePlayer;
       pWhitePlayer=pBlackPlayer;
       pBlackPlayer=p;
    }
}

/*!
    \fn draughts::forward()
 */
void draughts::forward()
{
    if( pNetPlayer ) return;
    
    //statusBar()->showMessage("half move FORWARD", 2000);
    if (pGame->forward())
    {
       AbstractPlayer *p=pWhitePlayer;
       pWhitePlayer=pBlackPlayer;
       pBlackPlayer=p;
    }
}


/*!
    \fn draughts::onSizeChanged()
 */
void draughts::onSizeChanged( int oldSize )
{
    int size=Conf().m_boardSize;
    //if( pGame ) delete pGame;
    
    engineFactory();
    Game *nG=new Game ( pEngine );
    boardModel->setGame( nG );
    moveListModel->setGame( nG );
    
    if( pGame ) delete pGame;
    pGame=nG;

    qDebug("rowCountChanged( oldSize %d, size %d)",oldSize, size );
    boardView->rowCountChanged( oldSize, size );
    boardView->columnCountChanged( oldSize, size);

    boardView->setMinimumSize( boardView->minimumSizeHint()  );
    boardView->setMaximumSize( boardView->minimumSizeHint()  );
    
    connect( pGame, SIGNAL(documentChanged()), this, SLOT(onDocumentChanged()));
    disconnect( engineInfoWidget );
    connect( pGame, SIGNAL(engineInfoChanged( const EngineInfo * )), engineInfoWidget, SLOT(onEngineInfoChanged( const EngineInfo * )));
    resize( sizeHint() );
    //wakeUp();
    newGame();
}


/*!
    \fn draughts::createView()
 */
void draughts::createView()
{
//menuBar ()->addMenu( new QMenu( "file" ) );
//menuBar ()->addMenu( new QMenu( "game" ) );

    setCentralWidget( new QWidget() );// uwaga to pomoglo zadzialal
    boardModel = new BoardModel( pGame );

    PieceDelegate *delegate = new PieceDelegate(this);
    //boardView->setItemDelegate(delegate);
    boardView=new BoardView( centralWidget(), delegate );
    boardView->setModel( boardModel );
    
    int size=Conf().m_boardSize;
    boardView->rowCountChanged( size, size );
    boardView->columnCountChanged( size, size);
    
    
    boardView->setMinimumSize( boardView->minimumSizeHint()  );
    boardView->setMaximumSize( boardView->minimumSizeHint()  );
    
    
    moveListView = new MoveListView();
    moveListModel=new MoveListModel(pGame);
    moveListView->setModel( moveListModel );

    engineInfoWidget =new EngineInfoWidget( this );
    connect( pGame, SIGNAL(engineInfoChanged( const EngineInfo * )), engineInfoWidget, SLOT(onEngineInfoChanged( const EngineInfo * )));
    
    QVBoxLayout *vLayout = new QVBoxLayout( );
    vLayout->addWidget(boardView);
    QSpacerItem * spacer=new QSpacerItem (1,1,QSizePolicy::Minimum, QSizePolicy::Expanding);
    vLayout->addItem( spacer );
    
    
    QVBoxLayout *vLayoutRight = new QVBoxLayout( );
    vLayoutRight->addWidget(engineInfoWidget);
    vLayoutRight->addWidget(moveListView);
    
    layout = new QHBoxLayout( );
    layout->addLayout( vLayout );
    layout->addLayout( vLayoutRight );


    centralWidget()->setLayout( layout );
    resize( sizeHint() );
}


/*!
    \fn draughts::engineFactory()
    produces engine depending of configuration
 */
AEngine * draughts::engineFactory()
{
    //remove old one
   if( pEngine) delete pEngine;pEngine=0;
   
   QString rules=Conf().m_Rules;
   if( rules=="International" )
   {
       //pEngine=new InternationalEngine();
       switch( Conf().m_boardSize )
       {
          case 8:
          pEngine=new EngineIntT<quint32,8>();
          break;
          
          case 10:
          pEngine=new EngineIntT<quint64,10>();
          break;
          
          default:
          QMessageBox::critical(this, "ERROR", 
             QString("size %1 is not implemented").arg( Conf().m_boardSize ) );
          Conf().m_boardSize=8;Conf().write();
       }
   }
   else if( rules=="American" )
   {
      switch( Conf().m_boardSize )
      {
          case 8:
          pEngine=new EngineAmT<quint32,8>();
          break;
          
          case 10:
          pEngine=new EngineAmT<quint64,10>();
          break;
          
          default:
          QMessageBox::critical(this, "ERROR", 
             QString("size %1 is not implemented").arg( Conf().m_boardSize ) );
          Conf().m_boardSize=8;Conf().write();
       }
   }
   else
   {
       QMessageBox::critical(this, "ERROR", 
           QString("game rules %1 are not implemented").arg( rules) );
       Conf().m_Rules="International";Conf().write();
   }
   return 0;
}


/*!
    \fn draughts::onGameOver(int)
 */
void draughts::onGameOver(int result)
{
    qDebug("draughts::onGameOver");
    QString msg;
    switch( result)
    {
       case Game::draw:
       msg="No winner, draw";
       break;
       
       case Game::blackWin:
       msg="BLACK player won";
       break;
       
        case Game::whiteWin:
       msg="WHITE player won";
       break;
    }
    QMessageBox::information(this, "Game Over", msg);
}


/*!
    \fn draughts::onDocumentChanged()
 */
void draughts::onDocumentChanged()
{
    Q_ASSERT( pGame );
    int whiteM, whiteK, blackM, blackK, moveNr,toDraw;
    int gameOver=pGame->getStatus( whiteM,  whiteK, blackM,  blackK,  moveNr,toDraw);
    
    QString toMove;
    toMove= ( moveNr%2 )? "Black to move":"White to move";
    if(gameOver) toMove="Game Over";
    QString status=QString("White: (%1+%2) Black: (%3+%4). %5 .moves to draw: %6").arg(whiteM).arg(whiteK).arg(blackM).arg(blackK).arg(toMove).arg(toDraw);
       
    statusBar()->showMessage(status);
}

/*!
    \fn draughts::onMoveFinished()
    player Signalled move finished
 */
void draughts::onMoveFinished( LegalMove LM )
{
    Q_ASSERT( pGame );
    pGame->makeMove (LM);
    
    Board *pB=pGame->getCurrentBoard();
    
    AbstractPlayer *p=0;
    if( pB->isWhiteTurn())
      p=pWhitePlayer;
    else
      p=pBlackPlayer;

    if( p->getType() == AbstractPlayer::engineType )
       ui_actionStop_thinking->setEnabled(true);
    p->yourMove(&LM);
    
    if( pGame->isGameOver() )
    {
       qDebug("is game over !!!");
       return;
    }
    
    ui_actionStop_thinking->setEnabled(false);
}

void draughts::setPlayers( newGameDialog *d )
{
    Q_ASSERT(d);
    AbstractPlayer::playerType reqWhite=d->getWhitePlayer();
    AbstractPlayer::playerType reqBlack=d->getBlackPlayer();
    disconnect( pBlackPlayer ,SIGNAL(moveFinished(LegalMove)),0,0);
    disconnect( pWhitePlayer ,SIGNAL(moveFinished(LegalMove)),0,0);
    {
       if( pWhitePlayer->getType()!=AbstractPlayer::networkType )
          delete pWhitePlayer;
       
       pWhitePlayer=0;
       if( reqWhite == AbstractPlayer::humanType )
          pWhitePlayer = new humanPlayer( this,boardModel );
          
       if( reqWhite == AbstractPlayer::engineType )
       {
          pWhitePlayer = new enginePlayer(this,pGame);
          pWhitePlayer->setLevel( Conf().m_WhiteLevel , Conf().m_WhiteLevelType );
       }
          
       if( reqWhite == AbstractPlayer::networkType )
          pWhitePlayer=pNetPlayer;
       Q_ASSERT(pWhitePlayer);
       connect( pWhitePlayer, SIGNAL(moveFinished(LegalMove)), this, SLOT(onMoveFinished(LegalMove)));
    }

    {
       if( pBlackPlayer->getType()!=AbstractPlayer::networkType )
          delete pBlackPlayer;
       pBlackPlayer=0;
       if( reqBlack == AbstractPlayer::humanType )
          pBlackPlayer = new humanPlayer( this,boardModel );
          
       if( reqBlack == AbstractPlayer::engineType )
       {
          pBlackPlayer = new enginePlayer(this,pGame);
          pBlackPlayer->setLevel( Conf().m_BlackLevel ,Conf().m_BlackLevelType );
       }
          
       if( reqBlack == AbstractPlayer::networkType )
          pBlackPlayer=pNetPlayer;
       Q_ASSERT(pBlackPlayer);
       connect( pBlackPlayer, SIGNAL(moveFinished(LegalMove)), this, SLOT(onMoveFinished(LegalMove)));
    }
    resize( sizeHint() );
}

int draughts::onSetPlayers()
{
  qDebug( "onSetPlayers ");
  
    int resp=0;
    newGameDialog *d=new newGameDialog(this);
    
    d->setWindowTitle( tr("Set Players"));
    d->setNetPlayerEnabled( pNetPlayer );
    d->setWhitePlayer( pWhitePlayer, true );
    d->setBlackPlayer( pBlackPlayer, true );
    d->resize( d->minimumSize() );
    resp=d->exec();
    
    if(resp !=QDialog::Accepted) {delete d;return resp;}
    qDebug("players confirmed");

    setPlayers( d );
    wakeUp();
    delete d;
    return resp;
}

AbstractPlayer*& draughts::getCurrentPlayer()
{
   Board *pB=pGame->getCurrentBoard();
   if( pB->isWhiteTurn())
      return pWhitePlayer;
    else
      return pBlackPlayer;
}
AbstractPlayer*& draughts::getNextPlayer()
{
   Board *pB=pGame->getCurrentBoard();
   if( !pB->isWhiteTurn() )
      return pWhitePlayer;
    else
      return pBlackPlayer;
}

/*!
    \fn draughts::onForceMove()
 */
void draughts::onForceMove()
{
   qDebug( "draughts::onForceMove()");
   
   AbstractPlayer*& rpCurPlayer=getCurrentPlayer();
   enum AbstractPlayer::playerType t= rpCurPlayer->getType();
   if( t != AbstractPlayer::humanType )return;
   
    delete rpCurPlayer;
    rpCurPlayer = new enginePlayer(this,pGame);

    if( rpCurPlayer== pWhitePlayer )
      rpCurPlayer->setLevel( Conf().m_WhiteLevel ,Conf().m_WhiteLevelType );
    else
      rpCurPlayer->setLevel( Conf().m_BlackLevel ,Conf().m_BlackLevelType );

    connect( rpCurPlayer, SIGNAL(moveFinished(LegalMove)), this, SLOT(onMoveFinished(LegalMove)));
    
    AbstractPlayer*& rpNextPlayer=getNextPlayer();
    t=rpNextPlayer->getType();
    if( t != AbstractPlayer::engineType )
    { 
       wakeUp();
       return;
    }

    rpNextPlayer = new humanPlayer( this,boardModel );
    connect( rpNextPlayer, SIGNAL(moveFinished(LegalMove)), this, SLOT(onMoveFinished(LegalMove)));
    wakeUp();
}

void draughts::wakeUp()
{
   ui_actionStop_thinking->setEnabled(true);
   Board *pB=pGame->getCurrentBoard();
   const LegalMove *pLastMove=pGame->getLastMove();
   if( pB->isWhiteTurn())
      pWhitePlayer->yourMove(pLastMove);
    else
      pBlackPlayer->yourMove(pLastMove);

   ui_actionStop_thinking->setEnabled(false);
}


void draughts::onNetGame()
{
   networkGameDialog *d=new networkGameDialog(this);
   connect( d, SIGNAL(connected(QAbstractSocket*)), this, SLOT(onConnected(QAbstractSocket*)) );
   connect( d, SIGNAL(disconnected()), this, SLOT(onDisconnected()) );
   d->show();
}
void draughts::onConnected(QAbstractSocket* pS)
{
   qDebug( "draughts::onConnected(QAbstractSocket* pS)" );
   Q_ASSERT( pS );
   
   pNetPlayer=new netPlayer( this, pS );
   connect( pNetPlayer, SIGNAL(gameReq(bool,bool)), this, SLOT(onGameReq(bool,bool)) );
}

void draughts::onDisconnected()
{
  qDebug( "draughts::onDisconnected()" );
  
  if(pNetPlayer==pWhitePlayer)
  {
    qDebug("net gral bialymi");
    pWhitePlayer = new humanPlayer( this,boardModel );
    connect( pWhitePlayer, SIGNAL(moveFinished(LegalMove)), this, SLOT(onMoveFinished(LegalMove)));
  }
  if(pNetPlayer==pBlackPlayer)
  {
    qDebug("net gral czarnymi");
    pBlackPlayer = new humanPlayer( this,boardModel );
    connect( pBlackPlayer, SIGNAL(moveFinished(LegalMove)), this, SLOT(onMoveFinished(LegalMove)));
  }
    
  if( pNetPlayer) delete pNetPlayer;
  pNetPlayer=0;
}

//----------------------------------------------------------------------
void draughts::onGameReq(bool normal,bool myColorIsWhite)
{
    qDebug( "gameReq received normal:%d myColorIsWhite:%d",normal ,myColorIsWhite);
    if( !normal )
    {
      qDebug("rejects not normal game");
      pNetPlayer->rejectGame();
      return;
    }
    
    int resp=0;
    newGameDialog *d=new newGameDialog(this);
    d->setWindowTitle( tr("New Game request"));

    d->setWhitePlayer( pWhitePlayer, true );
    d->setBlackPlayer( pBlackPlayer, true );
    d->setNetPlayWhite( !myColorIsWhite );
    
    resp=d->exec();
    
    if(resp !=QDialog::Accepted) 
    {
      pNetPlayer->rejectGame();
      return;
    }
    setPlayers( d );
    Q_ASSERT(pNetPlayer);
    
    pGame->newGame();
    boardView->update();
    pNetPlayer->acceptGame();
    wakeUp();
}
//----------------------------------------------------------------------
//tu powinien byc setPlayers
void draughts::onGameAcc(bool isAccepted)
{
    qDebug( "gameAcc received isAccepted:%d",isAccepted );
    if(isAccepted)
       wakeUp();
    else
       QMessageBox::warning(this, tr("Network player"),tr("New Game resquest was rejected"));
}

/*!
    \fn draughts::stopThinking()
 */
void draughts::stopThinking()
{
    Q_ASSERT( pEngine );
    
    
    Board *pB=pGame->getCurrentBoard();
    if( pB->isWhiteTurn() )
    {
      if( pBlackPlayer->getType()  == AbstractPlayer::engineType )
      {
        delete pBlackPlayer;
        pBlackPlayer = new humanPlayer( this,boardModel );
        connect( pBlackPlayer, SIGNAL(moveFinished(LegalMove)), this, SLOT(onMoveFinished(LegalMove)));
      }
      //if opponent plays net , we have to change current player
      if( pBlackPlayer->getType()  == AbstractPlayer::networkType )
      {
         delete pWhitePlayer;
         pWhitePlayer = new humanPlayer( this,boardModel );
         connect( pWhitePlayer, SIGNAL(moveFinished(LegalMove)), this, SLOT(onMoveFinished(LegalMove)));
      }
    }
    else
    {
      if( pWhitePlayer->getType()  == AbstractPlayer::engineType )
      {
        delete pWhitePlayer;
        pWhitePlayer = new humanPlayer( this,boardModel );
        connect( pWhitePlayer, SIGNAL(moveFinished(LegalMove)), this, SLOT(onMoveFinished(LegalMove)));
      }
      //if opponent plays net , we have to change current player
      if( pWhitePlayer->getType()  == AbstractPlayer::networkType )
      {
         delete pBlackPlayer;
         pBlackPlayer = new humanPlayer( this,boardModel );
         connect( pBlackPlayer, SIGNAL(moveFinished(LegalMove)), this, SLOT(onMoveFinished(LegalMove)));
      }
    }

    pEngine->stopThinking();
}

#include "draughts.moc"

