/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/


#ifndef DRAUGHTS_H
#define DRAUGHTS_H

#include <QObject>
#include <kxmlguiwindow.h>
#include <kaction.h>
#include "legalmove.h"


class QAction;
class BoardView;
class MoveListView;
class MoveListModel;
class BoardModel;
class netPlayer;
class AbstractPlayer;
class humanPlayer;
class newGameDialog;
class QAbstractSocket;
class QHBoxLayout;

class Game;
class AEngine;

class draughts : public KXmlGuiWindow
{
    Q_OBJECT

public:
    draughts();
    ~draughts();

protected:
    void closeEvent(QCloseEvent *event);

private slots:
    void open();
    bool save();
    void newFile();

    void newGame();
    void takeBack();
    void forward();
    bool saveAs();
    void onGameOver(int);
    void onDocumentChanged();

    void onNetGame();
    void stopThinking();

private:
    void connectActions();
    void loadBook();
    bool maybeSave();
    void loadFile(const QString &fileName);
    bool saveFile(const QString &fileName);
    void setCurrentFile(const QString &fileName);
    QString strippedName(const QString &fullFileName);
    void createView();
    AEngine * engineFactory();
    void setPlayers( newGameDialog *d );
    QHBoxLayout *layout;

    QToolBar *gameToolBar;
    QString curFile;
    BoardView *boardView;
    BoardModel *boardModel;
    
    MoveListView *moveListView;
    MoveListModel *moveListModel;
    
    QWidget *engineInfoWidget;
    
    Game *pGame;
    AEngine *pEngine;
    netPlayer *pNetPlayer;
    AbstractPlayer *pWhitePlayer;//humanPlayer
    AbstractPlayer *pBlackPlayer;
    AbstractPlayer*& getCurrentPlayer();
    AbstractPlayer*& getNextPlayer();

    QAction *ui_actionStop_thinking;
//     KAction *ui_actionNew;
//     KAction *ui_actionBack;
//     KAction *ui_actionForward; 
//     KAction *ui_actionConfiguration;
//     KAction *ui_actionForce_move;
// 
//     KAction *ui_actionOpen_Game;
//     KAction *ui_actionSave_As;
//     KAction *ui_actionSave_Game;
//     KAction *ui_actionSetPlayers;
//     KAction *ui_actionNetGame;
//     KAction *ui_actionInvert_board;


public slots:
    void preferencesDialog();
    void onSizeChanged( int oldSize );
    void onMoveFinished(LegalMove);
    int onSetPlayers();
    void onForceMove();
    void wakeUp();
    void onGameReq( bool normal ,bool followerIsWhite);
    void onGameAcc(bool);
    void onConnected(QAbstractSocket*);
    void onDisconnected();
};



#endif
