/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/
#ifndef DXPGAMEACC_H
#define DXPGAMEACC_H

#include <QByteArray>
#include "legalmove.h"

/**
	@author Piotr Banasiak <piotrbanasiak@onet.pl>
*/
class QDataStream;

class dxpGameReq
{
  public:
  dxpGameReq();
  ~dxpGameReq();
  QDataStream& writeTo( QDataStream& out )const;
  bool readFrom( QDataStream& in );
  void setFollowerWhite(bool);


  QByteArray header;
  QByteArray version;
  QByteArray initiatorName;
  QByteArray followerColor; // w or Z
  QByteArray thinkingTime; // time in minutes for whole game
  QByteArray numberOfMoves; //fot the whole game, can be Zero
  QByteArray startingPosition;// A -normal , B , use the next two fields
  QByteArray  colorToMoveFirst; // W white, Z black
  QByteArray position;// 
                    // 'e': empty field
                    // 'w': white piece
                    // 'z': black piece
                    // 'W': white king
                    // 'Z': black king
};

class dxpGameAcc 
{
  public:
  dxpGameAcc();
  ~dxpGameAcc();
  QDataStream& writeTo( QDataStream& out )const;
  bool readFrom( QDataStream& in );
  bool isAccepted();
  void setReject();

  QByteArray header;
  QByteArray initiatorName;//32
  QByteArray code; //  0 1 2 3 indian (but what exactly?)
  
};

class dxpMove 
{
  public:
  dxpMove();
  ~dxpMove();
  QDataStream& writeTo( QDataStream& out )const;
  bool readFrom( QDataStream& in );
  LegalMove toLegalMove();
  void fromLegalMove(const LegalMove &);

  QByteArray header;
  QByteArray time;       //4
  QByteArray fromField;  // 2
  QByteArray toField;    // 2
  QByteArray numberCaptured; // 2
  QByteArray capturedPieces;//variable
  private:
  int m_numberCapt;
};

class dxpGameEnd 
{
  public:
  dxpGameEnd();
  ~dxpGameEnd();
  QDataStream& writeTo( QDataStream& out )const;
  bool readFrom( QDataStream& in );

  QByteArray header;
  QByteArray reason;    //4
  QByteArray stopCode;  // 2
};

class dxpChat
{
  public:
  dxpChat();
  ~dxpChat();
  void setText(const QString &);
  QDataStream& writeTo( QDataStream& out )const;
  bool readFrom( QDataStream& in );

  QByteArray header;
  QByteArray text;
};
int toInt( const QByteArray& );
QByteArray fromInt( int x,int size );
QPoint pdn2point(int pieceNr);
int point2pdn( const QPoint & );
#endif
