/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/

#ifndef ENGINEINT64_H
#define ENGINEINT64_H

#include "abstractenginet.h"
#include "boardt.h"
#include "legalmovet.h"


/**
  *@author Piotr Banasiak
   international engine (rules) implemented as template
   T is quint32 for 8x8 board
        quint64 for 10x10 board
        uint72 for board 12x12
    T must support bits operations like shift and so
  */
template <class T, int bSize>
class EngineIntT : public AbstractEngineT<T,bSize>
{
  enum gamePhase
  {
     earlyPhase=0,
     middlePhase,
     gambitPhase
  };
  enum Score
  {
    scoreMan=1000,
    scoreKing=3500,
    scoreKingInGambit=3700,
    scoreNextKing=3200
  };
  
public: 
  EngineIntT();
  ~EngineIntT() ;
    
    virtual int gamePhase(const BoardT<T>* const pBoard)const;
    void setHash( QHash<BoardT<T>,HashNode<LegalMoveT<T> > > *pHash ){pH=pHash;};
    virtual void fillBook( QList<QList<LegalMove> > &Book );

    virtual void genPossibleMoves( QList<LegalMoveT<T> > &MoveList ,const BoardT<T>* const pBoard );
    virtual void genPossibleMoves( QList<LegalMove > &MoveList ,const Board* const pBoard );
    virtual void reFillSQV( int size,int phase );
    
    
    QHash<BoardT<T>,HashNode<LegalMoveT<T> > > *pH;
private:
  virtual int materialScore( const BoardT<T> * const pBoard )const;
  int earlyPhaseSQV(int x, int y);
  int middlePhaseSQV(int x, int y);
  int gambitPhaseSQV(int x, int y);
  void fillBook8( QList<QList<LegalMove> > &Book);
  void fillBook10( QList<QList<LegalMove> > &Book);
  
 void genNKingMove( QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B ,QList<LegalMove> *pList);
 inline void genKingMovesDirection( int s, QList<LegalMoveT<T> > &MoveList, T squares , int* D, QList<LegalMove> *pList);
 void genCapKingMove( QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B , QList<LegalMove> *pList=0)const;
 bool genCapKingMovePart( const int src0, int sqNr, int level, QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B ,
                      QList<LegalMove>*pList=0, LegalMove *pPart=0 )const;
 inline bool genKingCapDirection(const int src0, int s,int level, QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B, const int* D,
                                 QList<LegalMove>*pList, LegalMove *pPart )const;

 void genNManMove( QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B ,QList<LegalMove> *pList=0);
 inline void genManMovesDirection( int s, QList<LegalMoveT<T> > &MoveList, T squares, int* D ,QList<LegalMove> *pList);
 
 void genCapManMove( QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B , QList<LegalMove> *pList=0);
 void genCapPart(int src0, int srcNr, T opponent, int level, QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B, 
                 QList<LegalMove>*pList=0, LegalMove *pPart=0 );
 inline bool genCapPartDir(int src0, int srcNr, T opponent, int level, QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B, int* D,
                           QList<LegalMove>*pList, LegalMove *pPart );
 void leaveMaxCapture(QList<LegalMoveT<T> > &MoveList);
 void leaveMaxCapture( QList<LegalMove> &MoveList );///@todo join both functions

 inline bool canOpponentCapture( const BoardT<T> &B )const;  
 inline bool checkDirection( int sqNr, const T& opponent, const T& squares, const int * D )const;
};



/////////////////////////////////////////////////////////////////////////////////////////////
// implementation 
/////////////////////////////////////////////////////////////////////////////////////////////

template <class T, int bSize>
EngineIntT<T, bSize>::EngineIntT():AbstractEngineT<T,bSize>()
{
   Q_ASSERT( bSize==8 or bSize==10 or bSize==12 );
  
}
//------------------------------------------------------------------
template <class T, int bSize>
EngineIntT<T, bSize>::~EngineIntT()
{
}
//------------------------------------------------------------------
/** normal moves of men FIXME 
  bytes are xxxx 31,30 .... 2,1 ,0
  where xxx are unused in case variable is longer then need
*/
template <class T, int bSize>
void EngineIntT<T, bSize>::genNManMove( QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B ,QList<LegalMove> *pList)
{
  Q_ASSERT( !B.captured );
  
  T myMen;
  T squares= B.blackK | B.blackM | B.whiteK | B.whiteM;
  
  if( B.isWhiteTurn())
    myMen=B.whiteM;
  else
    myMen=B.blackM;

  int sqNr=0;// debug
  T srcMask=0x01;
  T shift=myMen;
  
  while( shift )
  {
    if(myMen & srcMask)
    {
      if( B.isWhiteTurn())
      {
        genManMovesDirection( sqNr, MoveList, squares, this->LU , pList);
        genManMovesDirection( sqNr, MoveList, squares, this->RU , pList);
      }
      else
      {
        genManMovesDirection( sqNr, MoveList, squares, this->LD , pList);
        genManMovesDirection( sqNr, MoveList, squares, this->RD , pList);
      }
    }

    srcMask <<= 1;
    // don't have to check all bits
    shift >>= 1; sqNr++;
  }
}
//-----------------------------------------------------------------------
template <class T, int bSize>
void EngineIntT<T, bSize>::genManMovesDirection( int s, QList<LegalMoveT<T> > &MoveList, T squares, int* D , QList<LegalMove> *pList)
{
  int d=D[s];
  if(d<0)return;//outside Board
  
  T M=0x01;
  M<<=d;
  
  bool noEmpty=squares & M;
  if( noEmpty )
    return;
    
    //add the move to the list
    LegalMoveT<T> lm;
    lm.src = s;
    lm.dst = d;
    MoveList.append( lm );
    
    if( pList )
    {
       QPoint start( this->Bx[s], this->By[s] );
       QPoint end( this->Bx[d], this->By[d] );
       LegalMove lm(start,end);
       
       pList->append( lm );
    }
}
//-----------------------------------------------------
/** generates all possible capture  man moves 
    pLM is LegalMove with intermediate squares used in Gui
*/
template <class T, int bSize>
void EngineIntT<T, bSize>::genCapManMove( QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B, QList<LegalMove> *pList )
{
  T myMen;

  Q_ASSERT( !B.captured );
  T opponent;
  
  if( B.isWhiteTurn())
  {
    myMen=B.whiteM;
    opponent=( B.blackM | B.blackK );
  }
  else
  {
    myMen=B.blackM;
    opponent=( B.whiteM | B.whiteK );
  }

  int sqNr=0;// debug
  T srcMask=0x01;
  T shift=myMen;

  while( shift )
  {
    bool candidate=myMen & srcMask;
    if(candidate)
    {
      if( pList )
      {
         LegalMove part;
         QPoint start( this->Bx[sqNr], this->By[sqNr] );
         part.insPath( start,0 );
         genCapPart(sqNr, sqNr, opponent, 0, MoveList, B, pList, &part);
      }
      else
         genCapPart(sqNr, sqNr, opponent, 0, MoveList, B, 0, 0);
    }

    srcMask <<= 1;
    // don't have to check all bits
    shift >>= 1; sqNr++;
  }
}
//-------------------------------------------------------------
// src0 very first position of man
// sqNr is where last multicapture is finished, we start checking from that position
template <class T, int bSize>
void EngineIntT<T, bSize>::genCapPart(int src0, int sqNr, T opponent, int level, QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B, 
                                      QList<LegalMove> *pList, LegalMove *pPart)
{
   bool atLeastOne=false;
   bool ok=false;
   
   ok=genCapPartDir(src0, sqNr, opponent, level , MoveList, B, this->LU, pList, pPart);
   if(ok) atLeastOne=true;
   
   ok=genCapPartDir(src0, sqNr, opponent, level , MoveList, B, this->RU, pList, pPart);
   if(ok) atLeastOne=true;
   
   ok=genCapPartDir(src0, sqNr, opponent, level , MoveList, B, this->LD, pList, pPart);
   if(ok) atLeastOne=true;
   
   ok=genCapPartDir(src0, sqNr, opponent, level , MoveList, B, this->RD, pList, pPart);
   if(ok) atLeastOne=true;
   
   if( !atLeastOne and level>0 )// or B.captured
   {
      LegalMoveT<T> lm;
      lm.src = src0;
      lm.dst=  sqNr;
      lm.captured=B.captured;
      
      MoveList.append( lm );
      
      // and pPart represent finished/full multicapture move
      // so add it to move list
      
      if( pList )
      {
         pList->append( *pPart );
      }
   }
}
//------------------------------------------------------------------
// check/generate next captures in multicapture move
// D direction table 
// pLm and pPart are to generate LegalMoves with intermediate steps.
template <class T, int bSize>
bool EngineIntT<T, bSize>::genCapPartDir(int src0, int srcNr, T opponent, int level, 
     QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B, int* D,  QList<LegalMove> *pList, LegalMove *pPart)
{
  bool captured=false;
  
  T squares= ( B.whiteM | B.whiteK | B.blackM | B.blackK | B.captured );
  
      int d1=D[srcNr];
      int d2=-1;
      if( d1>=0)d2=D[d1];
      if( d1>=0 and d2>=0 )//both on board
      {
        T d1Mask= (T(1) << d1);
        T d2Mask= (T(1) << d2);
        
        if( !(squares & d2Mask) and( d1Mask & opponent ) )
        {
          // generating intermediate information for Gui, building part of the move
          if( pPart )
          {
             QPoint start( this->Bx[srcNr], this->By[srcNr] );
             QPoint inter( this->Bx[d1], this->By[d1] );
             QPoint end( this->Bx[d2], this->By[d2] );
             
             pPart->insPath(end,level+1);
             pPart->insTaken(inter,level);
          }
          
          T inv = ~d1Mask;
          BoardT<T> b(B);
          
          //capture opponent
          b.captured |= d1Mask;
          b.whiteM &= inv;
          b.whiteK &= inv;
          b.blackM &= inv;
          b.blackK &= inv;
          
          //move my own
          T srcMask= 1;
          srcMask <<= srcNr;

          T opponent;
          const bool wTurn=b.isWhiteTurn();
          if( wTurn )
          {
            b.whiteM &= (~srcMask);
            b.whiteM |= d2Mask;
            opponent=( b.blackM | b.blackK );
          }
          else
          {
            b.blackM &= (~srcMask);
            b.blackM |= d2Mask;
            opponent=( b.whiteM | b.whiteK );
          }

          genCapPart( src0, d2 , opponent, level+1, MoveList, b, pList, pPart );
          captured=true;
        }
      }

  return captured;
}


//--------------------------------------------------------------
/** No descriptions */
template <class T, int bSize>
void EngineIntT<T, bSize>::genNKingMove( QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B ,QList<LegalMove> *pList=0)
{
  T myKings;
  if( B.isWhiteTurn() )
    myKings=B.whiteK;
  else
    myKings=B.blackK;

  T squares= B.blackK | B.blackM | B.whiteK | B.whiteM;
  
  int sqNr=0;// debug
  T srcMask=0x01;
  T shift=myKings;

  while( shift )
  {
    if(myKings & srcMask)
    {
      genKingMovesDirection( sqNr, MoveList, squares, this->LU , pList);
      genKingMovesDirection( sqNr, MoveList, squares, this->RU , pList);
      genKingMovesDirection( sqNr, MoveList, squares, this->LD , pList);
      genKingMovesDirection( sqNr, MoveList, squares, this->RD , pList);
    }

    srcMask <<= 1;
    // don't have to check all bits
    shift >>= 1; sqNr++;
  }
}
//------------------------------------------------------------------------------
/** D - direction table
*/
template <class T, int bSize>
void EngineIntT<T, bSize>::genKingMovesDirection( int s, QList<LegalMoveT<T> > &MoveList, T squares , int* D, QList<LegalMove> *pList)
{
  int  d=s;
  while( (d=D[d]) >=0 )// next move is on board
  {
    T M=0x01;
    M<<=d;

    bool noEmpty=squares & M;
    if( noEmpty )
      break;

    //add the move to the list
    LegalMoveT<T> lm;
    lm.src = s;
    lm.dst = d;
    MoveList.append( lm );
    
    if( pList )
    {
       QPoint start( this->Bx[s], this->By[s] );
       QPoint end( this->Bx[d], this->By[d] );
       LegalMove lm(start,end);
       
       pList->append( lm );
    }
  }
}
//----------------------------------------------------------------------------------------------
// generates fast 
template <class T, int bSize>
void EngineIntT<T, bSize>::genPossibleMoves( QList<LegalMoveT<T> > &MoveList ,const BoardT<T>* const pB )
{
  Q_ASSERT( pB );

  MoveList.clear();
  
  genCapManMove( MoveList, *pB );
  genCapKingMove( MoveList, *pB );
  
  int howMany=MoveList.count();
  /// @todo remove moves that has the same src,dst and captures

  //captures are mandatory
  if( howMany>0 )
  {
    leaveMaxCapture(MoveList);
    return ;
  }

  //no captures, gen normal moves
  genNManMove( MoveList, *pB );
  genNKingMove( MoveList, *pB );
 
   howMany=MoveList.count();
}
//----------------------------------------------------------------------------------------------
// only check if opponent has at least one capturing move
template <class T, int bSize>
bool EngineIntT<T, bSize>::canOpponentCapture( const BoardT<T> &B )const
{
  Q_ASSERT( !B.captured );
  T myPieces;
  T opponentMen;
  T opponentKings;
  T squares = B.blackM | B.blackK | B.whiteM | B.whiteK;
  if( B.isWhiteTurn())
  {
    myPieces= ( B.whiteM | B.whiteK);
    opponentMen=B.blackM ;
    opponentKings=B.blackK;
  }
  else
  {
    myPieces= (B.blackM | B.blackK);
    opponentMen=B.whiteM;
    opponentKings=B.whiteK;
  }

  int sqNr=0;// debug
  T M=0x01;
  T shift=opponentMen;

  while( shift )
  {
    bool candidate=opponentMen & M;
    bool ok=false;
    if(candidate)
    {
      ok=checkDirection( sqNr, myPieces, squares, this->LU);
      if(ok) return true;

      ok=checkDirection( sqNr, myPieces, squares, this->RU);
      if(ok) return true;

      ok=checkDirection( sqNr, myPieces, squares, this->LD);
      if(ok) return true;

      ok=checkDirection( sqNr, myPieces, squares, this->RD);
      if(ok) return true;
    }

    M <<= 1;
    // don't have to check all bits
    shift >>= 1; sqNr++;
  }
  
  if( opponentKings )
  {
    BoardT<T> newB(B);
    QList<LegalMoveT<T> > MoveList;
    newB.moveNr++; // pretend is opponent move
    
    genCapKingMove( MoveList, newB,0 );
    if( MoveList.count() )
       return true;
  }
  return false;
}
//----------------------------------
template <class T, int bSize>
bool EngineIntT<T, bSize>::checkDirection( int sqNr, const T& opponent, const T& squares, const int * D )const
{
  int d1=D[sqNr];
  int d2=-1;
  if( d1>=0)d2=D[d1];
  if( d1>=0 and d2>=0 )//both on board
  {
    T d1Mask= (T(1) << d1);
    T d2Mask= (T(1) << d2);
    
    if( !(squares & d2Mask) and( d1Mask & opponent ) )
      return true;
  }
  return false;
}
//----------------------------------------------------------------------------------------------
// LegalMove used in GUI needs intermediate results.
template <class T, int bSize>
void EngineIntT<T, bSize>::genPossibleMoves( QList<LegalMove > &MoveList ,const Board* const pB )
{
  Q_ASSERT( pB );

  BoardT<T> B( pB );
  
  MoveList.clear();
  QList<LegalMoveT<T> > MoveListNotUsed;
  
  genCapManMove( MoveListNotUsed, B, &MoveList );
  //AEngine::printMoveList( MoveList );
  genCapKingMove( MoveListNotUsed, B, &MoveList );
  
  int howMany=MoveList.count();
  /// @todo remove moves that has the same src,dst and captures

  //captures are mandatory
  if( howMany>0 )
  {
    leaveMaxCapture(MoveList);
    return ;
  }

  //no captures, gen normal moves
  genNManMove( MoveListNotUsed, B , &MoveList);
  genNKingMove( MoveListNotUsed, B, &MoveList);
 
   howMany=MoveListNotUsed.count();
}
//---------------------------------------------------------------
/** No descriptions */
template <class T, int bSize>
void EngineIntT<T, bSize>::genCapKingMove( QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B , QList<LegalMove> *pList) const
{
  Q_ASSERT( !B.captured );
  
  T myKings;
  if( B.isWhiteTurn() )
    myKings=B.whiteK;
  else
    myKings=B.blackK;

  int sqNr=0;// debug
  T srcMask=0x01;
  T shift=myKings;

  while( shift )
  {
    if(myKings & srcMask)
    {
      if( pList )
      {
         LegalMove part;
         QPoint start( this->Bx[sqNr], this->By[sqNr] );
         part.insPath( start,0 );
         genCapKingMovePart( sqNr, sqNr, 0, MoveList, B , pList, &part);
      }
      else
       genCapKingMovePart( sqNr, sqNr, 0, MoveList, B , 0, 0);
    }

    srcMask <<= 1;
    // don't have to check all bits
    shift >>= 1; sqNr++;
  }
  
}
//------------------------------------------------------------------
// try all directions
// return true if can capture anything
// if can continue from sqNr position
template <class T, int bSize>
bool EngineIntT<T, bSize>::genCapKingMovePart( const int src0, int sqNr, int level, QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B,
                                               QList<LegalMove>*pList, LegalMove *pPart )const
{
   bool nextCap=false;
   nextCap |= genKingCapDirection( src0, sqNr, level, MoveList, B, this->LU , pList, pPart );
   nextCap |= genKingCapDirection( src0, sqNr, level, MoveList, B, this->LD , pList, pPart );
   nextCap |= genKingCapDirection( src0, sqNr, level, MoveList, B, this->RU , pList, pPart );
   nextCap |= genKingCapDirection( src0, sqNr, level, MoveList, B, this->RD , pList, pPart );
   
   return nextCap;
}
//--------------------------------------------------------------
template <class T, int bSize>
bool EngineIntT<T, bSize>::genKingCapDirection(const int src0, int s,int level, QList<LegalMoveT<T> > &MoveList, const BoardT<T>& B, const int* D,
                                               QList<LegalMove>*pList, LegalMove *pPart)const
{
  int  d=s;

  T M;
  T sq= B.blackK | B.blackM | B.whiteK | B.whiteM | B.captured;

  //following the direction
  while( (d=D[d]) >=0 )//is on board
  {
    M=0x01;
    M<<=d;

    T noEmpty=sq & M;
    if( noEmpty )
      break;

  }//while

  if(d<0)//outside board
  {
    return false;
  }

  bool alreadyCaptured=M & B.captured;
  if( alreadyCaptured ) //already captured
  {
    // don't capture twice
    return false;
  }

  //is opponent there?
  T opponent;
  const bool wTurn=B.isWhiteTurn();
  if( wTurn )
    opponent=B.blackK | B.blackM;
  else
    opponent= B.whiteK | B.whiteM;

  if( !(opponent & M ))
  {
    return false;
  }
  
  
  bool anyCap=false;
  int capturedNr=d;

          
  T capMask=1;
   capMask<<=capturedNr;
  //all empty square after captured
  while( (d=D[d]) >=0 )//is on board
  {
    T M=0x01;
    M<<=d;

    if( sq & M )
      break;
  
  // generating intermediate information for Gui, building part of the move
  if( pPart )
  {
      QPoint capt( this->Bx[capturedNr], this->By[capturedNr] );
      pPart->insTaken(capt,level);
      
      QPoint inter( this->Bx[d], this->By[d] );
      pPart->insPath(inter,level+1);
  }
  
    //create copy of Borad, mark as captured and continue
    //move my own
    T srcMask= 1;
    srcMask<<=s;
    
    BoardT<T> lB( B );
    if( wTurn )
    {
      lB.whiteK &= (~srcMask);
      lB.whiteK |= M;
    }
    else
    {
      lB.blackK &= (~srcMask);
      lB.blackK |= M;
    }
    
    lB.captured |= capMask;
    anyCap |= genCapKingMovePart( src0, d, level+1, MoveList, lB , pList, pPart );// if can continue, then must
  }//of while


  //if(d<0) return false;
  if( anyCap ) return true;
  // generate moves if no captures on any squares in prev loop -----------
  capMask |= B.captured;
  d=capturedNr;
  while( (d=D[d]) >=0 )//is on board
  {
    T M=0x01;
    M<<=d;

    if( sq & M )
      break;
      
    anyCap=true;
    // end of multi capture move, src and dst masks
    T sM=1;
    T dM=1;
    sM <<= src0;
    dM <<= d;
    
    LegalMoveT<T> lm;
    lm.src = src0;
    lm.dst = d;
    lm.captured=capMask;
    MoveList.append( lm );
    
    // and pPart represent finished/full multicapture move
    // so add it to move list
    if( pList && pPart )
    {
        // generating intermediate information for Gui, building part of the move
        QPoint end( this->Bx[d], this->By[d] );
        pPart->insPath(end,level+1);
        pList->append( *pPart );
    }

  }//of while
  
  return anyCap;
}

//-------------------------------------------
template <class T, int bSize>
int EngineIntT<T, bSize>::materialScore( const BoardT<T> * const pBoard )const 
{
   int v=0;
   
   //------------ white Man
   T shift=pBoard->whiteM;
   T M=0x01;
   int sqNr=0;
   
   while( shift )
   {
     if( pBoard->whiteM & M )
     {
       v += this->SqvWm[sqNr];
       v += scoreMan;
     }
     
     M<<=1;
     shift >>= 1; sqNr++;
   }
   //------------ black Man
   shift=pBoard->blackM;
   M=0x01;
   sqNr=0;
   while( shift )
   {
     if( pBoard->blackM & M )
     {
       v -= this->SqvBm[sqNr];// minus because scores are calculated always for white
       v -= scoreMan;
     }
     
     M<<=1;
     shift >>= 1; sqNr++;
   }
   
   //------------ white Kings
   int WhiteKings=0;
   shift=pBoard->whiteK;
   M=0x01;
   sqNr=0;
   
   while( shift )
   {
     if( pBoard->whiteK & M )
     {
       v += this->SqvK[sqNr];
       if(++WhiteKings == 1)
         v += scoreKing;
       else
         v += scoreNextKing;
     }
     
     M<<=1;
     shift >>= 1; sqNr++;
   }
   //------------ black Kings
   int BlackKings=0;
   shift=pBoard->blackK;
   M=0x01;
   sqNr=0;
   
   while( shift )
   {
     if( pBoard->blackK & M )
     {
       v -= this->SqvK[sqNr];
       if(++BlackKings == 1)
         v -= scoreKing;
       else
         v -= scoreNextKing;
     }
     
     M<<=1;
     shift >>= 1; sqNr++;
   }

  if(! pBoard->isWhiteTurn() )
    v=-v;

  return v;
}

//-----------------------------------------------------------
template <class T, int bSize>
int EngineIntT<T, bSize>::gamePhase(const BoardT<T>* const pB)const
{
   ///@todo implement recognize sytuations where is practicaly drawAbstractEngine
   /// like 2 kings against one

    //calculating how many pieces at start position
  int rows=bSize/2 -1;
  int inRow=bSize/2;
  
  int starting=rows*inRow*2;
    
    int  white_M, white_K, black_M, black_K;
    pB->countPieces( white_M,  white_K, black_M,  black_K);
    
    if ( (white_M+black_M)< starting/3 ) return gambitPhase;
    if ( (white_M+black_M)<= starting*2/3 +2) return middlePhase;
    return earlyPhase;
}
//------------------------------------------------------------------------
template <class T, int bSize>
void EngineIntT<T, bSize>::reFillSQV( int size,int phase )
{
  Q_ASSERT( size==bSize);
  
  // just for safety
  for(int j=0;j< bSize*bSize/2 ; j++)
  {
     this->SqvWm[j]=0;
     this->SqvBm[j]=0;
     this->SqvK[j]=0;
  }
 
  for( int x=0;x<size;x++)
  for( int y=0;y<size;y++)
  {
    int vE=earlyPhaseSQV( x,y );
    int vM=middlePhaseSQV( x,y );
    int vG=gambitPhaseSQV( x,y );
    
    int wNr = this->m_byteNr[x][y];
    int bNr = this->m_byteNr[bSize-x-1][bSize-y-1];
    if( wNr<0 ) continue;//not used s
    
    switch(phase)
    {
      case earlyPhase:
      this->SqvWm[wNr] = vE;
      this->SqvBm[bNr] = vE;
      break;
      
      case middlePhase:
      this->SqvWm[wNr] = vM/2+vG/2;
      this->SqvBm[bNr] = vM/2+vG/2;
      break;
      
      case gambitPhase:
      this->SqvWm[wNr] = vG;
      this->SqvBm[bNr] = vG;
      break;
      
      default:
      qFatal("game phase is not recognized");
    }
  }
  
  if( this->randomness )
  {
    for(int j=0;j< bSize*bSize/2 ; j++)
    {
       int r1= rand() % (2* this->randomness);
       r1-= this->randomness;
       int tmp=this->SqvWm[j];
       this->SqvWm[j] = r1+tmp;
       
       int r2= rand() % (2* this->randomness);
       r2-= this->randomness;
       tmp=this->SqvBm[j];
       this->SqvBm[j] = r2+tmp;
    }
  }
  
  //king value
  if( phase == gambitPhase)
  {
    // that wiil make 4 Kings to catch one
    int sq;
    sq=this->m_byteNr[0][bSize-2];
    this->SqvK[sq]=150;
    
    sq=this->m_byteNr[1][bSize-1];
    this->SqvK[sq]=150;
  
    sq=this->m_byteNr[2][bSize-4];
    this->SqvK[sq]=100;
    
    sq=this->m_byteNr[3][bSize-3];
    this->SqvK[sq]=100;
  }
}
//------------------------------------------------------------------
template <class T, int bSize>
int EngineIntT<T, bSize>::earlyPhaseSQV(int x , int y)
{
   int X=bSize/2-0;
   int Y=bSize/2-1;
   
   if( (x+y) % 2 )
      return 0;

    if( x==4 and y==0 )return 30;
    if( x==0 and y==0 )return -20;
    if( x==1 and y==1 )return -30;
    if( x==1 and y==3 )return -30;
    
    int dx=abs( x-X );
    int dy=abs( y-Y );
    int v=(X-dx) + (Y-dy);
    v=v*10;
    if( y>x or y>(bSize-x) ) v=0;
    if( y > bSize/2 ) v=-10;
    
    return v;
}
//------------------------------------------------------------------
// control central line of the board, and the central squares
template <class T, int bSize>
int EngineIntT<T, bSize>::middlePhaseSQV( int x, int y )
{
   int centerY=bSize/2;

    if( (x+y) % 2 )
        return 0;

    if( x==4 and y==0 )return 30;
    if( x==0 and y==0 )return -20;
    if( x==1 and y==1 )return -30;
    if( x==1 and y==3 )return -30;
   
    //central line
    if( y==centerY   and x>1 and x<bSize-2 ) return 30;
    if( y==centerY-1 and x>1 and x<bSize-2 ) return 40;
    if( y==centerY-2 and x>1 and x<bSize-2 ) return 30;
    if( y==centerY-3 and x>1 and x<bSize-2 ) return 20;
    

    if( y== (bSize-2) ) return 510;//just before king line
    if( y== (bSize-3) ) return 250;

    return 0;

}
//------------------------------------------------------------------
template <class T, int bSize>
int EngineIntT<T, bSize>::gambitPhaseSQV(int x , int y)
{
   //int X=bSize/2-1;
   //int Y=bSize/2-1;
   
   if( (x+y) % 2 )
      return 0;
   
   if( x==(bSize-4) and y==0 )return 30;
   
   int v= y;
   v=v*10;
   if( y== (bSize-2) ) v=510;//just before king line
   if( y== (bSize-3) ) v=250;
   if( x==4 and y==0 ) v=80;
   return v;
}
//------------------------------------------------------------------
template <class T, int bSize>
void EngineIntT<T, bSize>::leaveMaxCapture(QList<LegalMoveT<T> > &MoveList)
{
    int captured=0;
    int c=MoveList.count();
    for(int j=0;j<c;j++)
    {
      int k=MoveList[j].takenPieces();
      if(captured<k)
        captured=k;
    }
    // delete other moves
    typename QList<LegalMoveT<T> >::Iterator it;


    int pos=0;
    while ( pos < MoveList.count() )
    {
       int t;
       t=MoveList[pos].takenPieces();

      if( t<captured )
      {
        MoveList.removeAt(pos);
        continue;
      }
      pos++;
    }
}
//------------------------------------------------------------------
template <class T, int bSize>
void EngineIntT<T, bSize>::leaveMaxCapture( QList<LegalMove> &MoveList )
{
    int captured=0;
    int c=MoveList.count();
    for(int j=0;j<c;j++)
    {
      int k=MoveList[j].takenPieces.count();
      if(captured<k)
        captured=k;
    }
    // delete other moves
    QList<LegalMove>::Iterator it;


    int pos=0;
    while ( pos < MoveList.count() )
    {
       int t;
       t=MoveList[pos].takenPieces.count();

      if( t<captured )
      {
        MoveList.removeAt(pos);
        continue;
      }
      pos++;
    }

}
//------------------------------------------------------------
template <class T, int bSize>
void EngineIntT<T, bSize>::fillBook( QList<QList<LegalMove> > &Book )
{
   switch( bSize )
   {
      case 8:
      fillBook8( Book );
      break;

      case 10:
      fillBook10( Book );
      break;
   }
   return;
}
//------------------------------------------------------------
template <class T, int bSize>
void EngineIntT<T, bSize>::fillBook8( QList<QList<LegalMove> > &Book)
{
   Book.clear();
   
  QList<LegalMove> bL;//bookLine
  
// Grupa I (1. c3-d4, ...):
// 1. c3 - d4, b6 - a5
// 1. c3 - d4, f6 - g5
// 1. c3 - d4, d6 - c5
// 1. c3 - d4, b6 - c5
// 1. c3 - d4, f6 - e5
// 1. c3 - d4, d6 - e5
//             c3 - d4, h6 - g5
  bL.clear();
  LegalMove LM0( QPoint(02,02), QPoint(03,03) );
  LegalMove LM1( QPoint(01,05),QPoint(00,04));//b6 - a5
  bL.append( LM0);
  bL.append( LM1);
  Book.append( bL );

  bL.clear();
  LM1=LegalMove( QPoint(05,05),QPoint(06,04));//f6 - g5
  bL.append( LM0);
  bL.append( LM1);
  Book.append( bL );
  
  bL.clear();
  LM1=LegalMove( QPoint(03,05), QPoint(02,04));//d6-c5
  bL.append( LM0);
  bL.append( LM1);
  Book.append( bL );

  bL.clear();
  LM1=LegalMove( QPoint(01,05) , QPoint(02,04));//b6-c5
  bL.append( LM0);
  bL.append( LM1);
  Book.append( bL );

  bL.clear();
  LM1=LegalMove( QPoint(05,05),QPoint(04,04));//f6-e5
  bL.append( LM0);
  bL.append( LM1);
  Book.append( bL );

  bL.clear();
  LM1=LegalMove( QPoint(03,05) ,QPoint(04,04) );//d6-e5
  bL.append( LM0);
  bL.append( LM1);
  Book.append( bL );

  bL.clear();
  LM1=LegalMove( QPoint(07,05),QPoint(06,04));//h6-g5
  bL.append( LM0);
  bL.append( LM1);
  Book.append( bL );
  
  
//Grupa II (1. c3-b4, ...):
// 1. c3 - b4, b6 - a5
// 1. c3 - b4, f6 - e5
// 1. c3 - b4, f6 - g5
  bL.clear();
  LM0=LegalMove( QPoint(02,02),QPoint(01,03));//c3-b4
  LM1=LegalMove( QPoint(01,05) ,QPoint (00,04));//b6-a5
  bL.append( LM0);
  bL.append( LM1);
  Book.append( bL );
  
  bL.clear();
  LM1=LegalMove( QPoint(05,05),QPoint(04,04));//f6-e5
  bL.append( LM0);
  bL.append( LM1);
  Book.append( bL );

  bL.clear();
  LM1=LegalMove( QPoint(05,05), QPoint(06,04));//f6-g5
  bL.append( LM0);
  bL.append( LM1);
  Book.append( bL );
  
      
//Grupa III (1.g3-h4, ...):
// 1. g3 - h4, b6 - a5
// 1. g3 - h4, f6 - e5
  bL.clear();
  LM0=LegalMove( QPoint(06,02),QPoint(07,03));//g3-h4
  LM1=LegalMove( QPoint(01,05),QPoint(00,04));//b6-a5
  bL.append( LM0);
  bL.append( LM1);
  Book.append( bL );

  bL.clear();
  LM1=LegalMove( QPoint(05,05),QPoint(04,04));//f6-e5
  bL.append( LM0);
  bL.append( LM1);
  Book.append( bL );
  
//Grupa IV (1. g3-f4, ...):
// 1. g3 - f4, f6 - e5
// 1. g3 - f4, b6 - a5
// 1. g3 - f4, d6 - c5
  
  bL.clear();
  LM0=LegalMove( QPoint(06,02),QPoint(05,03));//g3-f4
  LM1=LegalMove( QPoint(05,05),QPoint(04,04));//f6-e5
  bL.append( LM0);
  bL.append( LM1);
  Book.append( bL );

  bL.clear();
  LM1=LegalMove( QPoint(01,05),QPoint(00,04));//b6-a5
  bL.append( LM0);
  bL.append( LM1);
  Book.append( bL );

  bL.clear();
  LM1=LegalMove( QPoint(03,05),QPoint(02,04));//d6-c5
  bL.append( LM0);
  bL.append( LM1);
  Book.append( bL );
  
//Grupa V  (1. e3-f4, ...):
// 1. e3 - f4, f6 - g5
  bL.clear();
  LM0=LegalMove( QPoint(04,02),QPoint(05,03)); 
  LM1=LegalMove( QPoint(05,05),QPoint(06,04));//f6-g5
  bL.append( LM0);
  bL.append( LM1);
  Book.append( bL );
  
  
//Grupa VI (1. a3-b4, ...):
// 1. a3 - b4, b6 - a5
// 1. a3 - b4, h6 - g5
  bL.clear();
  LM0=LegalMove( QPoint(00,02),QPoint(01,03)); 
  LM1=LegalMove( QPoint(01,05),QPoint(00,04));//b6-a5
  bL.append( LM0);
  bL.append( LM1);
  Book.append( bL );
  
  bL.clear();
  LM1=LegalMove( QPoint(07,05),QPoint(06,04));//h6-g5
  bL.append( LM0);
  bL.append( LM1);
  Book.append( bL );


//Grupa VII (1. e3-d4, ...):
// 1. e3 - d4, e6 -d5 << ERROR (e6,d5) is invalid

}

//------------------------------------------------------------
template <class T, int bSize>
void EngineIntT<T, bSize>::fillBook10( QList<QList<LegalMove> > &Book)
{
   Book.clear();
   
  QList<LegalMove> bL;//bookLine
  // ------- rafaela -------------
  //amsterdamski
  //32-28
  //19-23
  //28-19x23
  //14-23x19
  {
    bL.clear();
    LegalMove LM0( QPoint(03,03), QPoint(04,04));
    LegalMove LM1( QPoint(06,06), QPoint(05,05));
    LegalMove LM2( QPoint(04,04), QPoint(06,06));LM2.insTaken( QPoint(05,05),0 );
    LegalMove LM3( QPoint(07,07), QPoint(05,05));LM3.insTaken( QPoint(06,06),0 );
    
    bL.append( LM0);
    bL.append( LM1);
    bL.append( LM2);
    bL.append( LM3);
    Book.append( bL );
  }

  //rozenburga
  // .. 16-21
  {
    bL.clear();
    LegalMove LM0( QPoint(03,03), QPoint(04,04));
    LegalMove LM1( QPoint(00,06), QPoint(01,05));
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  
  //
  // .. 17-21
  {
    bL.clear();
    LegalMove LM0( QPoint(03,03), QPoint(04,04));
    LegalMove LM1( QPoint(02,06), QPoint(01,05));//17-21
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }

  //. ..., 17-22 2. 28:17 12:21 
  {
    bL.clear();
    LegalMove LM0( QPoint(03,03), QPoint(04,04));
    LegalMove LM1( QPoint(02,06), QPoint(03,05));// 17-22
    LegalMove LM2( QPoint(04,04), QPoint(02,06));LM2.insTaken( QPoint(03,05),0 );//28:17x22
    LegalMove LM3( QPoint(03,07), QPoint(01,05));LM3.insTaken( QPoint(02,06),0 );//12-21x17
    
    bL.append( LM0);
    bL.append( LM1);
    bL.append( LM2);
    bL.append( LM3);
    Book.append( bL );
  }

  //1. ..., 17-22 2. 28:17 11:22 
  {
    bL.clear();
    LegalMove LM0( QPoint(03,03), QPoint(04,04));
    LegalMove LM1( QPoint(02,06), QPoint(03,05));// 17-22
    LegalMove LM2( QPoint(04,04), QPoint(02,06));LM2.insTaken( QPoint(03,05),0 );//28:17x22
    LegalMove LM3( QPoint(01,07), QPoint(03,05));LM3.insTaken( QPoint(02,06),0 );//11-22x17
    
    bL.append( LM0);
    bL.append( LM1);
    bL.append( LM2);
    bL.append( LM3);
    Book.append( bL );
  }

  //holenderski 18-22
  {
    bL.clear();
    LegalMove LM0( QPoint(03,03), QPoint(04,04));
    LegalMove LM1( QPoint(04,06), QPoint(03,05));
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  //holenderski 18-23
  {
    bL.clear();
    LegalMove LM0( QPoint(03,03), QPoint(04,04));
    LegalMove LM1( QPoint(04,06), QPoint(05,05));
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }

  //Dumonea  20-24
  {
    bL.clear();
    LegalMove LM0( QPoint(03,03), QPoint(04,04));
    LegalMove LM1( QPoint( 8,06), QPoint(07,05));
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  
  
   //springera 1. ..., 20-25
  {
    bL.clear();
    LegalMove LM0( QPoint(03,03), QPoint(04,04));
    LegalMove LM1( QPoint( 8,06), QPoint( 9,05));
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  
  // End of rafaela -
  
 //-------symetryczny- 33-28 -----------------
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(04,04));//33-28 
    LegalMove LM1( QPoint(04,06), QPoint(05,05));//klasyczny 18-23
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(04,04));//33-28 
    LegalMove LM1( QPoint(00,06), QPoint(01,05));//klasyczny 16-21
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(04,04));//33-28 
    LegalMove LM1( QPoint(02,06), QPoint(01,05));//klasyczny 17-21
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(04,04));//33-28 
    LegalMove LM1( QPoint(04,06), QPoint(03,05));//klasyczny 18-22
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(04,04));//33-28 
    LegalMove LM1( QPoint( 8,06), QPoint(07,05));//klasyczny 20-24
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(04,04));//33-28 
    LegalMove LM1( QPoint( 8,06), QPoint( 9,05));//klasyczny 20-25
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  

  //1. ..., 17-22 2. 28:17 12:21 
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(04,04));//33-28 
    LegalMove LM1( QPoint(02,06), QPoint(03,05));// 17-22
    LegalMove LM2( QPoint(04,04), QPoint(02,06));LM2.insTaken( QPoint(03,05),0 );//28:17x22
    LegalMove LM3( QPoint(03,07), QPoint(01,05));LM3.insTaken( QPoint(02,06),0 );//12:21x17
    
    bL.append( LM0);
    bL.append( LM1);
    bL.append( LM2);
    bL.append( LM3);
    Book.append( bL );
  }
  
  //1. ..., 17-22 2. 28:17 11:22 
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(04,04));//33-28 
    LegalMove LM1( QPoint(02,06), QPoint(03,05));// 17-22
    LegalMove LM2( QPoint(04,04), QPoint(02,06));LM2.insTaken( QPoint(03,05),0 );//28:17x22
    LegalMove LM3( QPoint(01,07), QPoint(03,05));LM3.insTaken( QPoint(02,06),0 );//11:22x17
    
    bL.append( LM0);
    bL.append( LM1);
    bL.append( LM2);
    bL.append( LM3);
    Book.append( bL );
  }
  // 1. ..., 19-23 2. 28:19 14:23 
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(04,04));//33-28 
    LegalMove LM1( QPoint(02,06), QPoint(03,05));// 17-22
    LegalMove LM2( QPoint(04,04), QPoint(06,06));LM2.insTaken( QPoint(05,05),0 );//28:19x23
    LegalMove LM3( QPoint(07,07), QPoint(05,05));LM3.insTaken( QPoint(06,06),0 );//14:23x19
    
    bL.append( LM0);
    bL.append( LM1);
    bL.append( LM2);
    bL.append( LM3);
    Book.append( bL );
  }
  //---- end of symmetric oppening
  //  --- Debiut Roozenburg'a 1. 33-29  -------------------
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(06,04));//33-29
    LegalMove LM1( QPoint(06,06), QPoint(05,05));//19-23 
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(06,04));//33-29
    LegalMove LM1( QPoint(02,06), QPoint(03,05));//17-22 system Keller'a: 
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(06,04));//33-29
    LegalMove LM1( QPoint(04,06), QPoint(03,05));//18-22 
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(06,04));//33-29
    LegalMove LM1( QPoint(00,06), QPoint(01,05));//16-21
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(06,04));//33-29
    LegalMove LM1( QPoint(02,06), QPoint(01,05));//17-21
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(06,04));//33-29
    LegalMove LM1( QPoint(06,06), QPoint(07,05));//19-24
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(06,04));//33-29
    LegalMove LM1( QPoint( 8,06), QPoint( 9,05));//20-25
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  
  //1. ..., 18-23 2. 29:18 13:22 
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(06,04));//33-29
    LegalMove LM1( QPoint(04,06), QPoint(05,05));// 18-23
    LegalMove LM2( QPoint(06,04), QPoint(04,06));LM2.insTaken( QPoint(05,05),0 );//29:18x
    LegalMove LM3( QPoint(05,07), QPoint(03,05));LM3.insTaken( QPoint(04,06),0 );//13:22x
    
    bL.append( LM0);
    bL.append( LM1);
    bL.append( LM2);
    bL.append( LM3);
    Book.append( bL );
  }
  //1. ..., 18-23 2. 29:18 12:23 
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(06,04));//33-29
    LegalMove LM1( QPoint(04,06), QPoint(05,05));// 18-23
    LegalMove LM2( QPoint(06,04), QPoint(04,06));LM2.insTaken( QPoint(05,05),0 );//29:18x
    LegalMove LM3( QPoint(03,07), QPoint(05,05));LM3.insTaken( QPoint(04,06),0 );//12:23x
    
    bL.append( LM0);
    bL.append( LM1);
    bL.append( LM2);
    bL.append( LM3);
    Book.append( bL );
  }
  //1. ..., 20-24 2. 29:20 15:24 
  {
    bL.clear();
    LegalMove LM0( QPoint(05,03), QPoint(06,04));//33-29
    LegalMove LM1( QPoint( 8,06), QPoint(07,05));// 20-24
    LegalMove LM2( QPoint(06,04), QPoint( 8,06));LM2.insTaken( QPoint(07,05),0 );//29:20x
    LegalMove LM3( QPoint( 9,07), QPoint(07,05));LM3.insTaken( QPoint( 8,06),0 );//15:24x
    
    bL.append( LM0);
    bL.append( LM1);
    bL.append( LM2);
    bL.append( LM3);
    Book.append( bL );
  }
  // -- end of  Roozenburg'a 
  
  //---- springer 31-26 ----------
  {
    bL.clear();
    LegalMove LM0( QPoint(01,03), QPoint(00,04));//31-26
    LegalMove LM1( QPoint(06,06), QPoint(05,05));//19-23
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(01,03), QPoint(00,04));//31-26
    LegalMove LM1( QPoint(04,06), QPoint(05,05));//18-23
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  //1. ..., 17-21 2. 26:17 12:21
  {
    bL.clear();
    LegalMove LM0( QPoint(01,03), QPoint(00,04));//31-26
    LegalMove LM1( QPoint(02,06), QPoint(01,05));// 17-21
    LegalMove LM2( QPoint(00,04), QPoint(02,06));LM2.insTaken( QPoint(01,05),0 );//26:17
    LegalMove LM3( QPoint(03,07), QPoint(01,05));LM3.insTaken( QPoint(02,06),0 );//12:21
    
    bL.append( LM0);
    bL.append( LM1);
    bL.append( LM2);
    bL.append( LM3);
    Book.append( bL );
  }
  //1. ..., 17-21 2. 26:17 11:22
  {
    bL.clear();
    LegalMove LM0( QPoint(01,03), QPoint(00,04));//31-26
    LegalMove LM1( QPoint(02,06), QPoint(01,05));// 17-21
    LegalMove LM2( QPoint(00,04), QPoint(02,06));LM2.insTaken( QPoint(01,05),0 );//26:17
    LegalMove LM3( QPoint(01,07), QPoint(03,05));LM3.insTaken( QPoint(02,06),0 );//11:22
    
    bL.append( LM0);
    bL.append( LM1);
    bL.append( LM2);
    bL.append( LM3);
    Book.append( bL );
  }
  // end of  springer
  
  // polish debiut  31-27 ----------
  {
    bL.clear();
    LegalMove LM0( QPoint(01,03), QPoint(02,04));//31-27
    LegalMove LM1( QPoint(06,06), QPoint(05,05));//19-23
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(01,03), QPoint(02,04));//31-27
    LegalMove LM1( QPoint(02,06), QPoint(01,05));//17-21
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(01,03), QPoint(02,04));//31-27
    LegalMove LM1( QPoint(02,06), QPoint(03,05));//17-22
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(01,03), QPoint(02,04));//31-27
    LegalMove LM1( QPoint( 8,06), QPoint( 9,05));//20-25
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(01,03), QPoint(02,04));//31-27
    LegalMove LM1( QPoint(04,06), QPoint(05,05));//18-23
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(01,03), QPoint(02,04));//31-27
    LegalMove LM1( QPoint( 8,06), QPoint(07,05));//20-24
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  //1. ..., 18-22 2. 27:18 13:22
  {
    bL.clear();
    LegalMove LM0( QPoint(01,03), QPoint(02,04));//31-27
    LegalMove LM1( QPoint(04,06), QPoint(03,05));// 18-22
    LegalMove LM2( QPoint(02,04), QPoint(04,06));LM2.insTaken( QPoint(03,05),0 );//27:18
    LegalMove LM3( QPoint(05,07), QPoint(03,05));LM3.insTaken( QPoint(04,06),0 );//13:22
    
    bL.append( LM0);
    bL.append( LM1);
    bL.append( LM2);
    bL.append( LM3);
    Book.append( bL );
  }
  // end of polish debiut
  //----------- french 34-30 ----------
  {
    bL.clear();
    LegalMove LM0( QPoint(07,03), QPoint( 8,04));//34-30
    LegalMove LM1( QPoint( 8,06), QPoint( 9,05));//20-25
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(07,03), QPoint( 8,04));//34-30
    LegalMove LM1( QPoint(02,06), QPoint(03,05));//17-22
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(07,03), QPoint( 8,04));//34-30
    LegalMove LM1( QPoint(04,06), QPoint(03,05));//18-22
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(07,03), QPoint( 8,04));//34-30
    LegalMove LM1( QPoint(06,06), QPoint(06,05));//19-23
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(07,03), QPoint( 8,04));//34-30
    LegalMove LM1( QPoint(02,06), QPoint(01,05));//17-21
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  //end of french
  
  // ----------- Farbe 34-29 ----------
  {
    bL.clear();
    LegalMove LM0( QPoint(07,03), QPoint(06,04));//34-29
    LegalMove LM1( QPoint(06,06), QPoint(05,05));//19-23
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(07,03), QPoint(06,04));//34-29
    LegalMove LM1( QPoint(02,06), QPoint(03,05));//17-22
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(07,03), QPoint(06,04));//34-29
    LegalMove LM1( QPoint(00,06), QPoint(01,05));//16-21
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(07,03), QPoint(06,04));//34-29
    LegalMove LM1( QPoint(04,06), QPoint(03,05));//18-22
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  {
    bL.clear();
    LegalMove LM0( QPoint(07,03), QPoint(06,04));//34-29
    LegalMove LM1( QPoint(02,06), QPoint(01,05));//17-21
    
    bL.append( LM0);
    bL.append( LM1);
    Book.append( bL );
  }
  // end of Farbe 34-29 ----------
  
  
  // Agafonow 35-30 
  //1. ..., 20-25 2. 33-29 15-20 3. 29-23!, 19:28 4. 32:23, 18:29 5. 34:23, 25:34 6.  40:29
  {
    bL.clear();
    LegalMove LM0( QPoint( 9,03), QPoint( 8,04));//35-30
    LegalMove LM1( QPoint( 8,06), QPoint( 9,05));// 20-25 
    LegalMove LM2( QPoint(05,03), QPoint(06,04));// 33-29
    LegalMove LM3( QPoint( 9,07), QPoint( 8,06));// 15-20
    LegalMove LM4( QPoint(06,04), QPoint(05,05));// 29-23!
    
    LegalMove LM5( QPoint(06,06), QPoint(04,04)) ;LM5.insTaken( QPoint(05,05),0 );//19-28x23
    LegalMove LM6( QPoint(03,03), QPoint(05,05)) ;LM6.insTaken( QPoint(04,04),0 );//32-23x28
    LegalMove LM7( QPoint(04,06), QPoint(06,04)) ;LM7.insTaken( QPoint(05,05),0 );//18-29x23
    LegalMove LM8( QPoint(07,03), QPoint(05,05)) ;LM8.insTaken( QPoint(06,04),0 );//34-23x29
    LegalMove LM9( QPoint( 9,05), QPoint(07,03)) ;LM9.insTaken( QPoint( 8,04),0 );//25-34x30
    LegalMove LM10( QPoint( 8,02), QPoint(06,04));LM10.insTaken( QPoint(07,03),0 );//40-29x34
    
    bL.append( LM0);
    bL.append( LM1);
    bL.append( LM2);
    bL.append( LM3);
    bL.append( LM4);
    bL.append( LM5);
    bL.append( LM6);
    bL.append( LM7);
    bL.append( LM8);
    bL.append( LM9);
    bL.append( LM10);
    Book.append( bL );
  }
}


#endif
