/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/

#include <QString>
#include "legalmove.h"
#include "board.h"



LegalMove::LegalMove()
{
	path.resize(0);	
	takenPieces.resize(0);
}
//------------------------------------------------
LegalMove::LegalMove(const LegalMove & LM)
{
	path=LM.path;//.copy();
	takenPieces=LM.takenPieces;//.copy();//deep copy
  Q_ASSERT ( path.size()<30 );
  Q_ASSERT ( takenPieces.size() <= 30 );
}
//-----------------------------------------------
LegalMove::~LegalMove()
{

}

bool LegalMove::operator==(const LegalMove& LM)const
{
   if( path!=LM.path )return false;
   if( takenPieces!=LM.takenPieces )return false;
   
   return true;
}
//------------------------------------------
//typical use for normal move
LegalMove::LegalMove(const QPoint &s,const QPoint &d)
{
	path.resize(2);
	path[0]=s;
	path[1]=d;
}
//---------------------------------------
void LegalMove::insPath(const QPoint &xy,int count)
{
	//int count=path.count();
	path.resize(count+1);
	path[count]=xy;
}
//---------------------------------------
void LegalMove::insTaken(const QPoint &xy,int count)
{
	//int count=takenPieces.count();
	takenPieces.resize(count+1);
	takenPieces[count]=xy;
}
//---------------------------------------------------------------
// 1 user input equall but not yet finished,ONLY then next is returned
// 2 (UInput together with xy) give exactly the same path
int LegalMove::comparePart(const QVector<QPoint> &UInput, const QPoint & next)const
{
	int countThis=path.count();
	int countU=UInput.count();

	// surely different, user input is longer
  // path must be one longer than user input to be exactly equal,  
	if(countU>=countThis)
		return 0;
	
	for(int j=0;j<countU;j++)
	{
		//check all elements
		QPoint uXY=UInput.at(j);
		QPoint thisXY=path.at(j);
		if(uXY != thisXY)
		{
			return 0;
		}
	}

	if( next == path.at(countU))
  {
     if( countU+1 == countThis ) return 2;
     return 1;
  }

	return 0;
}

void LegalMove::print()const
{
  qDebug( "BEGiN:---");
  for( int j=0;j<path.count();j++)
  {
     QPoint pos=path[j];
     qDebug( " path[%d]: (%d,%d)",j , pos.x() , pos.y());
  }
  
  qDebug( "takenPieces:---");
  for( int j=0;j<takenPieces.count();j++)
  {
     QPoint pos=takenPieces[j];
     qDebug( " takenPieces[%d]: (%d,%d)",j , pos.x() , pos.y());
  }
  qDebug( "END:---\n");
}

QString LegalMove::toString() const
{
   QPoint p1=path[0];
   QPoint p2=path[ path.count()-1 ];
   
   return QString("%1%2 , %3%4")
           .arg( QChar ( 'A'+p1.x() ))
           .arg( p1.y() +1 )
           .arg( QChar ( 'A'+p2.x()) )
           .arg( p2.y() +1 );
}
