/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/
#include "movelistmodel.h"
#include "game.h"
#include <QColor>

MoveListModel::MoveListModel( Game *pGame )
 : QAbstractTableModel()
{
   m_Game=pGame;
   bool ok=connect( m_Game, SIGNAL(historyChanged(int,int)), this, SLOT(onSizeChanged(int,int)));
   connect( m_Game, SIGNAL(currentMoveChanged(int)), this, SLOT(onCurrentMoveChanged(int)));
   if(!ok)qDebug( "connecting historyChanged MoveListModel::onSizeChanged failed");
   prevCurrent=-1;
   
}

MoveListModel::~MoveListModel()
{
}

void MoveListModel::setGame( Game *pGame )
{
    disconnect( m_Game , 0,0,0);
    m_Game=pGame;
    connect( m_Game, SIGNAL(historyChanged(int,int)), this, SLOT(onSizeChanged(int,int)));
    connect( m_Game, SIGNAL(currentMoveChanged(int)), this, SLOT(onCurrentMoveChanged(int)));
    
    //qDebug( "connecting historyChanged onSizeChanged");
}

int MoveListModel::rowCount ( const QModelIndex & /*parent*/  ) const
{
    const QList<LegalMove>& moveHistory=m_Game->getMoveHistory();
    int movesMade=moveHistory.count();
    if( movesMade<=0 )return 0;
    
    int rows=idx2row( movesMade-1 )+1;
    return rows;
}


int MoveListModel::columnCount ( const QModelIndex & /*parent*/  ) const
{
    return 2;
}

QVariant MoveListModel::data ( const QModelIndex & index, int role  ) const
{
    int r=index.row();
    int c=index.column();
    const QList<LegalMove>&  moveHistory=m_Game->getMoveHistory();
    int size=moveHistory.count();
    
    int idx=rowCol2Idx(r,c);
    
   
    if (!index.isValid())
        return QVariant();

    if (idx < 0 or idx >= size )
        return QVariant();

    //qDebug( "BoardModel::data() r: %d, c: %d   role:%d",r,c ,role);
    if (role == Qt::DisplayRole)
    {
       LegalMove LM=moveHistory[idx];
       QString s=LM.toString();
       return QVariant(s);
    }
    
    if (role == Qt::BackgroundColorRole)
    {
       if( m_Game->getCurrentMove() == idx+1 )
          return QVariant( QColor(0,255,0) );
       return QVariant();
    }
    
    if (role == Qt::UserRole)
    {
       m_Game->moveTo( idx );
    }

    return QVariant();
}

/*!
    \fn MoveListModel::onSizeChanged( int oldSize, int newSize )
    
 */
void MoveListModel::onSizeChanged( int oldSize, int newSize )
{
  //qDebug( "onSizeChanged %d %d", oldSize,newSize);
  
  QModelIndex idx0=createIndex ( idx2row(oldSize), idx2col(oldSize) );
  QModelIndex idx2=createIndex ( idx2row(newSize), idx2col(newSize) );
  
  //qDebug( "MoveListModel just about emitting dataChanged %d -> %d", oldSize, newSize);
  
  int rc0=idx2row(oldSize-1);
  if(oldSize<=0) rc0 = -1;
  int rc1=idx2row(newSize-1);
  int rows=rc1-rc0;
  if( rows>0 )
  {
     beginInsertRows(QModelIndex(), 0, rows);
     endInsertRows();
  }
}


void MoveListModel::onCurrentMoveChanged( int index )
{
  if ( index == prevCurrent )return;
  
  QModelIndex idx0=createIndex ( idx2row(prevCurrent-1), idx2col(prevCurrent-1));
  QModelIndex idx2=createIndex ( idx2row(index-1), idx2col(index-1));
  emit dataChanged( idx0, idx2 );
  
  prevCurrent=index;
}


int MoveListModel::idx2row( int idx ) const
{
   int row=(idx)/2;
   return row;
}


int MoveListModel::idx2col( int idx ) const
{
   int col=(idx)%2;
   return col;
}
   
int MoveListModel::rowCol2Idx( int row, int col ) const
{
   int idx=2*row+col;
   return idx;
}

QVariant MoveListModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role != Qt::DisplayRole)
        return QVariant();

    if (orientation == Qt::Horizontal)
    {
        if( section ==0)
          return QVariant("white");
        return QVariant("black"); 
    }
    else
        return QString("%1").arg(section+1);
}

#include "movelistmodel.moc"
