/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/
#include "networkgamedialog.h"
#include <QTcpSocket>
#include <QTcpServer>

networkGameDialog::networkGameDialog(QWidget *parent): QDialog(parent)
{
  ui.setupUi(this);
  pSocket=0;
  tcpServer=0;
  //pServer=0;
  connect ( ui.btnConnect, SIGNAL ( clicked()), this, SLOT( onConnectTo() ));
  connect ( ui.btnWait, SIGNAL ( clicked()), this, SLOT( onWait() ));
  connect ( ui.btnTerminate, SIGNAL ( clicked()), this, SLOT( onTerminate() ));
  ui.lineStatus->setText(i18n("do something"));
  
  connect ( this, SIGNAL ( finished(int)), this, SLOT( onDialogFinished(int) ));
}


networkGameDialog::~networkGameDialog()
{
    if(pSocket) delete pSocket;
    pSocket=0;
    if(tcpServer) delete tcpServer;
    tcpServer=0;
}

/*!
    \fn networkGameDialog::onConnectTo()
 */
void networkGameDialog::onConnectTo()
{
    if(pSocket) delete pSocket;
    pSocket=0;
    if(tcpServer) delete tcpServer;
    tcpServer=0;
    
    pSocket=new QTcpSocket( this );
    connect ( pSocket, SIGNAL ( connected()), this, SLOT( onConnected() ));
    connect ( pSocket, SIGNAL ( disconnected()), this, SLOT( onDisconnected() ));
    connect ( pSocket, SIGNAL ( destroyed()), this, SLOT( onSocketDestroyed() ));
    connect ( pSocket, SIGNAL ( error(QAbstractSocket::SocketError)), this, SLOT( onSocketError(QAbstractSocket::SocketError) ));
    
    ui.lineStatus->setText(i18n("connecting ..."));
    pSocket->connectToHost ( ui.lineHost->text() ,27531 );
}

void networkGameDialog::onSocketError(QAbstractSocket::SocketError)
{
   if (!pSocket)
	return;
   QString err=pSocket->errorString();
   ui.lineStatus->setText( err );
   pSocket->deleteLater();
   pSocket = 0;
}

/*!
    \fn networkGameDialog::onConnected()
 */
void networkGameDialog::onConnected()
{
    ui.lineStatus->setText(i18n("connected"));
    emit connected( pSocket );
    qDebug( "emit'ed connected( pSocket )");
}

/*!
    \fn networkGameDialog::onDisconnected()
 */
void networkGameDialog::onDisconnected()
{
    ui.lineStatus->setText(i18n("disconnected"));
    
    if(pSocket)pSocket->deleteLater();
    emit disconnected();
    qDebug( "emit'ed disconnected(  )");
}

void networkGameDialog::onSocketDestroyed()
{
   pSocket=0;
   qDebug("socket Destroyed");
   
   if(tcpServer)
      ui.lineStatus->setText(i18n("waiting Again"));
}

/*!
    \fn networkGameDialog::onWait()
 */
void networkGameDialog::onWait()
{
    ui.lineStatus->setText("waiting");
    if(pSocket) delete pSocket;
    pSocket=0;

    if(tcpServer)delete tcpServer;
    tcpServer=new QTcpServer(this);
    tcpServer->setMaxPendingConnections(1);
    bool ok=tcpServer->listen( QHostAddress::Any , 27531 );
    if(!ok)
    {
       ui.lineStatus->setText(i18n("listen failed"));
       delete tcpServer;
       tcpServer=0;
       return;
    }
       
    qDebug( "listen ok=%d", ok );
    connect ( tcpServer , SIGNAL (newConnection ()  ), this, SLOT(onNewConnection()) );
}

/*!
    \fn ::onNewConnection()
 */
void networkGameDialog::onNewConnection()
{
    qDebug("networkGameDialog::onNewConnection()");
    Q_ASSERT(!pSocket);
    
    pSocket=tcpServer->nextPendingConnection();

    ui.lineStatus->setText(i18n("CONNECTED"));
    
    connect ( pSocket, SIGNAL ( connected()), this, SLOT( onConnected() ));
    connect ( pSocket, SIGNAL ( disconnected()), this, SLOT( onDisconnected() ));
    connect ( pSocket, SIGNAL ( destroyed()), this, SLOT( onSocketDestroyed() ));
    connect ( pSocket, SIGNAL ( error(QAbstractSocket::SocketError)), this, SLOT( onSocketError(QAbstractSocket::SocketError) ));
    
    if( pSocket->state() == QAbstractSocket::ConnectedState )
    {
      emit connected( pSocket );
      qDebug( "onNewConnection(): -> emit'ed connected( pSocket )");
    }
}

void networkGameDialog::onTerminate()
{
   qDebug("networkGameDialog::onTerminate()");
   
   if(tcpServer)delete tcpServer;tcpServer=0;
   if(pSocket)pSocket->close();
   
   ui.lineStatus->setText("terminated");
}

void networkGameDialog::onDialogFinished(int result)
{
   qDebug( "onDialogFinished result%d",result);
   qDebug("networkGameDialog::closingDialog()");
   
   if(pSocket)pSocket->close();
   if(tcpServer)tcpServer->close();
}

#include "networkgamedialog.moc"
