/***************************************************************************
 *   Copyright (C) 2005 by Piotr Banasiak <piotrbanasiak@onet.pl>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to                             *
 *   the Free Software Foundation, 51 Franklin Street, Fifth Floor,        *
 *   Boston, MA 02110-1301, USA.                                           *
 ***************************************************************************/
#include "newgamedialog.h"
#include "enginedetailswidget.h"

newGameDialog::newGameDialog(QWidget *parent): QDialog(parent)
{
    ui.setupUi(this);
    whiteDetails=0;
    blackDetails=0;
    
    buttonGroup = new QButtonGroup(ui.groupThisSide);
    buttonGroup->addButton( ui.radioWhite, 0  );
    buttonGroup->addButton( ui.radioBlack, 1 );
    
    connect( buttonGroup, SIGNAL( buttonClicked(int)), this , SLOT(onClicked(int)) );
    
    connect( ui.btnWhite, SIGNAL( clicked()), this, SLOT(onDetails()));
    connect( ui.btnBlack, SIGNAL( clicked()), this, SLOT(onDetails()));
    connect( this, SIGNAL( finished(int)), this, SLOT(onDialogFinished(int)));
}


newGameDialog::~newGameDialog()
{
}


/*!
    \fn newGameDialog::onDialogFinished(int);
 */
void newGameDialog::onDialogFinished(int result)
{
    /// @todo implement me
    if(whiteDetails)whiteDetails->hide();
    if(blackDetails)blackDetails->hide();
    qDebug( "New Game onDialogFinished result %d",result);
    if( result !=QDialog::Accepted) return;
    
    qDebug("Accepted");
    if(whiteDetails)whiteDetails->updateConfig();
    if(blackDetails)blackDetails->updateConfig();
}

//network player side is alredy known and can not be changed
void newGameDialog::setNetPlayWhite(bool isWhite)
{
   ui.groupThisSide->setVisible( true );
   ui.radioWhite->setChecked( !isWhite );
   ui.radioBlack->setChecked( isWhite );
   ui.groupThisSide->setEnabled( false );
   
   ui.cmbWhitePlayer->setEnabled( ui.radioWhite->isChecked() );
   ui.cmbBlackPlayer->setEnabled( ui.radioBlack->isChecked() );
   ui.btnWhite->setEnabled( ui.cmbWhitePlayer->isEnabled() );
   ui.btnBlack->setEnabled( ui.cmbBlackPlayer->isEnabled() );
}

// that is if already 
void newGameDialog::setNetPlayerEnabled(bool enabled)
{
   ui.groupThisSide->setVisible( enabled );
   if( enabled )
   {
      ui.cmbWhitePlayer->setEnabled( ui.radioWhite->isChecked() );
      ui.cmbBlackPlayer->setEnabled( ui.radioBlack->isChecked() );
   }
   else
   {
      ui.cmbBlackPlayer->setEnabled( true );
      ui.cmbBlackPlayer->setEnabled( true );
   }
   ui.btnWhite->setEnabled( ui.cmbWhitePlayer->isEnabled() );
   ui.btnBlack->setEnabled( ui.cmbBlackPlayer->isEnabled() );
}

void newGameDialog::setWhitePlayer( AbstractPlayer *p, bool fixNetPlayer )
{
   Q_ASSERT( p );
   enum AbstractPlayer::playerType t=p->getType();
   switch( t )
   {
      case AbstractPlayer::networkType:
      ui.groupThisSide->setVisible( true );
      ui.radioWhite->setChecked( false );
      ui.radioBlack->setChecked( true );
      ui.cmbWhitePlayer->setEnabled( ui.radioWhite->isChecked() );
      ui.cmbBlackPlayer->setEnabled( ui.radioBlack->isChecked() );
      break;
      
      case AbstractPlayer::humanType:
      ui.cmbWhitePlayer->setCurrentIndex( 1 );
      break;
      
      case AbstractPlayer::engineType:
      ui.cmbWhitePlayer->setCurrentIndex( 0 );
      break;
   }
   ui.btnWhite->setEnabled( ui.cmbWhitePlayer->isEnabled() );
   ui.btnBlack->setEnabled( ui.cmbBlackPlayer->isEnabled() );
   
   ui.groupThisSide->setEnabled(  !fixNetPlayer );
}

void newGameDialog::setBlackPlayer( AbstractPlayer *p, bool fixNetPlayer)
{
   Q_ASSERT( p );
   enum AbstractPlayer::playerType t=p->getType();
   switch( t )
   {
      case AbstractPlayer::networkType:
      ui.groupThisSide->setVisible( true );
      ui.radioWhite->setChecked( true );
      ui.radioBlack->setChecked( false );
      ui.cmbWhitePlayer->setEnabled( ui.radioWhite->isChecked() );
      ui.cmbBlackPlayer->setEnabled( ui.radioBlack->isChecked() );
      break;
      
      case AbstractPlayer::humanType:
      ui.cmbBlackPlayer->setCurrentIndex( 1 );
      break;
      
      case AbstractPlayer::engineType:
      ui.cmbBlackPlayer->setCurrentIndex( 0 );
      break;
   }
   ui.btnWhite->setEnabled( ui.cmbWhitePlayer->isEnabled() );
   ui.btnBlack->setEnabled( ui.cmbBlackPlayer->isEnabled() );
   
   ui.groupThisSide->setEnabled(  !fixNetPlayer );
}

enum AbstractPlayer::playerType newGameDialog::getWhitePlayer()const
{
   if( !ui.cmbWhitePlayer->isEnabled() )//network connection
      return AbstractPlayer::networkType;
   
   if( ui.cmbWhitePlayer->currentIndex() ==0 )
      return AbstractPlayer::engineType;

   return AbstractPlayer::humanType;
}

enum AbstractPlayer::playerType newGameDialog::getBlackPlayer()const
{
   if( !ui.cmbBlackPlayer->isEnabled() )//network connection
      return AbstractPlayer::networkType;
   
   if( ui.cmbBlackPlayer->currentIndex() ==0 )
      return AbstractPlayer::engineType;

   return AbstractPlayer::humanType;
}


/*!
    \fn newGameDialog::onToggled(bool on)
 */
void newGameDialog::onClicked(int )
{
    setNetPlayerEnabled( true );
}

/*!
    \fn newGameDialog::onDetails()
    shows detail of player configuration
 */
void newGameDialog::onDetails()
{
    qDebug( "onDetails");
    if( !whiteDetails )
    {
      whiteDetails=new engineDetailsWidget(this,true);
      if( !blackDetails )
      blackDetails =new engineDetailsWidget(this,false);
    
    
      QLayout *hL=new QHBoxLayout();
      hL->addWidget( whiteDetails );
      hL->addWidget( blackDetails );
      
      ui.vboxLayout->addLayout( hL );
      whiteDetails->show();
      blackDetails->show();
    }
}

#include "newgamedialog.moc"
