/*
 * Copyright 2008 Marijn Kruisselbrink <m.kruisselbrink@student.tue.nl>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef ARRAY_H
#define ARRAY_H

#include <KDebug>

template<typename T> class Array {
public:
    Array() : m_size(0), m_data(0)
    {
    }

    explicit Array(int size) : m_size(size), m_data(new T[m_size]())
    {
    }

    ~Array()
    {
        delete[] m_data;
    }

    Array(const Array& a) : m_size(a.size()), m_data(new T[m_size]())
    {
        for (int i = 0; i < m_size; i++) {
            m_data[i] = a[i];
        }
    }

    Array& operator=(const Array& a)
    {
        if (this == &a) return;
        delete[] m_data;
        m_size = a.size();
        m_data = new T[m_size]();
        for (int i = 0; i < m_size; i++) {
            m_data[i] = a[i];
        }
        return *this;
    }

    void resize(int size)
    {
        delete[] m_data;
        m_size = size;
        m_data = new T[size]();
    }

    int size() const
    {
        return m_size;
    }

    const T& operator[](int index) const
    {
        Q_ASSERT(index >= 0);
        Q_ASSERT(index < m_size);
        return m_data[index];
    }

    T& operator[](int index)
    {
        Q_ASSERT(index >= 0);
        Q_ASSERT(index < m_size);
        return m_data[index];
    }
private:
    int m_size;
    T* m_data;
};

#endif // ARRAY_H
