/*
 * Copyright 2008 Marijn Kruisselbrink <m.kruisselbrink@student.tue.nl>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _CAMERA_H
#define _CAMERA_H

#include <Eigen/Core>

class Terrain;

class Camera {
public:
    Camera();
    Camera(const Eigen::Vector3f &pos);
    virtual ~Camera();

    void move(float dist);
    Eigen::Vector3f position() const;
    void setPosition(const Eigen::Vector3f &pos);
    void setPosition(float x, float y, float z);
    void rotateX(float angle);
    void rotateY(float angle);
    void rotateZ(float angle);
    void rotate(float xangle, float yangle, float zangle);
    void rotate(const Eigen::Vector3f &angle);
    void point(float x, float y, float z);
    void point(const Eigen::Vector3f &pos);
    void strafe(float dist);
    void render();
    virtual void update();
protected:
    Eigen::Vector3f m_up;
    Eigen::Vector3f m_view;
    Eigen::Vector3f m_right;
    Eigen::Vector3f m_pos;
    Eigen::Vector3f m_angle;

    void applyRotations();
};

class ThirdPersonCamera : public Camera {
public:
    ThirdPersonCamera(const Eigen::Vector3f &pos, float seaHeight, Terrain *ter);
    ThirdPersonCamera();
    virtual ~ThirdPersonCamera();

    void setCenter(const Eigen::Vector3f &pos);
    void setDist(float dist);
    void setSmooth(float smooth);
    void setAngSmooth(float smooth);
    void update();
    void moveCenter(float dist);
    void strafeCenter(float dist);

    Eigen::Vector3f targetAngle() const;
    void setTargetAngle(const Eigen::Vector3f& tAngle);
private:
    Eigen::Vector3f m_center;
    float m_dist;
    Eigen::Vector3f m_tPos;
    Eigen::Vector3f m_relPos;
    float m_smooth;
    Eigen::Vector3f m_oldCenter;
    Eigen::Vector3f m_tAngle;
    float m_angSmooth;
    unsigned int m_lastTime;
    float m_seaHeight;
    Terrain* m_ter;
};

#endif // _CAMERA_H
