/*
 * Copyright 2008 Marijn Kruisselbrink <m.kruisselbrink@student.tue.nl>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _GLOBALS_H
#define _GLOBALS_H

// FruitWars headers
#include "Fruit.h"
#include "Terrain.h"
#include "Players.h"
#include "Array.h"

#include <Eigen/Core>
#include <Eigen/Geometry>
#include <GL/gl.h>

class QGLContext;
class QTime;

static const float NEARF = 0.5;
static const float FARF = 512.0;
static const float FOV = 45.0;
static const float SEAHEIGHT = 4.0;
static const float MAXXANGLE = 85.0 / 180 * M_PI;
static const unsigned int DCDIF = 100;
static const float GLOBALGRAV = 20.0;
static const float BASKETPOSRADIUS = 45.0;
static const float BASKETROTSHIFT = -0.05;
static const float BASKETHEIGHTSHIFT[] = {0.0, 0.0, 0.1, 0.0, 0.0, -0.1, -0.05, 0.0};

extern unsigned int nplayers;
extern bool gl_13_available;
extern bool pointgame;
extern bool timegame;
extern int maxPoints;
extern long long int maxTime;
extern long long int theTime;
extern bool gameEnded;
extern bool showText;
extern bool sessionLost;
extern bool gameQuit;
extern int thaPerm[8];
extern Eigen::Transform3f fruitAttachMat;
extern int hitEffect;
extern float playerStunTime;
extern int diversityBonus;
extern bool oneFruitType;
extern FruitType theFruitType;
extern int minFruit;
extern int maxFruit;
extern float minWait;
extern float maxWait;
extern int teamMode;
extern int nTeams;
extern int nBaskets;
extern float spawnRadius;
extern bool spawnRandom;
extern bool friendlyFire;
extern int fruitAppleScore;
extern int fruitOrangeScore;
extern int fruitLemonScore;
extern int fruitAnanasScore;
extern int fruitStrawScore;
extern int screenWidth;
extern int screenHeight;
extern float mouseSpeed;
extern bool fullScreen;
extern int extraHitPoints;
extern int syncRate;
extern float throwInfluence;
extern int holdMode;
extern bool drunkMode;
extern int drunkSize;
extern bool nprMode;

extern void basicSetup();
extern void lightSetup();
extern void varInit();
extern void camInit();
extern void envInit();
extern void modelLoading();
extern void createEmptyTexture(GLuint& tex);

extern ThirdPersonCamera myCam;
extern bool keyState[514];
extern bool dKeyState[512];
extern bool dLastTime[512];
extern int mouseX, mouseY;
extern float moveAmount;
extern int halfWidth, halfHeight;
extern GLuint HUDRectTex;
extern float glblRotate;
extern Terrain myTer, mySea;
extern TerrainFinish myTerFinish, mySeaFinish;
extern SkyCube mySky;
extern float lightPos[4];
extern float lightDir[4];
extern float ambient[4];
extern float diffuse[4];
extern int frameCount, pastTime;
extern unsigned int oldTime, syncTime;
extern QString fpsString;
extern Model3D* playerModel;
extern Array<HumanPlayer> myPlayers;
extern Array<GLuint> myPlayerTex;
extern Eigen::Vector3f myPlayerColor[8];
extern FruitSpawner fruitSpawner;
extern Model3D* basketModel;
extern std::vector<FruitBasket> myBaskets;
extern std::vector<GLuint> myBasketTex;
extern Model3D* bananaModel;
extern Aim theCrossHair;
extern std::vector<Team> myTeams;
extern std::vector<int> playerInTeam;
extern Indicator theIndicator;
extern GLuint emptyTex;
extern GLuint curTex;
extern bool drunkActive;

extern unsigned int time();
extern float frandom();

extern QGLContext* glcontext;
extern QTime* globaltime;

#endif // _GLOBALS_H
