/*
 * Copyright 2008 Marijn Kruisselbrink <m.kruisselbrink@student.tue.nl>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _PLAYERS_H
#define _PLAYERS_H

#include "Weapons.h"
#include <QString>
#include <vector>

class BoneInstance;

static const float PLAYERSCALE = 0.02;
static const float PLAYERBSRAD = 70.0;
static const float PLAYERBSOFFSET[] = {0.0, 50.0, 0.0};
static const float PLAYERCOLRADIUS = 55.0;
static const float PLAYERHEIGHT = 2.50;
static const float PLAYERROTOFFSETY = M_PI;
static const int PLWALKANIM = 0;
static const int PLSTRAFEANIM = 0;
static const int PLSTUNANIM = 1;
static const float PLBLENDSPEED = 2.0;
static const float PLSTUNANIMFRAMESPEED = 10.0;
static const float PLAYERDODGEJUMPFAC = 0.50;
static const unsigned int PLAYERDODGETIMEDIF = 400;
static const int PLAYERRIGHTLEGVERT = 11;
static const int PLAYERLEFTLEGVERT = 26;
static const int PLAYERRIGHTARMVERT = 24;
static const int PLAYERRIGHTARMFRAME = 7;
static const int PLAYERLEFTARMVERT = 30;
static const int PLAYERLEFTARMFRAME = 8;
static const float PLAYERLOOKANGLE = M_PI * 0.04;
static const float FRUITTHROWSPEED = 8.0;
static const float FRUITMOVESPEED = 10.0;
static const float DEADSCALESPEED = 0.50;
static const float PLAYERWARPSIZE = 3.00;
static const float PLAYERWARPHEIGHT = 10.0;
static const float PLAYERWARPDIF = 6.0;
static const float PLAYERSHADOWRAD = 0.95;
static const float PLAYERWALKSPEED = 8.0;

struct AnimTrack {
    float frame;
    float tFrame;
    float frameSmooth;
    float frameSpeed;
    float dFrame;
    float blend;
    int setId;
    int priority;
    float maxFrame;
};

class HumanPlayer {
public:
    HumanPlayer(Model3D* model, int id);
    HumanPlayer();
    ~HumanPlayer();
    void startJump();
    void strafe(float speed);
    void turn(float angle);
    void move(float speed);
    void dodgeRight();
    void dodgeLeft();
    void dodgeForward();
    void dodgeBackward();
    void pickUpFruit(int fid);
    void tossFruit();
    void respawn();
    void die();

    void update();
    void nwUpdate();
    void nwUpdate(long long int rtime); // XXX message
    void updateAnim();
    void render();
    void warpRender();
    void warpRender2();
    void renderShadow();
    void sendUpdate();

    void scale(const Eigen::Vector3f& scale);
    void rotate(const Eigen::Vector3f& rot);
    void rotateX(float ang);
    void rotateY(float ang);
    void rotateZ(float ang);
    void position(const Eigen::Vector3f& pos);
    void positionBS(const Eigen::Vector3f& pos);
    Eigen::Vector3f getDirVec();
    Eigen::Vector3f getThrowVec();
    Eigen::Vector3f getScale();
    Eigen::Vector3f getRot();
    Eigen::Vector3f getPos();
    BS getColBS();
    Eigen::Vector3f getStrafePos();
    inline BoneInstance* getInst() { return inst; }
    inline void setIsAI(bool v) { isAI = v; }
    inline void setColorId(int i) { colorId = i; }
    inline void setBaskId(int i) { baskId = i; }
    inline Banana& getBanana() { return banana; }
    inline bool getIsJumping() const { return isJumping; }
    inline void setLastDodgeTime(unsigned int t) { lastDodgeTime = t; }
    inline void setIsJumping(bool b) { isJumping = b; }
    inline void setJumpSpeed(float s) { jumpSpeed = s; }
    inline void setDodgeSpeed(const Eigen::Vector3f& v) { dodgeSpeed = v; }
    inline void setHolds(int h) { holds = h; }
    inline int getHolds() const { return holds; }
    inline bool getDead() const { return dead; }
    inline void setOldPos(const Eigen::Vector3f& pos) { oldPos = pos; }
    inline const Eigen::Vector3f& getOldPos() { return oldPos; }
    inline int getBaskId() const { return baskId; }
    inline void setStunTime(float f) { stunTime = f; }
    int getTeamId() const { return teamId; }
    void setTeamId(int id) { teamId = id; }
    float getStunTime() const { return stunTime; }
    float getDt() const { return dt; }
    QString getNickName() const { return nickName; }
    void setRelTeamId(int id) { relTeamId = id; }
    int getColorId() const { return colorId; }

    EIGEN_MAKE_ALIGNED_OPERATOR_NEW
private:
    float dt;
    unsigned int lastTime;
    float grav;
    float jumpForce;
    float jumpSpeed;
    float dodgeForce;
    Eigen::Vector3f dodgeSpeed;
    unsigned int lastDodgeTime;
    float moveSpeed;
    float turnSpeed;
    bool isJumping;
    int playId;
    int teamId;
    int holds;
    float stunTime;
    bool dead;
    float deadScale;
    bool isAI;
    int baskId;
    int relTeamId;
    int colorId;
    Banana banana;
    Stars stars;
    Warp warp;
    unsigned int holdTime;
    BoneInstance *inst;
    Eigen::Vector3f oldPos;
    BS colBS;
    std::vector<AnimTrack> anim;
    unsigned int nwPlayerId;
    QString nickName;
    // lastIP
};

class Team {
public:
    Team(int tid);
    Team() {}
    ~Team();

    void addPlayer(int pid);
    int getPlayId(int index);
    int getScore();
    void setScore(int s);
    bool isEmpty();
    int getRelTeamId(int pid);
    inline int getPlayerCount() const { return nPlayers; }
    inline int getColorId() const { return colorId; }
    inline void setBaskId(int i) { baskId = i; }
    inline void setColorId(int i) { colorId = i; }
    inline int getBaskId() const { return baskId; }
    QString getTeamName() const { return teamName; }
private:
    unsigned int nPlayers;
    std::vector<int> players;
    int baskId;
    int teamId;
    QString teamName;
    int colorId;
};

#endif // _PLAYERS_H
