/*
 * Copyright 2008 Marijn Kruisselbrink <m.kruisselbrink@student.tue.nl>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef _WEAPONS_H
#define _WEAPONS_H

#include "GameMath.h"
#include "Particles.h"
#include <Eigen/Core>
#include <cmath>

class Instance;
class Model3D;

static const float WEAPONBANANASCALE = 0.015;
static const float WEAPONBANANABSRAD = 70.0;
static const float WEAPONBANANABSOFFSET[] = {12.0, 7.0, 2.0};
static const float WEAPONBANANATRAILOFFSET[] = {15.0, -40.0, 2.0, 25.0, 60.0, 2.0};
static const float WEAPONBANANACOLRADIUS = 60.0;
static const float WEAPONBANANATHROWSPEED = 60.0;
static const float WEAPONBANANASPINSPEED = 4.0 * M_PI;
static const float WEAPONBANANARANGE = 50.0;
static const float WEAPONBANANAUPACC = 30.0;
static const float WEAPONBANANAUPBRAKE = 0.50;
static const float WEAPONBANANARETURNFAC = 0.02;
static const float WEAPONBANANARETURNMAX = 4.0;
static const float BANANAMATRIX[] = {0.31,  0.65,  0.22, -145.24,
                                    0.68, -0.27, -0.15,  -59.41,
                                    -0.05,  0.26, -0.70,   23.66,
                                    0   ,  0   ,  0   ,   1};

class Banana {
public:
    Banana(Model3D* model, int owner);
    ~Banana();

    void update(float dt);
    void render();
    void trailRender();
    void throwIt(const Eigen::Vector3f& dir);
    void spin(float dt);
    void getIt();

    void scale(const Eigen::Vector3f& scale);
    void rotate(const Eigen::Vector3f& rot);
    void rotateX(float ang);
    void rotateY(float ang);
    void rotateZ(float ang);
    void position(const Eigen::Vector3f& pos);
    void positionBS(const Eigen::Vector3f& pos);
    Eigen::Vector3f getScale();
    Eigen::Vector3f getRot();
    Eigen::Vector3f getPos();
    BS getColBS();
    inline const Eigen::Vector3f& getThrowSpeed() const { return throwSpeed; }
    void setThrowSpeed(const Eigen::Vector3f& speed) { throwSpeed = speed; }
    inline Instance* getInst() { return inst; }
    inline Trail& getTrail() { return trail; }
    inline void setOldPos(const Eigen::Vector3f& pos) { oldPos = pos; }
    const Eigen::Vector3f& getOldPos() { return oldPos; }
    int getState() { return state; }
    void setState(int state) { this->state = state; }
    int getCarriedBy() { return carriedBy; }

    EIGEN_MAKE_ALIGNED_OPERATOR_NEW
private:
    Instance* inst;
    BS colBS;
    int carriedBy;
    int state;
    Eigen::Transform3f attachMat;
    Eigen::Vector3f throwSpeed;
    float throwTime;
    float upAcc;
    float breakFac;
    float returnTime;
    Eigen::Vector3f oldPos;
    Trail trail;
};

#endif // _WEAPONS_H
