/*
    genesissystem.cpp - Sega Genesis/Megadrive Gamefu system plugin

    Copyright (c) 2006      by Michaël Larouche       <larouche@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/
#include "genesissystem.h"

// KDE Includes
#include <kgenericfactory.h>
#include <kdebug.h>

// Gamefu includes
#include <rommetadatafactory.h>

#include "genesisrommetadata.h"

typedef KGenericFactory<GenesisSystem> GenesisSystemFactory;
K_EXPORT_COMPONENT_FACTORY( gamefu_genesis, GenesisSystemFactory( "gamefu_genesis" )  )

class GenesisSystem::Private
{
};

GenesisSystem::GenesisSystem(QObject *parent, const char *name, const QStringList& /*args*/)
 : Gamefu::System(GenesisSystemFactory::instance(), parent, name)
{
	
	if ( s_genesisSystem )
		kdDebug()<< "Sega Genesis/Megadrive system already initialized" << endl;
	else
		s_genesisSystem = this;

	d = new Private;

	registerSystemMimeType( QString::fromUtf8("application/x-rom-genesis") );
	Gamefu::RomMetaDataFactory::self()->addRomMetaDataFactory( QString::fromUtf8("application/x-rom-genesis"), new GenesisRomMetaData );
}


GenesisSystem::~GenesisSystem()
{
	delete d;
	s_genesisSystem = 0L;	
}

GenesisSystem *GenesisSystem::system()
{
	return s_genesisSystem;
}

GenesisSystem *GenesisSystem::s_genesisSystem = 0L;

#include "genesissystem.moc"
