

#include "hiqview.h"
#include "peg.h"

#include <qpainter.h>
#include <qlayout.h>

#include <kurl.h>

#include <ktrader.h>
#include <klibloader.h>
#include <kmessagebox.h>
#include <krun.h>
#include <klocale.h>
#include <qpushbutton.h>
#include <qstring.h>
#include <kmessagebox.h>
#include <kapplication.h>
#include <qwidget.h>
#include <kstandarddirs.h>
#include <qwmatrix.h>
#include <qscrollbar.h>

const QString defaultStyle("marble");
const int maxNumFrames = 500;

HiQView::HiQView(QCanvas * canv, QWidget *parent, const char * name, bool anim, QString &styleName, KApplication *kappl, int advanceP, int advancePFast)
: QCanvasView(canv, parent, name) {
    imageDir = (kappl->dirs()->findResourceDir("data", "hiq/images/" + defaultStyle + "/peg0000.png").append("/hiq/images/"));
    mousePressX = -1;
    mousePressY = -1;
    mousePressNum = -1;
    editable = false;
    animated = anim;
    canvas = canv;
    advancePeriod = advanceP;
    advancePeriodFast = advancePFast;
    mov = new QCanvasPixmapArray();
    img = new QCanvasPixmapArray();
    this->styleName = styleName;
    loadStyle();
    setFixedSize(img->image(0)->width(), img->image(0)->height());
    setHScrollBarMode(QScrollView::AlwaysOff);
    setVScrollBarMode(QScrollView::AlwaysOff);
    canv->resize(img->image(0)->width(), img->image(0)->height());
    bgSprite = new QCanvasSprite(img, canvas);
    bgSprite->setZ(0);
    bgSprite->setVisible(true);
    //setWorldMatrix(QWMatrix().scale(2, 2));

    for (int i = 0; i < 33; i++) {
        pegs[i] = 0;
    }
    reset();
    setAnimated(animated);
}

// takes an int and formats it to be 4 characters long (adds zeros to front)
static QString makeFourLong(int num) {
    QString ret = QString::number(num);
    while (ret.length() < 4) {
        ret = "0" + ret;
    }
    return ret;
}

int HiQView::findNumFrames() {
    int topIndex = maxNumFrames;
    int bottomIndex = 0;
    while (topIndex - bottomIndex > 1) {
        int curIndex = (topIndex + bottomIndex)/2;
        if (!((new QPixmap(imageDir + styleName + "/peg" + makeFourLong(curIndex) + ".png"))->size().isNull())) {
            bottomIndex = curIndex;
        } else {
            topIndex = curIndex;
        }
    }
    return topIndex;
}

void HiQView::loadStyle() {

    int numFrames = findNumFrames();
    img->readPixmaps(imageDir + styleName + "/board.png");
    mov->readPixmaps(imageDir + styleName + "/peg%1.png", numFrames);
    pegHeight = mov->image(0)->height();
    pegWidth = mov->image(0)->width();
    if (numFrames == 0 || !img->isValid() || !mov->isValid()) { // style not present
        if (styleName == *defaultStyle) {
            KMessageBox::error(this, i18n("Dfault style could be found.\nMake sure the styles are installed and that the image directory is set correctly."), i18n("Could not load style"));
            exit(1);
        } else {
            KMessageBox::sorry(this, QString(i18n("The Images for the requested style could not be found.\nTrying to load default style \"%1\" in stead.")).arg(*defaultStyle), i18n("Could not load syle"));
            styleName = *defaultStyle;
            loadStyle();
        }
    }

}

void HiQView::setStyle(QString nStyle) {
    if (nStyle != styleName) {
        styleName = nStyle;
        loadStyle();
        bgSprite->hide();
        bgSprite->show();
        for (int i = 0; i < 33; i++) {
            if (pegs[i] != 0) {
                pegs[i]->getSprite()->hide();
                pegs[i]->getSprite()->show();
            }
        }
    }
}
void HiQView::reset() {
    if (mousePressNum != -1) {
        pegs[mousePressNum]->moveBack();
        pegs[mousePressNum]->getSprite()->setAnimated(animated);
        mousePressNum = -1;
    }
    mousePressX = -1;
    mousePressY = -1;
    mousePressNum = -1;
    int index = 0;
    for (int i = 0; i < 7; i++) {
        for (int j = 0; j < 7; j++) {
            if (i >= 2 && i <= 4 ||
                    j >= 2 && j <= 4) {
                if (i == 3 && j == 3) {
                    if (pegs[index] != 0) {
                        pegs[index]->destroy();
                    }
                    pegs[index] = 0;
                } else if (pegs[index] == 0) { // keep existing ones
                    pegs[index] = new Peg(canvas, i, j, pegs, mov);
                    pegs[index]->getSprite()->setAnimated(animated);
                }
                index++;
            }
        }
    }
    history = new HistoryPtr();
    redoHistory = new HistoryPtr();
    createInitialBoard();
}

// sets the animated state of all pegs to 'yes'
void HiQView::setAnimated(bool yes) {
    animated = yes;
    if (yes) {
        canvas->setAdvancePeriod(advancePeriod);
    } else {
        canvas->setAdvancePeriod(advancePeriodFast);
    }

    for (int i = 0; i < 33; i++) {
        if (pegs[i] != 0) {
            pegs[i]->getSprite()->setAnimated(yes);
        }
    }
    if (mousePressNum != -1) {
        pegs[mousePressNum]->getSprite()->setAnimated(true);
    }
}

HiQView::~HiQView() {}

void HiQView::setEditable(bool yes) {
    editable = yes;
}

void HiQView::setUpdateDelay(int normal, int fast) {
    advancePeriod = normal;
    advancePeriodFast = fast;
    setAnimated(animated);
}

QString& HiQView::boardToString(bool b[]) {
    int counter = 0;
    QString q = "";
    for (int i = 0; i < 7; i++) {
        for (int j = 0; j < 7; j++) {
            bool iOOfBounds = (i < 2 && i >= 0) || (i < 7 && i >= 5);
            bool jOOfBounds = (j < 2 && j >= 0) || (j < 7 && j >= 5);
            if (iOOfBounds && jOOfBounds) {
                q += "  ";
            } else {
                if (b[counter]) {
                    q += " o";
                } else {
                    q += " .";
                }
            }
            counter++;
        }
    }
    return q;
}

QString& HiQView::toString() {
    QString ret = i18n("Initial Board:\n") + boardToString(initialBoard) + "\n\n";
    for (HistoryPart *p = history->first; p->next != 0; p = p->next) {
        ret += i18n("From row: %d and col: %d to row: %d and col: %d\n")
               .arg(p->fromRow).arg(p->fromCol).arg(p->toRow).arg(p->toCol);
    }
    return ret;
}

// returns false iff undo was unsuccessful
bool HiQView::undo() {
    if (history->first == 0) {
        return false; // empty history
    } else {
        pegs[Peg::convert(history->first->toRow, history->first->toCol)]->destroy();
        pegs[Peg::convert(history->first->toRow, history->first->toCol)] = 0;
        pegs[Peg::convert(history->first->fromRow, history->first->fromCol)]
        = new Peg(canvas, history->first->fromRow, history->first->fromCol, pegs, mov);
        pegs[Peg::convert(history->first->fromRow, history->first->fromCol)]
        ->getSprite()->setAnimated(animated);
        pegs[Peg::convert((history->first->fromRow+history->first->toRow)/2,
                          (history->first->fromCol+history->first->toCol)/2)]
        = new Peg(canvas, (history->first->fromRow+history->first->toRow)/2,
                  (history->first->fromCol+history->first->toCol)/2, pegs, mov);
        pegs[Peg::convert((history->first->fromRow+history->first->toRow)/2,
                          (history->first->fromCol+history->first->toCol)/2)]
        ->getSprite()->setAnimated(animated);
        HistoryPart * dummy = history->first;
        history->first = history->first->next;
        dummy->next = redoHistory->first;// feed into redo history
        redoHistory->first = dummy;
        if (history->first == 0) { // history is empty now
            emit undoIsLegal(false); // undoing illegal
        }
        emit redoIsLegal(true); // redoing is legal
        return true;
    }
}

// returns false iff redo was unsuccessful
bool HiQView::redo() {
    if (redoHistory->first == 0) {
        return false; // empty
    } else {
        pegs[Peg::convert(redoHistory->first->fromRow, redoHistory->first->fromCol)]->
        moveTo(redoHistory->first->toRow, redoHistory->first->toCol);
        HistoryPart * dummy = redoHistory->first;
        redoHistory->first = redoHistory->first->next;
        dummy->next = history->first;// feed back into normal history
        history->first = dummy;
        if (redoHistory->first == 0) { // redo history is empty
            emit redoIsLegal(false);
        }
        emit undoIsLegal(true); // undoing legal
        return true;
    }
}

void HiQView::createInitialBoard() {
    for (int i = 0; i < 33; i++) {
        initialBoard[i] = (pegs[i] != 0);
    }
    history->first = 0;
    redoHistory->first = 0;
    emit undoIsLegal(false); // undoing/redoing illegal
    emit redoIsLegal(false);
}

void HiQView::contentsMousePressEvent( QMouseEvent* e ) {
    int row = (int)(e->y()/(float)pegHeight);
    int col = (int)(e->x()/(float)pegWidth);
    int num = Peg::convert(row, col);
    if (num != -1) {
        if (editable) {
            if (pegs[num] == 0) {
                pegs[num] = new Peg(canvas, row, col, pegs, mov);
                pegs[num]->getSprite()->setAnimated(animated);
            } else {
                pegs[num]->destroy();
                pegs[num] = 0;
            }
            createInitialBoard();
        } else if (pegs[num] != 0) {
            mousePressX = pegWidth*col - ((QMouseEvent*)e)->x();
            mousePressY = pegHeight*row - ((QMouseEvent*)e)->y();
            mousePressNum = num;
            pegs[mousePressNum]->getSprite()->setZ(2);
            pegs[mousePressNum]->getSprite()->setAnimated(true);
        }
    }
}
void HiQView::contentsMouseMoveEvent ( QMouseEvent * e ) {
    if (mousePressNum != -1) {
        pegs[mousePressNum]->getSprite()->move(((QMouseEvent*)e)->x() + mousePressX * (float)mov->image(0)->width() / (float)pegWidth,
                                               ((QMouseEvent*)e)->y() + mousePressY * (float)mov->image(0)->height() / (float)pegHeight);
    }
}
void HiQView::contentsMouseReleaseEvent ( QMouseEvent * e ) {
    if (mousePressNum != -1 && !editable) {
        int x = (int) ((((QMouseEvent*)e)->x() + mousePressX)/(float)pegWidth+.5f);
        int y = (int) ((((QMouseEvent*)e)->y() + mousePressY)/(float)pegHeight+.5f);
        pegs[mousePressNum]->getSprite()->setAnimated(animated);
        HistoryPart * part = pegs[mousePressNum]->moveTo(y, x);
        if (part != 0) { // valid move
            part->next = history->first; // add to history
            history->first = part;
            redoHistory->first = 0; // redo history
            emit undoIsLegal(true);
            emit redoIsLegal(false);
        }
        mousePressX = -1;
        mousePressY = -1;
        mousePressNum = -1;
    }
}


void HiQView::print(QPainter *p, int height, int width) {
    // do the actual printing, here
    // p->drawText(etc..)
}
HistoryPtr* HiQView::getHistory() {
    return history;
}

#include "hiqview.moc"
