/* 
 * Copyright (C) 2005 <mikelima@cirulla.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation version 2.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */


#include <stdlib.h>
#include <string>

#include <klocale.h>
#include <kgenericfactory.h>
#include <kdebug.h>

#include <qcstring.h>
#include <qfile.h>

#include "kfile_zmachine.h"

typedef KGenericFactory<KZMachinePlugin> ZMachineFactory;

K_EXPORT_COMPONENT_FACTORY(kfile_zmachine, ZMachineFactory("kfile_zmachine"))

KZMachinePlugin::KZMachinePlugin(
	QObject *parent, const char *name, const QStringList &args) : 
KFilePlugin(parent, name, args)
{
    // set up our mime type
    KFileMimeTypeInfo *info = addMimeTypeInfo("application/x-zmachine");

    KFileMimeTypeInfo::GroupInfo *group;

    // info group
    group = addGroupInfo(info, "Info", i18n("Game Information"));
    addItemInfo(group, "GameFormat", i18n("Game Format"), QVariant::Int);
    addItemInfo(group, "Release", i18n("Release"), QVariant::Int);
    addItemInfo(group, "SerialCode", i18n("Serial Code"), QVariant::String);
}

bool 
KZMachinePlugin::readInfo(KFileMetaInfo& info, uint what)
{
    if (info.path().isEmpty()) return false;

    QFile f(info.path());
    if (!f.open(IO_ReadOnly)) return false;

    // Read the ZMachine file header. 
    // Layout information taken from
    // http://www.gnelson.demon.co.uk/zspec/sect11.html
    // 
    char data[0x37];
    if (f.readBlock(data, sizeof data) != sizeof data) return false;
    f.close();
    int gameFormat = data[0]; // Should be a number between 1 and 8
    if (gameFormat <= 0 || gameFormat > 8) return false;
    // From Appendix B. 
    int gameRelease = data[2] * 256 + data[3]; 
    QString serialCode = QString::fromLatin1(&data[0x12], 6);
    
    // Be sure the serial code is at least printable
    for (int i = 0; i < 6; i++) {
        if (!serialCode[i].isPrint()) return false;
    }    

    // If all is well, set the metadata
    KFileMetaInfoGroup group = appendGroup(info, "Info");
    appendItem(group, "GameFormat", gameFormat);
    appendItem(group, "Release", gameRelease);
    appendItem(group, "SerialCode", serialCode);
    
    return true;
}

#include "kfile_zmachine.moc"
