/***************************************************************************
 *   Copyright 2009 Zeng Huan <zh.issac@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "blender-delegate.h"
#include "blender-itemeditor.h"

#include <QApplication>
#include <QPainter>

const int DelegatePadding = 6;

Kolf::TextureListDelegate::TextureListDelegate( QObject *parent)
	: QAbstractItemDelegate(parent)
{
}

QSize Kolf::TextureListDelegate::sizeHint( const QStyleOptionViewItem &option,
		           const QModelIndex &index ) const
{
	Q_UNUSED(option);
	QPixmap thumbnail = index.data(Qt::DecorationRole).value<QPixmap>();
	int width = 2 * DelegatePadding + thumbnail.width();
	int height = 2 * DelegatePadding + thumbnail.height();
	return QSize(width, height);
}

void Kolf::TextureListDelegate::paint( QPainter *painter,
		            const QStyleOptionViewItem &option,
		            const QModelIndex &index ) const
{
	//highlight selected item
	QStyleOptionViewItemV4 opt = option;
	QStyle* style = opt.widget ? opt.widget->style() : QApplication::style();
	style->drawPrimitive(QStyle::PE_PanelItemViewItem, &opt, painter, opt.widget);
	//draw contents - thumbnail image
	QPixmap thumbnail = index.data(Qt::DecorationRole).value<QPixmap>();
	QPoint thumbnailPos = option.rect.topLeft() + QPoint(DelegatePadding, DelegatePadding);
	painter->drawPixmap(thumbnailPos, thumbnail);
	//draw contents - range description
	QString text = index.data(Qt::DisplayRole).toString();
	QRect textRect = option.rect.adjusted(2 * DelegatePadding + thumbnail.width(), DelegatePadding, -DelegatePadding, -DelegatePadding);
	painter->drawText(textRect, Qt::AlignVCenter | Qt::AlignLeft | Qt::TextSingleLine, text);
}

QWidget* Kolf::TextureListDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
	Kolf::BlenderItemEditor* editor = new Kolf::BlenderItemEditor(parent);
	editor->installEventFilter(const_cast<TextureListDelegate*>(this));
	return editor;
}

void Kolf::TextureListDelegate::updateEditorGeometry(QWidget* editor, const QStyleOptionViewItem& option, const QModelIndex& index) const
{
	editor->setGeometry(option.rect);
}

void Kolf::TextureListDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
{
	QVariantList range = index.data(Qt::EditRole).toList();
	Kolf::BlenderItemEditor* config = qobject_cast<Kolf::BlenderItemEditor*>(editor);
	config->setValue(range[0].toInt(), range[1].toInt(), range[2].toInt());
}

void Kolf::TextureListDelegate::setModelData(QWidget *editor, QAbstractItemModel *model,
		  const QModelIndex &index) const
{
	Kolf::BlenderItemEditor* config = qobject_cast<Kolf::BlenderItemEditor*>(editor);
	QVariantList range;
	range << config->minBox()->value();
	range << config->optBox()->value();
	range << config->maxBox()->value();
	model->setData(index, range, Qt::EditRole);
}

void Kolf::TextureListDelegate::editorClosed()
{
	Kolf::BlenderItemEditor* editor = qobject_cast<Kolf::BlenderItemEditor*>(sender());
	if (editor)
	{
		emit commitData(editor);
		emit closeEditor(editor);
	}
}

#include "blender-delegate.moc"
