/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "blender-texturepool.h"

#include <QImage>
#include <KStandardDirs>

Kolf::BlenderTexturePool::BlenderTexturePool()
{
	//find textures
	const QStringList imageFiles = KStandardDirs().findAllResources("data", QLatin1String("kolf-ng/textures/*.jpg"), KStandardDirs::NoDuplicates);
	foreach (const QString& imageFile, imageFiles)
	{
		//read data
		const QString imageName = imageFile.section('/', -1, -1).section('.', 0, 0);
		QImage image(imageFile);
		QPixmap thumbnail = QPixmap::fromImage(image.scaled(48, 48));
		//insert new item for texture
		QStandardItem* item = new QStandardItem;
		item->setData(imageName, Qt::DisplayRole);
		item->setData(image.scaled(48, 48), Qt::DecorationRole);
		item->setData(imageName, KeyRole);
		item->setData(image, ImageRole);
		item->setData(thumbnail, ThumbnailRole);
		item->setEditable(false);
		appendRow(item);
	}
}

Kolf::BlenderTexturePool* Kolf::BlenderTexturePool::instance()
{
	static Kolf::BlenderTexturePool instance;
	return &instance;
}

QModelIndex Kolf::BlenderTexturePool::findKey(const QString& key) const
{
	for (int row = 0; row < rowCount(); ++row)
	{
		QStandardItem* item = this->item(row);
		if (item->data(KeyRole) == key)
			return item->index();
	}
	return QModelIndex(); //no item with that key
}
