/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_GEOMETRY_WALL_H
#define KOLF_GEOMETRY_WALL_H

#include "geometry.h"
#include "math-pointf.h"

namespace Kolf
{
	//The reference point of this geometry is its center.
	//Note that the wall is always vertical! (Everything else would be very hard to display in the 2D view.)
	class WallGeometry : public Kolf::Geometry
	{
		public:
			WallGeometry(dSpaceID spaceID, Kolf::Object* object);
			virtual ~WallGeometry();

			Kolf::VectorF extent() const;
			void setExtent(const Kolf::VectorF& extent);

			static const qreal Thickness;
		protected:
			virtual void createOutlines();
		private:
			//Note on implementation: The wall extent is only applied in the X and Z directions, and the wall is then rotated around the Z axis. As this might interfere with the Kolf::Object::rotation, we encapsulate the actual wall geometry in a coordinate transformer geometry.
			dGeomID m_innerID;
			Kolf::VectorF m_extent;
	};
}

#endif // KOLF_GEOMETRY_WALL_H
