/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "geometry.h"
#include "object.h"
#include "overlay.h"
#include <ode/ode.h>

const int Kolf::Geometry::ActivationOutlinePadding = 10;

Kolf::Geometry::Geometry(dGeomID id, Kolf::Object* object)
	: m_id(id)
	, m_object(object)
{
	delete m_object->m_geometry;
	m_object->m_geometry = this;
	dGeomSetBody(id, m_object->m_bodyID);
	//use the data pointer of the ODE geom to provide a link back to this class
	dGeomSetData(m_id, (void*) this);
}

Kolf::Geometry::~Geometry()
{
	dGeomDestroy(m_id);
	m_object->m_geometry = 0;
}

Kolf::Object* Kolf::Geometry::object() const
{
	return m_object;
}

QPainterPath Kolf::Geometry::activationOutline() const
{
	return m_activationOutline;
}

void Kolf::Geometry::setActivationOutline(const QPainterPath& outline)
{
	m_activationOutline = outline;
}

QPainterPath Kolf::Geometry::interactionOutline() const
{
	return m_interactionOutline;
}

void Kolf::Geometry::setInteractionOutline(const QPainterPath& outline)
{
	m_interactionOutline = outline;
}

dGeomID Kolf::Geometry::id() const
{
	return m_id;
}

dMass Kolf::Geometry::mass() const
{
	dBodyID body = dGeomGetBody(m_id);
	if (!body)
		return dMass();
	dMass result;
	dBodyGetMass(body, &result);
	return result;
}

void Kolf::Geometry::setMass(const dMass& mass)
{
	dBodyID body = dGeomGetBody(m_id);
	if (body)
		dBodySetMass(body, &mass);
}

void Kolf::Geometry::setMass(qreal mass)
{
	dBodyID body = dGeomGetBody(m_id);
	if (!body)
		return;
	dMass massStructure;
	dBodyGetMass(body, &massStructure);
	dMassAdjust(&massStructure, mass);
	dBodySetMass(body, &massStructure);
}

void Kolf::Geometry::update()
{
	createOutlines();
	Kolf::Overlay* overlay = m_object->overlay(false);
	if (overlay)
		overlay->update();
}
