/***************************************************************************
 *   Copyright 2008-2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "handle-point.h"

#include <QGraphicsSceneHoverEvent>
#include <QGraphicsSceneMouseEvent>

const qreal PointHandleSize = 10.0;
const QPen Kolf::PointHandle::HandlePen(Qt::blue);
const QBrush Kolf::PointHandle::ActiveHandleBrush(QColor::fromHsv(240, 200, 255, 128));
const QBrush Kolf::PointHandle::NormalHandleBrush(QColor::fromHsv(240, 255, 255, 128));

Kolf::PointHandle::PointHandle(Kolf::PointHandle::Shape shape, QGraphicsItem* parent)
	: QGraphicsPathItem(parent)
{
	setAcceptHoverEvents(true);
	setAcceptedMouseButtons(Qt::LeftButton);
	setPen(HandlePen);
	setBrush(NormalHandleBrush);
	//create shape
	QPainterPath path;
	switch (shape)
	{
		case SquareShape:
			path.addRect(QRectF(-PointHandleSize, -PointHandleSize, 2 * PointHandleSize, 2 * PointHandleSize));
			break;
		case CircleShape:
			path.addEllipse(QPointF(0.0, 0.0), PointHandleSize, PointHandleSize);
			break;
		case TriangleShape:
			path.moveTo(QPointF(PointHandleSize, 0.0));
			path.lineTo(QPointF(-PointHandleSize, PointHandleSize));
			path.lineTo(QPointF(-PointHandleSize, -PointHandleSize));
			path.closeSubpath();
			break;
	}
	setPath(path);
}

void Kolf::PointHandle::hoverEnterEvent(QGraphicsSceneHoverEvent* event)
{
	event->accept();
	setBrush(ActiveHandleBrush);
}

void Kolf::PointHandle::hoverLeaveEvent(QGraphicsSceneHoverEvent* event)
{
	event->accept();
	setBrush(NormalHandleBrush);
}

void Kolf::PointHandle::mousePressEvent(QGraphicsSceneMouseEvent* event)
{
	event->accept();
	if (event->button() == Qt::LeftButton)
		emit moveStarted();
}

void Kolf::PointHandle::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
{
	event->accept();
	if (event->buttons() & Qt::LeftButton)
		emit moveRequest(event->scenePos());
}

void Kolf::PointHandle::mouseReleaseEvent(QGraphicsSceneMouseEvent* event)
{
	event->accept();
	if (event->button() == Qt::LeftButton)
		emit moveEnded();
}

#include "handle-point.moc"
