/***************************************************************************
 *   Copyright 2009 Zeng Huan <zh.issac@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "object-environment.h"
#include "objectregistry.h"
#include "overlay-empty.h"
#include "geometry-terrain.h"

#include <QPixmap>
#include <KStandardDirs>

REGISTER_KOLF_OBJECT(Kolf::EnvironmentObject, ki18n("Terrain"), KIcon())

namespace Kolf
{
	const QString HeightmapFileNamePattern("kolf-ng/courses/%1");
	const QString TextureFileNamePattern("kolf-ng/courses/%1");
}

Kolf::EnvironmentObject::EnvironmentObject(dWorldID worldID, dSpaceID spaceID)
	: Kolf::Object(worldID, spaceID)
	, m_geometry(new Kolf::TerrainGeometry(spaceID, this))
	, m_item(new QGraphicsPixmapItem(this))
{
}

Kolf::EnvironmentObject::~EnvironmentObject()
{
	delete m_item;
}

Kolf::Overlay* Kolf::EnvironmentObject::createOverlay()
{
	return new Kolf::EmptyOverlay(this);
}

void Kolf::EnvironmentObject::setPathToHeightmap(const QString& pathToHeightmap)
{
	//check input
	const QString fileName = KStandardDirs::locate("data", Kolf::HeightmapFileNamePattern.arg(pathToHeightmap));
	if (fileName.isEmpty())
		return;
	Kolf::HeightmapRenderer heightmapRenderer(fileName);
	if (!heightmapRenderer.isDataValid())
		return;
	//load heightmap
	m_pathToHeightmap = pathToHeightmap;
	m_geometry->setHeightmapRenderer(heightmapRenderer);
}

void Kolf::EnvironmentObject::setPathToTexture(const QString& pathToTexture)
{
	//check input
	const QString fileName = KStandardDirs::locate("data", Kolf::TextureFileNamePattern.arg(pathToTexture));
	if(fileName.isEmpty())
		return;
	//load texture
	m_pathToTexture = pathToTexture;
	m_geometry->setTexture(QImage(fileName));
	m_item->setPixmap(m_geometry->backdrop());
}

QImage Kolf::EnvironmentObject::heightmap() const
{
	return m_geometry->heightmapRenderer();
}

void Kolf::EnvironmentObject::setHeightmap(const QImage& heightmap)
{
	m_geometry->setHeightmapRenderer(heightmap);
}

const Kolf::TerrainGeometry* Kolf::EnvironmentObject::terrainGeometry() const
{
	return m_geometry;
}

#include "object-environment.moc"
