/***************************************************************************
 *   Copyright 2008-2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "object-hole.h"
#include "ball.h"
#include "geometry-sphere.h"
#include "objectregistry.h"
#include "overlay-empty.h"

#include <QGraphicsEllipseItem>

const int Kolf::HoleObject::HoleRadius = 14; // = 1.4 * Kolf::Ball::BallRadius

REGISTER_KOLF_OBJECT(Kolf::HoleObject, ki18n("Hole (target)"), KIcon())

Kolf::HoleObject::HoleObject(dWorldID worldID, dSpaceID spaceID)
	: Kolf::Object(worldID, spaceID)
	, m_holeItem(new QGraphicsEllipseItem(QRectF(-HoleRadius, -HoleRadius, 2 * HoleRadius, 2 * HoleRadius), this))
{
	Kolf::SphereGeometry* geometry = new Kolf::SphereGeometry(spaceID, this);
	geometry->setRadius(HoleRadius);
}

Kolf::HoleObject::~HoleObject()
{
}

Kolf::Overlay* Kolf::HoleObject::createOverlay()
{
	return new Kolf::EmptyOverlay(this);
}

Kolf::Object::SimulationFlags Kolf::HoleObject::simulationFlags() const
{
	return Kolf::Object::CustomInteraction;
}

void Kolf::HoleObject::doCustomInteraction(Kolf::Object* object)
{
	Kolf::Ball* ball = qobject_cast<Kolf::Ball*>(object);
	if (!ball)
		return;
	//capture ball
	ball->setPosition(position());
	ball->setVelocity(Kolf::VectorF());
	ball->setInHole(true);
}

#include "object-hole.moc"
