/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_OBJECTLIST_H
#define KOLF_OBJECTLIST_H

#include <QList>
#include <QObject>

namespace Kolf
{
	class Object;

	/**
	 * \class ObjectList
	 * \since 2.0
	 *
	 * This is a list of Kolf::Object instance pointers. Apart from the QList functionality, it offers a means to filter for specific object types.
	 */
	class ObjectList : public QList<Kolf::Object*>
	{
		public:
			///Returns a list of all Kolf::Object instances in this list that are of that given type.
			template <typename T> QList<T*> allOfType() const
			{
				QList<T*> resultList;
				foreach (Kolf::Object* object, *this)
				{
					T* castedObject = qobject_cast<T*>(object);
					if (castedObject)
						resultList << castedObject;
				}
				return resultList;
			}
			///Returns a list that contains all Kolf::Object instances except for those that are instances of the given type.
			template <typename T> Kolf::ObjectList allExceptType() const
			{
				Kolf::ObjectList resultList;
				foreach (Kolf::Object* object, *this)
					if (object && !qobject_cast<T*>(object))
						resultList << object;
				return resultList;
			}
	};
}

#endif // KOLF_OBJECTLIST_H
