/***************************************************************************
 *   Copyright 2008-2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "overlay-wall.h"
#include "handle-point.h"
#include "object.h"

#include <cmath>

Kolf::WallOverlay::WallOverlay(Kolf::Object* object)
	: Kolf::Overlay(object)
	, m_handle1(new Kolf::PointHandle(Kolf::PointHandle::SquareShape, this))
	, m_handle2(new Kolf::PointHandle(Kolf::PointHandle::SquareShape, this))
{
	addHandle(m_handle1);
	addHandle(m_handle2);
	connect(m_handle1, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
	connect(m_handle2, SIGNAL(moveRequest(const QPointF&)), this, SLOT(moveHandle(const QPointF&)));
}

void Kolf::WallOverlay::update()
{
	Kolf::Overlay::update();
	const Kolf::VectorF extent3D = object()->property("Extent").value<Kolf::PointF>();
	const QVectorF extent(extent3D.x(), extent3D.y());
	m_handle1->setPos(-extent);
	m_handle2->setPos(extent);
}

void Kolf::WallOverlay::moveHandle(const QPointF& handleScenePos)
{
	QPointF handlePos = mapFromScene(handleScenePos);
	const QObject* handle = sender();
	//get handle positions
	QPointF handle1Pos = m_handle1->pos();
	QPointF handle2Pos = m_handle2->pos();
	if (handle == m_handle1)
		handle1Pos = handlePos;
	else if (handle == m_handle2)
		handle2Pos = handlePos;
	//ensure minimum length
	static const qreal minLength = Kolf::Overlay::MinimumObjectDimension;
	const QVectorF positionDifference = handle1Pos - handle2Pos;
	const qreal length = sqrt(positionDifference * positionDifference);
	if (length < minLength)
	{
		const QVectorF additionalExtent = positionDifference * (minLength / length - 1);
		if (handle == m_handle1)
			handle1Pos += additionalExtent;
		else if (handle == m_handle2)
			handle2Pos -= additionalExtent;
	}
	//convert to extent/center
	const QVectorF extent2D = (handle2Pos - handle1Pos) / 2;
	const QPointF center2DOffset = (handle2Pos + handle1Pos) / 2;
	//merge with height information
	Kolf::VectorF extent3D = object()->property("Extent").value<Kolf::PointF>();
	extent3D.rx() = extent2D.x();
	extent3D.ry() = extent2D.y();
	object()->setProperty("Extent", qVariantFromValue(extent3D));
	Kolf::PointF center3D = object()->position();
	center3D.rx() += center2DOffset.x();
	center3D.ry() += center2DOffset.y();
	object()->setPosition(center3D);
}

#include "overlay-wall.moc"
