/***************************************************************************
 *   Copyright 2008-2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_COURSE_H
#define KOLF_COURSE_H

#include <QObject>

namespace Kolf
{
	class Scene;

	/**
	 * \class Course
	 * \since 2.0
	 *
	 * This class represents a course, consisting of many holes. It can be loaded from and saved to standard locations.
	 */
	class Course : public QObject
	{
		Q_OBJECT
		public:
			///Gives the file names of all available courses.
			static QStringList availableCourses();
			///Returns the title of the course with the given file name.
			static QString courseTitle(const QString& courseName);
			///Creates a new course object from the course file with the given \a name.
			Course(const QString& courseName);
			//TODO: a constructor that creates a new course with one sample hole
			~Course();

			///Returns the number of holes in this course. 0 indicates that the course file was invalid.
			///\note Hole numbering starts at zero internally.
			int holeCount() const;
			///Returns the name of this course, or an empty string if the course file was invalid.
			QString title() const;
			///Returns the par for the given \a hole.
			///\note Hole numbering starts at zero internally, so increment hole indexes in user-visible strings.
			int parForHole(int holeIndex) const;
			///Returns the scene for this \a hole.
			///\note Hole numbering starts at zero internally, so increment hole indexes in user-visible strings.
			Kolf::Scene* hole(int holeIndex) const;
		public Q_SLOTS:
			///Saves the changes that have been made to this course.
			void save() const;
		private:
			QString m_courseName, m_title;
			QList<int> m_parScores;
			QList<Kolf::Scene*> m_scenes;
	};
}

#endif // KOLF_COURSE_H
