/***************************************************************************
 *   Copyright 2008-2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "player.h"
#include "../elements/ball.h"
#include "../elements/scene.h"

Kolf::Player::Player(const QString& name, unsigned int holeCount)
	: m_ball(0)
	, m_scene(0)
	, m_hue(-1)
	, m_name(name)
	, m_scores(holeCount, 0)
	, m_totalScore(0)
	, m_currentHole(-1)
{
}

Kolf::Player::~Player()
{
	delete m_ball;
}

Kolf::Ball* Kolf::Player::ball() const
{
	if (!m_ball && m_scene)
	{
		m_ball = m_scene->addBall();
		m_ball->setColor(QColor::fromHsv(m_hue, 255, 255));
	}
	return m_ball;
}

int Kolf::Player::hue() const
{
	return m_hue;
}

QString Kolf::Player::name() const
{
	return m_name;
}

void Kolf::Player::setHue(int hue)
{
	if (m_hue != hue)
	{
		m_hue = hue;
		if (m_ball)
			m_ball->setColor(QColor::fromHsv(hue, 255, 255));
	}
}

void Kolf::Player::setName(const QString& name)
{
	m_name = name;
}

int Kolf::Player::currentScore() const
{
	return m_scores.value(m_currentHole);
}

int Kolf::Player::score(int hole) const
{
	return m_scores.value(hole);
}

int Kolf::Player::totalScore() const
{
	return m_totalScore;
}

void Kolf::Player::setScore(int hole, int score)
{
	if (hole >= 0 && hole < m_scores.count())
	{
		m_totalScore += score - m_scores[hole];
		m_scores[hole] = score;
		emit scoreChanged(this, hole);
	}
}

void Kolf::Player::setCurrentHole(int hole)
{
	m_currentHole = hole;
}

void Kolf::Player::setCurrentScore(int score)
{
	setScore(m_currentHole, score);
}

void Kolf::Player::setScene(Kolf::Scene* scene)
{
	if (m_scene == scene)
		return;
	if (m_scene)
		m_scene->removeBall(m_ball);
	m_scene = scene;
	delete m_ball;
	m_ball = 0;
}

#include "player.moc"
