/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_SCOREMODEL_H
#define KOLF_SCOREMODEL_H

#include <QAbstractTableModel>

namespace Kolf
{
	class Game;
	class Player;

	/**
	 * \class ScoreModel
	 * \since 2.0
	 *
	 * This model displays the scores of a certain game.
	 */
	class ScoreModel : public QAbstractTableModel
	{
		Q_OBJECT
		public:
			ScoreModel();

			Kolf::Game* game() const;
			void setGame(Kolf::Game* game);

			virtual int rowCount(const QModelIndex& parent = QModelIndex()) const;
			virtual int columnCount(const QModelIndex& parent = QModelIndex()) const;
			virtual Qt::ItemFlags flags(const QModelIndex& index) const;
			virtual QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const;
			virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
		public Q_SLOTS:
			void playerAdded(Kolf::Player* player);
			void currentHoleChanged(int newHoleIndex, int oldHoleIndex);
			void scoreChanged(Kolf::Player* player, int holeIndex);
		private:
			Kolf::Game* m_game;
	};
}

#endif // KOLF_SCOREMODEL_H
