/***************************************************************************
 *   Copyright 2008-2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "mainwindow.h"
#include "mainwindow_p.h"
#include "../engine/course.h"
#include "../engine/gameui.h"
#include "../engine/player.h"

#include <KAction>
#include <KActionCollection>
#include <KApplication>
#include <KIcon>
#include <KLocalizedString>
#include <KStandardAction>
#include <KStandardGameAction>
#include <KStatusBar>

//BEGIN Kolf::MainWindowPrivate

Kolf::MainWindowPrivate::MainWindowPrivate(Kolf::MainWindow* parent)
	: m_parent(parent)
	, m_tabWidget(new KTabWidget)
	, m_game(0)
	, m_newGameWidget(new Kolf::NewGameWidget)
{
	//new game widget
	connect(m_newGameWidget, SIGNAL(gameStarted(Kolf::Game*)), this, SLOT(startGame(Kolf::Game*)));
	newGame(); //causes the new game widget to be the first widget to be shown
}

Kolf::MainWindowPrivate::~MainWindowPrivate()
{
	delete m_game;
	delete m_newGameWidget;
	//m_tabWidget is deleted by ~KXmlGuiWindow
}

void Kolf::MainWindowPrivate::setupActions()
{
	KStandardGameAction::gameNew(this, SLOT(newGame()), m_parent->actionCollection());
	KStandardGameAction::quit(kapp, SLOT(quit()), m_parent->actionCollection());
}

void Kolf::MainWindowPrivate::newGame()
{
	if (m_tabWidget->indexOf(m_newGameWidget) == -1)
		m_tabWidget->addTab(m_newGameWidget, i18n("&New game"));
	m_tabWidget->setCurrentWidget(m_newGameWidget);
}

void Kolf::MainWindowPrivate::startGame(Kolf::Game* game)
{
	//remove "New game" tab and old game from interface
	m_tabWidget->removeTab(m_tabWidget->indexOf(m_newGameWidget));
	delete m_game;
	//start new game
	m_game = game;
	connect(game, SIGNAL(gameEnded()), this, SLOT(endGame()));
	game->startGame();
	//merge game interface (game is a subclass of KXMLGUIClient)
	Kolf::GameUIParent uiParent(m_parent, m_tabWidget);
	game->gameUI()->setUIParent(uiParent);
}

void Kolf::MainWindowPrivate::endGame()
{
	//TODO: necessary?
}

//END Kolf::MainWindowPrivate

//BEGIN Kolf::MainWindow

Kolf::MainWindow::MainWindow()
	: p(new Kolf::MainWindowPrivate(this))
{
	//the usual setup
	p->setupActions();
	setAutoSaveSettings();
	setupGUI();
	setCentralWidget(p->m_tabWidget);
	statusBar()->hide();
	show();
}

Kolf::MainWindow::~MainWindow()
{
	delete p;
}

//END Kolf::MainWindow

#include "mainwindow_p.moc"
