/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "newgame-coursewidget.h"
#include "newgame-coursemodel.h"

Kolf::NewGameCourseWidget::NewGameCourseWidget()
	: m_model(new Kolf::NewGameCourseModel(this))
{
	setModel(m_model);
	setSelectionMode(QAbstractItemView::SingleSelection);
}

void Kolf::NewGameCourseWidget::paintEvent(QPaintEvent* event)
{
	QListView::paintEvent(event);
	//ensure that something has been selected (not done in the constructor because the model delays course parsing to keep the application responsive)
	if (!selectionModel()->hasSelection() && m_model->rowCount() > 0)
		selectionModel()->select(m_model->index(0, 0), QItemSelectionModel::Select);
	//Note that this default selection is consistent with the fallback behavior in Kolf::NewGameCourseWidget::courseName().
}

QString Kolf::NewGameCourseWidget::courseName() const
{
	if (selectionModel()->hasSelection())
	{
		QModelIndex selectedIndex = selectionModel()->selectedIndexes().at(0);
		return selectedIndex.data(Qt::UserRole).toString();
	}
	//no selection -> if courses are available, fallback to the first course
	else if (m_model->rowCount() > 0)
	{
		return m_model->index(0, 0).data(Qt::UserRole).toString();
		//Note that this fallback behavior is consistent with the default selection in Kolf::NewGameCourseWidget::paintEvent (i.e., if the view would have been shown, this item would have been selected by default).
	}
	//no courses available
	else
		return QString();
}
