/***************************************************************************
 *   Copyright 2009 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "newgamewidget.h"
#include "newgame-coursewidget.h"
#include "newgame-playerlist.h"
#include "../engine/course.h"
#include "../engine/game-editor.h"
#include "../engine/game-normal.h"

#include <QApplication>
#include <QBoxLayout>
#include <QGroupBox>
#include <QPushButton>
#include <KIcon>
#include <KLocalizedString>

Kolf::NewGameWidget::NewGameWidget()
	: m_courseWidget(new Kolf::NewGameCourseWidget)
	, m_playerList(new Kolf::NewGamePlayerList)
{
	//some more widgets
	QGroupBox* courseGroupBox = new QGroupBox(i18n("Select a course"));
	QGroupBox* playerGroupBox = new QGroupBox(i18n("Select players"));
	QPushButton* startEditorButton = new QPushButton(KIcon("document-edit"), i18n("Open course for &editing"));
	connect(startEditorButton, SIGNAL(clicked()), this, SLOT(startEditorGame()));
	KIcon startNormalIcon(QApplication::isRightToLeft() ? "go-previous" : "go-next");
	QPushButton* startNormalButton = new QPushButton(startNormalIcon, i18n("&Start game"));
	connect(startNormalButton, SIGNAL(clicked()), this, SLOT(startNormalGame()));
	QPushButton* addPlayerButton = new QPushButton(KIcon("list-add-user"), i18n("Add player"));
	connect(addPlayerButton, SIGNAL(clicked()), m_playerList, SLOT(addPlayer()));
	//layouts for the coursße group box
	QHBoxLayout* courseButtonLayout = new QHBoxLayout;
	courseButtonLayout->addStretch(1);
	courseButtonLayout->addWidget(startEditorButton);
	QVBoxLayout* courseGroupLayout = new QVBoxLayout;
	courseGroupLayout->addWidget(m_courseWidget);
	courseGroupLayout->addLayout(courseButtonLayout);
	courseGroupBox->setLayout(courseGroupLayout);
	//layouts for the player group box
	QHBoxLayout* playerButtonLayout = new QHBoxLayout;
	playerButtonLayout->addStretch(1);
	playerButtonLayout->addWidget(addPlayerButton);
	playerButtonLayout->addWidget(startNormalButton);
	QVBoxLayout* playerGroupLayout = new QVBoxLayout;
	playerGroupLayout->addWidget(m_playerList);
	playerGroupLayout->addLayout(playerButtonLayout);
	playerGroupBox->setLayout(playerGroupLayout);
	//main layout
	QHBoxLayout* mainLayout = new QHBoxLayout;
	mainLayout->addWidget(courseGroupBox);
	mainLayout->addWidget(playerGroupBox);
	setLayout(mainLayout);
}

void Kolf::NewGameWidget::startEditorGame()
{
	const QString courseName = m_courseWidget->courseName();
	if (!courseName.isEmpty())
	{
		Kolf::Course* course = new Kolf::Course(courseName);
		Kolf::EditorGame* game = new Kolf::EditorGame(course);
		emit gameStarted(game);
	}
}

void Kolf::NewGameWidget::startNormalGame()
{
	const QString courseName = m_courseWidget->courseName();
	if (!courseName.isEmpty())
	{
		Kolf::Course* course = new Kolf::Course(courseName);
		Kolf::NormalGame* game = new Kolf::NormalGame(course);
		m_playerList->writePlayers(game);
		emit gameStarted(game);
	}
}

#include "newgamewidget.moc"
