/***************************************************************************
 *   Copyright 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_MANAGEDOBJECTGROUP_H
#define KOLF_MANAGEDOBJECTGROUP_H

#include "utils-undostack.h"
class KConfigGroup;

namespace Kolf
{
	class ManagedObject;
	class ObjectFactory;

	/**
	 * \class ManagedObjectGroup
	 * \since 2.0
	 *
	 * This class provides state tracking for multiple Kolf::Object instances.
	 * \sa Kolf::ManagedObject
	 *
	 * \warning This class assumes that all stacks in it are Kolf::ManagedObject instances. You should better not addStacks() etc. manually. Rely on the interface provided by this class, instead.
	 */
	class ManagedObjectGroup : public Utils::UndoGroup
	{
		Q_OBJECT
		public:
			///Creates a new managed object group.
			ManagedObjectGroup(Kolf::ObjectFactory* objectFactory);

			///Returns the managed objects in this managed object group.
			QList<Kolf::ManagedObject*> managedObjects() const;

			///Saves the current state of all contained objects and such into the KConfigGroup.
			void serialize(KConfigGroup* group);
			///Loads the serialized managed object group into this object.
			///\note It is highly recommended to use this method only with empty groups.
			void deserialize(KConfigGroup* group);
		Q_SIGNALS:
			///This signal forwards the objectCreated() signals of all Kolf::ManagedObject instances in this group.
			void objectCreated(Kolf::ManagedObject* managedObject);
			///This signal forwards the objectAboutToBeDeleted() signals of all Kolf::ManagedObject instances in this group.
			void objectAboutToBeDeleted(Kolf::ManagedObject* managedObject);
		private Q_SLOTS:
			void slotSubstatePushed(Utils::UndoStack* stack, int index);
		private:
			Kolf::ObjectFactory* m_objectFactory;
	};
}

#endif // KOLF_MANAGEDOBJECTGROUP_H
