/***************************************************************************
 *   Copyright 2008, 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_OBJECT_BALL_H
#define KOLF_OBJECT_BALL_H

#include "object.h"

namespace Kolf
{
	class CircleShape;

	/**
	 * \class BallObject
	 * \since 2.0
	 *
	 * The ball.
	 *
	 * \note This APIDOX brought to you by Captain Obvious.
	 */
	//This object's reference point is its center.
	class BallObject : public Kolf::Object
	{
		Q_OBJECT
		//The color is not a property because it should not be accessible through the editor.
		public:
			BallObject(b2World* world);

			///The color of this ball. One is only allowed to specify the hue.
			int hue() const;
			///Returns whether the ball has been captured by the hole.
			bool isInHole() const;
			void setHue(int hue);
			void setInHole(bool inHole);

			static const qreal BallRadius;
		Q_SIGNALS:
			void stopped();
		protected:
			virtual Kolf::Overlay* createOverlay();
			virtual void objectEvent(Kolf::Object::EventType type, const QVariant& data);
		private:
			Kolf::CircleShape* m_shape;
			QGraphicsEllipseItem* m_item;

			int m_hue;
			bool m_inHole;
			bool m_stoppedSignalSent;
	};
}

#endif // KOLF_OBJECT_BALL_H
