/***************************************************************************
 *   Copyright 2008, 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "object-rectangleblock.h"
#include "objectfactory.h"
#include "overlay-rectangle.h"
#include "shape-rectangle.h"

#include <QBrush>
#include <QPen>

REGISTER_KOLF_OBJECT(Kolf::RectangleBlockObject, {
	classObject->screenName = ki18n("Rectangular block");
	classObject->groupName = QLatin1String("02-basic-obstacles");
})

Kolf::RectangleBlockObject::RectangleBlockObject(b2World* world)
	: Kolf::Object(world)
	, m_shape(new Kolf::RectangleShape(QRectF(-1, -1, 2, 2)))
	, m_item(new QGraphicsRectItem(QRectF(-1, -1, 2, 2), this))
{
	m_item->setPen(QPen(Qt::gray));
	m_item->setBrush(Qt::gray);
	addShape(m_shape);
}

QSizeF Kolf::RectangleBlockObject::size() const
{
	return m_shape->rect().size();
}

void Kolf::RectangleBlockObject::setSize(const QSizeF& size)
{
	if (this->size() != size)
	{
		QRectF rect(QPointF(), size);
		rect.moveCenter(QPointF());
		m_shape->setRect(rect);
		m_item->setRect(rect);
		propagateUpdate();
	}
}

Kolf::Overlay* Kolf::RectangleBlockObject::createOverlay()
{
	return new Kolf::RectangleOverlay(this);
}

#include "object-rectangleblock.moc"
