/***************************************************************************
 *   Copyright 2008, 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "object-wall.h"
#include "objectfactory.h"
#include "overlay-wall.h"
#include "shape-wall.h"

REGISTER_KOLF_OBJECT(Kolf::WallObject, {
	classObject->screenName = ki18nc("as in \"a wall made of stone bricks\"", "Wall");
	classObject->groupName = QLatin1String("02-basic-obstacles");
})

Kolf::WallObject::WallObject(b2World* world)
	: Kolf::Object(world)
	, m_shape(new Kolf::WallShape(QPointF(1, 1)))
	, m_item(new QGraphicsLineItem(QLineF(1, 1, -1, -1), this))
{
	addShape(m_shape);
}

QPointF Kolf::WallObject::extent() const
{
	return m_shape->extent();
}

void Kolf::WallObject::setExtent(const QPointF& extent)
{
	if (this->extent() != extent)
	{
		m_item->setLine(QLineF(-extent, extent));
		m_shape->setExtent(extent);
		propagateUpdate();
	}
}

void Kolf::WallObject::setExtent(const QPointF& point1, const QPointF& point2)
{
	setPos((point2 + point1) / 2);
	setExtent((point2 - point1) / 2);
}

Kolf::Overlay* Kolf::WallObject::createOverlay()
{
	return new Kolf::WallOverlay(this);
}

#include "object-wall.moc"
