/***************************************************************************
 *   Copyright 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef KOLF_SCENE_H
#define KOLF_SCENE_H

#include <QGraphicsScene>

class b2World;

namespace Kolf
{
	class ContactListener;
	class Object;
	class ObjectFactory;

	/**
	 * \class Scene
	 * \since 2.0
	 *
	 * This QGraphicsScene subclass represents the physical world.
	 */
	class Scene : public QGraphicsScene
	{
		Q_OBJECT
		public:
			Scene(QObject* parent = 0);
			virtual ~Scene();

			///Returns whether the editing interfaces of the objects on this scene are active. These overlays are disabled by default.
			bool areOverlaysActive() const;
			///Returns whether the physics engine is active. The physics engine is disabled by default. This state can be toggled with the startSimulation() and stopSimulation() slots.
			bool isSimulationActive() const;
			///Returns a factory that can be used to create objects on this scene.
			Kolf::ObjectFactory* objectFactory() const;
			///Returns a list of all objects that are currently on this scene (and are subject to physics simulation).
			QList<Kolf::Object*> objects() const;
		public Q_SLOTS:
			void setOverlaysActive(bool overlaysActive);
			void startSimulation();
			void stopSimulation();
		protected:
			virtual void mousePressEvent(QGraphicsSceneMouseEvent* event);
			virtual void timerEvent(QTimerEvent* event);
		private Q_SLOTS:
			void addObject(Kolf::Object* object);
			void removeObject(Kolf::Object* object);
			void overlayStateChanged();
		private:
			b2World* m_world;
			Kolf::ContactListener* m_contactListener;
			Kolf::ObjectFactory* m_objectFactory;
			QList<Kolf::Object*> m_objects;
			int m_simulationTimerId;
			bool m_overlaysActive;
	};
}

#endif // KOLF_SCENE_H
