/***************************************************************************
 *   Copyright 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "shape-rectangle.h"

#include <Box2D/Collision/Shapes/b2PolygonShape.h>

Kolf::RectangleShape::RectangleShape(const QRectF& rect)
	: m_rect(rect)
{
	update();
}

QRectF Kolf::RectangleShape::rect() const
{
	return m_rect;
}

void Kolf::RectangleShape::setRect(const QRectF& rect)
{
	if (m_rect == rect)
		return;
	m_rect = rect;
	update();
}

b2Shape* Kolf::RectangleShape::createShape()
{
	b2PolygonShape* shape = new b2PolygonShape;
	shape->SetAsBox(
		m_rect.width() / 2, m_rect.height() / 2,
		b2Vec2(m_rect.center().x(), m_rect.center().y()),
		0 //intrinsic rotation angle
	);
	return shape;
}

void Kolf::RectangleShape::createOutlines()
{
	//interaction outline
	QPainterPath interactionOutline;
	interactionOutline.addRect(m_rect);
	setInteractionOutline(interactionOutline);
	//activation outline
	const QRectF rect = m_rect.adjusted(
		-Kolf::Shape::ActivationOutlinePadding,
		-Kolf::Shape::ActivationOutlinePadding,
		Kolf::Shape::ActivationOutlinePadding,
		Kolf::Shape::ActivationOutlinePadding
	);
	QPainterPath activationOutline;
	activationOutline.addRect(rect);
	setActivationOutline(activationOutline);
}
