/***************************************************************************
 *   Copyright 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "utils-bytearraycodec.h"

#include <KConfig>
#include <KConfigGroup>
#include <KStandardDirs>

static QString byteArrayCodecConfigName() {
	return KStandardDirs::locateLocal("data", "kolf/unitconverter.tmp");
}

K_GLOBAL_STATIC_WITH_ARGS(KConfig, byteArrayCodecConfig, (byteArrayCodecConfigName()))

QVariant Utils::ByteArrayCodec::decode(const QByteArray& value, const QVariant& defaultValue)
{
	//method 1: convert with plain QVariant methods
	QVariant newValue(value);
	if (newValue.convert(defaultValue.type()))
		return newValue;
	//method 2: convert by writing value into KConfig and reading it into a QVariant of the right type
	KConfigGroup cg(byteArrayCodecConfig, "Foo");
	cg.writeEntry("Bar", value);
	const QVariant result = cg.readEntry("Bar", defaultValue);
	cg.deleteEntry("Bar");
	return result;
}

QByteArray Utils::ByteArrayCodec::encode(const QVariant& value)
{
	//method 1: convert with plain QVariant methods
	QVariant newValue(value);
	if (newValue.convert(QVariant::ByteArray))
		return newValue.value<QByteArray>();
	//method 2: convert by writing value into KConfig and reading it as QByteArray
	KConfigGroup cg(byteArrayCodecConfig, "Foo");
	cg.writeEntry("Bar", value);
	const QByteArray result = cg.readEntry("Bar", QByteArray());
	cg.deleteEntry("Bar");
	return result;
}
