/***************************************************************************
 *   Copyright 2008, 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "player.h"
#include "../base/object-ball.h"
#include "../base/object-course.h"
#include "../base/objectfactory.h"
#include "../base/scene.h"
#include "../base/utils-listcasting.h"

Kolf::Player::Player(const QString& name, int hue)
	: m_hue(hue)
	, m_name(name)
{
}

Kolf::Player::~Player()
{
	delete m_ball;
}

Kolf::BallObject* Kolf::Player::ball(bool createIfNecessary) const
{
	if (!m_ball && m_scene && createIfNecessary)
	{
		//create a ball using the information contained in Kolf::CourseObject
		Kolf::CourseObject* courseObject = qlist_filter<Kolf::CourseObject*>(m_scene->objects()).value(0);
		if (courseObject)
			m_ball = courseObject->createBallObject();
		else
		{
			//If no course object is found, create a ball in the center of the scene.
			m_ball = qobject_cast<Kolf::BallObject*>(m_scene->objectFactory()->create("Kolf::BallObject"));
			m_ball->setPos(m_scene->sceneRect().center());
		}
		m_ball->setHue(m_hue);
		emit const_cast<Kolf::Player*>(this)->ballObjectCreated(m_ball.data());
	}
	return m_ball;
}

Kolf::Putter* Kolf::Player::putter() const
{
	return m_putter;
}

Kolf::Scene* Kolf::Player::scene() const
{
	return m_scene;
}

int Kolf::Player::hue() const
{
	return m_hue;
}

QString Kolf::Player::name() const
{
	return m_name;
}

void Kolf::Player::setScene(Kolf::Scene* scene, Kolf::Putter* putter)
{
	if (m_scene == scene)
		return;
	m_scene = scene;
	m_putter = putter;
	if (m_ball)
	{
		delete m_ball;
		m_ball = 0;
	}
}

#include "player.moc"
