/***************************************************************************
 *   Copyright 2009, 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef UTILS_TRANSPOSEPROXYMODEL_H
#define UTILS_TRANSPOSEPROXYMODEL_H

#include <QAbstractTableModel>

namespace Utils
{
	/**
	 * \class Kolf::TransposeProxyModel
	 * \since 2.0
	 *
	 * This model acts as a proxy model and changes the horizontal and vertical axis of a QAbstractTableModel. That means that, for instance, the model index (row = 23, column = 42) is mapped to (row = 42, column = 23). Header data is also mapped accordingly, as are any signals emitted by the source model.
	 */
	class TransposeProxyModel : public QAbstractTableModel
	{
		Q_OBJECT
		public:
			TransposeProxyModel();

			void setSourceModel(QAbstractTableModel* sourceModel);
			QAbstractTableModel* sourceModel() const;

			virtual int columnCount(const QModelIndex& parent = QModelIndex()) const;
			virtual int rowCount(const QModelIndex& parent = QModelIndex()) const;
			virtual Qt::ItemFlags flags(const QModelIndex& index) const;
			virtual QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const;
			virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
			virtual bool setData(const QModelIndex& index, const QVariant& value, int role = Qt::EditRole);
			virtual bool setHeaderData(int section, Qt::Orientation orientation, const QVariant& value, int role = Qt::EditRole);
		private Q_SLOTS:
			void slotColumnsAboutToBeInserted(const QModelIndex& parent, int start, int end);
			void slotColumnsAboutToBeRemoved(const QModelIndex& parent, int start, int end);
			void slotColumnsInserted(const QModelIndex& parent, int start, int end);
			void slotColumnsRemoved(const QModelIndex& parent, int start, int end);
			void slotDataChanged(const QModelIndex& topLeft, const QModelIndex& bottomRight);
			void slotHeaderDataChanged(Qt::Orientation orientation, int first, int last);
			void slotRowsAboutToBeInserted(const QModelIndex& parent, int start, int end);
			void slotRowsAboutToBeRemoved(const QModelIndex& parent, int start, int end);
			void slotRowsInserted(const QModelIndex& parent, int start, int end);
			void slotRowsRemoved(const QModelIndex& parent, int start, int end);
		private:
			QModelIndex mapFromSource(const QModelIndex& sourceIndex) const;
			QModelIndex mapToSource(const QModelIndex& proxyIndex) const;
			QAbstractTableModel* m_sourceModel;
	};
}

#endif // UTILS_TRANSPOSEPROXYMODEL_H
