/***************************************************************************
 *   Copyright 2010 Stefan Majewsky <majewsky@gmx.net>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "engine/course.h"
#include "engine/game-editor.h"
#include "engine/game-normal.h"
#include "interface/window.h"

#include <QDebug>
#include <KAboutData>
#include <KApplication>
#include <KCmdLineArgs>

int main(int argc, char** argv)
{
	KAboutData about("kolf2", "kolf", ki18n("<application>Kolf 2</application>"), "0.0.1", ki18n("KDE Minigolf Game"), KAboutData::License_GPL, ki18n("Copyright 2008 - 2010, the Kolf team"));
	about.addAuthor(ki18n("Stefan Majewsky"), KLocalizedString(), "majewsky@gmx.net", "http://majewsky.wordpress.com");

	KCmdLineArgs::init(argc, argv, &about);
	KApplication app;
	KGlobal::locale()->insertCatalog("libkdegames");

	Kolf::Window* window = new Kolf::Window;

	//load a sample course
	Kolf::Course* course = new Kolf::Course("test2");
	if (course->title().isEmpty() || course->holes().isEmpty())
	{
		qWarning("Error: Could not load course file. Did you `make install`?");
		return 0;
	}
#if 0
	Kolf::NormalGame* game = new Kolf::NormalGame(course, window->interface());
	game->addLocalPlayer("Konqi", 0);
	game->addLocalPlayer("Katie", 120);
#else
	//WARNING: This class is missing currently.
	Kolf::EditorGame* game = new Kolf::EditorGame(course, window->interface());
#endif
	window->setGame(game);

	window->show();
	return app.exec();
}
