#include "KGImageView.h"
#include "kolladev.h"

KGGraphicsNameItem::KGGraphicsNameItem(int argWidth, QString &argName, QGraphicsItem *parent) :
QGraphicsItem(parent)
{
    setData(ITEM_TYPE_KEY,ITEM_TYPE_NAME);
    width = argWidth;
    name = argName;
}
QRectF KGGraphicsNameItem::boundingRect() const
{
    return QRectF(0,0,width,TEXT_HEIGHT);
}
void KGGraphicsNameItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *, QWidget *)
{
    painter->drawText(0,0,width,TEXT_HEIGHT,Qt::AlignHCenter | Qt::AlignBottom,name);
}

KGGraphicsParentItem::KGGraphicsParentItem(int argWidth, int argHeight, QGraphicsItem *parent) :
QGraphicsItem(parent)
{
    setData(ITEM_TYPE_KEY,ITEM_TYPE_PARENT);
    width = argWidth;
    height = argHeight;
}
QRectF KGGraphicsParentItem::boundingRect() const{return QRectF(0,0,width,height);}
void KGGraphicsParentItem::paint(QPainter *, const QStyleOptionGraphicsItem *, QWidget *){}

KGGraphicsSelectionItem::KGGraphicsSelectionItem(int argWidth, int argHeight, QGraphicsItem *parent) :
QGraphicsItem(parent)
{
    width = argWidth;
    height = argHeight;
    setData(ITEM_TYPE_KEY,ITEM_TYPE_SELECTION);
}
void KGGraphicsSelectionItem::resize(int newWidth, int newHeight)
{
    width = newWidth;
    height = newHeight;
    prepareGeometryChange();
}
QRectF KGGraphicsSelectionItem::boundingRect() const
{
    return QRectF(0,0,width,height);
}
KGGraphicsSelectionItem::~KGGraphicsSelectionItem(){}
void KGGraphicsSelectionItem::paint(QPainter *painter, const QStyleOptionGraphicsItem *, QWidget *)
{
    QBrush brush(Qt::blue,Qt::Dense5Pattern);
    painter->fillRect(0,0,width,height,brush);
}

KGImageView::KGImageView(QWidget * parent, const char *) :
QGraphicsView(parent)
{
    maxImageWidth = 0;
    maxImageHeight = 0;
    setAlignment(Qt::AlignLeft | Qt::AlignTop);
    scene.setSceneRect(0, 0, 200, 1000);
    setScene(&scene);
    centerOn(0,0);
    itemSelected = false;
    selectionBox = new KGGraphicsSelectionItem(50,50);
    selectionBox->setPos(0,0);
    //scene.addItem(selectionBox);
    setMouseTracking (true);
}
KGImageView::~KGImageView()
{

}
void KGImageView::mousePressEvent(QMouseEvent *event)
{
    foreach(QGraphicsItem *graphicItem,items(event->pos()))
    {
	if(graphicItem->data(ITEM_TYPE_KEY) == ITEM_TYPE_PARENT)
	{
	    selectionBox->setParentItem(graphicItem);
	    selectionBox->setZValue(-1);
	    foreach(imageObject object,imageObjectList)
	    {
		if(object.parent == graphicItem)
		{
		    selectionChanged(object.image);
		    break;
		}
	    }
	    break;
	}
    }
}

void KGImageView::resizeEvent(QResizeEvent *)
{
    //TODO: This function needs a propery way to re-arange objects in a 
    // grid layout
    
    //this function uses improper methods in order to re-arange the boxes
    // in a grid layout. In addition, the function setSceneRect requires the 
    // height and width to be different in order for the scrollbars maximum value
    // and pageStep to be updated. So this functions alters the scene's height between 
    //1 and 2 pixels depending on the current height

    int imageAndNameWidth = maxImageWidth+(BORDER_SIZE*2);
    int imageAndNameHeight = maxImageHeight+(BORDER_SIZE*2)+TEXT_HEIGHT+5;
    
    int sceneWidth = width()/imageAndNameWidth*imageAndNameWidth;
    
    if(sceneWidth < imageAndNameWidth)
	sceneWidth = imageAndNameWidth;

    

    int sceneHeight = (imageObjectList.size()*(imageAndNameWidth*imageAndNameHeight)/sceneWidth-1)/(imageAndNameHeight)*(imageAndNameHeight)+(imageAndNameHeight);
    
    scene.setSceneRect(0, 0, width(), sceneHeight+(height()%2+1));

    int loopx = 0;
    int loopy = 0;
    
    foreach(imageObject object,imageObjectList)
    {
	object.parent->setPos(BORDER_SIZE/2+loopx,BORDER_SIZE/2+loopy);
	loopx+=imageAndNameWidth;
	if(loopx > width()-imageAndNameWidth)
	{
	    loopx = 0;
	    loopy += imageAndNameHeight;
	}
    }
}
void KGImageView::addItem(QGraphicsItem *item,QString name)
{
    //WARNING: this function is poorly designed and does not allow an item to be added multiple times
    if(!item)
	return;

    imageObject newObject;

    newObject.image = item;

    QRectF imageRect = item->boundingRect();

    if(imageRect.width() > maxImageWidth)
    {
	maxImageWidth = (int)imageRect.width();
	selectionBox->resize(maxImageWidth+BORDER_SIZE,maxImageHeight+TEXT_HEIGHT+5+BORDER_SIZE);
    }
    if(imageRect.height() > maxImageHeight)
    {
	maxImageHeight = (int)imageRect.height();
	selectionBox->resize(maxImageWidth+BORDER_SIZE,maxImageHeight+TEXT_HEIGHT+5+BORDER_SIZE);
    }
    
    int imageAndNameWidth = maxImageWidth+(BORDER_SIZE*2);
    int imageAndNameHeight = maxImageHeight+(BORDER_SIZE*2)+TEXT_HEIGHT+5;

    int sceneWidth = width()/imageAndNameWidth*imageAndNameWidth;

    if(sceneWidth < imageAndNameWidth)
	sceneWidth = imageAndNameWidth;
    
    int sceneHeight = (imageObjectList.size()*(imageAndNameWidth*imageAndNameHeight)/sceneWidth)/(imageAndNameHeight)*(imageAndNameHeight)+(imageAndNameHeight);

    scene.setSceneRect(0, 0, width(), sceneHeight);

    int imageX = (BORDER_SIZE/2)+(imageObjectList.size()*imageAndNameWidth) % (sceneWidth);
    int imageY = (BORDER_SIZE/2)+(imageObjectList.size()*imageAndNameWidth) / (sceneWidth) * imageAndNameHeight;

    newObject.parent = new KGGraphicsParentItem(imageAndNameWidth-BORDER_SIZE,imageAndNameHeight-BORDER_SIZE);
    newObject.parent->setPos(imageX,imageY);
    
    
    
    scene.addItem(newObject.parent);
    
    item->setParentItem(newObject.parent);
    item->setPos(BORDER_SIZE/2,BORDER_SIZE/2);
    item->setData(ITEM_TYPE_KEY, ITEM_TYPE_IMAGE);

    newObject.name = new KGGraphicsNameItem(maxImageWidth,name,newObject.parent);
    newObject.name->setPos(BORDER_SIZE/2,(BORDER_SIZE/2)+maxImageHeight+5);
    imageObjectList.push_back(newObject);

    if(!itemSelected)
    {
	itemSelected = true;
	selectionBox->setParentItem(newObject.parent);
    }    
}

#include "KGImageView.moc"
