#ifndef _H_KGNEWGAMEWIZARD_H_
#define _H_KGNEWGAMEWIZARD_H_

#include <QWizard>
#include <QWizardPage>
#include <QStringList>
#include <QLineEdit>
#include <KPageView>
#include <QStandardItemModel>

#include "KPageListViewDelegate.h"
#include "ui_newGamePage1.h"
#include "ui_newGamePage2.h"
#include "ui_newGamePage3.h"
#include "ui_newGamePage4.h"
#include "KGGameData.h"

#define GAMETYPE_ID 1 //0 is used to indicate no page
#define GAMESETTINGS_ID 2
#define GAMEINFO_ID 3
#define GAMESUMMARY_ID 4

/** 
    @author Keith Neuse<KNeuse@Gmail.com>
    this is the first page for the wizard
**/
class KGGameTypePage: public QWizardPage
{
    Ui::KGNewGamePage1Widget newGamePage1Widget;    
public:
    /**
	constructor
    **/
    KGGameTypePage();
};

/** 
    @author Keith Neuse<KNeuse@Gmail.com>
    this is the second page for the wizard
**/
class KGGameSettingsPage: public QWizardPage
{
    Q_OBJECT
    Ui::KGNewGamePage2Widget newGamePage2Widget;
public:
    KGGameSettingsPage();
    bool isRunningChecked();
    bool isJumpingChecked();
    bool isLeftChecked();
    bool isRightChecked();
    bool isUpChecked();
    bool isDownChecked();
    QString getFriction();
    QString getGravity();
protected slots:
    void enableTxtFriction(bool state);
    void enableTxtGravity(bool state);
};

/** 
    @author Keith Neuse<KNeuse@Gmail.com>
    this is the third page for the wizard
**/
class KGGameInfoPage: public QWizardPage
{
    Ui::KGNewGamePage3Widget newGamePage3Widget;
public:
    /**
	constructor
    **/
    KGGameInfoPage();

    /**
	gets the project name
	@return the project name
    **/
    QString getGameName();

    /**
	gets the intro text for the game
	@return intro text
    **/
    QString getIntroText();

    QLineEdit *getGameObject(void);
};

/** 
    @author Keith Neuse<KNeuse@Gmail.com>
    this is the last page for the wizard
**/
class KGGameSummaryPage: public QWizardPage
{
    Ui::KGNewGamePage4Widget newGamePage4Widget;
public:
    /**
	summary
    **/
    KGGameSummaryPage();

    /**
	sets the text for the summary object
	@PARAM summary = the summary to set
    **/
    void setSummary(QString summary);
    KUrlRequester *getLocation(void);
};

/** 
    @author Keith Neuse<KNeuse@Gmail.com>
    This object is the actually wizard
**/
class KGNewGameWizard: public QWizard 
{
    Q_OBJECT
    KGGameSummaryPage *summaryPage;
    KGGameInfoPage *infoPage;
    KGGameSettingsPage *settingsPage;
public:
    /**
	constructor
    **/
    KGNewGameWizard(QWidget *parent = 0);

    /**
	creates a gamedata object with all the information from the wizard
    **/
    KGGameData *getGameData(void);

    /**
	This function sets up the summary information before 
	the summary page is displayed
    **/
    void initializePage ( int id );
private slots:
    void validateFinishButton(void);
    void validateNextButton(void);
    void idChanged(int newID);
};

#endif
