#include "KGProject.h"

#include <KDebug>
#include <QHeaderView>
#include <QMessageBox>
#include "KGTabInterface.h"

//**************************************
//KGProject
//**************************************
KGProject::KGProject(QWidget *parent): 
QTreeView(parent)
{
    m_gameData = NULL;

    header()->hide();
    setSelectionMode(QAbstractItemView::SingleSelection);

    connect(this,SIGNAL(activated(const QModelIndex &)),this,SLOT(slotClicked(const QModelIndex &)));
}
//******************************************************************************
void KGProject::changeProjectName(QString newName)
    { m_gameData->setName(newName); }
//******************************************************************************
void KGProject::setGameData(KGGameData *gameData)
{ 
    if(m_gameData)
	delete m_gameData;
    m_gameData = gameData;
    setModel(m_gameData);
    connect(m_gameData,SIGNAL(itemRenamed(QString,QString,QString)),this,SLOT(renameItem(QString,QString, QString)));
}
void KGProject::renameItem(QString oldReference,QString newReference, QString newName)
    { emit itemRenamed(oldReference,newReference,newName); }
//******************************************************************************
KGTabInterface *KGProject::newLevel(void)
{
    clearSelection();
    return m_gameData->newLevel();
}
//******************************************************************************
KGTabInterface *KGProject::openLevel(QString reference)
    { return m_gameData->openLevel(reference); }
//******************************************************************************
//**************************************
//slots
//**************************************
void KGProject::closeEditor( QWidget * editor, QAbstractItemDelegate::EndEditHint hint )
{
    //prevents editor from closing until duplicate item is resolved
    if(!m_gameData->duplicateName())
	QTreeView::closeEditor(editor,hint);
    else
	QMessageBox::critical(this, tr("kolladev"), tr("This name already exist or is invalid. Please select another name."), QMessageBox::Ok);
}
//******************************************************************************
void KGProject::setSelection ( const QRect & rect, QItemSelectionModel::SelectionFlags command )
{
    if(m_gameData->isItemSelected())
	emit tabSelected(m_gameData->itemSelectedReference());
    QTreeView::setSelection(rect,command);
}
//******************************************************************************
void KGProject::slotClicked(const QModelIndex &index)
{
    if(m_gameData->isItemSelected())
	edit(m_gameData->selectedItem()->index());
}
//******************************************************************************
void KGProject::selectionChanged ( const QItemSelection & selected, const QItemSelection & deselected )
{
    m_gameData->selectionChanged(selected);
    if(m_gameData->isItemSelected())
	emit tabSelected(m_gameData->itemSelectedReference());
    QTreeView::selectionChanged(selected,deselected);
}
//**************************************
//events
//**************************************
void KGProject::mousePressEvent ( QMouseEvent * event )
{
    //disable mouse input until duplicate item resolved
    if(!m_gameData->duplicateName())
	QTreeView::mousePressEvent(event);
}
//******************************************************************************
void KGProject::keyPressEvent ( QKeyEvent * event )
{
    //disable keyboard input until duplicate item resolved
    if(!m_gameData->duplicateName())
	QTreeView::keyPressEvent(event);
}

#include "KGProject.moc"
