#include "KGSelectionInterface.h"
#include <KDebug>

//******************************************************************************
KGSelectionInterface::KGSelectionInterface()
{ 
    KIcon icon("draw-eraser");
    QPixmap eraser = icon.pixmap(TILE_SIZE,TILE_SIZE);
    eraserObject = new KGEraser(eraser);

    m_container = new KGContainer;
    m_container->preLoadAll(); 
    m_selectedObject = m_container->findFirstObject();
    m_workingLayerID = m_container->layerIDFromObject(m_selectedObject);
    updateLayerData(m_selectedObject,m_workingLayerID);

    m_eraserState = false;
}
//******************************************************************************
KGSelectionInterface::~KGSelectionInterface()
    { delete m_container; }
//******************************************************************************
KGContainer *KGSelectionInterface::container(void)
    { return m_container; }
//******************************************************************************
void KGSelectionInterface::setSelectedObject(KGObject *object)
{ 
    m_selectedObject = object;
    m_workingLayerID = m_container->layerIDFromObject(m_selectedObject);
    emit eraserModeChanged(false);
    emit currentObjectChanged(m_selectedObject, m_workingLayerID);
    emit layerIDChanged(m_workingLayerID);
    updateLayerData(m_selectedObject, m_workingLayerID);
}
//******************************************************************************
void KGSelectionInterface::updateLayerData(KGObject *selectedObject, int layerID)
{
    foreach(KGLayerData selectedLayer, m_layerDataList)
    {
	if(layerID == selectedLayer.layerID)
	{
	    selectedLayer.selectedObject = selectedObject;
	    return;
	}
    }
    KGLayerData layerData;
    layerData.selectedObject = selectedObject;
    layerData.layerID = layerID;
    m_layerDataList.push_back(layerData);
}
//******************************************************************************
KGObject *KGSelectionInterface::selectedObject()
    { return m_selectedObject; }
//******************************************************************************
KGObject *KGSelectionInterface::selectedObjectFromLayer(int layerID)
{
    foreach(KGLayerData selectedLayer, m_layerDataList)
    {
	if(layerID == selectedLayer.layerID)
	    return selectedLayer.selectedObject;
    }
    return NULL;
}
//******************************************************************************
K_GLOBAL_STATIC(KGSelectionInterface,g_instance)

KGSelectionInterface *KGSelectionInterface::instance()
    { return g_instance; }
//******************************************************************************
bool KGSelectionInterface::eraserMode(void)
    { return m_eraserState; }
//******************************************************************************
bool KGSelectionInterface::layerID(void)
    { return m_workingLayerID; }
//******************************************************************************
void KGSelectionInterface::setEraserMode(bool state)
{
    m_eraserState = state;
    if(state)
	emit currentObjectChanged(eraserObject,m_workingLayerID);
    else
	emit currentObjectChanged(selectedObjectFromLayer(m_workingLayerID),m_workingLayerID);
    emit eraserModeChanged(state);
}
//******************************************************************************
void KGSelectionInterface::setlayerID(int newLayerID)
{
    m_workingLayerID = newLayerID;
    emit layerIDChanged(m_workingLayerID); 
    if(!m_eraserState)
	emit currentObjectChanged(selectedObjectFromLayer(m_workingLayerID),m_workingLayerID);
}

#include "KGSelectionInterface.moc"
