#ifndef _KGCONTAINER_H_
#define _KGCONTAINER_H_

#include "KGObject.h"
#include "KGTile.h"
#include <QString>
#include <QDomDocument>
#include <QList>
#include <QFile>

#define INVALID_LAYER 0
#define BACKGROUND_LAYER 1
#define TILE_LAYER 2
#define OBJECT_LAYER 3

#define CATEGORY_TAG_NAME "category"
#define SYSTEM_FILE_NAME "defaults/defaultObjects.xml"

class KGContainer
{
public:
    KGContainer(KGContainer *parent = 0, int layerID = INVALID_LAYER);
    virtual ~KGContainer();
    KGObject *findObject(QString reference);
    KGContainer *findContainer(QString reference);
    void preLoadAll(void);
    void unloadAll(void);
    void addObject(KGObject *object);
    void addContainer(KGContainer *);

    QString reference(bool fullPath = false);
    void setReference(QString reference);

    QList<KGObject *> objects();
    QList<KGContainer *> containers();

    static KGObject *createObjectFromType(QString name);
    int layerID(void);
    int layerIDFromObject(KGObject *object);
    KGObject *findFirstObject();
private:
    KGObject *recursive_findFirstObject(KGContainer *parentContainer);
    QList<KGContainer *> containerList;
    QList<KGObject *> objectList;
    int m_layerID;
    KGContainer *m_parent;
    QString m_reference;

    void createContainer(QString name, int layerID);
    bool isCategory(QString name);
    void recursive_preLoadAllObjects(QDomElement rootElement,KGContainer *parentContainer, QString reference = QString());
    QDomElement findElementByAttribute(QDomElement parentElement, QString tagName, QString attributeName, QString attributeValue);
    QDomElement openSystemDefaults();
    KGObject *recursive_loadContainer(QDomElement rootElement, QString reference);
    KGContainer *loadContainerFromDefaults(KGContainer *container, QDomElement elementItem);
    KGObject *loadObjectFromDefaults(KGContainer *parentContainer, QDomElement elementItem, QString reference);    
};

#endif
