/***************************************************************************
 *   Copyright (C) 2007 by Michal Sciubidlo <michal.sciubidlo@gmail.com>   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "aiplayer.h"

#include <QList>
#include <QVector>

#include <kdebug.h>
#include "spectator.h"
#include "player.h"
#include "protocol.h"
#include "tile.h"
#include "tilebox.h"

//TODO: AI have to include final points deletion

aiPlayer::aiPlayer(const QString &nick, board *gameBoard, tileBox *tileBox) : player()
{
	spectator::m_nick = nick;
	m_board = gameBoard;
	m_tileBox = tileBox;
	m_ready = true;
}

aiPlayer::~aiPlayer()
{
	if (m_board) delete m_board;
	if (m_tileBox) delete m_tileBox;
}

bool aiPlayer::ready() const
{
	return true;
}

void aiPlayer::tellIfReady()
{
	emit readyChanged(this);
}

void aiPlayer::startGame(const QList<tile*> &tiles)
{
	m_board->reset();
	m_phase = PLAYING_GAME;
	m_tiles = tiles;
	m_tileBox->removeTiles(tiles);
}

void aiPlayer::sendNewTiles(const QList<tile*> &tiles)
{
	m_tiles << tiles;
	m_tileBox->removeTiles(tiles);
}

void aiPlayer::nextTurn(const QString &nick)
{
	m_phase = PLAYING_GAME;
	m_myturn = nick == m_nick;
	m_currnick = nick;

	if (!m_myturn) return;

	//qDeleteAll(m_tilesIHaveUsedInThisTurn);
	m_tilesIHaveUsedInThisTurn.clear();

	/*if ((playersList.size() == 2) && (m_tileBox.size() < 7))
	{
		//m_tileBox = other player tiles
	}
	else
	{
		//TODO:
	}*/

	QList<QList<tilePosition*> *> *allMoves = m_board->getPossibleMoves(m_tiles);
	kDebug() << "Possible words count: " << allMoves->size() << endl;
	if (allMoves->size() == 0)
	{
		//TODO: make it better to chose from pass turn and exchande tiles
		//TODO: on exchange tiles add them to box
		emit passTurn(this);
	}
	else
	{
		int maxPoints = 0;
		int maxWordIndex = 0;

		for (int i = 0; i < allMoves->size(); i++)
		{
			int temp = m_board->pointsForMove(*(*allMoves)[i]);
			if (temp > maxPoints)
			{
				maxPoints = temp;
				maxWordIndex = i;
			}
		}
		tilePositionList sendList;

		for (int i = 0; i < (*allMoves)[maxWordIndex]->size(); i++)
		{
			int index;
			if (m_board->isLocked((*(*allMoves)[maxWordIndex])[i]->x(), (*(*allMoves)[maxWordIndex])[i]->y()) == true )
				continue;
			
			//sendList << allMoves[maxWordIndex][i];

			for (int j = 0; j < m_tiles.size(); j++)
			{
				//found blank tile in rack
				if (m_tiles[j]->isBlank() && (*(*allMoves)[maxWordIndex])[i]->ourTile()->isBlank())
				{
					m_tiles[j]->setString((*(*allMoves)[maxWordIndex])[i]->ourTile()->string());

					sendList.append(tilePosition(m_tiles[j], (*(*allMoves)[maxWordIndex])[i]->x(), (*(*allMoves)[maxWordIndex])[i]->y()));
					m_tilesIHaveUsedInThisTurn.append(m_tiles.takeAt(j));
					break;
				}
				if (QString::compare(m_tiles[j]->string(), (*(*allMoves)[maxWordIndex])[i]->ourTile()->string()) == 0 )
				{
					sendList.append(tilePosition(m_tiles[j], (*(*allMoves)[maxWordIndex])[i]->x(), (*(*allMoves)[maxWordIndex])[i]->y()));
					m_tilesIHaveUsedInThisTurn.append(m_tiles.takeAt(j));
					break;
				}
			}
		}
		emit gotTiles((player *)this, sendList); //allMoves[maxWordIndex]); //sendList);  //allMoves[maxWordIndex]);
		//TODO: fill m_tilesIHaveUsedInThisTurn and emit gotTiles
	}

	while (!allMoves->isEmpty())
	{
		QList<tilePosition*> *tp = allMoves->at(0);
		while(!tp->isEmpty())
		{
			delete tp->at(0)->ourTile();
			delete tp->at(0);
			tp->removeFirst();
		}
		delete tp;
		allMoves->removeFirst();
	}
	delete allMoves;
}

void aiPlayer::sendExchangedTiles(const QList<tile*> &tiles)
{
	m_tiles << tiles;
	m_tileBox->removeTiles(tiles);
}

void aiPlayer::sendTiles(const tilePositionList &tiles)
{
	kDebug() << "AIPlayer::sendTiles, tiles count: "<< tiles.count() << endl;
	/*QList<tilePosition> *copy = new QList<tilePosition>();
	for (int i=0; i < tiles.size(); i++)
	{
		tile* tempTile = new tile(tiles[i].ourTile()->string());
		tempTile->setScore( tiles[i].ourTile()->score() );
		tempTile->setBlank( tiles[i].ourTile()->isBlank() );
		tilePosition* tempPosition = new tilePosition(tempTile, tiles[i].x(), tiles[i].y());
		copy->append(*tempPosition);
	}
	m_board->setPlayedTiles(*copy);*/
	
	m_board->setPlayedTiles(tiles);

}

void aiPlayer::correctWord(int score)
{
	m_phase = PLAYING_GAME;

	//points counting
	for(int i=0; i<playersList.size(); i++)
		if (playersList[i].first == m_currnick)
			playersList[i].second += score;

	//delete used tiles from AI bag but not this that AI have
	//AI tiles are deleted when placed on rack (startGame(), sendNewTiles(), sendExchangedTiles())
	if (m_currnick != m_nick)
		m_tileBox->removeTiles(m_board->getPlayedTiles().tiles());

	m_board->finalizePlayedTiles(ACCEPT);
}

void aiPlayer::unknownWords()
{
	m_votingState = VOTING_REJECTED;
	emit votingChanged();
}

void aiPlayer::rejectVotedTiles(const QString &nick)
{
	if (nick == m_nick)
	{
		m_phase = PLAYING_GAME;
		m_tiles << m_tilesIHaveUsedInThisTurn;
		m_tilesIHaveUsedInThisTurn.clear();
	}
	else
		m_board->finalizePlayedTiles(DELETE);
}

void aiPlayer::usesExclusiveTiles(const QString &nick)
{
	kDebug() << "AI usesExclusiveTiles" << endl;
	if (nick == m_nick)
	{
		m_phase = PLAYING_GAME;
		m_tiles << m_tilesIHaveUsedInThisTurn;
		m_tilesIHaveUsedInThisTurn.clear();
	}
	else
		m_board->finalizePlayedTiles(DELETE);
}

void aiPlayer::processNewMessage(const message &m)
{
	//never used
}


void aiPlayer::acceptedInGame(const QList< QPair<QString, int> > &players, const QString &language)
{
	for(int i=0; i<players.size(); i++)
		playersList << QPair<QString, int>(players[i].first, 0);
	playersList << QPair<QString, int>(m_nick, 0);
}

void aiPlayer::addPlayer(const QString &nick, bool isPlayer)
{
	if (isPlayer)
		playersList << QPair<QString, int>(nick, 0);
}

void aiPlayer::removePlayer(const QString &nick)
{
	for(int i=0; i<playersList.size(); i++)
	{
		if (playersList[i].first == nick)
		{
			playersList.removeAt(i);
			break;
		}
	}
}

//needed to take away from spectator
void aiPlayer::exchangedTiles(const QString &nick, int nTiles)
{
}
void aiPlayer::addPlayer(const QString &nick)
{
}
void aiPlayer::gameEnded(const QString &finisher, const QString &winner)
{
	m_phase = WAITING_TO_START_GAME;
	m_score = 0;
	m_myturn = false;
	m_ready = true;
	qDeleteAll(m_tilesIHaveUsedInThisTurn);
	m_tilesIHaveUsedInThisTurn.clear();
	qDeleteAll(m_tiles);
	m_tiles.clear();
}

void aiPlayer::passedTurn(const QString &nick)
{
}

void aiPlayer::sendStatus(const QString &nick, int which)
{
}

#include "aiplayer.moc"
