/***************************************************************************
 *   Copyright (C) 2007 by Michal Sciubidlo <michal.sciubidlo@gmail.com>   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef AIPLAYER_H 
#define AIPLAYER_H 

#include <QList>

#include "player.h"
#include "tileposition.h"
#include "board.h"

class tileBox;
class tile;

/**
 * This is the representation of a AI on the server 
 */ 

class aiPlayer : public player
{
Q_OBJECT
		
	public:
		aiPlayer(const QString &nick, board *gameBoard, tileBox *tileBox);
		~aiPlayer();
		
		bool ready() const;

		void startGame(const QList<tile *> &tiles);
		void sendNewTiles(const QList<tile *> &tiles);
		void nextTurn(const QString &nick);
		void sendTiles(const tilePositionList &tiles);
		void correctWord(int score);
		void unknownWords();
		void rejectVotedTiles(const QString &nick);
		void usesExclusiveTiles(const QString &nick);
		void sendExchangedTiles(const QList<tile*> &tiles);
		void tellIfReady();
		void addPlayer(const QString &nick, bool isPlayer);
		void addPlayer(const QString &nick);

		void exchangedTiles(const QString &nick, int nTiles);
		void acceptedInGame(const QList< QPair<QString, int> > &players, const QString &language);
		void passedTurn(const QString &nick);
		void gameEnded(const QString &finisher, const QString &winner);
		void sendStatus(const QString &nick, int which);
		void removePlayer(const QString &nick);
	signals:
		void readyChanged(player *p);
		/*sends move to server*/
		void gotTiles(player *p, const tilePositionList &tiles);
		void votingChanged();
		void passTurn(player *p);
		void exchangeTiles(player *p, const QList<tile*> &tiles);

	protected slots:
		void processNewMessage(const message &m);
	private:
		/*!
		 * 
		 */
		void endGameMove();
		board * m_board;
		tileBox * m_tileBox;
		QList<QPair<QString, int> > playersList;
		QString m_currnick;
};

#endif
