/***************************************************************************
 *   Copyright (C) 2005 by Albert Astals Cid <tsdgeos@terra.es>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef CHATWIDGET_H 
#define CHATWIDGET_H 

#include "ui_chatwidgetbase.h"

#include <QWidget>
#include <kombination_export.h>

class KOMBINATION_EXPORT chatWidget : public QWidget, private Ui_chatWidgetBase
{
Q_OBJECT
	public:
		explicit chatWidget(QWidget *parent);

		void setEnabled(bool enable);
		void setIAm(const QString &nick);

		const QStringList &ignoredNicks() const;
		void setIgnoredNicks(const QStringList &nicks);
		
		void addScoreCurrentPlayer(const QStringList &wordList, int score);
	
	signals:
		void typedMessage(const QString &message);

	public slots:
		void connected(const QString &languageCode);
		void addPlayer(const QString &nick, int which, bool isJoiningGame);
		void addMessage(const QString &nick, const QString &message);
		void startGame();
		void nextPlayer(const QString &nick, bool isItMe);
		void gameEnded(const QString &nickFinisher, const QString &nickWinner);
		void removePlayer(const QString &nick, const QString &game);
		void serverDown();
		void tilesRejected(int reason, const QString &nick);
		void someonePassedHisTurn(const QString &nick);
		void someoneExchangedHisTiles(const QString &nick, int nTiles);

	private slots:
		void sendMessage();
		void textEditChanged(const QString &text);

	private:
		void addMessageWithTime(const QString &text);

		QString m_nick;
		QString m_currentPlayer;
		bool m_ready;

		QStringList m_ignoredPlayers;
};

#endif
