/***************************************************************************
 *   Copyright (C) 2005 by Albert Astals Cid <tsdgeos@terra.es>            *
 *   Copyright (C) 2005 by Pino Toscano      <toscano.pino@tiscali.it>     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "miniprogressbar.h"

#include <QBrush>
#include <QColor>
#include <QPainter>
#include <QPen>
#include <QPoint>
#include <QRect>
#include <QSizePolicy>

miniProgressBar::miniProgressBar(Qt::Orientation orientation, QWidget *parent) : QFrame(parent)
{
	setMinimumSize(15, 15);
	setFrameStyle(NoFrame);
	setFrameShadow(Plain);
	m_orientation = orientation;
	m_progress = 0.50;
	update();
}

void miniProgressBar::setProgress(double progress)
{
	if ((progress < 0.0) || (progress > 100.0)) return;

	m_progress = progress / 100.0;
	update();
}

double miniProgressBar::progress() const
{
	return m_progress * 100.0;
}

void miniProgressBar::setOrientation(Qt::Orientation orientation)
{
	if (m_orientation != orientation)
	{
		m_orientation = orientation;
		update();
	}
}

Qt::Orientation miniProgressBar::orientation() const
{
	return m_orientation;
}

void miniProgressBar::paintEvent(QPaintEvent *event)
{
	Q_UNUSED(event);
	static int padding = 3;

	QPainter paint(this);

	QColor border = Qt::black;
	QColor innercolor = Qt::blue;

	paint.setPen(border);
	paint.setBrush(QBrush(Qt::NoBrush));
	QRect tmp = rect();
	tmp.setWidth(tmp.width() - 1);
	tmp.setHeight(tmp.height() - 1);
	paint.drawRect(tmp);

	paint.setPen(innercolor);
	QRect barrect = rect();
	barrect.translate(padding, padding);
	barrect.setWidth(barrect.width() - 2 * padding);
	barrect.setHeight(barrect.height() - 2 * padding);
	if (m_orientation == Qt::Horizontal)
	{
		barrect.setWidth(static_cast<int>(barrect.width() * m_progress));
	}
	else
	{
		barrect.setTop(barrect.top() + static_cast<int>(barrect.height() * (1 - m_progress)));
	}
	paint.fillRect(barrect, QBrush(innercolor));

	paint.end();
}

#include "miniprogressbar.moc"
