/***************************************************************************
 *   Copyright (C) 2005 by Albert Astals Cid <tsdgeos@terra.es>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "multiwidget.h"
#include "confirmwidget.h"
#include "protocol.h"

#include <QFont>
#include <QVBoxLayout>

#include <kdebug.h>

multiWidget::multiWidget(QWidget *parent) : QWidget(parent), m_ready(false), m_currentPlayer(0)
{
	setupUi(this);
	QStringList headers;
	headers << i18n("Name") << i18n("Score") << i18n("Status");
	treeWidget->setHeaderLabels(headers);
	setMinimumSize(100, 100);

	QWidget *foo = new QWidget(stackedWidget);
	stackedWidget->addWidget(foo);
	QVBoxLayout *swlay = new QVBoxLayout(foo);
	swlay->setMargin(0);
	m_confirmWidget = new confirmWidget(foo);
	m_confirmWidget->setTimeout(15);
	m_confirmWidget->show();
	swlay->addWidget(m_confirmWidget);

	connect(m_confirmWidget, SIGNAL(accepted()), this, SIGNAL(confirmAccepted()));
	connect(m_confirmWidget, SIGNAL(rejected()), this, SIGNAL(confirmRejected()));
	connect(m_confirmWidget, SIGNAL(accepted()), this, SLOT(restoreList()));
	connect(m_confirmWidget, SIGNAL(rejected()), this, SLOT(restoreList()));

	stackedWidget->setContentsMargins(0, 0, 0, 0);
	stackedWidget->setCurrentIndex(0);
}

void multiWidget::clearPlayerList()
{
	treeWidget->clear();
}

void multiWidget::setIAm(const QString &nick)
{
	m_nick = nick;
}

void multiWidget::setMyselfReady(bool ready)
{
	changeStatus(m_nick, ready ? STATUS_PLAYER_READY : STATUS_PLAYER_NOT_READY);
}

QString multiWidget::currentPlayerNick() const
{
	return m_currentPlayer->text(0);
}

QStringList multiWidget::getPlayersNicks( getPlayersNicksOption o )
{
	QStringList res;
	for (int i = 0; i < treeWidget->topLevelItemCount(); i++)
	{
		res << treeWidget->topLevelItem(i)->text(0);
	}
	if (o == DoNotGiveMyOwnNick) res.removeAll(m_nick);
	return res;
}

void multiWidget::addPlayer(const QString &nick, int which)
{
	QTreeWidgetItem *player = new QTreeWidgetItem(treeWidget);
	player->setText(0, nick);
	player->setText(1, "0");
	if (which == STATUS_PLAYER_READY) player->setText(2, i18n("Ready"));
	else if (which == STATUS_SPECTATOR) player->setText(2, i18n("Spectator"));
	else player->setText(2, i18n("Not Ready"));
	if (nick == m_nick)
	{
		QFont f = player->font(0);
		f.setBold(true);
		player->setFont(0, f);
	}
}

void multiWidget::changeStatus(const QString &nick, int which)
{
	if (which == STATUS_PLAYER_NOT_READY)
	{
		QList<QTreeWidgetItem*> items = treeWidget->findItems(nick, Qt::MatchExactly);
		if (items.size() == 1)
		{
			items[0]->setText(2, i18n("Not Ready"));
		}
	}
	else if (which == STATUS_PLAYER_READY)
	{
		QList<QTreeWidgetItem*> items = treeWidget->findItems(nick, Qt::MatchExactly);
		if (items.size() == 1)
		{
			items[0]->setText(2, i18n("Ready"));
		}
	}
	else if (which == STATUS_GAME_STARTED)
	{
		QList<QTreeWidgetItem*> items = treeWidget->findItems(".*", Qt::MatchRegExp | Qt::MatchRecursive);
		for (int i = 0; i < items.count(); i++)
		{
			if (items[i]->text(2) != i18n("Spectator"))
				items[i]->setText(2, i18n("Game started..."));
		}
	}
	else if (which == STATUS_PLAYER_PLACING_TILES)
	{
		QList<QTreeWidgetItem*> items = treeWidget->findItems(".*", Qt::MatchRegExp | Qt::MatchRecursive);
		for (int i = 0; i < items.count(); i++)
		{
			if (items[i]->text(2) != i18n("Spectator")) 
				items[i]->setText(2, items.at(i)->text(0) == nick ? i18n("Placing tiles") : i18n("Waiting"));
		}
	}
}

void multiWidget::startGame()
{
	changeStatus(QString(), STATUS_GAME_STARTED);
	for (int i = 0; i < treeWidget->topLevelItemCount(); i++)
	{
		treeWidget->topLevelItem(i)->setText(1, QString("0"));
	}
}

void multiWidget::nextPlayer(const QString &nick, bool isItMe)
{
	m_confirmWidget->setNick(nick);
	changeStatus(nick, STATUS_PLAYER_PLACING_TILES);
	m_currentPlayer = treeWidget->findItems(nick, Qt::MatchExactly).at(0);
}

void multiWidget::showAndStartTimeout()
{
	stackedWidget->setCurrentIndex(1);
	m_confirmWidget->startTimeout();
}

void multiWidget::restoreList()
{
	m_confirmWidget->stopTimeout();
	stackedWidget->setCurrentIndex(0);
}

void multiWidget::removePlayer(const QString &nick)
{
	QList<QTreeWidgetItem*> items = treeWidget->findItems(nick, Qt::MatchExactly, 0);
	delete items.at(0);
}

void multiWidget::serverDown()
{
	treeWidget->clear();
}

void multiWidget::addScoreCurrentPlayer(const QStringList &wordList, int score)
{
	const QString &nick = currentPlayerNick();
	m_currentPlayer->setText(1, QString::number(m_currentPlayer->text(1).toInt() + score));
}

#include "multiwidget.moc"
