/***************************************************************************
 *   Copyright (C) 2005 by Albert Astals Cid <tsdgeos@terra.es>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef PROTOCOL_H
#define PROTOCOL_H


// This file describes how the communication between
// the server and the client works

// First the client wants to connect to the server, then the server says
static const int VERSION = 1000;
// if the client does not support the protocol VERSION it will close the connection
// else the client answers if he wants to connect only as spectator or not
static const int CONNECT_AS_PLAYER = 0;
static const int CONNECT_AS_SPECTATOR = 1;
// and the server answers
static const int TOO_MANY_PLAYERS = 0;
// or
static const int SEND_NICK = 1;



// If the server said SEND_NICK the client will answer 
// with SEND_NICK followed by its nick
// and the server will say
static const int NICK_IN_USE = 2;
// if the nick is already taken
static const int SERVER_DOESNT_ALLOW_SPECTATORS = 3;
// if the server doesn't allow any spectator to a game
static const int ACCEPTED_INTO_GAME = 4;
// after ACCEPTED_INTO_GAME the client will receive the number of players 
// that there were already playing the game and then their nicks
// also the client will receive the languageCode in which the game is


// when the user has been accepted into the game it can send to the server
static const int TOGGLE_READY = 5;
static const int MESSAGE = 6;
// in case of MESSAGE it will be followed by the message
static const int STATUS = 7;
// in case of MESSAGE it will be followed by the message


// and the server can send to the user
static const int STARTING_GAME = 8;
// or MESSAGE
static const int NEW_PLAYER = 9;
static const int REMOVE_PLAYER = 10;
// in case of STARTING_GAME it will be followed by the seven initial tiles
// for that player
// in case of NEW_PLAYER it will be followed by the player nick
// in case of REMOVE_PLAYER it will be followed by the player nick


// Once the game has started the server will say
static const int NEXT_TURN = 11;
// or MESSAGE
// or SENDING_TILES - see below for more info
static const int PASSED_TURN = 12;
static const int EXCHANGED_TILES = 13;
static const int GAME_ENDED = 14;
// or REMOVE_PLAYER
// in case of NEXT_TURN it will be followed by the nick name of who plays
// in case of PASSED_TURN it will be followed by the nick name of who passed
// in case of EXCHANGED_TILES it will be followed by the nick name of who passed
// and the number of tiles he exchanged
// in case of GAME_ENDED it will be followed by the player nick that ended
// the game (emptied his deck) and by the name of the winner


// Once the game has started the users say to the server
// MESSAGE
static const int PASS_TURN = 15;
static const int EXCHANGE_TILES = 16;
static const int SENDING_TILES = 17;
// SENDING_TILES means a user has placed the word and is sending it to be
// validated by the server / users, it will be followed by the number of tiles
// the word has and the tiles themselves
static const int NEW_TILES = 18;
// NEW_TILES means that the server is sending us new tiles to add in our tile
// deck, it's followed by the number of tiles sent and then by the tiles
// themselves
static const int NEW_EXCHANGED_TILES = 19;
// NEW_EXCHANGED_TILES means that the server is sending us new tiles to add in our tile
// deck, it's followed by the number of tiles sent and then by the tiles
// themselves

// --- The trading phase --- //
// The trading phase begins when the user sends his SENDING_TILES to the server
// The server will then send the other users the SENDING_TILES followed either by
static const int CORRECT_WORDS = 101;
// or
static const int UNKNOWN_WORDS = 102;
// or
static const int INCORRECT_WORDS = 103;
// in case of CORRECT_WORDS the server will then send the score that words made and NEXT_TURN
// in case of INCORRECT_WORDS the server will then send the reason of incorrectness
static const int USES_EXCLUSIVE_TILES = 301;
// or
static const int LOST_VOTING = 302;
// in case of UNKNOWN_WORDS the clients have to answer with
static const int ACCEPT_WORDS = 104;
// or
static const int REJECT_WORDS = 105;
// the server will sum up the votations and will answer with
// CORRECT_WORDS followed by NEXT_TURN
// or
// INCORRECT_WORDS followed by LOST_VOTING and NEXT_TURN


// --- Statuses --- //
// a player is not ready
static const int STATUS_PLAYER_NOT_READY = 201;
// followed by the player name
// a player is ready
static const int STATUS_PLAYER_READY = 202;
// followed by the player name
// the game is started (but no new turn signal yet!)
static const int STATUS_GAME_STARTED = 203;
// a player is placing his/her tiles
static const int STATUS_PLAYER_PLACING_TILES = 204;
// followed by the player name
static const int STATUS_SPECTATOR = 205;


// BREAKING EVERYTHING!!!
static const int SERVER_IS_A_MULTISERVER = 1001;
static const int SERVER_IS_A_SINGLESERVER = 1002;
static const int ACCEPTED_INTO_SERVER = 1003;
static const int NEW_GAME = 1004;
static const int JOIN_GAME_AS_SPECTATOR = 1005;
static const int JOIN_GAME_AS_PLAYER = 1006;
static const int CREATE_GAME = 1007; // client to server
static const int OTHER_PLAYER_ENTERED_A_GAME = 1008;
#endif
