/***************************************************************************
 *   Copyright (C) 2005 by Albert Astals Cid <tsdgeos@terra.es>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "square.h"
#include "tile.h"
#include <kdebug.h>

square::square() : m_kind(NORMAL), m_tile(0), m_locked(false), m_state(EMPTY)
{
	m_nearPoints[HORIZONTAL] = 0;
	m_nearPoints[VERTICAL] = 0;
	possibleLetters[HORIZONTAL].resize(MAX_LETTERS_NUM);
	possibleLetters[HORIZONTAL].fill(true);
	possibleLetters[VERTICAL].resize(MAX_LETTERS_NUM);
	possibleLetters[VERTICAL].fill(true);
}

square::~square()
{
	delete m_tile;
}

void square::setKind(Kind k)
{
	m_kind = k;
}

square::Kind square::kind() const
{
	return m_kind;
}

void square::setTile(tile *t)
{
	//kDebug() << "square::setTile" << endl;
	m_tile = t;
}

tile* square::ourTile()
{
	return m_tile;
}

const tile* square::ourTile() const
{
	return m_tile;
}

void square::setLocked(bool lock)
{
	m_locked = lock;
}

bool square::isLocked() const
{
	return m_locked;
}

void square::setToAnchor()
{
	if (m_state == EMPTY)
		m_state = ANCHOR;
}

void square::setToTile()
{
	m_state = TILE;
}

int square::horizontalPoints() const
{
	return m_nearPoints[HORIZONTAL];
}

int square::verticalPoints() const
{
	return m_nearPoints[VERTICAL];
}

void square::addHorizontalPoints(int points)
{
	m_nearPoints[HORIZONTAL] += points;
}

void square::addVerticalPoints(int points)
{
	m_nearPoints[VERTICAL] += points;
}

square::State square::state() const
{
	return m_state;
}

bool square::isAnchor() const
{
	return (m_state == square::ANCHOR);
}

void square::turnOnCrossLetter(int index, Orientation orientation)
{
	possibleLetters[orientation].setBit(index);
}

void square::turnOffCrossLetter(int index, Orientation orientation)
{
	possibleLetters[orientation].clearBit(index);
}

bool square::crossLetterState(int index, Orientation orientation)
{
	//HACK: in square possibleLetters[0] = possible blank and 1=A
	// but is used from board code where 0=A so add 1 to index provided by board
	return possibleLetters[orientation].at(index+1);
}

void square::reset()
{
	m_tile = 0;
	m_locked = false;
	m_state = EMPTY;

	m_nearPoints[HORIZONTAL] = 0;
	m_nearPoints[VERTICAL] = 0;
	//possibleLetters[HORIZONTAL].resize(MAX_LETTERS_NUM);
	possibleLetters[HORIZONTAL].fill(true);
	//possibleLetters[VERTICAL].resize(MAX_LETTERS_NUM);
	possibleLetters[VERTICAL].fill(true);
}
