/***************************************************************************
 *   Copyright (C) 2005 by Albert Astals Cid <tsdgeos@terra.es>            *
 *   Copyright (C) 2006 by Pino Toscano <toscano.pino@tiscali.it>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "boardtest.h"
#include "../boardwidget.h"
#include "../tile.h"
#include "../tilefactory.h"
#include "../tilewidget.h"
#include "../tilebox.h"
#include "../renderer.h"
#include "../kombinationscene.h"

#include <kaboutdata.h>
#include <kapplication.h>
#include <kcmdlineargs.h>
#include <klocale.h>
#include <KStandardDirs>
#include <kdebug.h>

#include <QGraphicsView>
#include <QComboBox>
#include <QLabel>
#include <qlayout.h>
#include <QList>
#include <QSizePolicy>
#include <QSplitter>

int main(int argc, char *argv[])
{
	KAboutData about("kombination", 0, ki18n("boardtest"), "0.1", ki18n("boardtest"), KAboutData::License_GPL, ki18n("© 2005 Albert Astals Cid\n© 2006 Pino Toscano"));
	about.addAuthor(ki18n("Albert Astals Cid"), KLocalizedString(), "aacid@kde.org");
	about.addAuthor(ki18n("Pino Toscano"), KLocalizedString(), "toscano.pino@tiscali.it");
	KCmdLineArgs::init(argc, argv, &about);
	KApplication app;
	app.setTopWidget(new BoardTest());
	return app.exec();
}

BoardTest::BoardTest() : KMainWindow()
{
	QWidget *w = new QWidget(this);
	QVBoxLayout *lay = new QVBoxLayout(w);

	QSplitter *splitter = new QSplitter(w);
	lay->addWidget(splitter);

	QWidget *foo1 = new QWidget(w);
	QVBoxLayout *foo1lay = new QVBoxLayout(foo1);

	scene = new KombinationScene(this);
	scene->setSceneRect(0.0, 0.0, 32*15, 32*16);
	m_board = scene->getBoard();
	m_tileWidget = scene->getTileWidget();
	m_exchangeWidget = scene->getExchangeWidget();

	view = new QGraphicsView(scene);

	foo1lay->addWidget(view);

	splitter->addWidget(foo1);

	QWidget *foo2 = new QWidget(w);
	QVBoxLayout *foo2lay = new QVBoxLayout(foo2);

	QLabel * lbl = new QLabel(i18n("Style:"), foo2);
	lbl->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
	foo2lay->addWidget(lbl);

	m_style = new QComboBox(foo2);
	foo2lay->addWidget(m_style);

	foo2lay->addItem(new QSpacerItem(5, 5, QSizePolicy::Fixed, QSizePolicy::Expanding));

	splitter->addWidget(foo2);

	tileFactory tf;
	tf.readTileSet("en");
	m_board->setTileStringList(tf.tileStringList());
	m_box = new tileBox();
	m_box->newGame(tf.tiles());
	for (int i = 0; i < 7; ++i)
		m_tileWidget->addTile(m_box->getTiles(1)[0]);

	QString lookupDirectory = "/graphics/";
	KGlobal::dirs()->addResourceType("themeselector", "data", KGlobal::mainComponent().componentName() + lookupDirectory);
	QStringList themesAvailable;
	KGlobal::dirs()->findAllResources("themeselector", QString("*.desktop"), KStandardDirs::Recursive, themesAvailable);
	m_style->addItems(themesAvailable);

	connect(m_board, SIGNAL(tileSizeChanged(double)), m_tileWidget, SLOT(changeTileSize(double)));
	connect(m_board, SIGNAL(moveTileAway(tile *)), m_tileWidget, SLOT(addTile(tile *)));
	connect(m_style, SIGNAL(activated(int)), this, SLOT(changeStyle(int)));

	setCentralWidget(w);
	show();
}

BoardTest::~BoardTest()
{
}

void BoardTest::changeStyle(const QString &newStyle)
{
	renderer::self()->loadTheme(newStyle);
	//refresh?
}

#include "boardtest.moc"
