/***************************************************************************
 *   Copyright (C) 2007 by Albert Astals Cid <tsdgeos@terra.es>            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "messageping.h"

#include <KAboutData>
#include <KApplication>
#include <KCmdLineArgs>
#include <QTcpSocket>
#include <QTcpServer>

#include <kdebug.h>
//on windows #include "winsock2.h" ??
#include <sys/socket.h>

messageClient::messageClient()
{
	connect(&m_messageHandler, SIGNAL(newMessage(const message &)), this, SLOT(processNewMessage(const message &)));
	m_clientSocket = new QTcpSocket();
	connect(m_clientSocket, SIGNAL(connected()), this, SLOT(slotSocketConnected()));
	m_messageHandler.setSocket(m_clientSocket);
	m_clientSocket->connectToHost("localhost", 6123);
}

void messageClient::processNewMessage(const message &m)
{
	if (m.type == "ping")
	{
		message m2;
		m2.type = m.type;
		int value = m.data["value"].toInt();
		kDebug() << "Value " << value << endl;
		m2.data["value"] = QString::number(value + 1);
		m_messageHandler.sendMessage(m2);
	}
}

void messageClient::slotSocketConnected()
{
	kDebug() << "Client connected to the server, sending the first message" << endl;
	
	message m;
	m.type = "ping";
	m.data["value"] = "1";
	m_messageHandler.sendMessage(m);
}



messageServer::messageServer()
{
	connect(&m_messageHandler, SIGNAL(newMessage(const message &)), this, SLOT(processNewMessage(const message &)));
	m_serverSocket = new QTcpServer();
	int sockd = m_serverSocket->socketDescriptor();
	int turnOn = 1;
	setsockopt(sockd, SOL_SOCKET, SO_REUSEADDR, &turnOn, sizeof(turnOn));
	connect(m_serverSocket, SIGNAL(newConnection()), this, SLOT(slotReadyAccept()));
	if (m_serverSocket->listen(QHostAddress::Any, 6123))
	{
		kDebug() << "Server bound, creating client" << endl;
		new messageClient();
	}
}

void messageServer::slotReadyAccept()
{
	m_clientSocket = m_serverSocket->nextPendingConnection();
	m_messageHandler.setSocket(m_clientSocket);
	
	kDebug() << "Client connection accepted" << endl;
}

void messageServer::processNewMessage(const message &m)
{
	if (m.type == "ping")
	{
		message m2;
		m2.type = m.type;
		int value = m.data["value"].toInt();
		kDebug() << "Value " << value << endl;
		m2.data["value"] = QString::number(value + 1);
		m_messageHandler.sendMessage(m2);
	}
}




int main(int argc, char *argv[])
{
	KAboutData about("messageping", 0, ki18n("messageping"), "0.1", ki18n("messageping"), KAboutData::License_GPL, ki18n("© 2007 Albert Astals Cid"));
	about.addAuthor(ki18n("Albert Astals Cid"), KLocalizedString(), "aacid@kde.org");
	KCmdLineArgs::init(argc, argv, &about);
	KApplication app;
	new messageServer();
	return app.exec();
}

#include "messageping.moc"
