/***************************************************************************
 *   Copyright (C) 2005-2007 by Albert Astals Cid <aacid@kde.org>          *
 *   Copyright (C) 2005 by Pino Toscano      <toscano.pino@tiscali.it>     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "tileposition.h"
#include "tile.h"

#include <QList>

tilePosition::tilePosition(): m_tile(0), m_xpos(-1), m_ypos(-1)
{
}

tilePosition::tilePosition(tile *t, int x, int y): m_tile(t), m_xpos(x), m_ypos(y)
{
}

tilePosition::~tilePosition()
{
}

void tilePosition::setTile(tile *t)
{
	m_tile = t;
}

tile* tilePosition::ourTile() const
{
	return m_tile;
}

const tile* tilePosition::ourTileC() const
{
	return m_tile;
}

void tilePosition::setX(int x)
{
	m_xpos = x;
}

int tilePosition::x() const
{
	return m_xpos;
}

void tilePosition::setY(int y)
{
	m_ypos = y;
}

int tilePosition::y() const
{
	return m_ypos;
}

QMap<QString,QString> tilePosition::toMap(const QString &tag) const
{
	QMap<QString,QString> result;
	result = m_tile->toMap(tag);
	result[tag + 'X'] = QString::number(m_xpos);
	result[tag + 'Y'] = QString::number(m_ypos);
	return result;
}

void tilePosition::fromMap(const QMap<QString,QString> &data, const QString &tag)
{
	m_tile = new tile();
	m_tile->fromMap(data, tag);
	m_xpos = data[tag + 'X'].toInt();
	m_ypos = data[tag + 'Y'].toInt();
}

tilePositionList::tilePositionList(QList<tilePosition> t) : QList<tilePosition>(t)
{
}

tilePositionList::tilePositionList()
{
}

void tilePositionList::deleteTiles()
{
	QList<tilePosition>::iterator it, itEnd;
	it = begin();
	itEnd = end();
	for ( ; it != itEnd ; ++it)
		delete (*it).ourTile();
}

QList<tile*> tilePositionList::tiles() const
{
	QList<tile*> result;
	QList<tilePosition>::const_iterator it, itEnd;
	it = begin();
	itEnd = end();
	for ( ; it != itEnd; ++it)
		result << (*it).ourTile();
	return result;
}

bool tilePositionList::contains(int x, int y) const
{
	foreach(const tilePosition &tp, *this)
	{
		if (tp.x() == x && tp.y() == y) return true;
	}
	return false;
}

void tilePositionList::removeAtPos(int x, int y)
{
	for (int i = count() - 1; i >= 0; i--)
	{
		if (at(i).x() == x && at(i).y() == y) takeAt(i);
	}
}

QMap<QString,QString> tilePositionList::toMap() const
{
	QMap<QString,QString> result;
	result.insert("tiles", QString::number(count()));
	for (int i = 0; i < count(); ++i)
	{
		result.unite( at(i).toMap(QString("tile%1").arg(i)) );
	}
	return result;
}

void tilePositionList::fromMap(const QMap<QString,QString> &data)
{
	tilePosition tp;
	int tiles = data["tiles"].toInt();
	for (int i = 0; i < tiles; ++i)
	{
		tp.fromMap(data, QString("tile%1").arg(i));
		append( tp );
	}
}
