/***************************************************************************
 *   Copyright (C) 2005 by Albert Astals Cid <tsdgeos@terra.es>            *
 *   Copyright (C) 2005 by Pino Toscano      <toscano.pino@tiscali.it>     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "utils.h"

#include <QDrag>
#include <QPainter>
#include <QPixmap>
#include <QWidget>

#include "tile.h"
#include "tilefactory.h"

namespace Utils
{

bool checkRange(int value, int min, int max)
{
	return (value >= min) && (value < max);
}

bool isValidPoint(const QPoint &p)
{
	return isValidPoint(p.x(), p.y());
}

bool isValidPoint(int x, int y)
{
	return (x > -1) && (y > -1);
}

QPoint calcMousePositionPoint(const QPoint &p, double width)
{
	return QPoint(calcMousePositionInt(p.x(), width), calcMousePositionInt(p.y(), width));
}

int calcMousePositionInt(int pos, double width)
{
	return (int)((pos + width - 1) / width) - 1;
}

QDrag* createDragForTile(QWidget *dragparent, QObject * sender, const QPixmap &pixmap, tile *t, double size)
{
	if (!t) return 0;

	QDrag *drag = new QDrag(dragparent);
	sMimeData *mimeData = new sMimeData;
	mimeData->setSenderItem(sender);
	QPixmap tmpimg = pixmap;
	QPainter tmpp(&tmpimg);
	tmpp.setRenderHint(QPainter::Antialiasing, true);	
	tmpp.drawText(0, 0, 32, 32, Qt::AlignCenter, t->string());
	tmpp.end();
	mimeData->setText(tileFactory::simpleXMLSerialization(t));
	drag->setMimeData(mimeData);
	drag->setPixmap(tmpimg);
//	drag->setDragCursor(tmpimg, Qt::MoveAction);
//	drag->setDragCursor(tmpimg, Qt::CopyAction);
	drag->setHotSpot(QPoint(drag->pixmap().width()/2, drag->pixmap().height()/2));

	return drag;
}

}
