/*
 *  Koware - The ancient African game, also known as Mancala, for KDE
 *  Copyright (C) 2005  Michael Collins  <michaelcollins@ivorycity.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "game.h"
#include "alphabeta.h"

heuristic_t heuristics[256];

int  defaultheuristic(struct game* g)              { return g->player1score-g->player2score; }
void alphabeta_init()                              { heuristics[0]=defaultheuristic; }
void alphabeta_setheuristic(int id, heuristic_t h) { heuristics[id]=h;               }

int
alphabeta_ply( struct game* g, int heuristic, int depth, int  bestmax, int  bestmin, int* bestmove )
{
  struct game testgame;
  int         start;
  int         end;
  int         i;

  if(g->turn==PLAYER1) { start=6; end=12; }
  else                 { start=0; end=6;  }

  for(i=start; i<end; i++)
  {
    int result;
    int mybest;

    if( g->cups[i] == 0 )  continue; /*An empty cup*/
    testgame = *g;

    if( oware_move( &testgame, i ) < 0 ) continue; /*An illegal move*/

    if( depth==0 || testgame.gameover){ result = heuristics[heuristic](&testgame);          }
    else{                               result = alphabeta_ply( &testgame,heuristic,depth-1,bestmax,bestmin,&mybest); }

    if(g->turn==PLAYER1)
    {
      if(result>=bestmin){ *bestmove = i; return bestmin; }
      if(result>bestmax) { *bestmove = i; bestmax = result; }
    }
    else
    {
      if(result<=bestmax){ *bestmove = i; return bestmax; }
      if(result<bestmin) { *bestmove = i; bestmin = result; }
    }
  }

  if(g->turn==PLAYER1) return bestmax;
  else                 return bestmin;
}

int
alphabeta(struct game* g, int heuristic, int depth )
{
  int result;
  int bestmove;
  struct game testgame = *g;
  result = alphabeta_ply( &testgame, heuristic, depth, -INF, INF, &bestmove);
  return bestmove;
}



