/*
 *  Koware - The ancient African game, also known as Mancala, for KDE
 *  Copyright (C) 2005  Michael Collins  <michaelcollins@ivorycity.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "gamewidget.h"
#include <kapplication.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include <kglobalsettings.h>
#include <kstdaction.h>
#include <kstdgameaction.h>
#include <kstdguiitem.h>
#include <kaction.h>


struct game* ggame;

int
defense_heuristic( struct game* g)
{
  if(ggame->turn==PLAYER1) return (g->player1score)   - (g->player2score);
  else                     return (g->player1score*3) - (g->player2score);
}

GameWidget::GameWidget ( QWidget* parent, const char* name )
: QWidget( parent, name )
{
  QPixmap bg( locate("imgs","bg01.png") , "PNG" );
  setBackgroundMode(FixedPixmap);
  setBackgroundPixmap( bg );
  setMaximumSize(bg.width(),bg.height());

  animate_timer = new QTimer(this);
  animating     = 0;
  redo_top      = 0;

  QVBoxLayout* gamelayout  = new QVBoxLayout( this );
               gamelayout->addItem(new QSpacerItem(0,8));
  QHBoxLayout* scorelayout = new QHBoxLayout(gamelayout);

  p1ScoreLabel = new ScoreLabel(this);
  p1ScoreLabel->setBackgroundPixmap( bg );
  p1ScoreLabel->setPlayer(1);
  p1ScoreLabel->display(0);

  p2ScoreLabel = new ScoreLabel(this);

  p2ScoreLabel->setBackgroundPixmap( bg );
  p2ScoreLabel->setPlayer(2);
  p2ScoreLabel->display(0);

  owizard  = new OwareWizard(this);

  scorelayout->addWidget(p1ScoreLabel);
  scorelayout->addWidget(owizard);
  scorelayout->addWidget(p2ScoreLabel);

  gamelayout->addItem(new QSpacerItem(0,8));
  QHBoxLayout* cups2layout = new QHBoxLayout(gamelayout);
  QHBoxLayout* cups1layout = new QHBoxLayout(gamelayout);

  int i;

  for( i=0;i<12;i++){ cups[i] = new Cup(this,i>5?1:0); cups[i]->reset(4); }
  for( i=5;i>=0;i--){ cups2layout->addWidget(cups[i]);            }
  for( i=6;i<12;i++){ cups1layout->addWidget(cups[i]);            }

  connect(cups[6],  SIGNAL(clicked()), this, SLOT(pressed0()) );
  connect(cups[7],  SIGNAL(clicked()), this, SLOT(pressed1()) );
  connect(cups[8],  SIGNAL(clicked()), this, SLOT(pressed2()) );
  connect(cups[9],  SIGNAL(clicked()), this, SLOT(pressed3()) );
  connect(cups[10], SIGNAL(clicked()), this, SLOT(pressed4()) );
  connect(cups[11], SIGNAL(clicked()), this, SLOT(pressed5()) );

  gamelayout->setGeometry(QRect(0,0,bg.width(),bg.height()));

  aidepth      = 0;
  endoptimize  = 0;
  heuristic    = 0;
  whogoesfirst = HUMGOESFIRST;

  oware_init(&game);
  alphabeta_init();
  alphabeta_setheuristic(1,defense_heuristic);

  game.turn=PLAYER1;
  ggame = &game;
  newGame();
}

void GameWidget::pressed0(){ move1( 6);  }
void GameWidget::pressed1(){ move1( 7);  }
void GameWidget::pressed2(){ move1( 8);  }
void GameWidget::pressed3(){ move1( 9);  }
void GameWidget::pressed4(){ move1(10);  }
void GameWidget::pressed5(){ move1(11);  }

void
GameWidget::move1( int i )
{
  if(onmove(i)==0) return;
  for( int b=6; b<11;  b++ ) cups[b]->setEnabled(false);
}

void
GameWidget::move2( )
{
  owizard->setThinking();
  qApp->processEvents();

  if(endoptimize && (game.player1score+game.player2score >= 34) ) onmove(alphabeta(&game,heuristic,aidepth+10));
  else                                                            onmove(alphabeta(&game,heuristic,aidepth));
}

int
GameWidget::onmove(int distcup)
{
  if(animating) return 0;
  struct game tmpgame = game;
  int rslt = oware_move(&game,distcup);

  if( rslt<0 ){ QMessageBox::warning( this, i18n("Illegal Move"), illegaln2s(rslt), i18n("Okay")); game = tmpgame; return 0; }

  saved[saved_top++] = tmpgame;
  saved[saved_top  ] = game;
  redo_top = saved_top;

  for( int c=0; c<12; c++ ){
	  cups[c]->setRedIndicator(0);
	  cups[c]->setIndicator(0);
	  cups[c]->update();
  }

  animate_distcup   = distcup;
  animate_remaining = saved_last().cups[distcup];
  animate_cursor    = distcup;

  cups[distcup]->setRedIndicator(1);
  cups[distcup]->setIndicator(-animate_remaining);
  cups[distcup]->setNumber(0);
  cups[distcup]->update();

  connect( animate_timer, SIGNAL(timeout()), SLOT(animateCups()) );
  animate_timer->start(200,false);
  animating=1;

  return 1;
}

void
GameWidget::animateCups()
{
  if(animate_remaining == 0)
  {
    for(int i=0;i<12;i++)
    {                        
      if(i!=animate_distcup) 
        if( game.cups[i]<cups[i]->getNumber() ) cups[i]->setIndicator( game.cups[i] - cups[i]->getNumber() );

      cups[i]->setNumber(game.cups[i]);
      cups[i]->update();
    }

    p1ScoreLabel->display( game.player1score );
    p2ScoreLabel->display( game.player2score );

    disconnect(animate_timer,SIGNAL(timeout()),this,SLOT(animateCups()));
    animate_timer->stop();
    animating=0;

         if( saved_last().turn==PLAYER1 ) onEndAnimation1();
    else if( saved_last().turn==PLAYER2 ) onEndAnimation2();
  }

  else
  {
    animate_cursor++;
    if( animate_cursor == animate_distcup ) animate_cursor++;
    if( animate_cursor == 12              ) animate_cursor=0;

    cups[animate_cursor]->incNumber();
    cups[animate_cursor]->incIndicator();
    cups[animate_cursor]->update();
    animate_remaining--;
  }
}

void
GameWidget::onEndAnimation1()
{
  if( game.gameover ) onGameOver();
  else                move2();
}

void
GameWidget::onEndAnimation2()
{
  for( int b=6;b<12;b++ ) cups[b]->setEnabled(true);
  owizard->setResting();

  if( game.gameover ) onGameOver();
}

void
GameWidget::onGameOver()
{
  for( int b=6; b<12; b++ ) cups[b]->setEnabled(false);
  QMessageBox::warning( this, i18n("Game Over"), i18n("The game is over.\n"), i18n("Okay"));
}

void
GameWidget::newGame()
{
  oware_init(&game);
  alphabeta_init();

  for( int b=6; b<11; b++ ) cups[b]->setEnabled(true);
  for( int i=0; i<12; i++ ) cups[i]->reset(game.cups[i]);
  p1ScoreLabel->display( game.player1score );
  p2ScoreLabel->display( game.player2score );

  saved_top = 0;
  redo_top  = 0;

  if( whogoesfirst == WIZGOESFIRST )
  {
    game.turn=PLAYER2;
    saved_push(game);
    move2();
  }
  else
  {
    game.turn=PLAYER1;
    saved_push(game);
  }
}

void
GameWidget::undo()
{
  if( animating) return;
  if(!saved_top) return;

  if( saved_top>=2 ){
   saved_pop();
   game = saved_pop();
  }else{
    game=saved[0];
  }
  
  for( int i=0; i<12; i++ )
  {
    cups[i]->setIndicator(0);
    cups[i]->setRedIndicator(0);
    cups[i]->setNumber( game.cups[i] );
    cups[i]->update();
  }

  p1ScoreLabel->display( game.player1score );
  p2ScoreLabel->display( game.player2score );
}

void
GameWidget::redo()
{
  if( saved_top==redo_top ) return;

  saved_top+=2;
  game = saved[saved_top];

  for( int i=0; i<12; i++ )
  {
    cups[i]->setIndicator(0);
    cups[i]->setRedIndicator(0);
    cups[i]->setNumber( game.cups[i] );
    cups[i]->update();
  }

  p1ScoreLabel->display( game.player1score );
  p2ScoreLabel->display( game.player2score );
}

void
GameWidget::resizeEvent(QResizeEvent* )
{  
  owizard->setBackgroundPixmap(*backgroundPixmap());
  owizard->backgroundChanged();

  p1ScoreLabel->setBackgroundPixmap(*backgroundPixmap());
  p2ScoreLabel->setBackgroundPixmap(*backgroundPixmap());

  p1ScoreLabel->backgroundChanged();
  p2ScoreLabel->backgroundChanged();
}

#include "gamewidget.moc"

