/*
 *  Koware - The ancient African game, also known as Mancala, for KDE
 *  Copyright (C) 2005  Michael Collins  <michaelcollins@ivorycity.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qslider.h>
#include <qlcdnumber.h>
#include <qlayout.h>
#include <qlabel.h>
#include <qcheckbox.h>
#include <qbuttongroup.h>
#include <qradiobutton.h>
#include <qcombobox.h>
#include <qtextbrowser.h>
#include <qfile.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qaction.h>

#include "mainwindow.h"
#include "gamewidget.h"

#include <kapplication.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include <kglobalsettings.h>
#include <kstdaction.h>
#include <kstdgameaction.h>
#include <kstdguiitem.h>
#include <kaction.h>


class
AiMgr : public QWidget
{
  public:
  AiMgr(GameWidget* gw, QWidget* parent=0,const char* name=0);
};

class
AiMgr2 : public QWidget
{
  public:
  AiMgr2(GameWidget* gw, QWidget* parent=0,const char* name=0);
};

AiMgr::AiMgr(GameWidget* gw, QWidget* parent,const char* name)
: QWidget(parent,name)
{
  QHBoxLayout* toplayout   = new QHBoxLayout(this,4);
  QVBoxLayout* depthlayout = new QVBoxLayout(toplayout,4);

  QLabel* aml = new QLabel( i18n(" Set Wizard Intellect ") , this);
          aml->setAlignment( AlignCenter );
          aml->setFrameStyle( QFrame::Panel|QFrame::Sunken ); 
          aml->setLineWidth( 1 );

  depthlayout->addWidget(aml);

  QHBoxLayout* layout2 = new QHBoxLayout(depthlayout,4);

  QSlider* aislider = new QSlider(0,14,1,0,QSlider::Horizontal, this);
           aislider->setTickmarks(QSlider::Below);

  layout2->addWidget(aislider);

  QLCDNumber* lcd = new QLCDNumber(2,this);
              lcd->display(0);
              lcd->setSegmentStyle( QLCDNumber::Flat );

  layout2->addWidget(lcd);

  connect( aislider, SIGNAL(valueChanged(int)), lcd, SLOT(display(int)));
  connect( aislider, SIGNAL(valueChanged(int)),  gw, SLOT(setAiDepth(int)));

  depthlayout->addItem( new QSpacerItem(0,5) );

  QVBoxLayout* optimizelayout = new QVBoxLayout(depthlayout,4);

  QLabel* accl = new QLabel( i18n(" Ai Options "),this);
          accl->setAlignment( AlignCenter );
          accl->setFrameStyle( QFrame::Panel|QFrame::Sunken );
          accl->setLineWidth( 1 );

  optimizelayout->addWidget(accl);

  QCheckBox* endgame = new QCheckBox(i18n(" End Game Enhancer ") , this); endgame->setChecked(false); optimizelayout->addWidget(endgame);
  QCheckBox* defense = new QCheckBox(i18n(" Smarter Defense "  ) , this); defense->setChecked(false); optimizelayout->addWidget(defense);

  connect( endgame, SIGNAL(toggled(bool)), gw, SLOT(setEndOptimize(bool)) );
  connect( defense, SIGNAL(toggled(bool)), gw, SLOT(setDefenseHeuristic(bool)) );

}

AiMgr2::AiMgr2(GameWidget* gw, QWidget* parent,const char* name)
: QWidget(parent,name)
{
  QVBoxLayout* toplayout = new QVBoxLayout(this,4);

  QLabel* wgflbl = new QLabel( i18n("First Move By:") , this);
          wgflbl->setAlignment( AlignCenter );
          wgflbl->setFrameStyle( QFrame::Panel|QFrame::Sunken );
          wgflbl->setLineWidth( 1 );
          wgflbl->setMargin(0);

  toplayout->addWidget(wgflbl);

  QComboBox* whogoes = new QComboBox(this);
             whogoes->insertItem( i18n( "Human"   ) );
             whogoes->insertItem( i18n( "Computer") );

  connect(whogoes,SIGNAL(highlighted(int)),gw,SLOT(setWhoGoesFirst(int)));

  toplayout->addWidget(whogoes);
  
  QLabel* card    = new QLabel( this );
          card->setPixmap(QPixmap( locate("imgs","card.png")));

  toplayout->addWidget(card);
}

MainWindow::MainWindow( QWidget* parent, const char* name ) : KMainWindow( parent, name )
{
  setUsesBigPixmaps(true);
  setFixedWidth(800);

  GameWidget* gw = new GameWidget(this);
  setCentralWidget( gw );

  KStdGameAction::gameNew(gw,   SLOT(newGame()), actionCollection());
  KStdGameAction::undo   (gw,   SLOT(undo()), actionCollection());
  KStdGameAction::redo   (gw,   SLOT(redo()), actionCollection());
  KStdGameAction::quit   (qApp, SLOT(quit()), actionCollection());

  KAction* fileRulesAction = new KAction( i18n("Ru&les"),    CTRL+Key_F1, this, "rules" );
  connect( fileRulesAction, SIGNAL( activated() ), this, SLOT( appHelpActivated() ) );

  new KAction( i18n("Cup 0"), Qt::Key_1, gw, SLOT(pressed0()), actionCollection(), "sow_cup_1");  
  new KAction( i18n("Cup 1"), Qt::Key_2, gw, SLOT(pressed1()), actionCollection(), "sow_cup_2");  
  new KAction( i18n("Cup 2"), Qt::Key_3, gw, SLOT(pressed2()), actionCollection(), "sow_cup_3");  
  new KAction( i18n("Cup 3"), Qt::Key_4, gw, SLOT(pressed3()), actionCollection(), "sow_cup_4");  
  new KAction( i18n("Cup 4"), Qt::Key_5, gw, SLOT(pressed4()), actionCollection(), "sow_cup_5");  
  new KAction( i18n("Cup 5"), Qt::Key_6, gw, SLOT(pressed5()), actionCollection(), "sow_cup_6");  

  QToolBar*  gmgr = new QToolBar( i18n( "Game Manager" ),this);
  new QToolButton( QPixmap(locate("imgs","newgame.png")), i18n("New Game"), "Game Manager", gw,   SLOT(newGame()),  gmgr );
  new QToolButton( QPixmap(locate("imgs","quit.png")),    i18n("Quit"),     "Game Manager", qApp, SLOT(quit()),     gmgr );
  new QToolButton( QPixmap(locate("imgs","undo.png")),    i18n("Undo"),     "Game Manager", gw,   SLOT(undo()),     gmgr );
  new QToolButton( QPixmap(locate("imgs","redo.png")),    i18n("Redo"),     "Game Manager", gw,   SLOT(redo()),     gmgr );
  new QToolButton( QPixmap(locate("imgs","help.png")),    i18n("Help"),     "Game Manager", this, SLOT(appHelpActivated()), gmgr );

  QToolBar* aibar = new QToolBar( i18n("Ai Manager"),this);
  new AiMgr( gw, aibar );

  QToolBar* aibar2 = new QToolBar( i18n("Ai Manager"),this);
  new AiMgr2( gw, aibar2 );  

  QPopupMenu* helpMenu = new QPopupMenu( this );
  fileRulesAction->plug( helpMenu );

  setupGUI (ToolBar|Keys|StatusBar|Save|Create,"kowareui.rc");
}

#include "mainwindow.moc"














