/*
 *  Koware - The ancient African game, also known as Mancala, for KDE
 *  Copyright (C) 2005  Michael Collins  <michaelcollins@ivorycity.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <stdio.h>
#include "scorelabel.h"
#include "qimage.h"
#include "qpainter.h"
#include "qfontmetrics.h"
#include "alphas.h"
#include <kapplication.h>
#include <kstandarddirs.h>
#include <kglobalsettings.h>
#include <kstdaction.h>
#include <kstdgameaction.h>
#include <kstdguiitem.h>
#include <kaction.h>
#include <klocale.h>


ScoreLabel::ScoreLabel(QWidget* parent, const char* name)
: QWidget( parent, name )
{
  setBackgroundOrigin( ParentOrigin );
  setFixedSize( 140, 200 );

  trans = new QImage( locate("imgs","trans.png"), "PNG" );
  trans->setAlphaBuffer(true);

  buffer = new QPixmap(width(),height());
}

void
ScoreLabel::setPlayer( int n )
{
  player=n;
}

void
ScoreLabel::display( int n )
{
  if( n == number ) return;
  number = n;
  drawBuffer();
  update();
}

void 
ScoreLabel::moveEvent(QMoveEvent*)
{
  //backgroundChanged();
}

void
ScoreLabel::resizeEvent(QResizeEvent*)
{
  //backgroundChanged();
}

void
ScoreLabel::backgroundChanged()
{
  bg = backgroundPixmap()->convertToImage().copy(x(),y(),width(),height());
  *trans = trans->smoothScale(width(),height());
  alphaDraw(0,0,*trans);

  drawBuffer();
}

void
ScoreLabel::drawBuffer()
{
  QString scorestr;
  QString lblstr;

  scorestr.sprintf("%d",number);

  if(player==1) lblstr = i18n("Your\nScore");
  else          lblstr = i18n("Wizard's\nScore");

  QFont f0("arial",12,QFont::Bold);
  QFont f1("arial",60,QFont::Bold);

  QFontMetrics fm(f0);
  int fh = fm.height();

  QPainter p;
  p.begin(buffer);
  p.drawImage(0,0,bg);

  p.setPen   ( QColor( 0xffffff ) );
  p.setFont  ( f0 );
  p.drawText ( 0,2,width(),(fh*2)+6, Qt::AlignCenter, lblstr );
  p.setFont  ( f1 );
  p.drawText ( 0,0,width(),height(), Qt::AlignCenter, scorestr );

  p.end();
}

void
ScoreLabel::alphaDraw( int x, int y, QImage& img )
{
  int h   = img.height();
  int bpl = img.bytesPerLine();

  for( int i=0; i<h; i++ )
  {
    uchar* bgline = bg.scanLine(i+y); bgline  += 4*x;
    uchar* bfline = img.scanLine(i);
	
    for(int j=0; j<bpl; bfline+=4, bgline+=4, j+=4 )
	{ 
	  int alpha1 = bfline[3];
	  int alpha2 = 255-alpha1;

	  switch( alpha1 )
	  {
	    case 0: 
              break;

            case 255:
              *((int*)bgline) = *((int*)bfline);
              bgline[3]       = 0x00;
              break;

	    default:
              double d1 = alphas[alpha1];
              double d2 = alphas[alpha2];

              bgline[0]   = (uchar) ((uchar)(bgline[0]*d2)) + ((uchar)(bfline[0]*d1));
	      bgline[1]   = (uchar) ((uchar)(bgline[1]*d2)) + ((uchar)(bfline[1]*d1));
	      bgline[2]   = (uchar) ((uchar)(bgline[2]*d2)) + ((uchar)(bfline[2]*d1));
	  }
	}
  }
}

void
ScoreLabel::paintEvent(QPaintEvent*)
{
  bitBlt(this,0,0,buffer,0,0,width(),height());
}

#include "scorelabel.moc"


