/**
 * createwindow.cpp
 *
 * Copyright (C)  2004  Zack Rusin <zack@kde.org>
 * Copyright (C) 2008 Jeremy Whiting (jeremy@scitools.com)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
#include "createwindow.h"

#include "boardview.h"
#include "board.h"
#include "ui_editui.h"

#include <klineedit.h>
#include <kpushbutton.h>
#include <klocale.h>
#include <kglobal.h>
#include <kstandarddirs.h>
#include <kdebug.h>

#include <qbuttongroup.h>
#include <qlayout.h>
#include <qfile.h>
#include <qdatastream.h>
#include <qvector.h>

CreateWindow::CreateWindow( QWidget *parent )
    : KMainWindow( parent )
{
    QWidget * mainWidget = new QWidget(this);
    QHBoxLayout * main = new QHBoxLayout(mainWidget);
    QBoxLayout *left = new QVBoxLayout(this);
    QWidget * uiWidget = new QWidget(this);
    Ui::EditUI *ui = new Ui::EditUI;
    ui->setupUi(uiWidget);
    left->addWidget(uiWidget);
    connect( ui->m_titleEdit, SIGNAL(textChanged(const QString&)),
             SLOT(slotTitleChanged(const QString&)) );
    connect( ui->m_saveButton, SIGNAL(clicked()), SLOT(saveBoard()) );
    QButtonGroup * sizeGroup = new QButtonGroup(this);
    sizeGroup->addButton(ui->radioButton1, 0);
    sizeGroup->addButton(ui->radioButton2, 1);
    sizeGroup->addButton(ui->radioButton3, 2);
    sizeGroup->addButton(ui->radioButton4, 3);
    connect( sizeGroup, SIGNAL(buttonClicked(int)), SLOT(slotChangeSize(int)) );
    main->addLayout( left );

    QBoxLayout *right = new QVBoxLayout( this);

    QVector< QVector<bool> > pic( 20, QVector<bool>( 20, false ) );
    BoardXml xml;
    xml.name = "new board";
    Board *b = new Board(xml, this);
    b->setPicture( pic );
    m_view = new BoardView( b, this );
    right->addWidget( m_view );
    setCentralWidget( mainWidget );
    ui->m_titleEdit->setText( b->data().name );
    main->addLayout( right );
}

void CreateWindow::changeBoard( Board *board )
{
    m_view->setBoard( board );
}


void CreateWindow::slotTitleChanged( const QString& title )
{
    m_view->board()->setTitle( title );
}

void CreateWindow::saveBoard()
{
    Board *board = m_view->board();
    QString filePath = KStandardDirs::locateLocal( "appdata",
                                                   QString( "%1.pic" ).arg(
                                                       board->data().name));

    QVector< QVector<Board::TileState> > view = board->board();
    QVector< QVector<bool> > pic( board->rows(), QVector<bool>( board->columns(), false ) );

    for ( int i = 0; i < board->rows(); ++i ) {
        for ( int j = 0; j < board->columns(); ++j ) {
            if ( view[i][j] == Board::CheckedAsFilled )
                pic[i][j] = true;
        }
    }
    BoardXml xml; xml.name = board->data().name;
    Board *saveBoard = new Board(xml, this);
    saveBoard->setPicture( pic );

    kDebug()<<"Saving at "<<filePath<<endl;
    QFile file( filePath );
    file.open( QFile::WriteOnly );
    QDataStream str( &file );
    saveBoard->save( str );
    file.close();
    delete saveBoard;

    emit boardSaved();
}

void CreateWindow::slotChangeSize( int i )
{
    switch( i ) {
    case 0:
        if ( m_view->board()->rows() != 5 ) {
            QVector< QVector<bool> > pic( 5, QVector<bool>( 5, false ) );
            m_view->board()->setPicture( pic );
        }
        break;
    case 1:
        if ( m_view->board()->rows() != 10 ) {
            QVector< QVector<bool> > pic( 10, QVector<bool>( 10, false ) );
            m_view->board()->setPicture( pic );
        }
        break;
    case 2:
        if ( m_view->board()->rows() != 15 ) {
            QVector< QVector<bool> > pic( 15, QVector<bool>( 15, false ) );
            m_view->board()->setPicture( pic );
        }
        break;
    case 3:
        if ( m_view->board()->rows() != 20 ) {
            QVector< QVector<bool> > pic( 20, QVector<bool>( 20, false ) );
            m_view->board()->setPicture( pic );
        }
        break;
    }
}


#include "createwindow.moc"
