/**************************************************************************
*   Copyright (C) 2005 by Lukas Tinkl                                     *
*   lukas@kde.org                                                         *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
***************************************************************************/

#include "dlgNewServer.h"
#include "gameManager.h"

#include <qhbox.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qsizepolicy.h>

#include <kstdguiitem.h>
#include <klocale.h>

NewServerDlg::NewServerDlg( GameManager * manager, QWidget *parent, const char *name )
    : KDialogBase( parent, name, true, i18n( "Add New Server" ), User1|Cancel,
                   Ok, false, KStdGuiItem::add() )
{
     QHBox * page = makeHBoxMainWidget();
     ( void ) new QLabel( i18n( "Server:" ), page );

     m_addressLined = new QLineEdit( page, "address_edit" );
     m_addressLined->setInputMask( "000.000.000.000:00000" ); // stupid validity test :-)
     m_addressLined->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding,
                                                 QSizePolicy::Fixed ) );
     m_addressLined->setFocus();

     m_gameCombo = new GameComboBox( manager, page, "game_combo" );

     connect( this, SIGNAL( user1Clicked() ), this, SLOT( accept() ) );
}

QString NewServerDlg::address() const
{
    return m_addressLined->text();
}

QString NewServerDlg::gameID() const
{
    return m_gameCombo->gameID();
}

void NewServerDlg::setGameID( const QString & gameID )
{
    m_gameCombo->setGameID( gameID );
}

#include "dlgNewServer.moc"
