/**************************************************************************
*   Copyright (C) 2005 by Lukas Tinkl                                     *
*   lukas@kde.org                                                         *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
***************************************************************************/

#include "game.h"

#include <qpixmap.h>
#include <qstring.h>
#include <qpixmapcache.h>

#include <kstandarddirs.h>
#include <kdebug.h>

Game::Game( const QDomElement & e )
{
    m_dom = e;
}

QString Game::id() const
{
    return getNode( "id" );
}

QString Game::exec() const
{
    // FIXME there could be more commands, like a demo and a normal binary
    return getNode( "command" );
}

QString Game::name() const
{
    return getNode( "name" );
}

QString Game::serverOption() const
{
    return getNode( "qstat_option" );
}

QString Game::masterServerOption() const
{
    return getNode( "qstat_master_option" );
}

QString Game::masterServer() const
{
    // FIXME there can be multiple masters
    return getNode( "master" );
}

QPixmap Game::gameIcon() const
{
    QString name = getNode( "icon" );
    if ( !name.isNull() )
    {
        QPixmap pm;
        if ( !QPixmapCache::find( name, pm ) )
        {
            pm.load( locate( "data", "kqf/pics/games/" + name ), "XPM" );
            QPixmapCache::insert( name, pm );
        }

        return pm;
    }
    else
        return QPixmap();
}

QString Game::execClientType() const
{
    return getNode( "exec_client" );
}

QString Game::baseGame() const
{
    return getNode( "type" );
}

QString Game::getNode( const QString & name ) const
{
    if ( m_dom.isNull() )
        return QString::null;

    QDomNodeList list = m_dom.elementsByTagName( name );

    if ( list.count() > 0 )     // we just return the first item
    {
        QDomNode node = list.item( 0 );
        if ( !node.isNull() )
            return node.toElement().text();
    }

    return QString::null;
}

bool Game::operator<( const Game & rhs )
{
    return ( name() < rhs.name() );
}
