/***************************************************************************
 *   Copyright (C) 2005 by Lukas Tinkl                                     *
 *   lukas@kde.org                                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
 ***************************************************************************/


#ifndef _KQF_H_
#define _KQF_H_

#include <kapplication.h>
#include <kmainwindow.h>

#include "kqfview.h"

class KURL;

/**
 * This class serves as the main window for kqf.  It handles the
 * menus, toolbars, and status bars.
 *
 * @short Main window class
 * @author Lukas Tinkl <lukas@kde.org>
 */
class kqf: public KMainWindow
{
    Q_OBJECT
public:
    /**
     * Default constructor
     */
    kqf();

    /**
     * Default destructor
     */
    virtual ~kqf();

protected:
    /**
     * Overidden from KMainWindow to handle saving the configuration
     */
    virtual bool queryExit();

private slots:
    /**
     * Invoke the settings dialog
     */
    void optionsPreferences();

    /**
     * Toggle the "resolve hostnames" action
     */
    void slotToggleResolving();

    /**
     * Enable/disable actions pertaining to updating the server list
     */
    void slotEnableUpdateActions( bool enable );

    /**
     *  Enable/disable actions pertaining to server actions
     */
    void slotEnableServerActions( bool add, bool remove );

    /**
     * Show RMB popup menu for server in @p item, at position @p pos
     */
    void slotShowServerRMB( QListViewItem * item, const QPoint & pos, int col );

private:
    /**
     * Setup actions of this application
     */
    void setupActions();

    /**
     * @return whether qstat is present
     */
    bool checkQStat();

private:
    kqfView *m_view;
};

#endif // _KQF_H_

