/**************************************************************************
*   Copyright (C) 2005 by Lukas Tinkl                                     *
*   lukas@kde.org                                                         *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
***************************************************************************/

#include "player.h"

#include <qstring.h>
#include <qsimplerichtext.h>
#include <qregexp.h>
#include <qfontmetrics.h>
#include <qpainter.h>

#include <kdebug.h>

QString Player::text( int col ) const
{
    if ( col == 0 )
        return name();
    else if ( col == 1 )
        return QString::number( m_frags );
    else if ( col == 2 )
        return QString::number( m_ping );

    return QString::null;
}

int Player::compare( QListViewItem * i, int col, bool ascending ) const
{
    if ( col == 1 || col == 2 )             // integers
    {
        int us = 0;
        int them = 0;

        if ( col == 2 )         // ping
        {
            us = m_ping;
            them = static_cast<Player *>( i )->ping();
        }
        else if ( col == 1 )    // frags
        {
            us = m_frags;
            them = static_cast<Player *>( i )->frags();
        }

        if ( us < them )
            return -1;
        else if ( us > them )
            return 1;
        else
            return 0;
    }
    else
        return QListViewItem::compare( i, col, ascending );
}

// virtual
void Player::paintCell( QPainter * p, const QColorGroup & cg, int column, int width, int align )
{
    if ( column == 0 )          // player name
    {
        QRect r( 0, 0, width, height() );
        p->fillRect( r, Qt::gray );
        QSimpleRichText rich( m_name, listView()->font() );
        rich.draw( p, 0, 0, r, cg );
    }
    else
        QListViewItem::paintCell ( p, cg, column, width, align );
}

QString Player::name() const
{
    QRegExp rx( "</?font.*>" );
    rx.setMinimal( true );
    QString name = m_name;

    //kdDebug() << "Stripped name: " << name.remove( rx ) << endl;

    return name.remove( rx );
}

// virtual
int Player::width( const QFontMetrics & fm, const QListView * lv, int c ) const
{
    if ( c == 0 )               // player name
    {
        //kdDebug() << "Width: " << fm.width( name() ) << endl;
        return fm.width( name() );
    }
    else
        return QListViewItem::width( fm, lv, c );
}
