/**************************************************************************
*   Copyright (C) 2005 by Lukas Tinkl                                     *
*   lukas@kde.org                                                         *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
***************************************************************************/

#include "propDialog.h"

#include <qpixmap.h>
#include <qlabel.h>
#include <qstring.h>

#include <kstandarddirs.h>
#include <klocale.h>
#include <ksocketaddress.h>
#include <kdebug.h>

PropDialog::PropDialog( GameManager * manager, Server * server,
#ifdef HAVE_LIBGEOIP
                        GeoIP * gi,
#endif
                        QWidget *parent, const char *name, bool modal )
    : KDialogBase( parent, name, modal, i18n( "Server Properties" ), Close, Close )
{

    m_w = new PropWidget( this, "prop_widget" );
    setMainWidget( m_w );

    // fill the labels
    m_w->labelIcon->setPixmap( manager->gameIcon( server->gameID() ) );
    m_w->labelServerName->setText( server->strippedName() );

    m_w->lbHostname->setText( i18n( "Resolving..." ) );
    m_w->lbAddress->setText( server->address() );

#ifdef HAVE_LIBGEOIP
    m_w->lbCountryIcon->setPixmap( GameManager::flagIcon( server->countryCode( gi ) ) );
    m_w->lbCountry->setText( GeoIP_country_name_by_addr( gi, server->address().section( ":", 0, 0 ) ) );
#else
    m_w->lbCountryIcon->setText( "??" );
    m_w->lbCountry->setText( i18n( "Unknown" ) );
#endif

    m_w->lbPunkbuster->setText( server->hasPunkbuster() ? i18n( "Yes" ) : i18n( "No" ) );
    m_w->lbPassword->setText( server->isLocked() ? i18n( "Yes" ) : i18n( "No" ) );

    m_w->lbLastRefresh->setText( KGlobal::locale()->formatDateTime( server->lastRefresh() ) );
    m_w->lbSlots->setNum( server->privateSlots() );

    QString address = server->address().section( ":", 0, 0 );
    Q_UINT16 port = server->address().section( ":", 1, 1 ).toUInt();

    m_resolver = new KNetwork::KReverseResolver( KNetwork::KInetSocketAddress( KNetwork::KIpAddress( address ), port ) );
    connect( m_resolver, SIGNAL( finished( const KReverseResolver & ) ),
             this, SLOT( resolveHostname( const KReverseResolver & ) ) );

    m_resolver->start();

    kdDebug() << QString( "Resolution started with IP %1 and port %2" ).arg( address ).arg( port ) << endl;
}

PropDialog::~PropDialog()
{
    delete m_resolver;
}

void PropDialog::resolveHostname( const KReverseResolver & obj )
{
    m_w->lbHostname->setText( obj.node() );
}

#include "propDialog.moc"
