/*
Copyright (c) 2006, 2007, Nicolas Roffet, <nicolas-kde@roffet.com>

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
*/

#include "ksimiliboard.h"



#include <QApplication>
#include <QGridLayout>
#include <QWidget>


#include "ksimilicard.h"
#include "ksimilideckofcards.h"



KSimiliBoard::KSimiliBoard(QWidget *parent) : QFrame(parent)
{
	m_Layout = new QGridLayout;

	setLayout(m_Layout);

	m_deckOfCards = new KSimiliDeckOfCards(this);

	m_sizeBoard = 0;
	m_showedSolution = 1;
	setMinimumWidth(100);
	setMinimumHeight(50);
}


void KSimiliBoard::addCards(const int addMin, const int number2Add)
{
	int count;
	int add = addMin;

	do {
		for (int k=0; k < add; ++k) {
			int pos;
			if (m_freePlaces.count()>0) {
				// Free places in the board!
				pos = m_freePlaces.first();
				m_buttons[pos] = new KSimiliCard(this, this, m_deckOfCards->takeNewCard(), pos);
				m_freePlaces.removeFirst();
			} else {
				// Bigger board needed
				pos = m_sizeBoard;
				m_buttons[pos] = new KSimiliCard(this, this, m_deckOfCards->takeNewCard(), pos);
				m_sizeBoard++;
			}
			int j = pos/WIDTH_BOARD;
			m_Layout->addWidget( m_buttons[pos], j, pos - (j) * WIDTH_BOARD );
		}
		add = number2Add;
		count = countCombinations();
	} while ( ( (count==0) || ( (m_sizeBoard - m_freePlaces.count())< m_sizeBoardNormal ) ) && (add>0) );

	unHighlightHintAll();
	emit possibleCombinations(count);
}


void KSimiliBoard::chooseNewCards(const int number)
{
	m_sizeBoardNormal = number;
	
	for (int i = m_sizeBoard-1; i >=0 ; --i) {
		if (m_buttons[i]!=0)
			removeCard(m_buttons[i]);
	}
	m_selectedCards.clear();
	m_highlightedOkCards.clear();
	m_highlightedNotOkCards.clear();
	unHighlightHintAll();
	addCards(number, CARD_NUMBER_TO_ADD);
}


bool KSimiliBoard::compareCards(KSimiliCard *card1, KSimiliCard *card2, KSimiliCard *card3)
{
	if ((card1==0) || (card2==0) || (card3==0))
		return false;
	else {
		if ((((card1->form + card2->form + card3->form) % 3)==0) && (((card1->color + card2->color + card3->color) % 3)==0) && (((card1->number + card2->number + card3->number) % 3)==0) && (((card1->filling + card2->filling + card3->filling) % 3)==0))
			return true;
		else
			return false;
	}
}


int KSimiliBoard::countCombinations()
{
	// initialisation
	int found = 0;
	m_solutions.clear();
	m_numberOfDifferentCardsInKSimilis = 0;

	// Search of ksimilis
	for (int i = 0; i < m_sizeBoard-2; ++i) {
		for (int j = i+1; j < m_sizeBoard-1; ++j) {
			for (int k = j+1; k < m_sizeBoard; ++k) {
				if (compareCards(m_buttons[i], m_buttons[j], m_buttons[k])) {
					found++;

					if (!m_solutions.contains(m_buttons[i]))
						m_numberOfDifferentCardsInKSimilis++;
					m_solutions.append(m_buttons[i]);

					if (!m_solutions.contains(m_buttons[j]))
						m_numberOfDifferentCardsInKSimilis++;
					m_solutions.append(m_buttons[j]);

					if (!m_solutions.contains(m_buttons[k]))
						m_numberOfDifferentCardsInKSimilis++;
					m_solutions.append(m_buttons[k]);
				}
			}
		}
	}
	
	return found;
}


void KSimiliBoard::removeCard(KSimiliCard *card)
{
	m_deckOfCards->removeCardFromBoard(card->getCardId());
	int pos = card->position;
	if (pos==(m_sizeBoard-1)) {
		// Last card on the board
		m_sizeBoard--;
	} else
		m_freePlaces.append(pos);

	m_highlightedHintCards.removeAll(card);

	card->close();
	m_buttons[pos] = 0;
}


void KSimiliBoard::selectionChanged(KSimiliCard *card, const bool selection)
{
	if (selection) {
		if (!m_selectedCards.contains(card)) {
			m_selectedCards.append(card);
			m_highlightedNotOkCards.removeAll(card);
		}
	} else {
		m_selectedCards.removeAll(card);
	}

	if ((m_selectedCards.count()==3) && m_gameRunning) {
		emit selectionOf3Cards(m_selectedCards);
		if (compareCards(m_selectedCards[0],m_selectedCards[1],m_selectedCards[2])) {
			emit newCombinationFound();
			while (!m_selectedCards.isEmpty()) {
				m_highlightedOkCards.append(m_selectedCards.last());
				m_selectedCards.last()->highlightOk(true);
			}
		} else {
			emit newErrorFound();
			while (!m_selectedCards.isEmpty()) {
				m_highlightedNotOkCards.append(m_selectedCards.last());
				m_selectedCards.last()->highlightNotOk(true);
			}
		}
	}
}


void KSimiliBoard::setGameRunning(const bool gameRunning)
{
	m_gameRunning = gameRunning;
}


void KSimiliBoard::showHint()
{
	int hint;

	// Try to find a card that is not highlighted yet, if possible
	do {
		hint = rand() % m_solutions.count();
	} while((m_numberOfDifferentCardsInKSimilis>m_highlightedHintCards.count()) && (m_highlightedHintCards.contains(m_solutions[hint])));

	// unhighlight all cards
	unHighlightHintAll();

	// Then highlight the founded card
	m_highlightedHintCards.append(m_solutions[hint]);
	m_solutions[hint]->highlightHint(true);
}


void KSimiliBoard::showNextSolution()
{
	m_showedSolution++;
	if (m_showedSolution>m_solutions.count()/3)
		m_showedSolution = 1;
	// Note: assumption that there is at least a solution on the board


	// unselect all selected cards first
	unHighlightHintAll();

	// Then select a valid ksimili
	for (int i = 0; i<3; i++) {
		m_highlightedHintCards.append(m_solutions[((m_showedSolution-1)*3)+i]);
		m_solutions[((m_showedSolution-1)*3)+i]->highlightHint(true);
	}
}


void KSimiliBoard::timeLeftChanged()
{
	// Cards highligted as not found have to be displayed as normal cards.
	// They may be selected.
	while (!m_highlightedNotOkCards.isEmpty()) {
		KSimiliCard *c = m_highlightedNotOkCards.last();
		c->highlightNotOk(false);
		m_highlightedNotOkCards.removeAll(c);
	}

	// Cards highligted as found have to be removed of the board.
	// TODO: Use another timer to have a constant display time.
	if (!m_highlightedOkCards.isEmpty()) {
		while (!m_highlightedOkCards.isEmpty()) {
			KSimiliCard *c = m_highlightedOkCards.last();
			removeCard(c);
//			c->highlightOk(false);
			m_highlightedOkCards.removeAll(c);
		}
		addCards(0, CARD_NUMBER_TO_ADD);
	}
}


void KSimiliBoard::unHighlightHintAll() {
	while (!m_highlightedHintCards.isEmpty()) {
		KSimiliCard *c = m_highlightedHintCards.last();
		m_highlightedHintCards.removeAll(c);
		c->highlightHint(false);
	}
}

#include "ksimiliboard.moc"
