/*
Copyright (c) 2006, 2007, Nicolas Roffet, <nicolas-kde@roffet.com>

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
*/

#ifndef KSIMILIBOARD_H
#define KSIMILIBOARD_H



#define SUM_CARDS 81


#include <QFrame>
class QGridLayout;
#include <QList>
class QWidget;


class KSimiliCard;
class KSimiliDeckOfCards;



/**
 * @brief This is the board of the game.
 *
 * The cards are displayed on the board.
 */
class KSimiliBoard : public QFrame
{
	Q_OBJECT

	public:
		static int const WIDTH_BOARD = 4;
		static int const CARD_NUMBER_TO_ADD = 3;


		KSimiliBoard(QWidget *parent);

		
		void addCards(const int addMin, const int number2Add);
		void chooseNewCards(const int number);

		/**
		* @brief Check if the 3 cards are a valid ksimili.
		* 
		* true if KSimili, false if not
		*/
		bool compareCards(KSimiliCard *card1, KSimiliCard *card2, KSimiliCard *card3);

		/**
		* @brief Count the number of possible combinations on the board.
		*
		* This is useful to help the user know, if there are a lot of combinations (in this case it is quite easy to find one of them) or juste few of them (in this case it is much harder to find any of them).
		*/
		int countCombinations();

		void removeCard(KSimiliCard *card);
		void selectionChanged(KSimiliCard *card, const bool selection);
		void setGameRunning(const bool gameRunning);

		/**
		* @brief Show a hint
		* 
		* Highlight a card that is part of a (any) ksimili.
		*/
		void showHint();
		
		/**
		* @brief Show another solution
		*/
		void showNextSolution();

		/**
		* @brief Remove highlight (as hint) to all cards
		*
		* Remove the highlight of all the cards that are highlighted as hint.
		*/
		void unHighlightHintAll();


	public slots:
		void timeLeftChanged();


	signals:
		void newCombinationFound();
		void newErrorFound();
		void possibleCombinations(int ksimilis);
		void selectionOf3Cards(QList <KSimiliCard *>);


	private:
		// There are 81 different cards in the game
		KSimiliCard *m_buttons[SUM_CARDS];

		QList <KSimiliCard *> m_selectedCards;
		QList <KSimiliCard *> m_highlightedOkCards;
		QList <KSimiliCard *> m_highlightedNotOkCards;
		QList <KSimiliCard *> m_highlightedHintCards;
		QList <int> m_freePlaces;
		QList <KSimiliCard *> m_solutions;

		QGridLayout *m_Layout;

		int m_sizeBoard;
		int m_sizeBoardNormal;
		int m_showedSolution;

		// Used to choose every time a different hint
		int m_numberOfDifferentCardsInKSimilis;

		KSimiliDeckOfCards *m_deckOfCards;

		bool m_gameRunning;
};

#endif // KSIMILIBOARD_H
