/*
Copyright (c) 2006, 2007, Nicolas Roffet, <nicolas-kde@roffet.com>

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU Library General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
*/

#include "ksimilicard.h"



#include <QMouseEvent>
#include <QWidget>


#include <klocale.h>


#include "ksimiliboard.h"


KSimiliCard::KSimiliCard(QWidget *parent, KSimiliBoard *board, const cardForm form, const int color, const int number, const cardFilling filling, const int position) : QSvgWidget(parent)
{
	m_board = board;
	this->position = position;
	setAttribute( Qt::WA_DeleteOnClose );

	// cardId = form * 3^0 + color * 3^1 + (number - 1) * 3^2 + filling * 3^3
	setNewCardId(form*3 + color*3 + (number - 1)*9 + filling*27);
}


KSimiliCard::KSimiliCard(QWidget *parent, KSimiliBoard *board, const int cardId, const int position)
	: QSvgWidget(parent)
{
	m_board = board;
	this->position = position;
	setAttribute( Qt::WA_DeleteOnClose );

	setNewCardId(cardId);
}


void KSimiliCard::setNewCardId(const int newCardId)
{
	QString fillingTxt;
	QString colorTxt;
	QString formTxt;

	m_cardId = newCardId;
	
	if (m_cardId==-1) {
		filling = specialFilling;
		color = 3;
		form = specialForm;
		number = 0;
	} else {
		// cardId = form * 3^0 + color * 3^1 + (number - 1) * 3^2 + filling * 3^3
		switch (m_cardId / 27) {
			case 0:
				filling = opaque;
				fillingTxt = "opaque";
				break;
			case 1:
				filling = shading;
				fillingTxt = "shading";
				break;
			case 2:
				filling = transparent;
				fillingTxt = "transparent";
				break;
		}
		number = (m_cardId % 27) / 9 + 1;
		color = (m_cardId % 9) / 3;
		switch (color) {
			case 0:
				colorTxt = "blue";
				break;
			case 1:
				colorTxt = "red";
				break;
			case 2:
				colorTxt = "orange";
				break;
		}
		switch (m_cardId % 3) {
			case 0:
				form = ellipse;
				formTxt = "ellipse";
				break;
			case 1:
				form = rectangle;
				formTxt = "rectangle";
				break;
			case 2:
				form = losange;
				formTxt = "losange";
				break;
		}
		
		//TODO: Change this for i18n: setWhatsThis(i18n("This is a card with <b>%1 %2 %3 %4(s)</b>.", QByteArray::number(number), colorTxt, fillingTxt, formTxt));
	}

	m_selected = false;
	m_highlightedOk = false;
	m_highlightedNotOk = false;
	m_highlightedHint = false;

	createSvg(form, color, number, filling);
}


int KSimiliCard::getCardId()
{
	return m_cardId;
}


void KSimiliCard::createSvg(const cardForm form, const int color, const int number, const cardFilling filling)
{
	char const *colorTxt[] = {"blue", "red", "orange", "white"};

	QByteArray svgStart("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n <svg xmlns:dc=\"http://purl.org/dc/elements/1.1/\" xmlns:cc=\"http://web.resource.org/cc/\"  xmlns:rdf=\"http://www.w3.org/1999/02/22-rdf-syntax-ns#\" xmlns:svg=\"http://www.w3.org/2000/svg\" xmlns=\"http://www.w3.org/2000/svg\" width=\"200\" height=\"200\" version=\"1.0\">\n");


	QByteArray xmlNormalBorder("<rect  style=\"opacity:1;fill:none;fill-opacity:1;stroke:black;stroke-width:2;stroke-miterlimit:4;stroke-dasharray:2, 2;stroke-dashoffset:0;stroke-opacity:0.5\" width=\"180\" height=\"180\" x=\"10\" y=\"10\" />");

	QByteArray xmlSelectedBorder("<rect  style=\"opacity:1;fill:none;fill-opacity:1;stroke:black;stroke-width:8;stroke-miterlimit:4;stroke-opacity:1\" width=\"180\" height=\"180\" x=\"10\" y=\"10\" />");

	QByteArray xmlPartOfKSimiliBorder("<rect  style=\"opacity:0.8;fill:#beff3d;fill-opacity:1;stroke:none\" width=\"180\" height=\"180\" x=\"10\" y=\"10\" />");

	QByteArray xmlNotPartOfKSimiliBorder("<rect  style=\"opacity:0.8;fill:#ff3700;fill-opacity:1;stroke:none\" width=\"180\" height=\"180\" x=\"10\" y=\"10\" />");

	// Color RAL 1003
	QByteArray xmlHintBorder("<rect  style=\"opacity:0.8;fill:#ffcc00;fill-opacity:0.5;stroke:none\" width=\"180\" height=\"180\" x=\"10\" y=\"10\" />");



	QByteArray svg("");

	svg.append(getXmlDefs(colorTxt[color], filling)).append(" <g style=\"opacity:1\">\n ");
	switch(number) {
	case 0:
		//TODO: Add a nice sign like "?"
		break;
        case 1:
		svg.append(getXmlForm(form, colorTxt[color], filling, 60));
		break;
        case 2:
		svg.append(getXmlForm(form, colorTxt[color], filling, 30)).append(getXmlForm(form, colorTxt[color], filling, 90));
		break;
        case 3:
		svg.append(getXmlForm(form, colorTxt[color], filling, 0)).append(getXmlForm(form, colorTxt[color], filling, 60)).append(getXmlForm(form, colorTxt[color], filling, 120));
		break;
	}

	m_cardSvg = "";
	m_selectedSvg = "";

	m_cardSvg.append(svgStart).append(svg).append(xmlNormalBorder).append("</g>\n </svg>\n");
	m_selectedSvg.append(svgStart).append(svg).append(xmlSelectedBorder).append("</g>\n </svg>\n");

	m_partOfKSimiliSvg.append(svgStart).append(xmlPartOfKSimiliBorder).append(svg).append("</g>\n </svg>\n");
	m_notPartOfKSimiliSvg.append(svgStart).append(xmlNotPartOfKSimiliBorder).append(svg).append("</g>\n </svg>\n");
	m_hintSvg.append(svgStart).append(xmlHintBorder).append(svg).append("</g>\n </svg>\n");

	// The first time, we load the card
	updateSvg();
}


void KSimiliCard::mousePressEvent ( QMouseEvent* )
{
	if (!m_highlightedOk)
		selectCard(!m_selected);
}


QByteArray KSimiliCard::getXmlDefs(const QByteArray& color, cardFilling filling)
{
	QByteArray defs("");

	if (filling==shading)
		defs.append("<defs id=\"defs4\">\n <linearGradient id=\"lg\"><stop style=\"stop-color:white;stop-opacity:1;\" offset=\"0\" /><stop style=\"stop-color:").append(color).append(";stop-opacity:0.5;\" offset=\"1\" />\n </linearGradient>\n </defs>\n");

	return defs;
}


QByteArray KSimiliCard::getSvgStyleFill(const QByteArray& color, cardFilling filling )
{
	QByteArray style("opacity:1;");

	switch(filling) {
        case opaque:
		style.append("fill-opacity:1;fill:").append(color).append(";fill-rule:evenodd");
		break;
        case shading:
		style.append("fill:url(#lg)");
		break;
        case transparent:
		style.append("fill:none");
		break;
	case specialFilling:
		style.append("fill:none");
		break;
        }

	style.append(";stroke:").append(color).append(";stroke-width:4;stroke-opacity:1;stroke-dasharray:none;stroke-miterlimit:4");

	return style;
}


QByteArray KSimiliCard::getXmlForm(cardForm form, const QByteArray& color, cardFilling filling , int offsetX)
{
	QByteArray xmlForm("");
	int x = 0;

	switch(form) {
		case 0:
			x = 40 + offsetX;
			xmlForm.append(" <path style=\"").append(getSvgStyleFill(color, filling)).append("\" d=\"M ").append(QByteArray::number(x)).append(",180 A 20,80 0 0 0 ").append(QByteArray::number(x)).append(",20 A 20,80 0 0 0 ").append(QByteArray::number(x)).append(",180 z\"  />\n");
			break;
		case 1:
			x = 20 + offsetX;
			xmlForm.append("<rect style=\"").append(getSvgStyleFill(color, filling)).append("\" width=\"40\" height=\"160\" x=\"").append(QByteArray::number(x)).append("\" y=\"20\" />\n ");
			break;
		case 2:
			x = 40 + offsetX;
			xmlForm.append("<path style=\"").append(getSvgStyleFill( color, filling)).append("\" d=\"M ").append(QByteArray::number(x)).append(",20 l -20,80 l 20,80 l 20,-80 l -20,-80  z \" />\n ");
			break;
		default:
			break;
	}

	return xmlForm;
}


void KSimiliCard::updateSvg() {
	if (m_selected)
		load(m_selectedSvg);
	else {
		if (m_highlightedOk)
			load(m_partOfKSimiliSvg);
		else {
			if (m_highlightedNotOk)
				load(m_notPartOfKSimiliSvg);
			else {
				if (m_highlightedHint)
					load(m_hintSvg);
				else
					load(m_cardSvg);
			}
		}	
	}
	update();
}



//
// Slots
//

void KSimiliCard::selectCard(bool selected)
{
	if ((selected!=m_selected) && (m_board!=NULL)) {
		m_selected = selected;
		if (m_selected) {
			m_highlightedOk = false;
			m_highlightedNotOk = false;
		}
		updateSvg();
		m_board->selectionChanged(this, m_selected);
	}
}


void KSimiliCard::highlightOk(bool highlight)
{
	if (highlight!=m_highlightedOk) {
		m_highlightedOk = highlight;
		if (highlight)
			selectCard( false );
		updateSvg();
	}
}


void KSimiliCard::highlightNotOk(bool highlight)
{
	if (highlight!=m_highlightedNotOk) {
		m_highlightedNotOk = highlight;
		if (highlight) {
			selectCard( false );
		}
		updateSvg();
	}
}


void KSimiliCard::highlightHint(bool highlight)
{
	if (highlight!=m_highlightedHint) {
		m_highlightedHint = highlight;
		updateSvg();
	}
}

#include "ksimilicard.moc"
