/*
    This file is part of KSpiral.
    Copyright 2009 Felix Lemke <hobbyblobby@arcor.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef KSPIRAL_GAMEENGINE_H
#define KSPIRAL_GAMEENGINE_H

#include <QObject>
#include <cmath>
class QGraphicsItem;
class SvgReader;
class QPainterPath;
#include <QRectF>
class QTimer;

namespace KSpiral {
	class Ball;
	class GameWindow;
	class GameEngine : public QObject {
		Q_OBJECT
		public:
			GameEngine(SvgReader* path, KSpiral::GameWindow* window);
			~GameEngine();

			void startGame(int duration = 0);
			void setBarriers(const QList<QPainterPath* >& barriers) {m_barrierItems = barriers;}
			void setIgnoredItems(const QList<QGraphicsItem* >& items) {m_ignoredItems = items;}
			void setMaxBallcount(int count) {m_maxBallcount = count;}

		private:
			void ballCollided(KSpiral::Ball* cannonball, KSpiral::Ball* chainBall);
			void ballAtBorder(KSpiral::Ball* cannonball);

			void deleteBall(KSpiral::Ball* ball);
			void deleteBalls(QList<KSpiral::Ball* > balls);

			qreal euklid(const QPointF& point) {return sqrt(pow(point.x(),2) + pow(point.y(),2));}
			qreal euklid(const qreal& x, const qreal& y) {return euklid(QPointF(x, y));}

			void gameEnds();

			KSpiral::Ball* ballAtIndex(int index) const; //return the ball, which is on position [index] from the first ball (firstBall = 0)

			void executeChainStep(qreal distance, KSpiral::Ball* startBall, int direction);
			//startBall: from this Ball the chain moves (until find start/end ball)
			//direction: from startBall you can "push" or "pull" the chain, +1 means, the chainballs coming next are moved; -1 means, the chainballs before startball are moved;
			void removeTypeCombo(KSpiral::Ball* refBall);

		public Q_SLOTS:
			void chainStep(qreal distance = 0);
			void cannonballStepped(KSpiral::Ball* cannonball);
			void arrangeStep(KSpiral::Ball* cannonball);

			void ballsLost(int count);
		Q_SIGNALS:
			void gameIsOver(int score);

		private:
			SvgReader* m_path;
			int m_score;
			int m_maxBallcount;
			int m_ballCounter;
			//ball arrange
// 			int m_arrangePosition;
// 			QPointF m_insertPos;
			//new structure
			KSpiral::GameWindow* m_window;
			KSpiral::Ball* m_firstBall;
			KSpiral::Ball* m_lastBall;
			//main chain animation
			QList<KSpiral::Ball* > m_cannonBalls;
			bool m_isInChainStep;
			QTimer* m_timer;
			qreal m_distancePerStep;
			//important constants
			qreal m_ballDiameter;
			//colliding check
			QList<QGraphicsItem* > m_ignoredItems;
			QList<QPainterPath* > m_barrierItems;
	};
}

#endif //KSPIRAL_GAMEENGINE_H
