/*
    This file is part of KSpiral.
    Copyright 2009 Felix Lemke <hobbyblobby@arcor.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef KSPIRAL_GAMEWINDOW_H
#define KSPIRAL_GAMEWINDOW_H

#include <KXmlGuiWindow>
class QGraphicsItem;
class QGraphicsRectItem;
class QGraphicsScene;
class KGameRenderedItem;
class KGameRenderedObjectItem;
class QGraphicsView;
class QMouseEvent;
#include <QStringList>
class KGameRenderer;
class KActionMenu;
class SvgReader;

namespace KSpiral {
	class Ball;
	class Cannon;
	class GameEngine;
	class MouseView;

    enum GameState {
        beforeFirstStart,
        inGame
    };

	class GameWindow : public KXmlGuiWindow {
		Q_OBJECT
		public:
			GameWindow();
			virtual ~GameWindow();

			virtual void mouseMoveEvent(QMouseEvent* event);
			virtual void mousePressEvent(QMouseEvent* );

			KSpiral::Ball* addBall(KSpiral::Ball* newBall);
			KSpiral::Ball* addBall(const QPointF& point, QString ballType = QString());

			QString randomBalltype() const {return m_ballTypes.value(qrand() % m_ballTypes.size());}

        protected:
            virtual void resizeEvent(QResizeEvent* event);

		public Q_SLOTS:
			void startGame();
			void showHighscores();
			void gameIsOver(int score);
			void difficultyChanged();

		private:
			void drawGameElements();

			//drawing elements
			QString m_imageFile;
			KSpiral::MouseView* m_view;
			QGraphicsScene* m_scene;
			KGameRenderedObjectItem* m_background;
			KGameRenderedObjectItem* m_cannonTop;
      //background elements (before start game)
      QGraphicsRectItem* m_backRect;
			//game elements
      KSpiral::GameState m_gameState;
			KSpiral::GameEngine* m_engine;
			KSpiral::Cannon* m_cannon;
			int m_cannonSpeed;
			SvgReader* m_path;
			KGameRenderer* m_renderer;
			QStringList m_ballTypes;
			QString m_levelName;
			QString m_unlocalizedLevelName;
			//needed by gameEgine:
			QStringList m_barriers;
			QList<QPainterPath* > m_barrierItems;
			QList<QGraphicsItem* > m_ignoredItems;
			QList<int > m_maxBallcount;
			QList<int > m_duration;
	};
}

#endif //KSPIRAL_GAMEWINDOW_H
