/*
    This file is part of KSpiral.
    Copyright 2009 Felix Lemke <hobbyblobby@arcor.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include "svgreader.h"
#include <cmath>
#include <kstandarddirs.h>
#include <QFile>
#include <QDomDocument>
#include <QDomElement>
#include <QDomNode>
#include <QDomNodeList>
#include <QDomNamedNodeMap>
#include <QGraphicsView>
#include <QGraphicsScene>
#include <QPainterPath>

SvgReader::SvgReader()
  : QPainterPath()
{
}

SvgReader::~SvgReader()
{
}

// QPointF SvgReader::pointAtLength(const qreal& length) const
// {
// 	return pointAtPercent(percentAtLength(length));
// }

// QPointF SvgReader::C(const qreal& t) {
// // 	return pointAtLength(t * length());
// 	return pointAtPercent(t);
// }

void SvgReader::openPathFromSvgFile (QFile* file, const QString& id )
{
  QString pathdata("");
  QDomDocument* document = new QDomDocument();
  document->setContent(file);
  QDomNodeList nodes(document->elementsByTagName("path"));
  for(int i = 0; i < nodes.count(); ++i) {
    QDomNode node(nodes.at(i));
    if (node.isElement()) {
      QDomElement element(node.toElement());
      if (element.attribute("id") == id)
	if (element.hasAttribute("d"))
	  pathdata = element.attribute("d");
    }
  }
  QStringList way = pathdata.split(QRegExp("[,\\s]")); //TODO: Parsing ohne Leerzeichen erlauben

  QPointF lastControlPoint(0.0, 0.0);
  for(int i = 0; i < way.count(); ++i) {
    if (way[i] == "M") {
      moveTo(way[i+1].toFloat(), way[i+2].toFloat());
      lastControlPoint = currentPosition();
      i += 2;
    } else if (way[i] == "m" ) {
      moveTo(currentPosition() + QPointF(way[i+1].toFloat(), way[i+2].toFloat()));
      lastControlPoint = currentPosition();
      i += 2;
    } else if (way[i] == "L") {
      lineTo(way[i+1].toFloat(), way[i+2].toFloat());
      lastControlPoint = currentPosition();
      i += 2;
    } else if (way[i] == "l") {
      lineTo(QPointF(way[i+1].toFloat(), way[i+2].toFloat()) + currentPosition());
      lastControlPoint = currentPosition();
      i += 2;
    } else if (way[i] == "H") {
      lineTo(way[i+1].toFloat(), currentPosition().y());
      lastControlPoint = currentPosition();
      ++i;
    } else if (way[i] == "h") {
      lineTo(QPointF(way[i+1].toFloat(), 0.0) + currentPosition());
      lastControlPoint = currentPosition();
      ++i;
    } else if (way[i] == "V") {
      lineTo(currentPosition().x(), way[i+1].toFloat());
      lastControlPoint = currentPosition();
      ++i;
    } else if (way[i] == "v") {
      lineTo(QPointF(0.0, way[i+1].toFloat()) + currentPosition());
      lastControlPoint = currentPosition();
      ++i;
    } else if (way[i] == "C") {
      cubicTo(way[i+1].toFloat(), way[i+2].toFloat(), //Anfasser 1
		   way[i+3].toFloat(), way[i+4].toFloat(), //Anfasser 2
		   way[i+5].toFloat(), way[i+6].toFloat());//Endpunkt
      lastControlPoint = QPointF(way[i+3].toFloat(), way[i+4].toFloat());
      i += 6;
    } else if (way[i] == "c") {
      cubicTo(QPointF(way[i+1].toFloat(), way[i+2].toFloat()) + currentPosition(), //Anfasser 1
		   QPointF(way[i+3].toFloat(), way[i+4].toFloat()) + currentPosition(), //Anfasser 2
		   QPointF(way[i+5].toFloat(), way[i+6].toFloat()) + currentPosition());//Endpunkt
      lastControlPoint = QPointF(way[i+3].toFloat(), way[i+4].toFloat()) + currentPosition();
      i += 6;
    } else if (way[i] == "S") { //der erste Kontrollpunkt ist der gespiegelte zweite Kontrollpunkt
				//der vorherigen Kurve. Falls das keine Kurvenoperation war, dann
				//ist der Kontrollpunkt die currentPosition
      QPointF newControlPoint(2 * currentPosition() - lastControlPoint);
      cubicTo(newControlPoint,
		   QPointF(way[i+1].toFloat(), way[i+2].toFloat()),
		   QPointF(way[i+3].toFloat(), way[i+4].toFloat()));//Endpunkt
      lastControlPoint = QPointF(way[i+1].toFloat(), way[i+2].toFloat());
      i += 4;
    } else if (way[i] == "s") { //siehe "S"
      QPointF newControlPoint(2 * currentPosition() - lastControlPoint);
      cubicTo(newControlPoint,
		   QPointF(way[i+1].toFloat(), way[i+2].toFloat()) + currentPosition(),
		   QPointF(way[i+3].toFloat(), way[i+4].toFloat()) + currentPosition());
      lastControlPoint = QPointF(way[i+1].toFloat(), way[i+2].toFloat()) + currentPosition();
      i += 4;
    } else if (way[i] == "Q") {
      quadTo(way[i+1].toFloat(), way[i+2].toFloat(),
		  way[i+3].toFloat(), way[i+4].toFloat());
      lastControlPoint = QPointF(way[i+1].toFloat(), way[i+2].toFloat());
      i += 4;
    } else if (way[i] == "q") {
      quadTo(QPointF(way[i+1].toFloat(), way[i+2].toFloat()) + currentPosition(),
		  QPointF(way[i+3].toFloat(), way[i+4].toFloat()) + currentPosition());
      lastControlPoint = QPointF(way[i+1].toFloat(), way[i+2].toFloat()) + currentPosition();
      i += 4;
    } else if (way[i] == "T") { //aehnlich wie S
      QPointF newControlPoint(2 * currentPosition() - lastControlPoint);
      quadTo(newControlPoint,
		  QPointF(way[i+1].toFloat(), way[i+2].toFloat()));
      lastControlPoint = newControlPoint;
      i += 2;
    } else if (way[i] == "t") { //siehe "S"
      QPointF newControlPoint(2 * currentPosition() - lastControlPoint);
      quadTo(newControlPoint,
		  QPointF(way[i+1].toFloat(), way[i+2].toFloat()) + currentPosition());
      lastControlPoint = newControlPoint;
      i += 2;
    } else if (way[i] == "Z" || way[i] == "z") {
      closeSubpath();
      lastControlPoint = currentPosition();
    }
  }
}


QPointF SvgReader::C(qreal t) {
	return pointAtPercent(percentAtLength(t));
}

QPointF SvgReader::CAtPercent(qreal progress) {
	return C(progress / length());
}
