/*
    This file is part of KSpiral.
    Copyright 2009 Felix Lemke <hobbyblobby@arcor.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef SVGREADER_H
#define SVGREADER_H

#include <QFile>
#include <QMap>
#include <QPainterPath>

class SvgReader : public QPainterPath {
  public:
    SvgReader();
    ~SvgReader();

    void openPathFromSvgFile(QFile* file, const QString& id);
		/**
		* @brief use this function to get the path pointwise
		*
		* this function will use the QPainterPath::pointAtPercent function. The problem of this function is, that the distance between point isn't constant, so this function will normalize this distances
		* @return ruturns the point lying on the path
		* @param progress takes the progress between 0 and 1
		*/
//     QPointF pointAtLength(const qreal& length) const;
		QPointF C(qreal t); //t between 0 and length() as distance from start
		QPointF CAtPercent(qreal progress); //progress between 0 and 1
		/**
		* Use this functions to get point at the path in distance to a point given by the progress. The difference between the function is, that the distance on the one hand is calculated using the air-line distance and on the other hand using the real distance along the path.
		* @return returns the Point lying at the path in distance to the point at progress
		* @param progress use for origin Point
		* @param distance is the distance between the given progress and the return point
		*/
// 		QPointF pointAtLinearDistance(const qreal& progress, const qreal& distance) const {};
// 	private:
};

#endif // SVGREADER_H
