/* -*- coding: utf-8 -*- */
/* This file is part of Ktank.
   Created by sacha schutz and olivier gueudelot

   KTank is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation, version 2.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
   02110-1301, USA
*/

#include "ktblockitem.h"
 #include <QGraphicsSceneMouseEvent>
#include <QDebug>
#include <KMenu>
#include <KStandardAction>
#include <math.h>
//=================================================================
KTBlockItem::KTBlockItem(QGraphicsItem * parent ):QGraphicsRectItem(parent)
{

setFlags(QGraphicsItem::ItemIsMovable|QGraphicsItem::ItemIsSelectable);
setAcceptHoverEvents (true);
grabbed = false;
grab    = NO;
vBehavior = "BLOCKS";
vTexturePath = "no_texture";
pix     = QPixmap(10,10);
repeatTex = QPoint(1,1);
pix.fill(QColor(255,0,0,50));



name    ="no_name";
}

//================================================================
void KTBlockItem::paint ( QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget ) 
{





   painter->drawPixmap (option->rect, pix);






if ( isSelected () == true)
drawGrapper(painter,option);




}

//=================================================================
QPixmap KTBlockItem::copyPix(QPixmap original, int rx, int ry)
{



QPixmap final(original.width()*rx, original.height()*ry);
final.fill(Qt::transparent);
QPainter painter(&final);

for ( int y=0; y< original.height()*ry; y+=original.height())
  {
    for (int x=0; x<original.width()*rx; x+=original.width() )
    {
    painter.drawPixmap(x,y,original.width(),original.height(), original);
    }
}
return final;
}



//=================================================================
void KTBlockItem::drawGrapper(QPainter * painter, const QStyleOptionGraphicsItem * option)
{
int sc = 4;   //size of corner

if ( grab != NO)
{
QString strRes = QString::number(option->rect.width())+"x"+QString::number(option->rect.height());
painter->drawText(option->rect, Qt::AlignCenter, strRes);
}
painter->setBrush(QColor(0,255,0,50));
painter->setPen(Qt::NoPen);
painter->drawRect(option->rect.x(),option->rect.y(),option->rect.width(),sc );
painter->drawRect(option->rect.x(),option->rect.y()+option->rect.height(),option->rect.width(),-sc );

painter->drawRect(option->rect.x(),option->rect.y(),sc,option->rect.height());
painter->drawRect(option->rect.x()+option->rect.width(),option->rect.y(),-sc,option->rect.height());
}
//=====================================================================
void KTBlockItem::mouseMoveEvent ( QGraphicsSceneMouseEvent * event )
{

QPointF mouse = event->pos();
if ( grab != NO)
{

mouse.setX( arr(mouse.x(),10) );
mouse.setY( arr(mouse.y(),10) );
}

//resize!
switch ( grab)
{
case NO : QGraphicsRectItem::mouseMoveEvent(event); 
	  setPos(arr(pos().x(),10), arr(pos().y(),10));
	  break;

case HD : setRect(rect().x(),
		  rect().y(), 
		  rect().width(),
		    mouse.y()-rect().y());
		  break;

case HU : setRect(rect().x(),
		  mouse.y(),
		  rect().width(),
		  rect().height()+(rect().y()-mouse.y()));
		  break;

case VR : setRect(rect().x(), 
		  rect().y(),
		  mouse.x()-rect().x(), 
		  rect().height());
		  break;

case VL : setRect(mouse.x(),
		  rect().y(), 
		  rect().width()+(rect().x()-mouse.x()),
		  rect().height());
		  break;


case CDR :setRect(rect().x(),
		  rect().y(),
		  mouse.x()-rect().x(), 
		  mouse.y()-rect().y());
		  break; 

case CUR :setRect(rect().x(),
		  mouse.y(),
		  mouse.x()-rect().x(), 
		  rect().height()+(rect().y()-mouse.y()));
		  break;

case CDL :setRect(mouse.x(),
		  rect().y(),
		  rect().width()+(rect().x()-mouse.x()),
		  mouse.y()-rect().y());
		  break; 

case CUL :setRect(mouse.x(), 
		  mouse.y(),
		  rect().width()+(rect().x()-mouse.x()),
		  rect().height()+(rect().y()-mouse.y()));
		  break; 

}


//TEMPORARY ... IF somebody can correct it!! Currently, if we grab in bad side, with mouse, it's bug! Because coord is < 0.
setRect(abs(rect().x()), 
	abs(rect().y()),
	abs(rect().width()),
	abs(rect().height()));


update();



}
//===========================================================================
void KTBlockItem::hoverMoveEvent ( QGraphicsSceneHoverEvent * event )
{

grab = getCorner(event->pos());

if ( (grab == CUL) || (grab == CDR))
setCursor(Qt::SizeFDiagCursor);

if ( (grab == CUR) || (grab == CDL))
setCursor(Qt::SizeBDiagCursor);

if ( (grab == HU) || (grab == HD))
setCursor(Qt::SizeVerCursor);

if ( (grab == VL) || (grab == VR))
setCursor(Qt::SizeHorCursor);

if ( grab == NO)
setCursor(Qt::ArrowCursor);




QGraphicsRectItem::hoverEnterEvent(event);


}
//=====================================================================
   void KTBlockItem::contextMenuEvent(QGraphicsSceneContextMenuEvent *event)
  {
	if ( isSelected())
	{
KMenu menu("The Box");
menu.addAction("Copy"); 	
menu.addAction("Cut");   
menu.addAction("delete");    
menu.exec(event->screenPos());
	}    
}


//====================================================================
corner KTBlockItem::getCorner( QPointF mouse)
{

double x = rect().x();
double y = rect().y();
double h = rect().height();
double w = rect().width();

QRectF cul (x,y, 10,10);
QRectF cur (x+w-10,y, 10,10);
QRectF cdl (x,y+h-10,10,10);
QRectF cdr (x+w-10,y+h-10,10,10);

QRectF hu (x+10,y,w-20,10);
QRectF hd (x+10,y+h-10,w-20,10);
QRectF vl (x,y+10,10,h-20);
QRectF vr (x+w-10,y+10,10,h-20);

if ( cul.contains(mouse) ) return CUL;
if ( cur.contains(mouse) ) return CUR;
if ( cdl.contains(mouse) ) return CDL;
if ( cdr.contains(mouse) ) return CDR;
if ( hu.contains(mouse) ) return HU;
if ( hd.contains(mouse) ) return HD;
if ( vl.contains(mouse) ) return VL;
if ( vr.contains(mouse) ) return VR;

return NO;
}

//==========================================================================
KTBlockItem::KTBlockItem(const KTBlockItem &original)  //Constructeur de copy
{
  
  setRect(original.rect());

}

//=============================================================================

int KTBlockItem::arr(double value, int fac)
{
int val = (int)(value);

int reste  = val % fac;

if ( reste <= (int)(fac/2))
val -= reste;

if ( reste > (int)(fac/2))
val+=(fac - reste);
return val;
}
//=================================================================================
void KTBlockItem::arrondirRect(int fac)
{

setRect(
arr(rect().x(),fac),
arr(rect().y(),fac),
arr(rect().width(),fac),
arr(rect().height(),fac)
);
}
//====================================================================================
 QRectF KTBlockItem::arrondirRect( QRectF rect, int fac)
{

rect.setRect(
arr(rect.x(),fac),
arr(rect.y(),fac),
arr(rect.width(),fac),
arr(rect.height(),fac));


return rect;

}



