/***************************************************************************
 *   Copyright (C) 2003-2007 by Albert Astals Cid                          *
 *   tsdgeos@terra.es                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef BOARD_H
#define BOARD_H

#include <qobject.h>
#include <qpoint.h>

#include "kcase.h"

class boardEvaluation;

class board : public QObject
{
Q_OBJECT
	// methods
	public:
		board(QObject *o, bool createKcases);

		// copies the board to another one
		board(const board *original);
		
		virtual ~board();

		// Starts a new game in the board
		virtual void newGame();

		// Ends the current game of the board
		virtual void endGame();

	signals:
		// player has win
		// int is the number of the player
		// bool = true -> it was human
		// both players where of the same kind
		void playerwin(int, bool);

		// human player wins versus the computer
		void win();

		// computer player wins versus the human
		void loose();

		// the game has ended in a draw
		void draw();

		// someone has made a movement
		void movement();

		// someone has toggled a selection
		// true the selection has been made
		// false the selection has been unmade
		void selection(bool);

	protected:
		// currentPlayer placed piece in x, y
		// next turn is also called
		void place(int x, int y);

		// currentPlayer selected x, y
		void select(int x, int y);

		// currentPlayer deselects the selected piece
		void unselect();

		// currentPlayer moves the selected piece to x, y
		void placeSelected(int x, int y);

		// returns a matrix of the size of the board with evaluations of each position
		// the higher the better for player 1
		// steps are the recursive steps to evaluate
		boardEvaluation* suggestMove(int steps = 1);

		// is the game beign played?
		bool playing();
		
		// sets if the game is beign player or not
		void setPlaying(bool b);

		// a player has moved, let's process things before the next turn comes
		virtual void nextTurn();

		// makes the move
		void moveComputer();

		// returns true if all cases are occupied i.e. the game has ended
		bool allCasesOccupied() const;

		// returns true if thre is a winner
		bool ttt() const;

		// returns true if a selected piece can be moved to the new location
		// in PARTIAL mode, if in TOTAL mode returns true
		bool canMoveFrom(int selected_x, int selected_y, int x, int y) const;

		// returns the puntuation of the current board
		int evaluate();

	private:
		// returns true if the horizontal line i is of the same player
		bool checkH(int i) const;

		// returns true if the vertical line i is of the same player
		bool checkV(int i) const;

		// returns true if the diagonal line i is of the same player
		// i = 0 is (0,0) (1,1) and (2,2)
		// i = 1 is (0,2) (1,1) and (2,0)
		bool checkD(int i) const;

		// returns the score of the horizontal line i
		int scoreH(int i) const;

		// returns the score of the horizontal line i
		int scoreV(int i) const;

		// returns the score of the horizontal line i
		// i = 0 is (0,0) (1,1) and (2,2)
		// i = 1 is (0,2) (1,1) and (2,0)
		int scoreD(int i) const;

		// returns the number of times o appears on the horizontal line i
		// if is is alone
		int numberPlayerOcurrencesH(kcase::PLAYER o, int i) const;

		// returns the number of times o appears on the vertical line i
		// if is is alone
		int numberPlayerOcurrencesV(kcase::PLAYER o, int i) const;

		// returns the number of times o appears on the diagonal line i
		// if is is alone
		// i = 0 is (0,0) (1,1) and (2,2)
		// i = 1 is (0,2) (1,1) and (2,0)
		int numberPlayerOcurrencesD(kcase::PLAYER o, int i) const;

	// atributes
	protected:
		// the cases of the board
		kcase ***cases;

		// the current player
		kcase::PLAYER currentPlayer;

		// the current game mode
		int currentMode;

		// who controls player 1
		int player1;

		// who controls player 2
		int player2;

		// how namy pieces has player 1 left, only relevant in PARTIAL or TOTAL modes
		int player1PiecesLeft;

		// how namy pieces has player 2 left, only relevant in PARTIAL or TOTAL modes
		int player2PiecesLeft;

		// is there a selected piece?? and its position
		bool selected;
		int selected_x;
		int selected_y;
		
		//is the game begin played
		bool p_playing;
};

#endif
