/***************************************************************************
 *   Copyright (C) 2003 by Albert Astals Cid                               *
 *   tsdgeos@terra.es                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef BOARDWIDGET_H
#define BOARDWIDGET_H

#include <qframe.h>

#include "board.h"

class boardWidget : public board
{
Q_OBJECT
	public:
		boardWidget(QWidget *parent);
		~boardWidget();

		// Starts a new game in the board
		void newGame();

		// Ends the current game in the board
		void endGame();

		// returns the frame of the boardWidget
		QFrame* getFrame();

	private:
		// a player has moved, let's process things before the next turn comes
		void nextTurn();

		// emits the appropriate statusbar message to say who should play next
		void changeStatusBar();

		// returns true if the current player can place a piece in x, y
		bool processClassic(int x, int y);

		// the currentPlayer is trying to do something at x, y, it can be
		// place a new piece, select and existant or move a selected one,
		// the game mode is Partial
		void processPartial(int x, int y);

		// the currentPlayer is trying to do something at x, y, it can be
		// place a new piece, select and existant or move a selected one
		// the game mode is Total
		void processTotal(int x, int y);

	public slots:
		// the settings of the config dialog have changed
		void readSettings();

	private slots:
		// the user has clicked at x y
		void boardClicked(int x, int y);

	signals:
		// the receiver should put the QString in the statusbar
		void statusBarMessage(const QString&);

	private:
		QFrame *frame;
};

#endif
