/***************************************************************************
 *   Copyright (C) 2003-2007 by Albert Astals Cid                          *
 *   tsdgeos@terra.es                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "kttt.h"

#include <kaction.h>
#include <kactioncollection.h>
#include <kconfigdialog.h>
#include <klocale.h>
#include <knotification.h>
#include <knotifyconfigwidget.h>
#include <kstatusbar.h>
#include <kstandardaction.h>
#include <kstandardgameaction.h>

#include "boardwidget.h"
#include "colorsconfig.h"
#include "generalconfig.h"
#include "settings.h"
#include "shapesconfig.h"

kttt::kttt() : KXmlGuiWindow()
{
	p_board = new boardWidget(this);
	setCentralWidget(p_board->getFrame());

	KStandardGameAction::gameNew(this, SLOT(newGame()), actionCollection());
	KStandardGameAction::end(this, SLOT(endGame()), actionCollection());
	KStandardGameAction::quit(this, SLOT(close()), actionCollection());

	KStandardAction::configureNotifications(this, SLOT(configureNotifications()), actionCollection());
	KStandardAction::preferences(this, SLOT(configureSettings()), actionCollection());

	statusBar(); /* create the statusbar so that setupGUI creates the menu entry */
	setupGUI(Keys | StatusBar | Save | Create);
	show();
	connect(p_board, SIGNAL(statusBarMessage(const QString&)), statusBar(), SLOT(message(const QString&)));
	connect(p_board, SIGNAL(playerwin(int, bool)), this, SLOT(playerwin(int, bool)));
	connect(p_board, SIGNAL(win()), this, SLOT(humanwin()));
	connect(p_board, SIGNAL(loose()), this, SLOT(humanloose()));
	connect(p_board, SIGNAL(draw()), this, SLOT(draw()));
	connect(p_board, SIGNAL(movement()), this, SLOT(movement()));
	connect(p_board, SIGNAL(selection(bool)), this, SLOT(selection(bool)));
	newGame();
}

void kttt::newGame()
{
	p_board -> newGame();
	actionCollection() -> action("game_end") -> setEnabled(true);
}

void kttt::endGame()
{
	p_board -> endGame();
	actionCollection() -> action("game_end") -> setEnabled(false);
}

void kttt::playerwin(int number, bool human)
{
	Q_ASSERT(number == 1 || number == 2);
	if (number == 1)
	{
		if (human) KNotification::event("humanwin", i18n("Player 1 wins!!!"));
		else KNotification::event("computerwin", i18n("Player 1 wins!!!"));
	}
	else
	{
		if (human) KNotification::event("humanwin", i18n("Player 2 wins!!!"));
		else KNotification::event("computerwin", i18n("Player 2 wins!!!"));
	}
	actionCollection() -> action("game_end") -> setEnabled(false);
}

void kttt::humanwin()
{
	KNotification::event("win", i18n("You win!!!"));
	actionCollection() -> action("game_end") -> setEnabled(false);
}

void kttt::humanloose()
{
	KNotification::event("loose", i18n("You lose!!!"));
	actionCollection() -> action("game_end") -> setEnabled(false);
}

void kttt::draw()
{
	KNotification::event("draw", i18n("The game is drawn!!!"));
	actionCollection() -> action("game_end") -> setEnabled(false);
}

void kttt::movement()
{
	KNotification::event("move");
}

void kttt::selection(bool select)
{
	if (select) KNotification::event("select");
	else KNotification::event("unselect");
}

void kttt::configureNotifications()
{
	KNotifyConfigWidget::configure(this);
}

void kttt::configureSettings()
{
	if ( KConfigDialog::showDialog("settings") ) return;

	KConfigDialog *dialog = new KConfigDialog(this, "settings", ktttSettings::self());
	dialog->addPage(new generalConfig(), i18n("General"), "package_settings");
	dialog->addPage(new colorsConfig(), i18n("Colors"), "colors");
	dialog->addPage(new shapesConfig(), i18n("Shapes"), "kttt");
	connect(dialog, SIGNAL(settingsChanged(QString)), p_board, SLOT(readSettings()));
	dialog->exec();
	delete dialog;
}

#include "kttt.moc"
