//Magazynier, a warehouse assistant game.
//Copyright (C) 2008, 2009 Tadeusz Andrzej Kadlubowski yess@hell.org.pl
//
//This program is free software; you can redistribute it and/or modify
//it under the terms of the GNU General Public License as published by
//the Free Software Foundation; either version 2 of the License, or
//(at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License along
//with this program; if not, write to the Free Software Foundation, Inc.,
//51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

#ifndef COMMANDS_H
#define COMMAND_H

#include <QPoint>
#include <QUndoCommand>

#include "levelModel.h"

class LevelModel;

class MagazynierCommand : public QUndoCommand
{
	public:
		explicit MagazynierCommand(LevelModel *levelModel, QUndoCommand *parent = 0) :
			QUndoCommand(parent),
			m_levelModel(levelModel),
			m_moveCount(0),
			m_shiftCount(0)
			{}

		explicit MagazynierCommand(LevelModel *levelModel, QPoint pos, QPoint delta, QUndoCommand *parent = 0) :
			QUndoCommand(parent),
			m_pos(pos),
			m_delta(delta),
			m_levelModel(levelModel),
			m_moveCount(0),
			m_shiftCount(0)
			{}

		virtual void undo();
		virtual void redo();

		void setCounts(int moveCount, int shiftCount);
	protected:
		QPoint m_pos;
		QPoint m_delta;
		LevelModel *m_levelModel;
		int m_moveCount;
		int m_shiftCount;
};

class MoveFromEmptyToEmptyCommand : MagazynierCommand
{
	public: 
		MoveFromEmptyToEmptyCommand(LevelModel *levelModel, QPoint pos, QPoint delta, QUndoCommand *parent = 0) : MagazynierCommand(levelModel, pos, delta, parent) {}
		void redo(void) { m_levelModel->moveFromEmptyToEmpty(m_pos, m_delta); }
		void undo(void) { m_levelModel->moveFromEmptyToEmpty(m_pos + m_delta, -m_delta); }
};

class MoveFromEmptyToGoalCommand : public MagazynierCommand
{
	public: 
		MoveFromEmptyToGoalCommand(LevelModel *levelModel, QPoint pos, QPoint delta, QUndoCommand *parent = 0) : MagazynierCommand(levelModel, pos, delta, parent) {}
		void redo(void) { m_levelModel->moveFromEmptyToGoal(m_pos, m_delta); }
		void undo(void) { m_levelModel->moveFromGoalToEmpty(m_pos + m_delta, -m_delta); }
};

class MoveFromGoalToEmptyCommand : public MagazynierCommand
{
	public: 
		MoveFromGoalToEmptyCommand(LevelModel *levelModel, QPoint pos, QPoint delta, QUndoCommand *parent = 0) : MagazynierCommand(levelModel, pos, delta, parent) {}
		void redo(void) { m_levelModel->moveFromGoalToEmpty(m_pos, m_delta); }
		void undo(void) { m_levelModel->moveFromEmptyToGoal(m_pos + m_delta, -m_delta); }
};

class MoveFromGoalToGoalCommand : public MagazynierCommand
{
	public: 
		MoveFromGoalToGoalCommand(LevelModel *levelModel, QPoint pos, QPoint delta, QUndoCommand *parent = 0) : MagazynierCommand(levelModel, pos, delta, parent) {}
		void redo(void) { m_levelModel->moveFromGoalToGoal(m_pos, m_delta); }
		void undo(void) { m_levelModel->moveFromGoalToGoal(m_pos + m_delta, -m_delta); }
};

#endif
