/***************************************************************************
 *   Copyright (C) 2005 by Joshua Keel <joshuakeel@gmail.com>              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include <QPainter>
#include <QPixmap>
#include <QPushButton>
#include <QBrush>
#include <QColor>
#include <QSizePolicy>

#include <kdebug.h>

#include "market.h"

Market::Market() : QWidget(0, 0)
{
	setAttribute(Qt::WA_NoBackground, true);
	setMouseTracking(true);
	setMinimumSize(640, 480);

	show();
}

Market::~Market()
{
}

void Market::paintEvent(QPaintEvent *)
{
	uint wWidth = width();
	uint wHeight = height();

	QPixmap buf(wWidth, wHeight);
	buf.fill();
	
	QPainter p(&buf);

	p.setRenderHint(QPainter::Antialiasing);

	//fill the rectangles
	p.fillRect(0, 0, wWidth, wHeight / 20, QColor(230, 230, 230));
	p.fillRect(0, wHeight / 20, wWidth / 2, wHeight - (wHeight / 20 + wHeight / 4), QColor(255, 210, 192));
	p.fillRect(wWidth / 2, wHeight / 20, wWidth / 2, wHeight - (wHeight / 20 + wHeight / 4), QColor(205, 255, 187));
		
	//draw all the lines necessary
	QPen pen("black");
	pen.setWidth(3);
	p.setPen(pen);
	p.drawLine(0, wHeight * 0.05, wWidth, wHeight * 0.05);
	p.drawLine(wWidth * 0.5, wHeight * 0.05, wWidth * 0.5, wHeight);
	pen.setWidth(2);
	p.setPen(pen);
	p.drawLine(0, wHeight * 0.75, wWidth, wHeight * 0.75);
	
	//draw first of semi-rectangles on left and right sides
	//These rectangles contain the stock information
	pen.setStyle(Qt::DotLine);
	p.setPen(pen);
	p.drawLine(0, wHeight * 0.25, wWidth * 0.42, wHeight * 0.25);
	p.drawLine(wWidth * 0.42, wHeight * 0.25, wWidth * 0.42, wHeight * 0.66);
	p.drawLine(wWidth * 0.42, wHeight * 0.66, 0, wHeight * 0.66);

	//draw second of the semi-rectangles
	p.drawLine(wWidth, wHeight * 0.25, wWidth * 0.58, wHeight * 0.25);
	p.drawLine(wWidth * 0.58, wHeight * 0.25, wWidth * 0.58, wHeight * 0.66);
	p.drawLine(wWidth * 0.58, wHeight * 0.66, width(), wHeight * 0.66);

	//draw the two center buttons
	pen.setStyle(Qt::SolidLine);
	p.setPen(pen);
	p.fillRect(wWidth * 0.45, wHeight * 0.25, wWidth * 0.10, wWidth * 0.10, QColor(52, 168, 56));
	p.drawRect(wWidth * 0.45, wHeight * 0.25, wWidth * 0.10, wWidth * 0.10);
	p.fillRect(wWidth * 0.45, wHeight * 0.66 - wWidth * 0.10, wWidth * 0.10, wWidth * 0.10, QColor(255, 96, 125));
	p.drawRect(wWidth * 0.45, wHeight * 0.66 - wWidth * 0.10, wWidth * 0.10, wWidth * 0.10);

	//blit to the screen
#warning "kde4: port it"
	//bitBlt(this, 0, 0, &buf);
}

void Market::resizeEvent(QResizeEvent *)
{
	//make sure that every time the window is resized, the paintEvent() gets called
	update();
}

//a stupid slot, serving no purpose
void Market::mySlot()
{
}

#include "market.moc"
