/***************************************************************************
 *   Copyright (C) 2008 Copyright 2008  Tom Vollerthun <vollerthun@gmx.de>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/


#include "description.h"

#include <QDomDocument>

#include <KDebug>
#include <KLocale>

const QString Description::ROOT_TAG = QString("nonogram_description");

const QString Description::NAME_TAG = "name";
const QString Description::HINT_TAG = "hint";
const QString Description::DESCRIPTION_TAG = "description";

const QString Description::VALUE_TAG = "value";

const QString Description::LOCALE_ATT = "locale";

Description * Description::read(const QByteArray & data) {

	QDomDocument d;
	d.setContent(data);

	return read(d);
}


Description * Description::read(const QString & data) {

	QDomDocument d;
	d.setContent(data);

	return read(d);
}

Description * Description::read(const QDomDocument & document)
{
	Description * retval = new Description();
	QDomElement root = document.firstChildElement(ROOT_TAG);

	/*handle the name-stuff*/
	QString name = getValueOf(root, NAME_TAG);
	retval->setName(name);

	/*handle the hint-stuff*/
	QString hint = getValueOf(root, HINT_TAG);
	retval->setHint(hint);

	/*handle description stuff*/
	QString description = getValueOf(root, DESCRIPTION_TAG);
	retval->setDescription(description);

	return retval;
}

QString Description::getValueOf(const QDomElement & el, const QString tag)
{
	QString retval;

	QDomElement name = el.firstChildElement(tag);
	for (QDomElement aValue = name.firstChildElement(VALUE_TAG); !aValue.isNull(); aValue = aValue.nextSiblingElement(VALUE_TAG)) {

		QDomAttr locale = aValue.attributeNode(LOCALE_ATT);

		if (!locale.isNull()) {
			/*TODO: prefer correct locale*/
			QString v = aValue.firstChild().nodeValue();
		} else {
			//this is the default locale. Use it if nothing else helps.
			QString v = aValue.firstChild().nodeValue();
			retval = v;
		}
	}

	return retval;
}

void Description::setName(const QString & name) {
	m_name = name;
}

const QString Description::name() const {
	return m_name;
}

void Description::setDescription(const QString & description) {
	m_description = description;
}

const QString Description::description() const {
	return m_description;
}

void Description::setHint(const QString & hint) {
	m_hint = hint;
}

const QString Description::hint() const {
	return m_hint;
}

