/***************************************************************************
 *   Copyright (C) 2008 Copyright 2008  Tom Vollerthun <vollerthun@gmx.de>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef A_GRID_H_
#define A_GRID_H_

#include <QSvgWidget>
#include <QPainter>
#include <QPen>

#include "ds/data.h"
#include <qrubberband.h>

/**
 * This class provides some common methods used by both: the two grid headers and the center.
 * It is not really abstract (all methods common). Instead it is never used on it's own, so
 * it's almost the same :)
 * The cause for not making it abstract is not technical: I can't seem to get it work.
 */
class Abstract_Grid :
    public QSvgWidget
{
    Q_OBJECT

public:
    Abstract_Grid(QWidget *parent);
    ~Abstract_Grid() {}

    Data * data();

    static int HORIZONTAL_LEFT_OFFSET;

    static int VERTICAL_UPPER_OFFSET;

public slots:
    virtual void setData(Data * data);
    void selectCell(QPoint * selectedCell);
    virtual void readConfig();
    void autoselectColor();

signals:
    void cellSelected(QPoint * selectedCell);
    void wantColor(QColor * color);
    void settingsChanged();


protected:
    void paintEvent(QPaintEvent *event);
    virtual void mouseMoveEvent(QMouseEvent * event);


    virtual bool doPaintHorizontal() = 0;
    virtual bool doPaintVertical() = 0;

    virtual void paintAdditionalHorizontal(QPainter * painter, int pos_y, int row) = 0;
    virtual void paintAdditionalVertical(QPainter * painter, int pos_x, int column) = 0;

    virtual int widthOfLastLine() = 0;
    virtual int heightOfLastLine() = 0;

    void paintHorizontal(QPainter * painter);
    void paintVertical(QPainter * painter);

    void calculatePositions();
    QPoint * getGridPositions (QMouseEvent * event);


    Data * m_data;

    QColor gridColor;
    QColor highlightColor;

    int m_cols;
    int m_rows;

    int m_maxHeight;
    int m_maxWidth;

    int m_v_spacing;
    int m_h_spacing;

    QPoint * selectedCell;
    QRubberBand * verticalRubber1;
    QRubberBand * verticalRubber2;

    QRubberBand * horizontalRubber1;
    QRubberBand * horizontalRubber2;

    bool autoselectClickColor;
    int waitBeforeAutoselect;
};

#endif /*A_GRID_H_*/
