/***************************************************************************
 *   Copyright (C) 2008 Copyright 2008  Tom Vollerthun <vollerthun@gmx.de>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef GRID_HEADER_H
#define GRID_HEADER_H

#include <QWidget>
#include <QColor>
#include <QMouseEvent>

#include "gui/a_grid.h"
#include "ds/data.h"

/**
 * This class provides a column/line-header which can display the numbers
 * of boxes that must be filled in the corresponding row.
 * On resizing, it resizes accordingly with the central grid-widget.
 *
 * The class knows it's orientation: horizontal or vertical
 */
class GridHeader : public Abstract_Grid
{
    Q_OBJECT

    public:
        GridHeader(QWidget *parent, bool isHorizontal);
        ~GridHeader()
        {
        }

    public slots:
        virtual void readConfig();

    protected:
        bool doPaintHorizontal();
        bool doPaintVertical();

        int widthOfLastLine();
        int heightOfLastLine();

        void paintAdditionalHorizontal(QPainter * painter, int pos_y, int row);
        void paintAdditionalVertical(QPainter * painter, int pos_x, int column);

        void paintHint(QPainter * painter, QRect * inset, Data::HintEntry * hintEntry);
        void mousePressEvent(QMouseEvent * event);

    private:
        void calculateHintRect(QPainter * painter);
        bool hasHorizontalOrientation;

        QRect * m_hintRect;
        QColor hintCrossthroughColor;

};

#endif
