/***************************************************************************
 *   Copyright (C) 2008 Copyright 2008  Tom Vollerthun <vollerthun@gmx.de>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#ifndef NOG_FORMAT_H_
#define NOG_FORMAT_H_

#include <QString>
#include <QXmlDefaultHandler>

#include "io/nonogram_io.h"
#include "io/nog_out.h"
#include "ds/data.h"

/**
 * This class provides the reading part of the nonogram file format.
 * Writing is performed by the NogOut class which is defined in nog_out.h
 */
class NogFormat : public NonogramIo, public QXmlDefaultHandler
{
    Q_OBJECT

    public:

        NogFormat(QObject *parent=0);

        Data * readFile(const QString & fileName);

        Data * readData(const QByteArray & data);

        bool startElement(const QString &namespaceURI,
                const QString &localName, const QString &qName,
                const QXmlAttributes &attributes);
        bool endElement(const QString &namespaceURI, const QString &localName,
                const QString &qName);
        bool characters(const QString &str);
        bool fatalError(const QXmlParseException &exception);

   signals:
       void dataRead(Data * data);

    protected:
        Data* handleXml(const QXmlInputSource &);
        QString m_currentText;
        Data * m_data;
        bool m_isGame;
};

#endif /*NOG_FORMAT_H_*/
