/***************************************************************************
 *   Copyright (C) 2008 Copyright 2008  Tom Vollerthun <vollerthun@gmx.de>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "nog_out.h"

#include <QFile>
#include <QString>
#include <QTextStream>

#include <KDebug>

const QString NogOut::GAME_TAG = "nonogram_game";

const QString NogOut::NAME_TAG = "name";
const QString NogOut::HINT_TAG = "hint";
const QString NogOut::DESCRIPTION_TAG = "description";

const QString NogOut::ROWS_TAG = "rowHints";
const QString NogOut::COLUMNS_TAG = "columnHints";
const QString NogOut::MATRIX_TAG = "matrix";


void NogOut::save(Data * data, const QString & outfile, bool saveMatrix )
{
	QFile o(outfile);
    if (o.open(QFile::WriteOnly | QFile::Truncate)) {
        QTextStream out(&o);

        QString output = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
        output.append(createOutput(data, saveMatrix) );
        out << output.toUtf8();

        o.close();
    }
    else {
        kDebug() << "Could not write to file [" << outfile << "]";
    }
}

QString NogOut::createOutput(Data * data, bool saveMatrix)
{
    Description * d = data->description();

    QString name;
    QString description;
    QString hint;

    if (d != NULL) {
        if (d->name() != NULL) {
            name = d->name();
        }

        if (d->description() != NULL) {
            description = d->description();
        }

        if (d->hint() != NULL) {
            hint = d->hint();
        }
    }

    QString retval;
    retval.append('<' + NogOut::GAME_TAG + ">\n");
    retval.append("\t<" + NogOut::NAME_TAG + '>' + name + "</"
            + NogOut::NAME_TAG + ">\n");
    retval.append("\t<" + NogOut::HINT_TAG + '>' + hint + "</"
                + NogOut::HINT_TAG + ">\n");
    retval.append("\t<" + NogOut::DESCRIPTION_TAG + '>' + description + "</"
                + NogOut::DESCRIPTION_TAG + ">\n");

    retval.append("\t<" + NogOut::ROWS_TAG + '>' + Data::toString(data->rowHints()) + "</"
                    + NogOut::ROWS_TAG + ">\n");

    retval.append("\t<" + NogOut::COLUMNS_TAG + '>' + Data::toString(data->columnHints()) + "</"
                    + NogOut::COLUMNS_TAG + ">\n");

    if (saveMatrix){
        retval.append("\t<" + NogOut::MATRIX_TAG + '>' + Data::toString(data->matrix()) + "</"
                        + NogOut::MATRIX_TAG + ">\n");
    }

    retval.append("</" + NogOut::GAME_TAG + '>');

    return retval;
}

