/***************************************************************************
 *   Copyright (C) 2008 Copyright 2008  Tom Vollerthun <vollerthun@gmx.de>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 ***************************************************************************/

#include "mainwindow.h"

#include <KApplication>
#include <KAction>
#include <KLocale>
#include <KActionCollection>

#include <KFileDialog>
#include <KMessageBox>
#include <KIO/NetAccess>
#include <KSaveFile>
#include <KStandardDirs>
#include <KDebug>
#include <KUrl>
#include <kconfigdialog.h>
#include <KXMLGUIFactory>


#include <QGridLayout>
#include <QVBoxLayout>
#include <QDir>
#include <QStringList>
#include <QImageReader>

#include <KDE/KStandardGameAction>

#include "settings.h"


#include "gui/grid.h"
#include "gui/game_info.h"
#include "gui/config/colorconfig.h"
#include "gui/config/clevernessconfig.h"

#include "ds/package.h"
#include "ds/solver.h"

#include "io/nop_format.h"
#include "io/io_factory.h"


MainWindow::MainWindow(QWidget *parent) :
    KXmlGuiWindow(parent)
{
    grid = new Grid(this);

    GameInfo * info = new GameInfo(this);
    connect(info, SIGNAL(changedGameTo(Data *)), grid, SLOT(setData(Data *)));
    info->readPackages();

    int max = 5;
    QGridLayout *layout = new QGridLayout;
    layout->addWidget(info, 0, 0);
    layout->addWidget(grid, 1, 0, max, 1);

    QWidget *window = new QWidget;
    window->setLayout(layout);

    setCentralWidget(window);
    setupActions();
    kDebug() << "Sending settingsChanged";
    emit settingsChanged();
}

void MainWindow::setupActions()
{
    // Game
    KStandardGameAction::restart(this, SLOT(newGame()), actionCollection());
    KStandardGameAction::load(this, SLOT(load()), actionCollection());
    KStandardGameAction::quit(this, SLOT(close()), actionCollection());


    KStandardAction::preferences(this, SLOT(configureSettings()), actionCollection());
    KStandardAction::keyBindings(guiFactory(), SLOT(configureShortcuts()), actionCollection());


//     statusBar(); /* create the statusbar so that setupGUI creates the menu entry */
//     setupGUI(Keys | StatusBar | Save | Create);
    createGUI();
}

void MainWindow::newGame()
{
    kDebug() << "New game requested";
    grid->data()->resetGame();
}

void MainWindow::load()
{

    QList<QByteArray> imageFormats = QImageReader::supportedImageFormats();

    QString openFormats;
    for (int i = 0; i < imageFormats.size(); ++i) {
        QString aFormat = QString(imageFormats.at(i));
        openFormats.append("*."+aFormat+' ');
    }

    QString fileNameFromDialog = KFileDialog::getOpenFileName(KUrl("kfiledialog:///load"),
            openFormats + '|' + i18n("Images"),
            this);

    kDebug() << "Opening file: "<<fileNameFromDialog;

    if (! fileNameFromDialog.isEmpty()) {
        QString tmpFile;
        if (KIO::NetAccess::download(fileNameFromDialog, tmpFile, this)) {

            Data * data = Io_Factory::readFile(tmpFile);

            grid->setData(data);

            KIO::NetAccess::removeTempFile(tmpFile);

        }
        else {
            KMessageBox::error(this, KIO::NetAccess::lastErrorString());
        }
    }
}

void MainWindow::configureSettings() {
    if ( KConfigDialog::showDialog("settings") ) return;

    KConfigDialog *dialog = new KConfigDialog(this, "settings", nonogramSettings::self());
    dialog->addPage(new ColorConfig(), i18n("Colors"), "wizard");
    dialog->addPage(new ClevernessConfig(), i18n("Cleverness"), "wizard");

    connect(dialog, SIGNAL(settingsChanged(const QString&)), SIGNAL(settingsChanged()));
    dialog->show();
}



