/* mainwindow.h

    Parsek - free client for playing 4X space strategy games
    Copyright (C) 2006-2007  Jure Repinc <jlp@holodeck1.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/
#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <kmainwindow.h>
#include <kapplication.h>
#include <kaction.h>
#include <kicon.h>
#include <kled.h>
#include <kmenu.h>
#include <kmenubar.h>
#include <kmessagebox.h>
#include <kstatusbar.h>
#include <kstandardaction.h>

#include <QObject>
#include <QDockWidget>
#include <QString>
#include <QLabel>
#include <QTableView>
#include <QTimer>
#include <QTreeView>
#include <QListView>
#include <QVariant>

#include <tpproto/gameinfo.h>
#include <tpproto/gamelayer.h>
#include <tpproto/gamestatuslistener.h>

#include "parsekstatuslistener.h"
#include "parsekeventloop.h"
#include "loggerwidget.h"
#include "connecttoserverdialog.h"
#include "statuswidget.h"
#include "connecttogamedialog.h"
#include "version.h"
#include "objectmodel.h"
#include "objectview.h"

class QLabel;
class QTimer;
class KAction;
class KLed;

using namespace std;
using namespace TPProto;

class MainWindow : public KMainWindow
{
    Q_OBJECT
public:
    MainWindow(QWidget *parent = 0);
private Q_SLOTS:
    void connectToServer();
    void disconnect();
    void quitGame();
    void updateClock();
    void flashClock();

    //Status Watcher Signals
    void connected();
    void disconnected();
    void accountCreated(bool);
    void loggedIn(bool);
    void eotStarted();
    void eotEnded();
    void timeToEot(uint32_t timetoeot);
private:
    //Setup
    void setupActions();
    void setupMenus(bool connected);
    void setupStatusBar();
    void setupGameWindows();
    void setupGame();
    void setupStatusWidget();
    bool setupGameList();


    //Game Objects
    GameLayer *game;
    ParsekEventLoop *eventloop;
    ParsekStatusListener *statuslistener;
    string serveraddress;
    string servergame;
    string clientuser;
    string clientpass;

    //Widgets + Models
    LoggerWidget *logger;
    StatusWidget *status;
    ObjectView *objectview;

    //Basic Interface Objects
    KAction *connectAction;
    KAction *quitAction;
    KAction *disconnectAction;
    KMenu *gameMenu;
    KMenu *help;
    QLabel *statusLabel;
    QLabel *timeLabel;
    KLed *connectionLed;

    //Clock Stuff
    QLabel *clockLabel;
    QLabel *clockDisplay;
    QTimer *clockUpdate;
    uint32_t clock;

};

#endif // MAINWINDOW_H
