/*
    Copyright 2008 Gábor Lehel <illissius@gmail.com>

    This file is part of Superpong.

    Superpong is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Superpong is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with Superpong (files COPYING and COPYING.LESSER).
    If not, see <http://www.gnu.org/licenses/>.
*/

#include <KAction>
#include <KLocale>
#include <KMenu>
#include <KMenuBar>
#include <KStandardAction>
#include <KStandardGameAction>

#define withgl

#ifdef withgl
#include <QGLWidget>
#endif

#include "pongwindow.h"
#include "pongapplication.h"
#include "pongscene.h"
#include "pongview.h"

PongWindow::PongWindow()
{
    PongScene *ps = PongScene::instance();
    PongView  *pv = new PongView( ps, this );
#ifdef withgl
    QGLFormat f;
    f.setSampleBuffers( true );
    pv->setViewport( new QGLWidget( f, pv ) );
#endif
    pv->setRenderHints( QPainter::Antialiasing | QPainter::HighQualityAntialiasing );
    pv->setBackgroundBrush( Qt::black );
    setCentralWidget( pv );

    KAction *newgame = KStandardGameAction::gameNew( ps, SLOT( newGame() ), this );
    KAction *quit = KStandardAction::quit( PongApplication::instance(), SLOT( quit() ), this );

    KMenu *file = new KMenu( i18n( "&File" ) );
    file->addAction( newgame );
    file->addSeparator();
    file->addAction( quit );
    menuBar()->addMenu( file );
    menuBar()->addMenu( helpMenu() );
}
